// Filter.cpp

#include "Filter.h"

#include "Common/Common.h"
#include "Common/ConvertString.h"
#include "Common/ParamList.h"
#include "Base/AppConfig.h"
#include "FileExclude/FileExclude.h"
#include "FileOpt/FileOpt.h"
#include "GDLog/GDLog.h"
#include "GDLog/GDTrace.h"
#include "gd3_define/gd3_errcode.h"

#include <sstream>
#include <algorithm>
#include <iostream>

using namespace std;

#ifndef NO_ERROR
#	define NO_ERROR 0
#endif


list<CMonitorInformation> CFilter::m_monitorList;
long CFilter::m_nVersion = 2;


CFilter::CFilter()
{
}

CFilter::~CFilter()
{
}

bool CFilter::Initialize( const char * sParams )
{
	//ɺtrace
	CGDTrace trace( LOG_L6, "CFilter:Initialize(%s)", sParams );
	
	bool bRet = true;
	//÷ָб
	// CParamList paramList( sParams, "|" );
	
	//MessageQueueKeyϢkeybufferС32kڽverifierData// by liu_yfeng 2009-01-05
	if ( NO_ERROR != m_messageQueue.Init( KEY_MEM_MQUE_VERIFIER, 512 ) ) // 512K
	{
		bRet = false;
		GDPrint( "Init() fail\n" );
		return trace.SetRetResult( bRet );
	}
	//if ( NO_ERROR != m_messageQueue.CreateQueue( KEY_MEM_MQUE_VERIFIER, 512 ) ) // 512K
	//{
	//	bRet = false;
	//	GDPrint( "CreateMemory() fail\n" );
	//	return trace.SetRetResult( bRet );
	//}
	
	//SharedBufferKeyǹڴkeybufferС32kʶverifiermonitorInfoͨ
	if ( NO_ERROR != m_sharedBuffer.CreateBuffer( KEY_MEM_MONITOR_VERIFIER, 32 ) ) // 32K
	{
		bRet = false;
		GDPrint( "CreateMemory() fail\n" );
		return trace.SetRetResult( bRet );
	}
	return trace.SetRetResult( bRet );
}


bool CFilter::Uninitialize()
{
	CGDTrace trace( LOG_L6, "CFilter:Uninitialize()" );
	
	bool bRet = true;
	//ɺtrace
	//release buffer
	if ( NO_ERROR != m_sharedBuffer.ReleaseBuffer() )
	{
		bRet = false;
		cout << "ReleaseBuffer error!" << endl;
		return trace.SetRetResult( bRet );
	}
	
	//release queue	// delete by liu_yfeng 2009-01-05
	//if ( NO_ERROR != m_messageQueue.ReleaseQueue() )
	//{
	//	bRet = false;
	//	cout << "ReleaseQueue error!" << endl;
	//	return trace.SetRetResult( bRet );
	//}
	return trace.SetRetResult( bRet );
}

bool CFilter::StartFilter()
{
	//ɺtrace
	CGDTrace trace( LOG_L6, "CFilter:StartFilter()" );
	return trace.SetRetResult( true );
}

bool CFilter::StopFilter()
{
	//ɺtrace
	CGDTrace trace( LOG_L6, "CFilter:StopFilter()" );
	return trace.SetRetResult( true );
}

bool CFilter::WatchDirectory( CMonitorInformation& moniInfo )
{
	CGDTrace trace( LOG_L6, "CFilter:WatchDirectory(%s)", moniInfo.m_monitorPath.c_str() );

	//ɺtrace
	string smoniInfo = toString(moniInfo);
	
	bool bRet = false;
	//ڱбһĿ¼
	list<CMonitorInformation>::iterator iter;
	iter = find(m_monitorList.begin(),m_monitorList.end(),moniInfo);
	if ( iter != m_monitorList.end() )
	{
		m_monitorList.remove(moniInfo);				
	}
	m_monitorList.push_back(moniInfo);
	
	//SharedBufferʽ֪ͨverifier¼Ŀ¼
	if ( SetSharedBufferToVerifier( m_monitorList ) )
		bRet = true;

	trace.SetRetResult( bRet );
	return bRet;
}

bool CFilter::UnwatchDirectory( const char * sDirId )
{
	//ɺtrace
	CGDTrace trace( LOG_L6, "CFilter:UnwatchDirectory(%s)", sDirId );

	bool bRet = false;
	//ȴӱбĿ¼
	list<CMonitorInformation>::iterator iter = m_monitorList.begin();
	while ( iter != m_monitorList.end() )
	{
		if ( (*iter).m_sDirId == sDirId )
			iter = m_monitorList.erase( iter ); // ע÷
		else
			++iter;
	}

	//SharedBufferʽ֪ͨverifier¼Ŀ¼
	if ( SetSharedBufferToVerifier( m_monitorList ) )
		bRet = true;

	trace.SetRetResult( bRet );
	return bRet;
}

bool CFilter::IsWatched( const char * sDirId )
{
	//ɺtrace
	CGDTrace trace( LOG_L6, "CFilter:IsWatched(%s)", sDirId );
	
	bool bRet = false;
	//IsWatchedᴥһļάбĲҲ
	list<CMonitorInformation>::iterator iter;
	for ( iter = m_monitorList.begin(); iter != m_monitorList.end(); iter++ )
	{
		if ( (*iter).m_sDirId == sDirId )
		{
			bRet = true;
		}
	}

	trace.SetRetResult( bRet );
	return bRet;
}

bool CFilter::IsWatchedByWebsiteId( const char * sWebsiteId )
{
	return true;
}

bool CFilter::ReadEvent( EVENT_S& event )
{
	// ͨѶ
	//ϢжһϢ,CMessageQueueһ¼۸ļָϢ
	int nSize = sizeof(EVENT_S);
	bool bHave = false;
	long nRet = m_messageQueue.GetMessageB( (void*)&event, nSize ); 
	if ( NO_ERROR == nRet )
	{
		//istringstream iss( p );
		//iss >> event;
		return true;
	}
	else if ( MSG_MEM_NOMSG != nRet )
	{
		// ɺtrace
		CGDTrace::GDTrace( LOG_ERR, "CFilter:ReadEvent() ret=[%d]", nRet );
	}
	
	return false;
}

bool CFilter::SetSharedBufferToVerifier( list<CMonitorInformation> &m_monitorList )
{
	// ־ʾϢ
	
	bool bRet = false;
#ifndef HPUX
	ostringstream oss; 
	oss << m_nVersion++ << endl;
	
	long num = m_monitorList.size();
	oss << num << endl;
	
	for ( list<CMonitorInformation>::iterator iter = m_monitorList.begin(); iter != m_monitorList.end(); ++iter )
	{
		oss << *iter;
	}
	
	//m_monitorListתstring
	string monitorString = "";
	monitorString = oss.str();
#else
	string monitorString;
	monitorString = CGDString::ltoa( 1 );
	monitorString = monitorString + " ";
	
	long num = m_monitorList.size();
	monitorString = monitorString + CGDString::ltoa( num );
	monitorString = monitorString + " ";
	
	for ( list<CMonitorInformation>::iterator iter = m_monitorList.begin(); iter != m_monitorList.end(); ++iter )
	{
		string tmp;
		(*iter).toString( tmp, (*iter) );
		monitorString += tmp;
	}
	CGDLog::WriteSysLog( "2037:HPUX Set Shared Buffer %s", monitorString.c_str() );
#endif

	//ɺtrace
	CGDTrace trace( LOG_L6, "CFilter:SetSharedBufferToVerifier(%s)", monitorString.c_str() );
	
	//ͨCSharedBuffer,֪ͨverifier¼Ŀ¼
	if ( NO_ERROR != m_sharedBuffer.SetData( (void*)monitorString.c_str(), monitorString.length() + 1 ) )
	{
		cout << "SetData() fail" << endl;
	} 
	else
	{
		bRet = true;
	}

	trace.SetRetResult( bRet );
	return bRet;
}

