//
// FindChange.cpp
// 


#include "FindChange.h"
#include "EventInfoScanner.h"
#include "PathTree.h"
#include "MonitorPathTree.h"
#include "SyncPathTree.h"

#include "Common/Common.h"
#include "Common/GDString.h"
#include "Common/GDTimer.h"
#include "GDLog/GDLog.h"
#include "GDLog/GDTrace.h"
#include "GDLog/LogFile.h"
#include "GDLog/ThreadData.h"
#include <time.h>
#include <iostream>

using namespace std;


namespace InforGuard{
namespace FileScanner{


CFindChange::CFindChange( CMonitorInformation& moniInfo, const long nSeconds )
: InforGuard::GDThreadPool::CThread( ( std::string( "FileScanner-" ) + moniInfo.m_monitorPath ).c_str() ), m_nSeconds( nSeconds ), m_pPathTree(NULL)
{
	m_moniInfo.Clone( moniInfo );
	m_isRunning = true;
}


CFindChange::~CFindChange()
{
}

bool CFindChange::setPolicy(CMonitorInformation& moniInfo)
{
    m_moniInfo.Clone( moniInfo );

    if( m_pPathTree )
    {
        m_pPathTree->setPolicy(moniInfo);
    }

    return true;
}

long CFindChange::Run( void* param )
{
	// scanner־
	CThreadData::SetCurrentThreadLogFile( "scan", true ); // ߳()־

	// һĿ¼
	if ( !strcmp( "ma", GetAppName() ) )
	{
		m_pPathTree = new CMonitorPathTree( m_moniInfo );
	}
	else
	{
		m_pPathTree = new CSyncPathTree( m_moniInfo );
	}

	if ( !m_pPathTree->isExist() )
	{
		CGDLog::WriteErrLog( "1067:CFindChange::Run -- Path[%s] is not exist", m_moniInfo.m_monitorPath.c_str() );
	}
	else
	{
		CGDTrace::GDTrace( LOG_INF, "CFindChange::Run watch(%s)", m_moniInfo.m_monitorPath.c_str() );
	}
    
	
    CGDLog::WriteThreadState("scan thread started");          //wu_binд߳״̬
	while( InforGuard::GDThreadPool::TS_STOPING != m_status )
	{
		
		try
		{
			CGDLog::WriteThreadState("scan task started");
			m_pPathTree->Scan();
			CGDLog::WriteThreadState("scan task end");
		}
		catch( ... )
		{
			// 쳣
			CGDLog::WriteErrLog( "1068:CFindChange:Run catch ..." );
		}
		for ( int i=0; i<m_nSeconds && InforGuard::GDThreadPool::TS_STOPING != m_status; i++ )
		{
			CDateTime::Sleep( 1000 );
		}		
	}
	CGDLog::WriteThreadState("scan thread exited");          //wu_binд߳״̬

	if ( m_pPathTree )
	{
		delete m_pPathTree;
		m_pPathTree = NULL;
	}
	CGDTrace::GDTrace( LOG_INF, "CFindChange::Run exit watch(%s)", m_moniInfo.m_monitorPath.c_str() );

	return 0;
}


}	// namespace FileScanner
}	// namespace InforGuard






