/***********************************************************************
*
* ļ        GDLockWin32.cpp
* ģĿģʵֻWindowsƽ̨ļ໥̻߳⡢̻⣩
* ߼ڣ  wang_jin 2008-05-05
* 汾          1.0
* ȨϢ      CVICSE Middleware
* ļ¼	  wang_jin 2008-07-05 д
*
***********************************************************************/

#include <stdio.h> // for sprintf()
#include <assert.h> // for assert()

#include "gd3_define/gd3_errcode.h"
#include "GDLock/GDLockWin32.h"
#include "Common/ExceptionError.h"


// ̻߳

CThreadMutex::CThreadMutex()
{
	InitializeCriticalSection( &m_mutex );
}

CThreadMutex::~CThreadMutex()
{
	DeleteCriticalSection( &m_mutex );
}

void CThreadMutex::Lock()
{
	try
	{
		EnterCriticalSection( &m_mutex );
	}
	catch ( ... )
	{
		CExceptionError ex;
		ex.m_nErrCode = GetLastError();
		ex.AddErrText( "ϵͳ: EnterCriticalSection()[%d][%s]", ex.m_nErrCode, GetErrText(ex.m_nErrCode) ); 
		throw ex;
	}
}

void CThreadMutex::Unlock()
{
	LeaveCriticalSection( &m_mutex ); // [2008/6/17 2320]ڴ쳣
}


// ̻

CProcessMutex::CProcessMutex()
{
	m_mutex = INVALID_HANDLE_VALUE;
	m_nKey = 0;
}

CProcessMutex::CProcessMutex( long nKey )
{
	m_mutex = INVALID_HANDLE_VALUE;
	m_nKey = nKey;
}

CProcessMutex::~CProcessMutex()
{}

long CProcessMutex::Initialize( long nKey )
{
	DWORD dwErr = 0;

	// ȫ
    char domainBuffer[80] = { 0 };
    SID_NAME_USE snu;

	memset(m_aclBuffer, 0 ,sizeof(m_aclBuffer));
	m_pacl = (PACL)&m_aclBuffer;
	memset(m_sidBuffer, 0, sizeof(m_sidBuffer));
	m_psid = (PSID)&m_sidBuffer;

	DWORD sidBufferSize = sizeof(m_sidBuffer);
	DWORD domainBufferSize = sizeof(domainBuffer);

	BOOL bRet = InitializeSecurityDescriptor(&m_sd, SECURITY_DESCRIPTOR_REVISION);
	if ( ! bRet )
	{
		dwErr = GetLastError();
		//CGDLog::WriteErrLog( "InitializeSecurityDescriptor() return false" );
		GDPrint( "InitializeSecurityDescriptor() return false errno=[%d]", dwErr );
		return dwErr;
	}
	bRet = InitializeAcl(m_pacl, 1024, ACL_REVISION);
	if ( ! bRet )
	{
		dwErr = GetLastError();
		//CGDLog::WriteErrLog( "InitializeAcl() return false" );
		GDPrint( "InitializeAcl() return false errno=[%d]", dwErr );
		return dwErr;
	}
	bRet = LookupAccountName(0, "Everyone", m_psid, &sidBufferSize, domainBuffer,
		&domainBufferSize, &snu);
	if ( ! bRet )
	{
		dwErr = GetLastError();
		//CGDLog::WriteErrLog( "LookupAccountName() return false" );
		GDPrint( "LookupAccountName() return false errno=[%d]", dwErr );
		return dwErr;
	}
	bRet = AddAccessAllowedAce(m_pacl, ACL_REVISION, GENERIC_ALL, m_psid);
	if ( ! bRet )
	{
		dwErr = GetLastError();
		//CGDLog::WriteErrLog( "AddAccessAllowedAce() return false" );
		GDPrint( "AddAccessAllowedAce() return false errno=[%d]", dwErr );
		return dwErr;
	}
	bRet = SetSecurityDescriptorDacl(&m_sd, TRUE, m_pacl, FALSE);
	if ( ! bRet )
	{
		dwErr = GetLastError();
		//CGDLog::WriteErrLog( "SetSecurityDescriptorDacl() return false" );
		GDPrint( "SetSecurityDescriptorDacl() return false errno=[%d]", dwErr );
		return dwErr;
	}
	m_sa.nLength = sizeof(SECURITY_ATTRIBUTES);
	m_sa.bInheritHandle = FALSE;
	m_sa.lpSecurityDescriptor = &m_sd;

	// 
	m_nKey = nKey;
	char szKey[32] = {0};
	sprintf( szKey, "InforGuardLock%x", nKey );
	m_mutex = CreateMutex(&m_sa, FALSE, szKey);
	if ( INVALID_HANDLE_VALUE == m_mutex )
	{
		dwErr = GetLastError();
		//CGDLog::WriteErrLog( "SetSecurityDescriptorDacl() return false" );
		GDPrint( "CreateMutex() return -1 errno=[%d]", dwErr );
		return dwErr;
	}

	return dwErr;
}

bool CProcessMutex::Uninitialize()
{
    if ( INVALID_HANDLE_VALUE != m_mutex )
	{
        ReleaseMutex( m_mutex );
        CloseHandle( m_mutex );
    }

	return true;
}

bool CProcessMutex::Lock( unsigned int ms )
{
    if ( INVALID_HANDLE_VALUE == m_mutex )
	{
		if ( Initialize( m_nKey ) )
		{
			return false;
		}
	} 
	
	DWORD dwRtn = WaitForSingleObject( m_mutex, ms );

	switch ( dwRtn )
	{
	case WAIT_OBJECT_0:
	case WAIT_ABANDONED:
		return true;
	case WAIT_FAILED:
		return false;
	case WAIT_TIMEOUT:
		CloseHandle(m_mutex);
		return false;
	default:
		assert(0);
	}

	return false;
}

bool CProcessMutex::Unlock()
{
    if ( INVALID_HANDLE_VALUE == m_mutex )
	{
		return false;
	} 
	
	if ( ReleaseMutex( m_mutex ) )
	{
		return true;
	}
	else
	{
		return false;
	}
}


// д(̼)(дȵ)

const int MAXCOUNT = 0x12345678; // 㹻ĹԴĿ

CProcessRWMutex::CProcessRWMutex()
{
	m_hRWMutex = NULL;
	m_hRMutex = NULL;
	m_hWMutex = NULL;
	m_hCountMutex = NULL;
}

CProcessRWMutex::CProcessRWMutex( long nKey )
{
	m_hRWMutex = NULL;
	m_hRMutex = NULL;
	m_hWMutex = NULL;
	m_hCountMutex = NULL;
	m_nKey = nKey;
}

CProcessRWMutex::~CProcessRWMutex()
{
	if ( m_hRWMutex )
	{
		CloseHandle( m_hRWMutex );
	}
	if ( m_hRMutex )
	{
		CloseHandle( m_hRMutex );
	}
	if ( m_hWMutex )
	{
		CloseHandle( m_hWMutex );
	}

	if ( m_hCountMutex )
	{
		CloseHandle( m_hCountMutex );
	}
}
	
long CProcessRWMutex::Initialize( long nKey )
{
	// ȫ
	char domainBuffer[80] = { 0 };
	SID_NAME_USE snu;

	memset(m_aclBuffer, 0 ,sizeof(m_aclBuffer));
	m_pacl = (PACL)&m_aclBuffer;
	memset(m_sidBuffer, 0, sizeof(m_sidBuffer));
	m_psid = (PSID)&m_sidBuffer;

	DWORD sidBufferSize = sizeof(m_sidBuffer);
	DWORD domainBufferSize = sizeof(domainBuffer);

	BOOL bRet = InitializeSecurityDescriptor(&m_sd, SECURITY_DESCRIPTOR_REVISION);
	if ( ! bRet )
	{
		//CGDLog::WriteErrLog( "InitializeSecurityDescriptor() return false" );
		GDPrint( "InitializeSecurityDescriptor() return false" );
		return -1;
	}
	bRet = InitializeAcl(m_pacl, 1024, ACL_REVISION);
	if ( ! bRet )
	{
		//CGDLog::WriteErrLog( "InitializeAcl() return false" );
		GDPrint( "InitializeAcl() return false" );
		return -2;
	}
	bRet = LookupAccountName(0, "Everyone", m_psid, &sidBufferSize, domainBuffer,
		&domainBufferSize, &snu);
	if ( ! bRet )
	{
		//CGDLog::WriteErrLog( "LookupAccountName() return false" );
		GDPrint( "LookupAccountName() return false" );
		return -3;
	}
	bRet = AddAccessAllowedAce(m_pacl, ACL_REVISION, GENERIC_ALL, m_psid);
	if ( ! bRet )
	{
		//CGDLog::WriteErrLog( "AddAccessAllowedAce() return false" );
		GDPrint( "AddAccessAllowedAce() return false" );
		return -4;
	}
	bRet = SetSecurityDescriptorDacl(&m_sd, TRUE, m_pacl, FALSE);
	if ( ! bRet )
	{
		//CGDLog::WriteErrLog( "SetSecurityDescriptorDacl() return false" );
		GDPrint( "SetSecurityDescriptorDacl() return false" );
		return -5;
	}
	m_sa.nLength = sizeof(SECURITY_ATTRIBUTES);
	m_sa.bInheritHandle = FALSE;
	m_sa.lpSecurityDescriptor = &m_sd;

	// 
	char sName[32] = {0};
	sprintf( sName, "InforGuardRWLock%x01", nKey );
	m_hRWMutex = CreateEvent( &m_sa, FALSE, TRUE, sName );
	if ( NULL == m_hRWMutex )
	{
		GDPrint( "CreateEvent1 error: [%d]\n", GetLastError()); 
		return -6;
	}

	sprintf( sName, "InforGuardRWLock%x02", nKey );
	m_hRMutex = CreateEvent( &m_sa, FALSE, TRUE, sName );
	if ( NULL == m_hRMutex )
	{
		GDPrint( "CreateEvent2 error: [%d]\n", GetLastError());
		return -7;
	}

	sprintf( sName, "InforGuardRWLock%x03", nKey );
	m_hWMutex = CreateEvent( &m_sa, FALSE, TRUE, sName );
	if ( NULL == m_hWMutex )
	{
		GDPrint( "CreateEvent3 error: [%d]\n", GetLastError());
		return -8;
	}

	/*
	sprintf( sName, "InforGuardRWLock%x03", nKey );
	m_hWMutex = CreateSemaphore( &m_sa, MAXCOUNT, 2*MAXCOUNT, sName ); 
	if ( NULL == m_hWMutex )
	{
		GDPrint( "CreateSemaphore error: [%d]\n", GetLastError());
		return -8;
	}
	*/

	sprintf( sName, "InforGuardRWLock%x04", nKey );
	m_hCountMutex = CreateSemaphore( &m_sa, 0, MAXCOUNT, sName ); 
	if ( NULL == m_hCountMutex )
	{
		GDPrint( "CreateSemaphore error: [%d]\n", GetLastError());
		return -9;
	}

	return 0;
}

long CProcessRWMutex::Uninitialize()
{
	if ( m_hRWMutex )
	{
		CloseHandle( m_hRWMutex );
		m_hRWMutex = NULL;
	}
	if ( m_hRMutex )
	{
		CloseHandle( m_hRMutex );
		m_hRMutex = NULL;
	}
	if ( m_hWMutex )
	{
		CloseHandle( m_hWMutex );
		m_hWMutex = NULL;
	}

	if ( m_hCountMutex )
	{
		CloseHandle( m_hCountMutex );
		m_hCountMutex = NULL;
	}

	return 0;
}

long CProcessRWMutex::Lock( enumLockMode LockMode, unsigned long ms )
{
	// ûгʼʼ
	if ( NULL == m_hRWMutex )
	{
		long nRet = Initialize( m_nKey );
		if ( nRet )
		{
			return nRet;
		}
	}

	if ( LOCK_EXCL == LockMode )
	{
		P( m_hRWMutex );
		P( m_hWMutex );
	}
	else // ( LOCK_SHAR == LockMode )
	{
		P( m_hRWMutex );
		P( m_hRMutex );
		if ( INC() == 1 )
		{
			P( m_hWMutex );
		}
		V( m_hRMutex );
		V( m_hRWMutex );
	}
	
	m_LockMode = LockMode;
	return 0;
}


long CProcessRWMutex::Unlock()
{
	if ( LOCK_EXCL == m_LockMode )
	{
		V( m_hWMutex );
		V( m_hRWMutex );
	}
	else // ( LOCK_SHAR == m_LockMode )
	{
		P( m_hRMutex );
		if ( DEC() == 0 )
		{
			V( m_hWMutex );
		}
		V( m_hRMutex );
	}
	
	return NO_ERROR;
}


long CProcessRWMutex::P( HANDLE hMutex )
{
	if ( INVALID_HANDLE_VALUE == hMutex )
	{
		return -100;
	} 
	
	DWORD dwRtn = WaitForSingleObject( hMutex, INFINITE );
	
	switch ( dwRtn )
	{
	case WAIT_OBJECT_0:
	case WAIT_ABANDONED:
		return 0;
	case WAIT_FAILED:
		return -100;
	case WAIT_TIMEOUT:
		CloseHandle(hMutex);
		return INFINITE;
	default:
		assert(0);
	}
	
	return -1;
}


long CProcessRWMutex::V( HANDLE hMutex )
{
	if ( INVALID_HANDLE_VALUE == hMutex )
	{
		return -1;
	} 
	
	if ( SetEvent( hMutex ) )
	{
		return 0;
	}
	else
	{
		return -1;
	}
}

long CProcessRWMutex::INC()
{	
	long nPrevCount = -1;
	BOOL bRet = ReleaseSemaphore( m_hCountMutex, 1, &nPrevCount );
	return bRet ? nPrevCount + 1 : (-1);
}

long CProcessRWMutex::DEC()
{	
	long nPrevCount = -1;
	BOOL bRet = ReleaseSemaphore( m_hCountMutex, 1, &nPrevCount );
	WaitForSingleObject( m_hCountMutex, INFINITE );
	WaitForSingleObject( m_hCountMutex, INFINITE );
	return bRet ? nPrevCount - 1 : (-1);
}

// Ŀǰûʵ֣ΪWindowsϲ漰̼¼־
int CProcessRWMutex::SetValue( int nIndex, int nValue )
{
	return 0;
}

int CProcessRWMutex::GetValue( int nIndex )
{
	return 0;
}
