// IISFILTER.CPP - Implementation file for your Internet Server
//    IIS Filter

#include "stdafx.h"
#include "IISFilter.h"

#include "GDVerifier/GDVerifier.h"

#define DEFAULT_TMPLOG_FILENAME "gd-iis.log"
#define TIME_LENGTH 32

bool bLog = true;

void TmpLog( const char* fmt, ... )
{
	if ( bLog )
	{
		char tmplog_path[ MAX_PATH ] = {0};
		FILE *tmp_logfile = NULL;
		
#ifndef _WIN32
		sprintf( tmplog_path, "/tmp/%s", DEFAULT_TMPLOG_FILENAME );
		tmp_logfile = fopen(tmplog_path, "a+");
#else
		sprintf( tmplog_path, "C:\\tmp\\%s", DEFAULT_TMPLOG_FILENAME );
		tmp_logfile = fopen( tmplog_path, "a+" );
#endif
		
		if( tmp_logfile )
		{
			char time_string[ TIME_LENGTH ] = {0};
			time_t nowtime = time( NULL );
			
			va_list args;
			va_start( args, fmt );
			
			// Format is month day hour:minute:second (24 time) 
			strftime( time_string, TIME_LENGTH, "%b %d %H:%M:%S", localtime(&nowtime) );
			
#if defined _WIN32
			fprintf( tmp_logfile, "%s [%u] - ", time_string, GetCurrentThreadId() );
#else
			fprintf( tmp_logfile, "%s [%u] - ", time_string, pthread_self() );
#endif
			
			vfprintf( tmp_logfile, fmt, args );
			va_end(args);
			
			fprintf( tmp_logfile, "\n" );
			fclose(tmp_logfile);
			tmp_logfile = NULL;
		}
	}
}

//<<TEST
#include <windows.h>
#include <comdef.h>
#include <initguid.h>
#include <objbase.h>
#include <iads.h>
#include <adshlp.h>
#include <string>
#include <vector>
using namespace std;

#pragma comment(lib,"ActiveDS.lib") 
#pragma comment(lib,"Adsiid.lib")
#pragma comment(lib,"comsupp.lib")


#include <iostream>

bool GetSiteDefaultDoc(BSTR& aDpath, string& sDirPath, string& sDefaultDoc)
{
	HRESULT hr;
	IADs *pSite = NULL;
	bool bSuccess = false;

	hr = ADsGetObject(aDpath, IID_IADs, (void**)&pSite);
	if ( SUCCEEDED(hr) )
	{
		//Virtual Directory*****************
		char sPath[256] = "";
		_bstr_t bText = aDpath;
		char* lpszText = bText;
		
		sprintf(sPath, "%s/ROOT", lpszText);

		IADs *pVDir;
		BSTR bstrPath = _bstr_t(sPath); 
		hr = ADsGetObject(bstrPath,IID_IADs, (void**)&pVDir);
		if ( SUCCEEDED(hr) )
		{
			_variant_t var;

			pVDir->Get(L"Path",&var);
			bText = var.bstrVal;
			lpszText = bText;
			sDirPath = lpszText;
			
			pVDir->Get(L"DefaultDoc",&var);
			bText = var.bstrVal;
			lpszText = bText;
			sDefaultDoc = lpszText;
			
			//**************
			VariantClear(&var);
			pVDir->Release();
			pVDir = NULL;

			bSuccess = true;
		}

		SysFreeString(bstrPath);

		pSite->Release();
		pSite = NULL;
	}

	TmpLog( "GetSiteDefaultDoc() return [%s]", bSuccess?"true":"false" );

	return bSuccess;
}

string GetHomePage( const char* homePage, int index )
{
	const char* p = homePage;
	int n = index;
	while ( n > 0 && p && *p )
	{
		p = strchr(p,',');
		if ( p && *p )
		{
			p++;
		}
		n--;
	}
	if ( p && *p )
	{
		string page = p;
		int nPos = page.find_first_of( "," );
		if ( nPos != std::string::npos )
		{
			page = page.substr( 0, nPos );
		}
		return page;
	}
	return "";
}

long GetDefaultDoc( vector<HomePage>& vDefaultDoc )
{
	HRESULT hr;
	IADs *pADs = NULL;
	
	hr = CoInitialize( NULL );
	if ( FAILED( hr ) )
	{
		TmpLog( "CoInitialize failed. Error 0x%0x", hr );
		return -1;
	}
	else
	{
		TmpLog( "CoInitialize OK" );
	}

	IADsContainer *pServer = NULL;

	TmpLog( "step1" );
	hr = ADsGetObject(L"IIS://localhost/w3svc",IID_IADsContainer, (void**)&pServer);
	if (!SUCCEEDED(hr))
	{
		TmpLog("Error Connecting to IIS! Is IIS Installed? Is IIS running?");
		return -1;
	}
	else
	{
		TmpLog( "ADsGetObject OK" );
	}

	IEnumVARIANT *pEnum = NULL;
	hr = ADsBuildEnumerator(pServer, &pEnum );
	if (SUCCEEDED(hr))
	{
		_variant_t vSite;
		ULONG lFetch;
		IADs *pSite=NULL;

		while(SUCCEEDED(ADsEnumerateNext( pEnum, 1, &vSite, &lFetch )) && lFetch == 1)
		{
			int item = 0;
			IDispatch *pDisp;
			pDisp = V_DISPATCH(&vSite);
			hr = pDisp->QueryInterface(IID_IADs, (void**) &pSite);
			if (SUCCEEDED(hr))
			{
				BSTR bstrClass;

				pSite->get_Class(&bstrClass);

				_bstr_t bText = bstrClass;
				char* lpszText = bText;

				if(strcmp(lpszText,"IIsWebServer")==0)
				{
					_variant_t	var;
					BSTR aDpath;
					
					pSite->get_ADsPath(&aDpath);
					//pSite->get_Name(&tSite.name);
					//pSite->Get(L"ServerComment",&var);
					//tSite.comment = var.bstrVal;
					//pSite->Get(L"ServerState",&var);
					//tSite.state = var.lVal;
					//pSite->Get(L"ServerBindings",&var);

					string sDirPath;
					string sDefaultDoc;

					if (GetSiteDefaultDoc(aDpath, sDirPath, sDefaultDoc))
					{
						HomePage hp;
						hp.path = sDirPath + "\\";
						// 
						for ( int i=0; ; i++ )
						{
							string file = GetHomePage( sDefaultDoc.c_str(), i );
							if ( file.empty() )
								break;
							hp.file.push_back( file );
							TmpLog( "GetSiteDefaultDoc[%d]=[%s]", i, file.c_str() );
						}
						vDefaultDoc.push_back( hp );
					}

					VariantClear(&vSite);
					SysFreeString(aDpath);
				}

				SysFreeString(bstrClass);

				pSite->Release();
				pSite = NULL;
				VariantClear(&vSite);
			}
		}
	}
	else
	{
		TmpLog("Error enumerating sites!");
	}

	if (pEnum)
	{
		ADsFreeEnumerator(pEnum);
	}

	pServer->Release();
	pServer = NULL;

	TmpLog( "GetDefaultDoc() return [%d] with docs=[%d]", hr, vDefaultDoc.size() );

	CoUninitialize();
	return hr;
}

bool GetHomePages( vector<HomePage>& vDefaultDoc, const char* path, vector<string>& homePages )
{
	int nSize = vDefaultDoc.size();
	for ( int i=0; i<nSize; i++ )
	{
		TmpLog( "GetHomePages [%s][%s]", vDefaultDoc[i].path.c_str(), path );
		if ( vDefaultDoc[i].path == path )
		{
			homePages = vDefaultDoc[i].file;
			return true;
		}
	}
	return false;
}
//>>

///////////////////////////////////////////////////////////////////////
// The one and only CWinApp object
// NOTE: You may remove this object if you alter your project to no
// longer use MFC in a DLL.

CWinApp theApp;

///////////////////////////////////////////////////////////////////////
// The one and only CIISSqlFilter object

CIISFilter theFilter;

///////////////////////////////////////////////////////////////////////
// CIISFilter implementation

CIISFilter::CIISFilter()
{
	m_nCount = 9;

	TmpLog( "IIS۸Ĺ-Begin" );

#if 0
	//<<Guard Code
	long nRet = GetDefaultDoc( m_defaultDoc );
	if ( 0 == nRet )
	{
		for ( int i=0; i<m_defaultDoc.size(); i++ )
		{
			for ( int ii=0; ii<m_defaultDoc[i].file.size(); ii++ )
			{
				TmpLog( "DefaultDocument:[%d][%s][%d][%s]", i, m_defaultDoc[i].path.c_str(), ii, m_defaultDoc[i].file[ii].c_str() );
			}
		}
	}
	else
	{
		TmpLog( "GetDefaultDoc() return [%d]", nRet );
	}
	//>>
#endif
	TmpLog( "IIS۸Ĺ-OK" );
}

CIISFilter::~CIISFilter()
{
	TmpLog( "IIS۸Ĺر" );
}

BOOL CIISFilter::GetFilterVersion(PHTTP_FILTER_VERSION pVer)
{
	// Call default implementation for initialization
	CHttpFilter::GetFilterVersion(pVer);

	// Clear the flags set by base class
	pVer->dwFlags &= ~SF_NOTIFY_ORDER_MASK;

	// Set the flags we are interested in
	pVer->dwFlags |= SF_NOTIFY_ORDER_LOW | SF_NOTIFY_NONSECURE_PORT | SF_NOTIFY_URL_MAP;

	// Load description CString
	TCHAR sz[SF_MAX_FILTER_DESC_LEN+1] = {0};
	ISAPIVERIFY(::LoadString(AfxGetResourceHandle(),
			IDS_FILTER, sz, SF_MAX_FILTER_DESC_LEN));
	_tcscpy(pVer->lpszFilterDesc, sz);

	return TRUE;
}

DWORD CIISFilter::OnUrlMap(CHttpFilterContext* pCtxt,
						   PHTTP_FILTER_URL_MAP pMapInfo)
{
	//<<**** InforGuard Code ****
	if ( m_nCount > 0 )
	{
		m_nCount --;
		if ( m_nCount == 0 )
		{
			bLog = false;
		}
	}

	TmpLog( "OnUrlMap(%s) begin", pMapInfo->pszPhysicalPath );

	static CVerifier* verifier = CVerifier::Instance();

	char sAppRooPath[ MAX_PATH ] = {0};
	DWORD dwSize = sizeof( sAppRooPath );
	DWORD ret = pCtxt->GetServerVariable( "APPL_PHYSICAL_PATH", sAppRooPath, &dwSize ); 

	int nCode = 0;

	// ֤
	if ( pMapInfo->pszPhysicalPath[ strlen(pMapInfo->pszPhysicalPath) - 1 ] == '\\' )
	{
		// ĬҳĿ¼ҳ
		TmpLog( "Ĭҳ[%s][%s]", pMapInfo->pszPhysicalPath, sAppRooPath );
		// ȡĬҳ
		vector<string> homePages;
		if ( GetHomePages( m_defaultDoc, sAppRooPath, homePages ) )
		{
			// ֤ҳ
			string sHomePage;
			string sHomeFile;
			for ( int i=0; i<homePages.size(); i++ )
			{
				sHomePage = pMapInfo->pszPhysicalPath;
				sHomePage += homePages[i];
				TmpLog( "֤ҳ[%d][%s]", i, sHomePage.c_str() );
				if ( i == 0 )
				{
					nCode = verifier->Verify( 'I', sHomePage.c_str(), sAppRooPath );
				}
				else if ( nCode == 40404 ) // 40404 ʾûˮӡ
				{
					nCode = verifier->Verify( 'I', sHomePage.c_str(), sAppRooPath );
				}
				else
				{
					verifier->Verify( 'I', sHomePage.c_str(), sAppRooPath );
				}
			}
		}
		else
		{
			TmpLog( "OnUrlMap(%s) ȡĬҳʧ[%s]", pMapInfo->pszPhysicalPath, sAppRooPath );
			nCode = verifier->Verify( 'I', pMapInfo->pszPhysicalPath, sAppRooPath );
		}
	}
	else
	{
		// ָҳ
		TmpLog( "ָҳ[%s]", pMapInfo->pszPhysicalPath, sAppRooPath );
		nCode = verifier->Verify( 'I', pMapInfo->pszPhysicalPath, sAppRooPath );
	}

	if ( nCode )
	{
		TmpLog( "OnUrlMap(%s)-Verify() Deny Access", pMapInfo->pszPhysicalPath );

		HackResponse(pCtxt, pMapInfo);
		TmpLog( "OnUrlMap(%s) return KO", pMapInfo->pszPhysicalPath );

		return SF_STATUS_REQ_FINISHED_KEEP_CONN;
	}

	TmpLog( "OnUrlMap(%s) return OK", pMapInfo->pszPhysicalPath );
	//>>**** InforGuard Code ****

	return CHttpFilter::OnUrlMap(pCtxt, pMapInfo);
}


//liu_yfeng 2009-05-22
#define CR '\r'
#define LF '\n'
#define CRLF "\r\n"
#define CRLFCRLF "\r\n\r\n"
#define CONTEXT_NEWREQUEST			NULL
void CIISFilter::HackResponse(CHttpFilterContext* pfc, PHTTP_FILTER_URL_MAP pMapInfo)
{
	static CVerifier* verifier = CVerifier::Instance();
	char* errMessage = (char*) verifier->GetErrMessage();
	int len = strlen( errMessage );
	char nTemp[32] = {0};
	sprintf( nTemp, "%d", len );

	CString buf("HTTP/1.1 ");
	buf += "200 OK";
	_tzset();
	CString d = CTime::GetCurrentTime().FormatGmt("%a, %d %b %Y %H:%M:%S GMT");
	buf += "\r\nDate: " + d;
	buf += "\r\nContent-Type: text/html; charset=GB2312";
	buf += "\r\nContent-Length: ";
	buf += nTemp;			// ûֵĻIEʾհҳ
	buf += "\r\nPragma: no-cache";
	buf += "\r\nCache-control: no-cache";
	buf += "\r\nExpires: " + d;
	buf += CRLFCRLF;
	
	buf += errMessage;
	DWORD szbuf(buf.GetLength());
	pfc->WriteClient(buf.GetBuffer(0),&szbuf);
	pfc->m_pFC->pFilterContext = CONTEXT_NEWREQUEST; //reset for OnReadRawData

	/*	// ǿת
	CHAR szRedirect[1024] = {0};
	CHAR szServerName[256] = {0};
	ULONG nSize = sizeof(szServerName);
	pfc->GetServerVariable( "SERVER_NAME", szServerName, &nSize );
	CHAR szServerPort[256] = {0};
	nSize = sizeof(szServerPort);
	pfc->GetServerVariable( "SERVER_PORT", szServerPort, &nSize );
	// replace URL with desired server
	

	const char* pErrPage = verifier->GetErrPage();
	if ( !pErrPage || !(*pErrPage) )
	{
		pErrPage = "forbidden.html";
	}
	
	sprintf( szRedirect, "Location: http://%s:%s/%s\r\n\r\n", szServerName, szServerPort, pErrPage );
	
	// ǰҳǴҳ棬Ҫضˣֱӱ
	if ( strstr( pMapInfo->pszPhysicalPath, pErrPage ) )
	{
		TmpLog( "OnUrlMap(%s) Ignore Redirect", pMapInfo->pszPhysicalPath );
	//	return SF_STATUS_REQ_ERROR;
	}
	
	BOOL bRet = pfc->ServerSupportFunction ( SF_REQ_SEND_RESPONSE_HEADER,
		(LPVOID) "302 Redirect", 
		(DWORD *) szRedirect, 
		0 );
	if ( ! bRet )
	{
		DWORD dwErr = GetLastError();
		TmpLog( "Redirect fail: [%d]", dwErr ); 
	}
	else
	{
		TmpLog( "OnUrlMap(%s) Redirect(%s)", pMapInfo->pszPhysicalPath, szRedirect );
	}
	// we are done with this request
	*/
}

// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CIISFilter, CHttpFilter)
	//{{AFX_MSG_MAP(CIISFilter)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

///////////////////////////////////////////////////////////////////////
// If your extension will not use MFC, you'll need this code to make
// sure the extension objects can find the resource handle for the
// module.  If you convert your extension to not be dependent on MFC,
// remove the comments arounn the following AfxGetResourceHandle()
// and DllMain() functions, as well as the g_hInstance global.

/****

static HINSTANCE g_hInstance;

HINSTANCE AFXISAPI AfxGetResourceHandle()
{
	return g_hInstance;
}

BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ulReason,
					LPVOID lpReserved)
{
	if (ulReason == DLL_PROCESS_ATTACH)
	{
		g_hInstance = hInst;
	}

	return TRUE;
}

****/

/*
IISص[2008-08-19]
ļĬҳΣ(c:\inetpub\wwwroot\,c:\inetpub\wwwroot\)(c:\inetpub\wwwroot\Default.htm,)
ļĬҳһΣ(c:\inetpub\wwwroot\default.htm,c:\inetpub\wwwroot\)
ļʷĬҳһΣ(c:\inetpub\wwwroot\index.htm,c:\inetpub\wwwroot\)
ļʷĬҳһΣ(c:\inetpub\wwwroot\hi.html,c:\inetpub\wwwroot\)
ܽ᣺ΪʱǰͬΪʱ֤Ĭҳֻ֤ǰ
˵֤̰ݹ
*/
