/***********************************************************************
*
* ļ        MessageQueue.cpp
* ģĿģʵϢ
* ߼ڣ  wang_jin 2008-05-05
* 汾          1.0
* ȨϢ      CVICSE Middleware
* ļ¼  
*
***********************************************************************/

#include "DataManager.h"
#include "MemoryManager.h"

#include "GDLock/GDLock.h"
#include "GDLog/GDLog.h"
#include "GDLog/GDTrace.h"
#include "GDQue/MessageQueue.h"

CMessageQueue::CMessageQueue()
:m_pDataMgr( new CSharedDataManager )
,m_pProcessMutex( new CProcessMutex )
{
	m_pThreadMutex = NULL;
	m_pThreadMutex = new CThreadMutex();
	m_bCreate = false;
}

CMessageQueue::~CMessageQueue()
{
	if ( m_pDataMgr )
	{
		m_pDataMgr->ReleaseMemory();
		delete m_pDataMgr;
		m_pDataMgr = NULL;
	}
	if ( m_pThreadMutex )
	{
		delete m_pThreadMutex;
	}
}

long CMessageQueue::Init( long nKey, long nSize )
{
	CGDTrace trace( LOG_L6, "CMessageQueue:Init(%x,%d)", nKey, nSize);
	m_nKey = nKey;
	m_nSize = nSize;
	CThreadLock lock( *m_pThreadMutex );
	long nRet = m_pProcessMutex->Initialize( m_nKey ); // ź빲ڴͬKEY
	if ( nRet )
	{
		CGDLog::WriteErrLog( "1109:CMessageQueue::Init(%x,%d) Initialize Mutex failed[%d]", m_nKey, m_nSize, nRet );
   		return trace.SetRetValue( nRet );
	}
	return nRet;
}

long CMessageQueue::CreateQueue( long nKey, long nSize )
{
    CGDTrace trace( LOG_L6, "CMessageQueue:CreateQueue(%x,%d)", nKey, nSize);	// del by liu_yfeng 2009-01-09

	long nRet = 0;

//	CProcessLock lock( *m_pProcessMutex );
	nRet = m_pDataMgr->CreateMemory( nKey, nSize );
    if ( nRet )
    {
        CGDLog::WriteErrLog( "1110:CMessageQueue::CreateQueue(%x,%d) ret=[%d]", nKey, nSize, nRet );
		return trace.SetRetValue( nRet );
    }

	return trace.SetRetValue( nRet );
}

long CMessageQueue::ReleaseQueue()
{
	CGDTrace trace( LOG_L6, "CMessageQueue:ReleaseQueue()" );	// del by liu_yfeng 2009-01-09
//	CProcessLock lock( *m_pProcessMutex );
	long nRet = m_pDataMgr->ReleaseMemory();
	return trace.SetRetValue( nRet );
}

long CMessageQueue::PutMessage( void* pMsg, int nSize )
{
	CGDTrace trace( LOG_L6, "CMessageQueue:PutMessage(%d)", nSize );
	CProcessLock lock( *m_pProcessMutex );
	if ( !m_bCreate )
	{
		if ( NO_ERROR == CreateQueue( m_nKey, m_nSize ) )
		{
			m_bCreate = true;
		}
	}
	
	long nRet = m_pDataMgr->PutBlock( pMsg, nSize );
	//ReleaseQueue();
	return trace.SetRetValue( nRet );
}

long CMessageQueue::GetMessageB( void* pMsg, int& nSize )
{
	CProcessLock lock( *m_pProcessMutex );
	if ( !m_bCreate )
	{
		if ( NO_ERROR == CreateQueue( m_nKey, m_nSize ) )
		{
			m_bCreate = true;
		}
	}
	long nRet = m_pDataMgr->GetBlock( pMsg, nSize );
	//ReleaseQueue();
	return nRet;
}

bool CMessageQueue::IsOwner()
{
	CGDTrace trace( LOG_L6, "CMessageQueue:IsOwner()" );
	return trace.SetRetResult( m_pDataMgr->IsOwner());
}

int CMessageQueue::GetOffset( void* pBuf )					// Чڴַתƫ
{
	CGDTrace trace( LOG_L6, "CMessageQueue:GetOffset()" );
    if( pBuf )
    {
		int nRet = m_pDataMgr->GetOffset( pBuf );
	    return trace.SetRetValue( nRet );
    }

    return trace.SetRetValue( 0 );
}

void* CMessageQueue::GetRealAddress( int nOffset )				// ƫתЧڴַ
{
	CGDTrace trace( LOG_L6, "CMessageQueue:GetRealAddress(%d)", nOffset );
	if ( nOffset )
	{
		return m_pDataMgr->GetRealAddress( nOffset );
	}

	return NULL;
}

