//
// PcreRex.h

#include "PcreRex/PcreRex.h"
#include "pcre.h"



const int PcreRex::OVEC_SIZE = 4096;


PcreRex::PcreRex(const string& pattern, int options, bool study): _pcre(0), _extra(0)
{
	const char* error;
	int offs;
	_pcre = gd_pcre_compile(pattern.c_str(), options, &error, &offs, 0);
	if (!_pcre)
	{
		throw PcreRexException( "PcreRex new error" );
	}
	if (study)
		_extra = gd_pcre_study(_pcre, 0, &error);
}


PcreRex::~PcreRex()
{
	if (_pcre)  gd_pcre_free(_pcre);
	if (_extra) gd_pcre_free(_extra);
}


int PcreRex::match(const string& subject, string::size_type offset, Match& mtch, int options) const
{
	if ( offset >= subject.length() )
	{
		throw PcreRexException( "offset exception" );
	}

	int ovec[OVEC_SIZE];
	int rc = gd_pcre_exec(_pcre, _extra, subject.c_str(), int(subject.size()), int(offset), options & 0xFFFF, ovec, OVEC_SIZE);
	if (rc == PCRE_ERROR_NOMATCH)
	{
		mtch.offset = string::npos;
		mtch.length = 0;
		return 0;
	}
	else if (rc == PCRE_ERROR_BADOPTION)
	{
		throw PcreRexException("bad option");
	}
	else if (rc == 0)
	{
		throw PcreRexException("too many captured substrings");
	}
	else if (rc < 0)
	{
		throw PcreRexException( "Pcre match error rc < 0" );
	}
	mtch.offset = ovec[0] < 0 ? string::npos : ovec[0];
	mtch.length = ovec[1] - mtch.offset;
	return rc;
}


int PcreRex::match(const string& subject, string::size_type offset, MatchVec& matches, int options) const
{
	if ( offset >= subject.length() )
	{
		throw PcreRexException( "offset exception" );
	}

	matches.clear();

	int ovec[OVEC_SIZE];
	int rc = gd_pcre_exec(_pcre, _extra, subject.c_str(), int(subject.size()), int(offset), options & 0xFFFF, ovec, OVEC_SIZE);
	if (rc == PCRE_ERROR_NOMATCH)
	{
		return 0;
	}
	else if (rc == PCRE_ERROR_BADOPTION)
	{
		throw PcreRexException("bad option");
	}
	else if (rc == 0)
	{
		throw PcreRexException("too many captured substrings");
	}
	else if (rc < 0)
	{
		throw PcreRexException( "Pcre match error rc < 0" );
	}
	matches.reserve(rc);
	for (int i = 0; i < rc; ++i)
	{
		Match m;
		m.offset = ovec[i*2] < 0 ? string::npos : ovec[i*2] ;
		m.length = ovec[i*2 + 1] - m.offset;
		matches.push_back(m);
	}
	return rc;
}


bool PcreRex::match(const string& subject, string::size_type offset) const
{
	Match mtch;
	match(subject, offset, mtch, RE_ANCHORED | RE_NOTEMPTY);
	return mtch.offset == offset && mtch.length == subject.length() - offset;
}


bool PcreRex::match(const string& subject, string::size_type offset, int options) const
{
	Match mtch;
	match(subject, offset, mtch, options);
	return mtch.offset == offset && mtch.length == subject.length() - offset;
}


bool PcreRex::match(const string& subject, const string& pattern, int options)
{
	int ctorOptions = options & (RE_CASELESS | RE_MULTILINE | RE_DOTALL | RE_EXTENDED | RE_ANCHORED | RE_DOLLAR_ENDONLY | RE_EXTRA | RE_UNGREEDY | RE_UTF8 | RE_NO_AUTO_CAPTURE);
	int mtchOptions = options & (RE_ANCHORED | RE_NOTBOL | RE_NOTEOL | RE_NOTEMPTY | RE_NO_AUTO_CAPTURE | RE_NO_UTF8_CHECK);
	PcreRex re(pattern, ctorOptions, false);
	return re.match( subject, string::size_type(0), mtchOptions );
}


