
//
// Thread.cpp
//

#pragma warning( disable: 4786 )


#include "Common/DateTime.h"
#include "GDThreadPool/Thread.h"

#ifdef WIN32
	#include <windows.h>
#else
#	include <unistd.h>
#	include <pthread.h>
#endif
#include <stdio.h>
#include <time.h>


namespace InforGuard   {
namespace GDThreadPool {

CThread::CThread( const char* name )
{
	m_threadName = name;
	m_param = NULL;
	m_status = TS_NULL;
}


CThread::CThread()
{
	m_threadName = "NoName";
	m_param = NULL;
	m_status = TS_NULL;
}

CThread::~CThread()
{
	Uninit();
}


long CThread::Init( void* param )
{
	m_param = param;
	return 0;
}


long CThread::Uninit()
{
	Stop();
	m_param = NULL;
	return 0;
}


long CThread::Start()
{
	if ( TS_NULL != m_status && TS_STOPED != m_status )
	{
		return 0;
	}
	m_status = TS_STARTING;
#ifdef WIN32
	m_hThread = (HANDLE)_beginthreadex( 
		NULL, 
		0, 
		ThreadProc, 
		this, 
		0, 
		&m_threadId );

	if ( m_hThread )
	{
		CloseHandle( m_hThread );
		m_hThread = 0;
		return 0;
	}
#else
	m_threadId = pthread_create( &m_hThread,
		NULL,
		ThreadProc,
		this );

	if ( 0 == m_threadId )
	{
		pthread_detach( m_hThread ); // ͨ pthread_detachԴȨ̣߳һ߳˳pthread_createΪԴõЧͷš
		m_hThread = 0;
		return 0;
	}
#endif

	m_status = TS_NULL;
	return GetLastError();
}


bool CThread::Stop( unsigned int ms )
{
	if ( TS_FREE != m_status && TS_BUSY != m_status && TS_STARTED != m_status )
	{
		if ( TS_NULL == m_status || TS_STOPED == m_status )
		{
			return true;
		}
		return false;
	}
	m_status = TS_STOPING;

	if ( 0 == ms )
	{
		return false;
	}

	//
	while ( m_status != TS_STOPED )
	{
		if ( ms > 100 )
		{
			CDateTime::Sleep( 100 );
			ms -= 100;
		}
		else
		{
			break;
		}
	}

	if ( TS_STOPED == m_status )
	{
		return true;
	}

	return false;
}


enumThreadStatus CThread::GetStatus()
{
	return m_status;
}


THREADPROC CThread::ThreadProc( void* param )
{
	CThread* pThread = ( CThread* ) param;

	pThread->m_status = TS_STARTED;

	pThread->Run( pThread->m_param );

	pThread->m_status = TS_STOPED;
	return 0;
}

}
}
