//
// ParamListTest.cpp
//


#include "CppUnit/TestCaller.h"
#include "CppUnit/TestSuite.h"
#include "WaterMarkDBTest.h"
#include "WaterMark/WaterMarkDB.h"


const std::string Test_Path_Base	= "D:\\src\\model\\src\\WaterMark\\WaterMarkTest\\Data";
const std::string WM_LOCK_DB_PATH	= Test_Path_Base + "\\ma_Lock";
const std::string WM_BAD_DB_PATH	= Test_Path_Base + "\\ma_Bad";
const std::string WM_DB_PATH		= Test_Path_Base + "\\ma";
const std::string WM_DB_NAME		= "wm.db";


const std::string Test_File_Path	= Test_Path_Base;
const std::string Test_File_Name1	= Test_Path_Base + "\\test1.txt";
const std::string Test_File_Name2	= Test_Path_Base + "\\test2.txt";
const std::string Test_File_Name3	= Test_Path_Base + "\\test3.txt";
const std::string Test_File_NoExit	= Test_Path_Base + "\\testNoExit.txt";


const long DB_TREE_KEY			= 0x12345678;	// ݿKey
const long DB_TREE_SIZE			= 10240;	// ݿ⹲ڴС(k)
const long DB_TREE_SAVE_COUNT	= 10;		// ͬ޸ĵĸ
const long DB_TREE_SAVE_SECONDS	= 60;		// ͬݿʱ


WaterMarkDBTest::WaterMarkDBTest(const std::string& name): CppUnit::TestCase(name)
{
}


WaterMarkDBTest::~WaterMarkDBTest()
{
}


void WaterMarkDBTest::testInit()
{
	CWaterMarkDB* pWmDb1 = new CWaterMarkDB();

	long nRet = 0;

	// ·ΪʱĬΪǰĿ¼
	nRet = pWmDb1->Init( WM_DB_PATH.c_str(), WM_DB_NAME.c_str(), DB_TREE_KEY, DB_TREE_SIZE, DB_TREE_SAVE_COUNT, DB_TREE_SAVE_SECONDS );
	assert( 0 == nRet );

	nRet = pWmDb1->Init( WM_DB_PATH.c_str(), WM_DB_NAME.c_str(), DB_TREE_KEY, DB_TREE_SIZE, DB_TREE_SAVE_COUNT, DB_TREE_SAVE_SECONDS );
	assert( 0 == nRet );

	delete pWmDb1;
	pWmDb1 = NULL;

	CWaterMarkDB* pWmDb2 = new CWaterMarkDB();

	nRet = pWmDb2->Init( WM_DB_PATH.c_str(), "", DB_TREE_KEY, DB_TREE_SIZE, DB_TREE_SAVE_COUNT, DB_TREE_SAVE_SECONDS );
	assert( -1 == nRet );

	nRet = pWmDb2->Init( NULL, NULL, DB_TREE_KEY, DB_TREE_SIZE, DB_TREE_SAVE_COUNT, DB_TREE_SAVE_SECONDS );
	assert( -1 == nRet );

	delete pWmDb2;
	pWmDb2 = NULL;
}


void WaterMarkDBTest::testAdd()
{
	CWaterMarkDB* pWmDb = new CWaterMarkDB();

	long nRet = 0;

	nRet = pWmDb->Init( WM_DB_PATH.c_str(), WM_DB_NAME.c_str(), DB_TREE_KEY, DB_TREE_SIZE, DB_TREE_SAVE_COUNT, DB_TREE_SAVE_SECONDS );
	assert( 0 == nRet );

	nRet = pWmDb->Remove( Test_File_Name1.c_str() );
	assert( 0 == nRet );

	bool bRet = pWmDb->Find( Test_File_Name1.c_str() );
	assert( !bRet );

	// ˮӡ
	char szWm[ WMARK_LEN] = {0};
	nRet = CWaterMarkDB::BuildWaterMark( Test_File_Name1.c_str(), szWm );
	assert( 0 == nRet );

	nRet = pWmDb->Add( Test_File_Name1.c_str(), szWm );
	assert( 0 == nRet );

	char szWmGet[ WMARK_LEN] = {0};
	nRet = pWmDb->Get( Test_File_Name1.c_str(),  szWmGet );
	assert( 0 == strcmp( szWmGet, szWm ) );

	nRet = pWmDb->Add( Test_File_Name1.c_str(), szWm );
	assert( 0 == nRet );

	nRet = pWmDb->Add( "", "" );
	assert( 0 != nRet );

	nRet = pWmDb->Add( NULL, NULL );
	assert( 0 != nRet );

	delete pWmDb;
	pWmDb = NULL;
}


void WaterMarkDBTest::testGet()
{
	CWaterMarkDB* pWmDb = new CWaterMarkDB();

	long nRet = 0;

	nRet = pWmDb->Init( WM_DB_PATH.c_str(), WM_DB_NAME.c_str(), DB_TREE_KEY, DB_TREE_SIZE, DB_TREE_SAVE_COUNT, DB_TREE_SAVE_SECONDS );
	assert( 0 == nRet );

	// ˮӡ
	char szWm[ WMARK_LEN] = {0};
	nRet = CWaterMarkDB::BuildWaterMark( Test_File_Name1.c_str(), szWm );
	assert( 0 == nRet );

	nRet = pWmDb->Add( Test_File_Name1.c_str(), szWm );
	assert( 0 == nRet );

	char szWmGet[ WMARK_LEN] = {0};
	nRet = pWmDb->Get( Test_File_Name1.c_str(),  szWmGet );
	assert( 0 == nRet && 0 == strcmp( szWmGet, szWm ) );

	nRet = pWmDb->Remove( Test_File_Name1.c_str() );
	assert( 0 == nRet );

	nRet = pWmDb->Remove( "" );
	assert( 0 != nRet );

	nRet = pWmDb->Get( Test_File_Name1.c_str(),  szWmGet );
	assert( 0 != nRet );

	nRet = pWmDb->Get( "",  szWmGet );
	assert( 0 != nRet );

	nRet = pWmDb->Get( NULL,  NULL );
	assert( 0 != nRet );


	delete pWmDb;
	pWmDb = NULL;
}


void WaterMarkDBTest::testFind()
{
	CWaterMarkDB* pWmDb = new CWaterMarkDB();

	long nRet = 0;

	nRet = pWmDb->Init( WM_DB_PATH.c_str(), WM_DB_NAME.c_str(), DB_TREE_KEY, DB_TREE_SIZE, DB_TREE_SAVE_COUNT, DB_TREE_SAVE_SECONDS );
	assert( 0 == nRet );

	// ˮӡ
	char szWm[ WMARK_LEN] = {0};
	nRet = CWaterMarkDB::BuildWaterMark( Test_File_Name1.c_str(), szWm );
	assert( 0 == nRet );

	nRet = pWmDb->Add( Test_File_Name1.c_str(), szWm );
	assert( 0 == nRet );

	bool bRet = false;
	bRet = pWmDb->Find( Test_File_Name1.c_str() );
	assert( bRet );

	bRet = pWmDb->Find( "AA\\vv" );
	assert( !bRet );

	bRet = pWmDb->Find( "" );
	assert( !bRet );

	bRet = pWmDb->Find( NULL );
	assert( !bRet );

	delete pWmDb;
	pWmDb = NULL;
}


void WaterMarkDBTest::testUpdate()
{
	CWaterMarkDB* pWmDb = new CWaterMarkDB();

	long nRet = 0;

	nRet = pWmDb->Init( WM_DB_PATH.c_str(), WM_DB_NAME.c_str(), DB_TREE_KEY, DB_TREE_SIZE, DB_TREE_SAVE_COUNT, DB_TREE_SAVE_SECONDS );
	assert( 0 == nRet );

	// ˮӡ
	char szWm[ WMARK_LEN] = {0};
	nRet = CWaterMarkDB::BuildWaterMark( Test_File_Name1.c_str(), szWm );
	assert( 0 == nRet );

	nRet = pWmDb->Add( Test_File_Name1.c_str(), szWm );
	assert( 0 == nRet );

	char szWmGet[ WMARK_LEN] = {0};
	char szNewWm[ WMARK_LEN] = {1};
	nRet = pWmDb->Update( Test_File_Name1.c_str(), true, szNewWm );
	assert( 0 == nRet );
	nRet = pWmDb->Get( Test_File_Name1.c_str(),  szWmGet );
	assert( 0 == nRet && 0 == strcmp( szWmGet, szNewWm ) );

	nRet = pWmDb->Update( "", true, szNewWm );
	assert( 0 != nRet );

	nRet = pWmDb->Update( NULL, true, szNewWm );
	assert( 0 != nRet );

	nRet = pWmDb->Remove( Test_File_NoExit.c_str() );
	assert( 0 == nRet );

	nRet = pWmDb->Update( Test_File_NoExit.c_str(), true, szNewWm );
	assert( 0 == nRet );
	nRet = pWmDb->Get( Test_File_NoExit.c_str(),  szWmGet );
	assert( 0 == nRet && 0 == strcmp( szWmGet, szNewWm ) );

	nRet = pWmDb->Remove( Test_File_NoExit.c_str() );
	assert( 0 == nRet );

	bool bRet = false;
	bRet = pWmDb->Find( Test_File_NoExit.c_str() );
	assert( !bRet );

	nRet = pWmDb->Update( "", false, szNewWm );
	assert( 0 != nRet );

	nRet = pWmDb->Update( NULL, false, szNewWm );
	assert( 0 != nRet );

	delete pWmDb;
	pWmDb = NULL;
}


void WaterMarkDBTest::testRemove()
{
	CWaterMarkDB* pWmDb = new CWaterMarkDB();

	long nRet = 0;

	nRet = pWmDb->Init( WM_DB_PATH.c_str(), WM_DB_NAME.c_str(), DB_TREE_KEY, DB_TREE_SIZE, DB_TREE_SAVE_COUNT, DB_TREE_SAVE_SECONDS );
	assert( 0 == nRet );

	// ˮӡ
	char szWm[ WMARK_LEN] = {0};
	nRet = CWaterMarkDB::BuildWaterMark( Test_File_Name1.c_str(), szWm );
	assert( 0 == nRet );

	nRet = pWmDb->Add( Test_File_Name1.c_str(), szWm );
	assert( 0 == nRet );

	bool bRet = false;
	bRet = pWmDb->Find( Test_File_Name1.c_str() );
	assert( bRet );

	nRet = pWmDb->Remove( Test_File_Name1.c_str() );
	assert( 0 == nRet );
	bRet = pWmDb->Find( Test_File_Name1.c_str() );
	assert( !bRet );

	nRet = pWmDb->Remove( Test_File_Path.c_str() );
	assert( 0 == nRet );

	nRet = pWmDb->Remove( Test_File_NoExit.c_str() );
	assert( 0 == nRet );

	nRet = pWmDb->Remove( Test_File_NoExit.c_str() );
	assert( 0 == nRet );

	nRet = pWmDb->Remove( "" );
	assert( 0 != nRet );

	nRet = pWmDb->Remove( NULL );
	assert( 0 != nRet );
	
	delete pWmDb;
	pWmDb = NULL;
}


void WaterMarkDBTest::testRename()
{
	CWaterMarkDB* pWmDb = new CWaterMarkDB();

	long nRet = 0;

	nRet = pWmDb->Init( WM_DB_PATH.c_str(), WM_DB_NAME.c_str(), DB_TREE_KEY, DB_TREE_SIZE, DB_TREE_SAVE_COUNT, DB_TREE_SAVE_SECONDS );
	assert( 0 == nRet );

	// ˮӡ
	char szWm[ WMARK_LEN] = {0};
	nRet = CWaterMarkDB::BuildWaterMark( Test_File_Name1.c_str(), szWm );
	assert( 0 == nRet );

	nRet = pWmDb->Add( Test_File_Name1.c_str(), szWm );
	assert( 0 == nRet );

	bool bRet = false;
	bRet = pWmDb->Find( Test_File_Name1.c_str() );
	assert( bRet );

	nRet = pWmDb->Rename( Test_File_Name1.c_str(), Test_File_Name2.c_str() );
	assert( 0 == nRet );
	bRet = pWmDb->Find( Test_File_Name1.c_str() );
	assert( !bRet );
	bRet = pWmDb->Find( Test_File_Name2.c_str() );
	assert( bRet );

	nRet = pWmDb->Rename( Test_File_Path.c_str(), "aa" );
	assert( 0 == nRet );

	nRet = pWmDb->Rename( Test_File_NoExit.c_str(), "aaa.txt" );
	assert( 0 != nRet );

	nRet = pWmDb->Rename( "", "" );
	assert( 0 != nRet );

	nRet = pWmDb->Rename( NULL, NULL );
	assert( 0 != nRet );
	
	delete pWmDb;
	pWmDb = NULL;
}


void WaterMarkDBTest::testBuildWaterMark()
{
	long nRet = 0;
	char szWm1[ WMARK_LEN] = {0};
	nRet = CWaterMarkDB::BuildWaterMark( Test_File_Name1.c_str(), szWm1 );
	assert( 0 == nRet );
	char szWm2[ WMARK_LEN] = {0};
	nRet = CWaterMarkDB::BuildWaterMark( Test_File_Name1.c_str(), szWm2 );
	assert( 0 == nRet );
	assert( 0 == strcmp( szWm1, szWm2 ) );
	// Բڵļˮӡ
	char szWm3[ WMARK_LEN] = {0};
	try
	{
		nRet = CWaterMarkDB::BuildWaterMark( Test_File_NoExit.c_str(), szWm3 );
		fail( "BuileWaterMark fail" );
	}
	catch( CExceptionError& e )
	{
		printf( "%s\n", e.GetErrText().c_str() );
	}
	
	nRet = CWaterMarkDB::BuildWaterMark( "", szWm3 );
	assert( 0 != nRet );
		
	nRet = CWaterMarkDB::BuildWaterMark( NULL, szWm3 );
	assert( 0 != nRet );
}


void WaterMarkDBTest::setUp()
{
}


void WaterMarkDBTest::tearDown()
{
}


CppUnit::Test* WaterMarkDBTest::suite()
{
	CppUnit::TestSuite* pSuite = new CppUnit::TestSuite("WaterMarkDBTest");

	CppUnit_addTest(pSuite, WaterMarkDBTest, testInit);
	CppUnit_addTest(pSuite, WaterMarkDBTest, testAdd);
	CppUnit_addTest(pSuite, WaterMarkDBTest, testGet);
	CppUnit_addTest(pSuite, WaterMarkDBTest, testFind);
	CppUnit_addTest(pSuite, WaterMarkDBTest, testUpdate);
	CppUnit_addTest(pSuite, WaterMarkDBTest, testRemove);
	CppUnit_addTest(pSuite, WaterMarkDBTest, testRename);
	CppUnit_addTest(pSuite, WaterMarkDBTest, testBuildWaterMark);

	return pSuite;
}
