/******************************************************************************
*
* ļ              AppConfig.cpp
* ģĿģ      ࣬ʵļ
* ߼ڣ        zhang_dpeng 2008-03-03
* 汾                1.0
* ȨϢ            CVICSE Middleware
* ļ¼            
*
******************************************************************************/

#include "Base/MonitorDef.h"
#include "Base/AppConfig.h"
#include "Common/GDString.h"
#include "FileOpt/FileOpt.h"
#include "GDLog/GDLog.h"
#include "GDLog/GDTrace.h"
#include "Groundwork/Groundwork.h"
#include "Groundwork/Path.h"
#include "Groundwork/File.h"
#include "Groundwork/String.h"
#include "Groundwork/DateTimeParser.h"
#include "Groundwork/DateTimeFormat.h"

#include <iostream>


using namespace InforGuard::Groundwork;


CAppConfig* CAppConfig::m_pInstance = NULL;


CAppConfig& CAppConfig::Instance()
{
	assert(m_pInstance != NULL);
	return *m_pInstance;
}


CAppConfig::CAppConfig() : m_bUpdated(false)
{
	m_pInstance = this;
}


CAppConfig::~CAppConfig()
{
	m_pInstance = NULL;
}


bool CAppConfig::MakeSureTmpDirExist(const std::string& localFileName)
{
	CGDTrace trace( LOG_L6, "CAppConfig:MakeSureTmpDirExist(%s)", localFileName.c_str() );
	
	if (localFileName.length() < 2)
		return trace.SetRetResult( false );
	
	try
	{
#ifdef WIN32
		std::string root = localFileName.substr(0, 3);
		
		CFileOpt::MakeSureDirExist((root + "GDWorkPath\\Dustbin").c_str());
		CFileOpt::MakeSureDirExist((root + "GDWorkPath\\Evidence").c_str());
		CFileOpt::MakeSureDirExist((root + "GDWorkPath\\Temporary").c_str());
#else
		std::string root = GetInforGuardPath();
		
		CFileOpt::MakeSureDirExist((root + "/GDWorkPath/Dustbin").c_str());
		CFileOpt::MakeSureDirExist((root + "/GDWorkPath/Evidence").c_str());
		CFileOpt::MakeSureDirExist((root + "/GDWorkPath/Temporary").c_str());
#endif
	}
	catch ( CExceptionError & ex )
	{
		CGDTrace::GDTrace( LOG_INF, "CAppConfig::MakeSureTmpDirExist(%s)\n%s", localFileName.c_str(), ex.GetErrText().c_str() );
		CGDLog::WriteErrLog( "1011:CAppConfig::MakeSureTmpDirExist error (%s)\n%s", localFileName.c_str(), ex.GetErrText().c_str());
		return trace.SetRetResult( false );
	}
	
	return trace.SetRetResult( true );
}


bool CAppConfig::IsFileInDirectory(const std::string& filePath, const std::string& parentDir)
{
	CGDTrace trace( LOG_L6, "CAppConfig::IsFileInDirectory:(%s)", filePath.c_str() );
	if (icompare(filePath, parentDir) == 0 ) // ǸĿ¼
		return trace.SetRetResult( true );
	
	std::string tmpPath = parentDir + Path::separator();
	if (icompare( filePath.substr( 0, tmpPath.length() ), tmpPath ) == 0 ) // localFileNameǷڸĿ¼
		return trace.SetRetResult( true );
	
	return trace.SetRetResult( false );
}

long CAppConfig::GetConfigFileContent(std::string& configFileContent)
{
	CGDTrace trace( LOG_L6, "CAppConfig:GetConfigFileContent(%s)", configFileContent.c_str() );
	
	FastMutex lock(_mutex);
	
	try
	{
		TiXmlDocument doc;
		
		if (!doc.LoadFile(m_configFileName.c_str())) 
		{
			throw XmlParseError(doc);
		}
		
		TiXmlPrinter printer;
		doc.Accept( &printer );
		configFileContent = printer.CStr();
	}
	catch (XmlParseError& exc)
	{
		std::cerr << exc.DisplayText() << std::endl;
	}
	
	return trace.SetRetValue( 0 );
}

bool CAppConfig::RegisterObserver( CConfigObserver* pObserver )
{
	CGDTrace trace( LOG_L6, "CAppConfig:RegisterObserver()" );
	
	m_cfgObserver.push_back( pObserver );

	return trace.SetRetResult( true );
}

long CAppConfig::Save(const std::string& configFileContent)
{
	//CGDTrace trace( LOG_L6, "CAppConfig:Save(%s)", configFileContent.c_str() );
	CGDTrace trace( LOG_L6, "CAppConfig:Save(...)" );
	
	FastMutex lock(_mutex);
	
	try
	{
		TiXmlDocument doc;
		
		if ( 0 == doc.Parse(configFileContent.c_str()) ) 
		{
			throw XmlParseError(doc);
		}
		
		if (!doc.SaveFile(m_configFileName.c_str()))
		{
			throw XmlParseError(doc);
		}
		
		m_bUpdated = true;
		m_nLastMofifyTime = time(NULL);
	}
	catch (XmlParseError& exc)
	{
		std::cerr << exc.DisplayText() << std::endl;
	}
	
	// Add by wang_jin at 2008-11-28 Ψһ۲߼عɶ˿ڲʧܶ۲ģʽ(Ϊ̼߳)
	// Add by wang_jin at 2008-4-8 for ۲ģʽˢĿ¼״̬
	std::vector<CConfigObserver*>::iterator iter;
	for ( iter = m_cfgObserver.begin(); iter != m_cfgObserver.end(); ++iter )
	{
		if ( (*iter)->Routine() )
		{
			// ɹ
		}
		else
		{
			// ʧ
		}
	}
	
	return trace.SetRetValue( 0 );
}


std::string CAppConfig::GetHostId()
{
	//CGDTrace trace( LOG_L6, "CAppConfig::GetHostId()" );
	FastMutex lock(_mutex);
	CheckUpdate();
	
	return m_hostId;
}


std::string CAppConfig::GetLocalIp()
{
	//CGDTrace trace( LOG_L6, "CAppConfig::GetLocalIp()" );
	FastMutex lock(_mutex);
	CheckUpdate();
	
	return m_localIp;
}


std::string CAppConfig::GetLocalPort()
{
	//CGDTrace trace( LOG_L6, "CAppConfig::GetLocalPort()" );
	FastMutex lock(_mutex);
	CheckUpdate();
	
	return m_localPort;
}


std::string CAppConfig::GetMcIp()
{
	FastMutex lock(_mutex);
	CheckUpdate();
	return m_mcIp;
}


std::string CAppConfig::GetMcPort()
{	
	FastMutex lock(_mutex);
	CheckUpdate();	
	return m_mcPort;
}

void CAppConfig::GetMonitors(std::vector<Monitor>& monitors)
{
	CGDTrace trace( LOG_L6, "CAppConfig:GetMonitors()" );
	
	FastMutex lock(_mutex);
	CheckUpdate();
	
    monitors = m_monitors;
}


bool CAppConfig::IsFileMonitoring(const std::string& localFileName)
{
	CGDTrace trace( LOG_L6, "CAppConfig:IsFileMonitoring(%s)", localFileName.c_str() );
	
	FastMutex lock(_mutex);
	CheckUpdate();

	bool bRet = false;
	std::vector<WebSite>::iterator itWebSite;
	for (itWebSite = m_webSites.begin(); itWebSite != m_webSites.end(); ++itWebSite)
	{
		std::vector<SiteDirectory>::iterator itDirectory;
		for (itDirectory = (*itWebSite).siteDirectorys.begin(); itDirectory != (*itWebSite).siteDirectorys.end(); ++itDirectory)
		{
			if (IsFileInDirectory(localFileName, (*itDirectory).localPath)) // localFileNameǷڸĿ¼
			{
				bRet = ((*itWebSite).monitorState == 1);
				return trace.SetRetResult( bRet );
			}
		}
	}
	return trace.SetRetResult( bRet );
}


void CAppConfig::GetWebSites(std::vector<WebSite>& webSites)
{
	CGDTrace trace( LOG_L6, "CAppConfig:GetWebSites()" );
	
	FastMutex lock(_mutex);
	CheckUpdate();
	
	webSites = m_webSites;
}


void CAppConfig::GetRemoteInfoByLocalFileName(const std::string& localFileName, std::vector<RemotePath>& remotePaths)
{
	CGDTrace trace( LOG_L6, "CAppConfig:GetRemoteInfoByLocalFileName(%s)", localFileName.c_str() );
	
	FastMutex lock(_mutex);
	CheckUpdate();
	
	std::vector<WebSite>::iterator itWebSite;
	for (itWebSite = m_webSites.begin(); itWebSite != m_webSites.end(); ++itWebSite)
	{
		std::vector<SiteDirectory>::iterator itDirectory;
		for (itDirectory = (*itWebSite).siteDirectorys.begin(); itDirectory != (*itWebSite).siteDirectorys.end(); ++itDirectory)
		{
			if (IsFileInDirectory(localFileName, (*itDirectory).localPath)) // localFileNameǷڸĿ¼
			{
				std::vector<RemotePath>::iterator itRemotePath;
				for (itRemotePath = (*itDirectory).remotePaths.begin(); itRemotePath != (*itDirectory).remotePaths.end(); ++itRemotePath)
				{
					remotePaths.push_back((*itRemotePath));
				}
				return;
			}
		}
	}
}


std::string CAppConfig::GetWebsiteIdByLocalFileName( const std::string& localFileName )
{	
	CGDTrace trace( LOG_L6, "CAppConfig:GetWebsiteIdByLocalFileName(%s)", localFileName.c_str() );
	
	FastMutex lock(_mutex);
	CheckUpdate();
	
	std::vector<WebSite>::iterator itWebSite;
	for (itWebSite = m_webSites.begin(); itWebSite != m_webSites.end(); ++itWebSite)
	{
		std::vector<SiteDirectory>::iterator itDirectory;
		for (itDirectory = (*itWebSite).siteDirectorys.begin(); itDirectory != (*itWebSite).siteDirectorys.end(); ++itDirectory)
		{
			if (IsFileInDirectory(localFileName, (*itDirectory).localPath)) // localFileNameǷڸĿ¼
			{
				return trace.SetRetText( (*itWebSite).id.c_str() );
			}
		}
	}
	return trace.SetRetText( "" );
}


std::string CAppConfig::GetWebsiteIdByDirectoryId( const std::string& sDirId )
{	
	CGDTrace trace( LOG_L6, "CAppConfig:GetWebsiteIdByDirectoryId(%s)", sDirId.c_str() );
	
	FastMutex lock(_mutex);
	CheckUpdate();
	
	std::vector<WebSite>::iterator itWebSite;
	for (itWebSite = m_webSites.begin(); itWebSite != m_webSites.end(); ++itWebSite)
	{
		std::vector<SiteDirectory>::iterator itDirectory;
		for (itDirectory = (*itWebSite).siteDirectorys.begin(); itDirectory != (*itWebSite).siteDirectorys.end(); ++itDirectory)
		{
			if ( (*itDirectory).id == sDirId ) // localFileNameǷڸĿ¼
			{
				return trace.SetRetText( (*itWebSite).id.c_str() );
			}
		}
	}
	return trace.SetRetText( "" );
}

void CAppConfig::GetAllLoaclPath(std::vector<std::string>& localPaths, bool excludeNotMonitoringSite)
{
	CGDTrace trace( LOG_L6, "CAppConfig:GetAllLoaclPath()" );
	
	FastMutex lock(_mutex);
	CheckUpdate();
	
	std::vector<WebSite>::iterator itWebSite;
	for (itWebSite = m_webSites.begin(); itWebSite != m_webSites.end(); ++itWebSite)
	{
		if (excludeNotMonitoringSite && ((*itWebSite).monitorState == 0)) // Ƿųδڼ״̬վ
			continue;
		
		std::vector<SiteDirectory>::iterator itDirectory;
		for (itDirectory = (*itWebSite).siteDirectorys.begin(); itDirectory != (*itWebSite).siteDirectorys.end(); ++itDirectory)
		{
			localPaths.push_back((*itDirectory).localPath);
		}
	}
}

//ļжϼĿ¼ѷGetSiteDirectoryById
void CAppConfig::GetSiteDirectoryByLocalFileName(const std::string& localFileName, SiteDirectory &siteDirectory)
{
	CGDTrace trace( LOG_L6, "CAppConfig:GetSiteDirectoryByLocalFileName(%s)", localFileName.c_str() );
	
	FastMutex lock(_mutex);
	CheckUpdate();
	
	std::vector<WebSite>::iterator itWebSite;
	for (itWebSite = m_webSites.begin(); itWebSite != m_webSites.end(); ++itWebSite)
	{
		std::vector<SiteDirectory>::iterator itDirectory;
		for (itDirectory = (*itWebSite).siteDirectorys.begin(); itDirectory != (*itWebSite).siteDirectorys.end(); ++itDirectory)
		{
			if (IsFileInDirectory(localFileName, (*itDirectory).localPath)) // localFileNameǷڸĿ¼
			{
				siteDirectory = (*itDirectory);
	
				trace.SetRetText("OK");
				return;
			}
		}
	}
	trace.SetOutParam( "NONE" );
}

//IDȡĿ¼ add by wu_bin
void CAppConfig::GetSiteDirectoryById(const char * sDirId, SiteDirectory &siteDirectory)
{
	CGDTrace trace( LOG_L6, "CAppConfig:GetSiteDirectoryById(%s)", sDirId );
	
	FastMutex lock(_mutex);
	CheckUpdate();
	
	std::vector<WebSite>::iterator itWebSite;
	for (itWebSite = m_webSites.begin(); itWebSite != m_webSites.end(); ++itWebSite)
	{
		std::vector<SiteDirectory>::iterator itDirectory;
		for (itDirectory = (*itWebSite).siteDirectorys.begin(); itDirectory != (*itWebSite).siteDirectorys.end(); ++itDirectory)
		{
			if ((*itDirectory).id == sDirId) // ж¼IdĿ¼Ƿ
			{
				siteDirectory = (*itDirectory);
				
				trace.SetRetText("OK");
				return;
			}
		}
	}
	trace.SetOutParam( "NONE" );

}

bool CAppConfig::GetSiteById(const std::string& websiteId, WebSite& website)
{
	CGDTrace trace( LOG_L6, "CAppConfig:GetSiteById(%s)", websiteId.c_str() );
	
	FastMutex lock(_mutex);
	CheckUpdate();
	
	std::vector<WebSite>::iterator itWebSite;
	for (itWebSite = m_webSites.begin(); itWebSite != m_webSites.end(); ++itWebSite)
	{
		if ((*itWebSite).id == websiteId)
		{
			website = (*itWebSite);
			return trace.SetRetResult( true );
		}
	}
	
	return trace.SetRetResult( false );
}


bool CAppConfig::IsValidEvent(const std::string& localFileName, int eventType)
{
	CGDTrace trace( LOG_L6, "CAppConfig:IsValidEvent(%s, %d)", localFileName.c_str(), eventType );
	
	FastMutex lock(_mutex);
	CheckUpdate();
	
    bool validPath = false;
	
	std::vector<WebSite>::iterator itWebSite;
	for (itWebSite = m_webSites.begin(); itWebSite != m_webSites.end(); ++itWebSite)
	{
		std::vector<SiteDirectory>::iterator itDirectory;
		for (itDirectory = (*itWebSite).siteDirectorys.begin(); itDirectory != (*itWebSite).siteDirectorys.end(); ++itDirectory)
		{
			if (IsFileInDirectory(localFileName, (*itDirectory).localPath)) // localFileNameǷڸĿ¼
			{
				validPath = true;
				break;
			}
		}
		
		if (validPath)
			break;
	}
	
	if (!validPath) // ļĳվĿ¼?
	{
		return trace.SetRetResult( false );
	}
	
	if (!(*itWebSite).policy.monitorPolicy.available) //δüع˲
	{
		return trace.SetRetResult( true );
	}
	
	switch ( eventType & 0x0f ) // ж¼
	{
	case MONITOR_EVENT_CHANGE:
		{
			if ((*itWebSite).policy.monitorPolicy.excludeChangeEvent)
				return trace.SetRetResult( false );
			break;
		}
	case MONITOR_EVENT_CREATE:
		{
			if ((*itWebSite).policy.monitorPolicy.excludeCreateEvent)
				return trace.SetRetResult( false );
			break;
		}
	case MONITOR_EVENT_DELETE:
	case MONITOR_EVENT_MOVEOUT:
		{
			if ((*itWebSite).policy.monitorPolicy.excludeDeleteEvent)
				return trace.SetRetResult( false );
			break;
		}
	case MONITOR_EVENT_RENAME:
		{
			if ((*itWebSite).policy.monitorPolicy.excludeRenameEvent)
				return trace.SetRetResult( false );
			break;
		}
	default: // δ֪¼
		break;
	}
	
	if ((*itWebSite).policy.monitorPolicy.excludeTypeAvailable) // Ƿļ͹
	{
		std::string::size_type extPos;
		extPos = localFileName.rfind('.');
		
		if (extPos != std::string::npos) // չŹļ
		{
			std::string ext;
			ext = localFileName.substr(extPos + 1, localFileName.size());
			
			std::vector<std::string>::iterator it;
			for (it = (*itWebSite).policy.monitorPolicy.excludeType.begin(); it != (*itWebSite).policy.monitorPolicy.excludeType.end(); ++it)
			{
				if (icompare(ext, (*it)) == 0) // ųļ
					return trace.SetRetResult( false );
			}
		}
	}
	
	if ((*itWebSite).policy.monitorPolicy.excludeBlurAvailable) // Ƿģ
	{
		// liu_yfeng	2008-07-29
		std::string name = localFileName;
		std::string::size_type last = name.find_last_of( S_PATHLASH );
		if ( last != std::string::npos )
		{
			name = name.substr( last + 1 );
		}
		std::vector<std::string>::iterator it;
		for (it = (*itWebSite).policy.monitorPolicy.excludeBlur.begin(); it != (*itWebSite).policy.monitorPolicy.excludeBlur.end(); ++it)
		{
			//add   here  zhangquan 2008-10-7
			if ( (*it).find('*') !=string::npos ) 
			{
				//add by wu_bin 2009-7-14
				//ж¼ͣų ɾ¼
				if ( (eventType & 0x0f) != MONITOR_EVENT_DELETE )
				{
					if ( !CFileOpt::IsDirectory( localFileName.c_str() ) )
					{
						if ( CGDString::MatchWithPattern( (*it).c_str(), name.c_str() ) )
						{
							return trace.SetRetResult( false );
						}
					}
				}
				
			}
		}
	}
	
	if ((*itWebSite).policy.monitorPolicy.excludePathAvailable) // ǷųĿ¼
	{
		std::vector<std::string>::iterator it;
		for (it = (*itWebSite).policy.monitorPolicy.excludePath.begin(); it != (*itWebSite).policy.monitorPolicy.excludePath.end(); ++it)
		{
			std::string path = (*it);
			std::string file = localFileName;
			
			if (path.size() <= file.size())
			{
				std::string filepath;
				filepath = file.substr(0, path.size() + 1);
				
				if (icompare(filepath, path + Path::separator()) == 0) // ųĿ¼
					return trace.SetRetResult( false );
			}
		}
	}
	
	if ((*itWebSite).policy.monitorPolicy.excludeFileAvailable) // Ƿųļ
	{
		std::vector<std::string>::iterator it;
		for (it = (*itWebSite).policy.monitorPolicy.excludeFile.begin(); it != (*itWebSite).policy.monitorPolicy.excludeFile.end(); ++it)
		{
			if (icompare(localFileName, (*it)) == 0) // ųļ
			   return trace.SetRetResult( false );
		}
	}
	
	return trace.SetRetResult( true );
}

bool CAppConfig::IsValidFile(const std::string& localFileName)
{
	CGDTrace trace( LOG_L6, "CAppConfig:IsValidFile(%s)", localFileName.c_str() );
	
	FastMutex lock(_mutex);
	CheckUpdate();
	
    bool validPath = false;
	
	// λļ/Ŀ¼վ
	std::vector<WebSite>::iterator itWebSite;
	for (itWebSite = m_webSites.begin(); itWebSite != m_webSites.end(); ++itWebSite)
	{
		std::vector<SiteDirectory>::iterator itDirectory;
		for (itDirectory = (*itWebSite).siteDirectorys.begin(); itDirectory != (*itWebSite).siteDirectorys.end(); ++itDirectory)
		{
			if (IsFileInDirectory(localFileName, (*itDirectory).localPath)) // localFileNameǷڸĿ¼
			{
				validPath = true;
				break;
			}
		}
		
		if (validPath)
			break;
	}
	
	if (!validPath)
	{
		return trace.SetRetResult( false ); // ļ/Ŀ¼κվ
	}
	
	if (!(*itWebSite).policy.filePolicy.available)
	{
		return trace.SetRetResult( true ); // δù˲
	}
	
	// жϵĿ¼ļ
	// ģƥĿ¼ļֵ֧
	if ((*itWebSite).policy.filePolicy.excludePathAvailable) // ǷųĿ¼
	{
		std::vector<std::string>::iterator it;
		for (it = (*itWebSite).policy.filePolicy.excludePath.begin(); it != (*itWebSite).policy.filePolicy.excludePath.end(); ++it)
		{
			std::string path = (*it);
			std::string file = localFileName;
	
			if (icompare(file, path) == 0)// ųĿ¼
				return trace.SetRetResult( false );
				
			if (path.size() <= file.size())
			{
				std::string filepath;
				filepath = file.substr(0, path.size() + 1);
				if (icompare(filepath, path + Path::separator()) == 0) // ųĿ¼
					return trace.SetRetResult( false );
			}

			if ( (*it).find('*') != string::npos ) 
			{
				if (CGDString::MatchWithPattern((*it).c_str(), file.c_str()))
					return trace.SetRetResult(false);
			}		
		}
	}
	
	if ( CFileOpt::IsDirectory( localFileName.c_str() ) )     //add by wubin жļĿ¼2010-02-26
		return true;

	if ((*itWebSite).policy.filePolicy.excludeTypeAvailable) // Ƿļ͹
	{
		std::string::size_type extPos;
		extPos = localFileName.rfind('.');
		
		if (extPos != std::string::npos) // չŹļ
		{
			std::string ext;
			ext = localFileName.substr(extPos + 1, localFileName.size());
			
			std::vector<std::string>::iterator it;
			for (it = (*itWebSite).policy.filePolicy.excludeType.begin(); it != (*itWebSite).policy.filePolicy.excludeType.end(); ++it)
			{
				if (icompare(ext, (*it)) == 0) // ųļ
					return trace.SetRetResult( false );
			}
		}
	}
	
	if ((*itWebSite).policy.filePolicy.excludeBlurAvailable) // Ƿģ
	{
		// ʵ
		// liu_yfeng	2008-07-08
		std::string name = localFileName;
		std::string::size_type last = name.find_last_of( S_PATHLASH );
		if ( last != std::string::npos )
		{
			name = name.substr( last + 1 );
		}
		std::vector<std::string>::iterator it;
		for (it = (*itWebSite).policy.filePolicy.excludeBlur.begin(); it != (*itWebSite).policy.filePolicy.excludeBlur.end(); ++it)
		{
			if ( CGDString::MatchWithPattern( (*it).c_str(), name.c_str() ) )
			{
				return trace.SetRetResult( false );
			}
		}
	}
	
	if ((*itWebSite).policy.filePolicy.excludeFileAvailable) // Ƿųļ
	{
		std::vector<std::string>::iterator it;
		for (it = (*itWebSite).policy.filePolicy.excludeFile.begin(); it != (*itWebSite).policy.filePolicy.excludeFile.end(); ++it)
		{
			if (icompare(localFileName, (*it)) == 0) // ųļ
				return trace.SetRetResult( false );
		} 
	}
	
	File localFile(localFileName);
	bool isDir = localFile.isDirectory();
	
	if ((*itWebSite).policy.filePolicy.startDateAvailable) // жļ޸
	{
		int tzd;
#ifdef WIN32
		if ((localFile.getLastModified() + 28800000000) < DateTimeParser::parse(DateTimeFormat::SORTABLE_FORMAT, (*itWebSite).policy.filePolicy.startDate, tzd).timestamp())
#else
		if ((localFile.getLastModified() + 28800000000LL) < DateTimeParser::parse(DateTimeFormat::SORTABLE_FORMAT, (*itWebSite).policy.filePolicy.startDate, tzd).timestamp())
#endif
		{
			return trace.SetRetResult( false );
		}
	}
	
	if ((*itWebSite).policy.filePolicy.endDateAvailable) // жļ޸
	{
		int tzd;
#ifdef WIN32
		if ((localFile.getLastModified() + 28800000000) > DateTimeParser::parse(DateTimeFormat::SORTABLE_FORMAT, (*itWebSite).policy.filePolicy.endDate, tzd).timestamp())
#else
		if ((localFile.getLastModified() + 28800000000LL) > DateTimeParser::parse(DateTimeFormat::SORTABLE_FORMAT, (*itWebSite).policy.filePolicy.endDate, tzd).timestamp())
#endif
		{
			return trace.SetRetResult( false );
		}
	}
	
	if ((*itWebSite).policy.filePolicy.maxFileSizeAvailable && !isDir) // жļֵ
	{
		if (localFile.getSize() > static_cast<File::FileSize>((*itWebSite).policy.filePolicy.maxFileSize))
		{
			return trace.SetRetResult( false );
		}
	}
	
	if ((*itWebSite).policy.filePolicy.minFileSizeAvailable && !isDir) // жļСֵ
	{
		if (localFile.getSize() < static_cast<File::FileSize>((*itWebSite).policy.filePolicy.minFileSize))
		{
			return trace.SetRetResult( false );
		}
	}
	
	return trace.SetRetResult( true );
}

//ûиĿ¼µļģƥ䣬Ϊǵúᱻ
// Ŀ¼IDжǷϷļ(ֶ)
bool CAppConfig::IsValidFile(const std::string& localFileName, const std::string& sWebSiteId)
{
	CGDTrace trace( LOG_L6, "CAppConfig:IsValidFile(%s)", localFileName.c_str() );
	
	FastMutex lock(_mutex);
	CheckUpdate();
	
    bool validPath = false;
	
	// λļ/Ŀ¼վ
	std::vector<WebSite>::iterator itWebSite;
	int nSite = m_webSites.size();
	for (itWebSite = m_webSites.begin(); itWebSite != m_webSites.end(); ++itWebSite)
	{
		std::vector<SiteDirectory>::iterator itDirectory;
		for (itDirectory = (*itWebSite).siteDirectorys.begin(); itDirectory != (*itWebSite).siteDirectorys.end(); ++itDirectory)
		{
			if (IsFileInDirectory(localFileName, (*itDirectory).localPath) && ( (*itWebSite).id == sWebSiteId )) // localFileNameǷڸĿ¼
			{
				validPath = true;
				break;
			}
		}
		
		if (validPath)
			break;
	}
	
	if (!validPath)
	{
		return trace.SetRetResult( false ); // ļ/Ŀ¼κվ
	}
	
	if (!(*itWebSite).policy.filePolicy.available)
	{
		return trace.SetRetResult( true ); // δù˲
	}
	
//	{
		// Ŀ¼
	if ((*itWebSite).policy.filePolicy.excludePathAvailable) // ǷųĿ¼
	{
		std::vector<std::string>::iterator it;
		for (it = (*itWebSite).policy.filePolicy.excludePath.begin(); it != (*itWebSite).policy.filePolicy.excludePath.end(); ++it)
		{
			std::string path = (*it);
			std::string file = localFileName;
		
			if (icompare(file, path) == 0)// ųĿ¼
				return trace.SetRetResult( false );
				
			if (path.size() <= file.size())
			{
				std::string filepath;
				filepath = file.substr(0, path.size() + 1);
					
				if (icompare(filepath, path + Path::separator()) == 0) // ųĿ¼
					return trace.SetRetResult( false );
			}

			if ( (*it).find('*') != string::npos ) 
			{
				if (CGDString::MatchWithPattern((*it).c_str(), file.c_str()))
					return trace.SetRetResult(false);
			}		
		}
	}
		
	if ( CFileOpt::IsDirectory( localFileName.c_str() ) )     //add by wubin жļĿ¼2010-02-26
		return true;

	if ((*itWebSite).policy.filePolicy.excludeTypeAvailable) // Ƿļ͹
	{
		std::string::size_type extPos;
		extPos = localFileName.rfind('.');
		
		if (extPos != std::string::npos) // չŹļ
		{
			std::string ext;
			ext = localFileName.substr(extPos + 1, localFileName.size());
			
			std::vector<std::string>::iterator it;
			for (it = (*itWebSite).policy.filePolicy.excludeType.begin(); it != (*itWebSite).policy.filePolicy.excludeType.end(); ++it)
			{
				if (icompare(ext, (*it)) == 0) // ųļ
					return trace.SetRetResult( false );
			}
		}
	}
	
	if ((*itWebSite).policy.filePolicy.excludeBlurAvailable) // Ƿģ
	{
		// ʵ
		// liu_yfeng	2008-07-08
		std::string name = localFileName;
		std::string::size_type last = name.find_last_of( S_PATHLASH );
		if ( last != std::string::npos )
		{
			name = name.substr( last + 1 );
		}
		std::vector<std::string>::iterator it;
		for (it = (*itWebSite).policy.filePolicy.excludeBlur.begin(); it != (*itWebSite).policy.filePolicy.excludeBlur.end(); ++it)
		{
			if ( CGDString::MatchWithPattern( (*it).c_str(), name.c_str() ) )
			{
				return trace.SetRetResult( false );
			}
		}
	}
	
	if ((*itWebSite).policy.filePolicy.excludeFileAvailable) // Ƿųļ
	{
		std::vector<std::string>::iterator it;
		for (it = (*itWebSite).policy.filePolicy.excludeFile.begin(); it != (*itWebSite).policy.filePolicy.excludeFile.end(); ++it)
		{
			if (icompare(localFileName, (*it)) == 0) // ųļ
				return trace.SetRetResult( false );
		} 
	}
	
	File localFile(localFileName);
	bool isDir = localFile.isDirectory();
	
	if ((*itWebSite).policy.filePolicy.startDateAvailable) // жļ޸
	{
		int tzd;
#ifdef WIN32
		if ((localFile.getLastModified() + 28800000000) < DateTimeParser::parse(DateTimeFormat::SORTABLE_FORMAT, (*itWebSite).policy.filePolicy.startDate, tzd).timestamp())
#else
		if ((localFile.getLastModified() + 28800000000LL) < DateTimeParser::parse(DateTimeFormat::SORTABLE_FORMAT, (*itWebSite).policy.filePolicy.startDate, tzd).timestamp())
#endif
		{
			return trace.SetRetResult( false );
		}
	}
	
	if ((*itWebSite).policy.filePolicy.endDateAvailable) // жļ޸
	{
		int tzd;
#ifdef WIN32
		if ((localFile.getLastModified() + 28800000000) > DateTimeParser::parse(DateTimeFormat::SORTABLE_FORMAT, (*itWebSite).policy.filePolicy.endDate, tzd).timestamp())
#else
		if ((localFile.getLastModified() + 28800000000LL) > DateTimeParser::parse(DateTimeFormat::SORTABLE_FORMAT, (*itWebSite).policy.filePolicy.endDate, tzd).timestamp())
#endif
		{
			return trace.SetRetResult( false );
		}
	}
	
	if ((*itWebSite).policy.filePolicy.maxFileSizeAvailable && !isDir) // жļֵ
	{
		if (localFile.getSize() > static_cast<File::FileSize>((*itWebSite).policy.filePolicy.maxFileSize))
		{
			return trace.SetRetResult( false );
		}
	}
	
	if ((*itWebSite).policy.filePolicy.minFileSizeAvailable && !isDir) // жļСֵ
	{
		if (localFile.getSize() < static_cast<File::FileSize>((*itWebSite).policy.filePolicy.minFileSize))
		{
			return trace.SetRetResult( false );
		}
	}
	
	return trace.SetRetResult( true );


}

bool CAppConfig::IsIncludePolicy(const std::string& localFileName)
{
	CGDTrace trace( LOG_L6, "CAppConfig:IsIncludePolicy(%s)", localFileName.c_str() );
	
	FastMutex lock(_mutex);
	CheckUpdate();
	
    bool validPath = false;
	
	// λļ/Ŀ¼վ
	std::vector<WebSite>::iterator itWebSite;
	for (itWebSite = m_webSites.begin(); itWebSite != m_webSites.end(); ++itWebSite)
	{
		std::vector<SiteDirectory>::iterator itDirectory;
		for (itDirectory = (*itWebSite).siteDirectorys.begin(); itDirectory != (*itWebSite).siteDirectorys.end(); ++itDirectory)
		{
			if (IsFileInDirectory(localFileName, (*itDirectory).localPath)) // localFileNameǷڸĿ¼
			{
				validPath = true;
				break;
			}
		}
		
		if (validPath)
			break;
	}
	
	if (!validPath)
	{
		return trace.SetRetResult( true ); // ļ/Ŀ¼κվ
	}
	
	if (!(*itWebSite).policy.filePolicy.available)
	{
		return trace.SetRetResult( false ); // δù˲
	}

    // ǷųĿ¼
    if ((*itWebSite).policy.filePolicy.excludePathAvailable) 
	{
        // жϵĿ¼ļ
	    if ( CFileOpt::IsDirectory( localFileName.c_str() ) )
		{
			std::vector<std::string>::iterator it;
			for (it = (*itWebSite).policy.filePolicy.excludePath.begin(); it != (*itWebSite).policy.filePolicy.excludePath.end(); ++it)
			{
				std::string path = (*it);
				
                // ƥųĿ¼
				if (icompare(localFileName, localFileName.length(), path) == 0)
					return trace.SetRetResult( true );
			}
		}
	}
	
    // Ƿļ͹
	if ((*itWebSite).policy.filePolicy.excludeTypeAvailable) 
	{
        if( (*itWebSite).policy.filePolicy.excludeType.size() > 0 )
        {
            return trace.SetRetResult( true );
        }
	}
	
    // Ƿģ
	if ((*itWebSite).policy.filePolicy.excludeBlurAvailable) 
	{
        if( (*itWebSite).policy.filePolicy.excludeBlur.size() > 0 )
        {
            return trace.SetRetResult( true );
        }
	}
	
    // Ƿųļ
	if ((*itWebSite).policy.filePolicy.excludeFileAvailable)
	{
		std::vector<std::string>::iterator it;
		for (it = (*itWebSite).policy.filePolicy.excludeFile.begin(); it != (*itWebSite).policy.filePolicy.excludeFile.end(); ++it)
		{
            std::string path = (*it);
            // ųļ
			if (icompare(localFileName, localFileName.length(), path) == 0) 
				return trace.SetRetResult( true );
		} 
	}
	
	return trace.SetRetResult( false );
}
/*****************************************
***		ļ
***		жǷųĿ¼ļ
***		ųtruefalse
***		add by wubin 2010-03-01
*****************************************/
bool CAppConfig::IsExcludeFile(const std::string& localFileName)
{
	CGDTrace trace( LOG_L6, "CAppConfig:IsExcludeFile(%s)", localFileName.c_str() );
	
	FastMutex lock(_mutex);
	CheckUpdate();
	
    bool validPath = false;
	
	// λļ/Ŀ¼վ
	std::vector<WebSite>::iterator itWebSite;
	for (itWebSite = m_webSites.begin(); itWebSite != m_webSites.end(); ++itWebSite)
	{
		std::vector<SiteDirectory>::iterator itDirectory;
		for (itDirectory = (*itWebSite).siteDirectorys.begin(); itDirectory != (*itWebSite).siteDirectorys.end(); ++itDirectory)
		{
			if (IsFileInDirectory(localFileName, (*itDirectory).localPath)) // localFileNameǷڸĿ¼
			{
				validPath = true;
				break;
			}
		}
		
		if (validPath)
			break;
	}
	
	if (!validPath)
	{
		return trace.SetRetResult( true ); // ļ/Ŀ¼κվ
	}
	if (!(*itWebSite).policy.monitorPolicy.available)
	{
		return trace.SetRetResult( false ); // δù˲
	}
	
	// жϵĿ¼ļ
	//if ( CFileOpt::IsDirectory( localFileName.c_str() ) )     //add by wubin жļĿ¼2010-02-26
	//{
		// Ŀ¼
		if ((*itWebSite).policy.monitorPolicy.excludePathAvailable) // ǷųĿ¼
		{
			std::vector<std::string>::iterator it;
			for (it = (*itWebSite).policy.monitorPolicy.excludePath.begin(); it != (*itWebSite).policy.monitorPolicy.excludePath.end(); ++it)
			{
				std::string path = (*it);
				std::string file = localFileName;
				
				if (icompare(file, path) == 0)// ųĿ¼
					return trace.SetRetResult( true );
				
				if (path.size() <= file.size())
				{
					std::string filepath;
					filepath = file.substr(0, path.size() + 1);
					
					if (icompare(filepath, path + Path::separator()) == 0) // ųĿ¼
						return trace.SetRetResult( true );
				}

				if ( (*it).find('*') != string::npos ) 
				{
					if (CGDString::MatchWithPattern((*it).c_str(), file.c_str()))
						return trace.SetRetResult(true);
				}		
			}
		}
		//return trace.SetRetResult( true );
	//}
	if ( CFileOpt::IsDirectory( localFileName.c_str() ) )    
		return false;
	
	if ((*itWebSite).policy.monitorPolicy.excludeTypeAvailable) // Ƿļ͹
	{
		std::string::size_type extPos;
		extPos = localFileName.rfind('.');
		
		if (extPos != std::string::npos) // չŹļ
		{
			std::string ext;
			ext = localFileName.substr(extPos + 1, localFileName.size());
			
			std::vector<std::string>::iterator it;
			for (it = (*itWebSite).policy.monitorPolicy.excludeType.begin(); it != (*itWebSite).policy.monitorPolicy.excludeType.end(); ++it)
			{
				if (icompare(ext, (*it)) == 0) // ųļ
					return trace.SetRetResult( true );
			}
		}
	}
	
	if ((*itWebSite).policy.monitorPolicy.excludeBlurAvailable) // Ƿģ
	{
		// ʵ
		// liu_yfeng	2008-07-08
		std::string name = localFileName;
		std::string::size_type last = name.find_last_of( S_PATHLASH );
		if ( last != std::string::npos )
		{
			name = name.substr( last + 1 );
		}
		std::vector<std::string>::iterator it;
		for (it = (*itWebSite).policy.monitorPolicy.excludeBlur.begin(); it != (*itWebSite).policy.monitorPolicy.excludeBlur.end(); ++it)
		{
			if ( CGDString::MatchWithPattern( (*it).c_str(), name.c_str() ) )
			{
				return trace.SetRetResult( true );
			}
		}
	}
	
	if ((*itWebSite).policy.monitorPolicy.excludeFileAvailable) // Ƿųļ
	{
		std::vector<std::string>::iterator it;
		for (it = (*itWebSite).policy.monitorPolicy.excludeFile.begin(); it != (*itWebSite).policy.monitorPolicy.excludeFile.end(); ++it)
		{
			if (icompare(localFileName, (*it)) == 0) // ųļ
				return trace.SetRetResult( true );
		} 
	}
	
	return trace.SetRetResult( false );
}
