//
// CorbaService.cpp
//
//

#include "CorbaService.h"
#include "MsgTran/MsgTran_impl.h"
#include <STAR/CORBA.h>
#include <STAR/AssistAdaptorManager.h>
#include <process.h>


CORBA::ORB_var CorbaService::_orb;

CorbaService::CorbaService(const string& localIp, int port):
	_stopped(false),
	_ip(localIp),
	_port(NumberFormatter::format(port))
{
}


CorbaService::~CorbaService()
{
	stop();
}


void CorbaService::start()
{
	poco_assert (!_stopped);

	_thread.start(*this);
		
	while ( CORBA::is_nil(  _orb ) )
	{
		CDateTime::Sleep(300);
	}
}


void CorbaService::stop()
{
	if (!_stopped)
	{
		_stopped = true;
		_thread.join();
	}
}


void CorbaService::run()
{
	while (true)
	{
		InitOrb();
		
		if (_stopped)
		{
			break;
		}
	}
}


long CorbaService::InitOrb()
{
	int state = 0;
	int nFileTranTreadPoolNum = 5;// ļ̳߳
	int nNetWaterMarkTreadPoolNum = 2;// ˮӡ̳߳
	
	try
	{
		// ʼ
		char* v_argv[] = {"-OAhost", (char*)_ip.c_str(), "-OAport", (char*)_port.c_str(), "-ORBthreaded" };
		int v_argc = sizeof(v_argv) / sizeof(v_argv[0]);

		// ʼORB

		_orb = CORBA::ORB_init(v_argc, v_argv);

		// servant
		// MsgTran
		MsgTran_impl* msgTranImpl = new MsgTran_impl();
		PortableServer::ServantBase_var servantMsgTran = msgTranImpl;

		// ȡRootPOA
		CORBA::Object_var poaObj = _orb -> resolve_initial_references("RootPOA");
		STARPortableServer::POA_var rootPOA = STARPortableServer::POA::_narrow(poaObj);
		// ȡRootPOAManager
		PortableServer::POAManager_var manager = rootPOA -> the_POAManager();

		// AssistAdaptorManager
		CORBA::Object_var aamObj = _orb->resolve_initial_references("AssistAdaptorManager");
		if( CORBA::is_nil(aamObj) )
		{
			printf("AssistAdaptorManager resole reference failed\n");
			state = -1;
			return 0;
		}
		STAR::AssistAdaptorManager_var aam = STAR::AssistAdaptorManager::_narrow(aamObj);

		//
		//ɷԹ
		//
		CORBA::Object_var dsfObj = _orb -> resolve_initial_references("RequestLevelDSFactory");
		STAR::RequestLevelDSFactory_var dsf = STAR::RequestLevelDSFactory::_narrow(dsfObj);

		//
		// ɷԶ 
		//
		
		// same thread
		STAR::DispatchRequestLevelStrategy_var ds_st;
		try
		{
			ds_st = dsf -> create_same_thread_strategy();
		}
		catch(CORBA::NO_IMPLEMENT&)
		{
			cout<<"system donnot implement this"<<endl;
		}

		// thread_per_request
		STAR::DispatchRequestLevelStrategy_var ds_tpr;
		try
		{
			ds_tpr = dsf -> create_thread_per_request_strategy();
		}
		catch(CORBA::NO_IMPLEMENT&)
		{
			cout<<"system donnot implement this"<<endl;
		}
		// filetran thread pool
		STAR::DispatchRequestLevelStrategy_var ds_tp_fileTran;
		STAR::ThreadPoolId tpId_fileTran;
		try
		{
			tpId_fileTran = dsf -> create_thread_pool(nFileTranTreadPoolNum);
			ds_tp_fileTran = dsf -> create_thread_pool_strategy(tpId_fileTran);
		}
		catch(CORBA::NO_IMPLEMENT&)
		{
			cout<<"system donnot implement this"<<endl;
		}

		// netwatermark thread pool
		STAR::DispatchRequestLevelStrategy_var ds_tp_netWaterMark;
		STAR::ThreadPoolId tpId_netWaterMark;
		try
		{
			tpId_netWaterMark = dsf -> create_thread_pool(nNetWaterMarkTreadPoolNum);
			ds_tp_netWaterMark = dsf -> create_thread_pool_strategy(tpId_netWaterMark);
		}
		catch(CORBA::NO_IMPLEMENT&)
		{
			cout<<"system donnot implement this"<<endl;
		}
		// netlog thread pool
		STAR::DispatchRequestLevelStrategy_var ds_tp_netLog;
		STAR::ThreadPoolId tpId_netLog;
		try
		{
			tpId_netLog = dsf -> create_thread_pool(2);
			ds_tp_netLog = dsf -> create_thread_pool_strategy(tpId_netLog);
		}
		catch(CORBA::NO_IMPLEMENT&)
		{
			cout<<"system donnot implement this"<<endl;
		}
		// netDirectory thread pool
		STAR::DispatchRequestLevelStrategy_var ds_tp_netDirectory;
		STAR::ThreadPoolId tpId_netDirectory;
		try
		{
			tpId_netDirectory = dsf -> create_thread_pool(2);
			ds_tp_netDirectory = dsf -> create_thread_pool_strategy(tpId_netDirectory);
		}
		catch(CORBA::NO_IMPLEMENT&)
		{
			cout<<"system donnot implement this"<<endl;
		}
		//
		// ɷԵPOA
		//     - same thread
		//     - thread per request
		//     - thread pool
		//
		CORBA::PolicyList policies(2);
		policies.length(2);

		policies[0] = rootPOA -> create_lifespan_policy(PortableServer::PERSISTENT);// ־
		policies[1] = rootPOA -> create_requestlevel_dispatch_policy(ds_st);
		PortableServer::POA_var stPOA =	rootPOA -> create_POA("stPOA", manager, policies);
		policies[1] = rootPOA -> create_requestlevel_dispatch_policy(ds_tpr);
		PortableServer::POA_var tprPOA = rootPOA -> create_POA("tprPOA", manager, policies);
		policies[1] = rootPOA -> create_requestlevel_dispatch_policy(ds_tp_fileTran);
		PortableServer::POA_var tpFileTranPOA = rootPOA -> create_POA("tpFileTranPOA", manager, policies);
		policies[1] = rootPOA -> create_requestlevel_dispatch_policy(ds_tp_netWaterMark);
		PortableServer::POA_var tpNetWaterMarkPOA = rootPOA -> create_POA("tpNetWaterMarkPOA", manager, policies);
		policies[1] = rootPOA -> create_requestlevel_dispatch_policy(ds_tp_netLog);
		PortableServer::POA_var tpNetLogPOA = rootPOA -> create_POA("tpNetLogPOA", manager, policies);
		policies[1] = rootPOA -> create_requestlevel_dispatch_policy(ds_tp_netDirectory);
		PortableServer::POA_var tpNetDirectoryPOA = rootPOA -> create_POA("tpNetDirectory", manager, policies);


		// ObjectId
		PortableServer::ObjectId_var objIdMsgTran = PortableServer::string_to_ObjectId("MsgTran");

		/*	===========ò=========
			ļˮӡƵʱ-̳߳ز
			same thread?
			һMsgTranthread_per_request,ԴͬʱϢ?
			=======================	*/
		PortableServer::ObjectId_var idMsgTran= stPOA -> activate_object(servantMsgTran);

		// ö
		MsgTran_var objMsgTran = InforGuard::MsgTran::_narrow(stPOA -> servant_to_reference(msgTranImpl));
		
		// 
		aam->binding_objref(objIdMsgTran, objMsgTran);

		// POAManagerORB
		manager -> activate();

		printf( "ORBThread started ...\n" );

		_orb -> run();
	}
	catch( const CORBA::Exception& ex )
	{
		cerr << ex << endl;
		state = -1;
	}
	
	if(!CORBA::is_nil(_orb))
	{
		try
		{
			_orb -> destroy();
		}
		catch( const CORBA::Exception& ex )
		{
			cerr << ex << endl;
			state = -1;
		}
	}
	
	return 0;
}
