//
// Exception.cpp
//
// $Id: Exception.cpp,v 1.3 2008/06/17 06:33:43 zhang_dpeng Exp $
//


#include "Groundwork/Exception.h"
#include <typeinfo>


namespace InforGuard {
namespace Groundwork {


Exception::Exception(int code): _pNested(0), _code(code)
{
}


Exception::Exception(const string& msg, int code): _msg(msg), _pNested(0), _code(code)
{
}


Exception::Exception(const string& msg, const string& arg, int code): _msg(msg), _pNested(0), _code(code)
{
	if (!arg.empty())
	{
		_msg.append(": ");
		_msg.append(arg);
	}
}


Exception::Exception(const string& msg, const Exception& nested, int code): _msg(msg), _pNested(nested.clone()), _code(code)
{
}


Exception::Exception(const Exception& exc):
	exception(exc),
	_msg(exc._msg),
	_code(exc._code)
{
	_pNested = exc._pNested ? exc._pNested->clone() : 0;
}

	
Exception::~Exception() throw()
{
	delete _pNested;
}


Exception& Exception::operator = (const Exception& exc)
{
	if (&exc != this)
	{
		delete _pNested;
		_msg     = exc._msg;
		_pNested = exc._pNested ? exc._pNested->clone() : 0;
		_code    = exc._code;
	}
	return *this;
}


const char* Exception::name() const throw()
{
	return "Exception";
}


const char* Exception::className() const throw()
{
	return typeid(*this).name();
}


const char* Exception::what() const throw()
{
	return name();
}

	
string Exception::displayText() const
{
	string txt = name();
	if (!_msg.empty())
	{
		txt.append(": ");
		txt.append(_msg);
	}
	return txt;
}


Exception* Exception::clone() const
{
	return new Exception(*this);
}


void Exception::rethrow() const
{
	throw *this;
}


GROUNDWORK_IMPLEMENT_EXCEPTION(LogicException, Exception, "Logic exception")
GROUNDWORK_IMPLEMENT_EXCEPTION(AssertionViolationException, LogicException, "Assertion violation")
GROUNDWORK_IMPLEMENT_EXCEPTION(NullPointerException, LogicException, "Null pointer")
GROUNDWORK_IMPLEMENT_EXCEPTION(BugcheckException, LogicException, "Bugcheck")
GROUNDWORK_IMPLEMENT_EXCEPTION(InvalidArgumentException, LogicException, "Invalid argument")
GROUNDWORK_IMPLEMENT_EXCEPTION(NotImplementedException, LogicException, "Not implemented")
GROUNDWORK_IMPLEMENT_EXCEPTION(RangeException, LogicException, "Out of range")
GROUNDWORK_IMPLEMENT_EXCEPTION(IllegalStateException, LogicException, "Illegal state")
GROUNDWORK_IMPLEMENT_EXCEPTION(InvalidAccessException, LogicException, "Invalid access")
GROUNDWORK_IMPLEMENT_EXCEPTION(SignalException, LogicException, "Signal received")
GROUNDWORK_IMPLEMENT_EXCEPTION(UnhandledException, LogicException, "Signal received")

GROUNDWORK_IMPLEMENT_EXCEPTION(RuntimeException, Exception, "Runtime exception")
GROUNDWORK_IMPLEMENT_EXCEPTION(NotFoundException, RuntimeException, "Not found")
GROUNDWORK_IMPLEMENT_EXCEPTION(ExistsException, RuntimeException, "Exists")
GROUNDWORK_IMPLEMENT_EXCEPTION(TimeoutException, RuntimeException, "Timeout")
GROUNDWORK_IMPLEMENT_EXCEPTION(SystemException, RuntimeException, "System exception")
GROUNDWORK_IMPLEMENT_EXCEPTION(RegularExpressionException, RuntimeException, "Error in regular expression")
GROUNDWORK_IMPLEMENT_EXCEPTION(LibraryLoadException, RuntimeException, "Cannot load library")
GROUNDWORK_IMPLEMENT_EXCEPTION(LibraryAlreadyLoadedException, RuntimeException, "Library already loaded")
GROUNDWORK_IMPLEMENT_EXCEPTION(NoThreadAvailableException, RuntimeException, "No thread available")
GROUNDWORK_IMPLEMENT_EXCEPTION(PropertyNotSupportedException, RuntimeException, "Property not supported")
GROUNDWORK_IMPLEMENT_EXCEPTION(PoolOverflowException, RuntimeException, "Pool overflow")
GROUNDWORK_IMPLEMENT_EXCEPTION(NoPermissionException, RuntimeException, "No permission")
GROUNDWORK_IMPLEMENT_EXCEPTION(OutOfMemoryException, RuntimeException, "Out of memory")
GROUNDWORK_IMPLEMENT_EXCEPTION(DataException, RuntimeException, "Data error")

GROUNDWORK_IMPLEMENT_EXCEPTION(DataFormatException, DataException, "Bad data format")
GROUNDWORK_IMPLEMENT_EXCEPTION(SyntaxException, DataException, "Syntax error")
GROUNDWORK_IMPLEMENT_EXCEPTION(CircularReferenceException, DataException, "Circular reference")
GROUNDWORK_IMPLEMENT_EXCEPTION(PathSyntaxException, SyntaxException, "Bad path syntax")
GROUNDWORK_IMPLEMENT_EXCEPTION(IOException, RuntimeException, "I/O error")
GROUNDWORK_IMPLEMENT_EXCEPTION(FileException, IOException, "File access error")
GROUNDWORK_IMPLEMENT_EXCEPTION(FileExistsException, FileException, "File exists")
GROUNDWORK_IMPLEMENT_EXCEPTION(FileNotFoundException, FileException, "File not found")
GROUNDWORK_IMPLEMENT_EXCEPTION(PathNotFoundException, FileException, "Path not found")
GROUNDWORK_IMPLEMENT_EXCEPTION(FileReadOnlyException, FileException, "File is read-only")
GROUNDWORK_IMPLEMENT_EXCEPTION(FileAccessDeniedException, FileException, "Access to file denied")
GROUNDWORK_IMPLEMENT_EXCEPTION(CreateFileException, FileException, "Cannot create file")
GROUNDWORK_IMPLEMENT_EXCEPTION(OpenFileException, FileException, "Cannot open file")
GROUNDWORK_IMPLEMENT_EXCEPTION(WriteFileException, FileException, "Cannot write file")
GROUNDWORK_IMPLEMENT_EXCEPTION(ReadFileException, FileException, "Cannot read file")
GROUNDWORK_IMPLEMENT_EXCEPTION(UnknownURISchemeException, RuntimeException, "Unknown URI scheme")


GROUNDWORK_IMPLEMENT_EXCEPTION(ApplicationException, Exception, "Application exception")
GROUNDWORK_IMPLEMENT_EXCEPTION(BadCastException, RuntimeException, "Bad cast exception")

} // namespace Groundwork
}	// namespace InforGuard
