// FileList.cpp

#pragma warning( disable: 4786 )


#ifdef WIN32
#else
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#endif

#include <algorithm>

#include "Common/GDString.h"
#include "Common/DateTime.h"
#include "FileOpt/FileOpt.h"
#include "FileOpt/FileList.h"
#include "GDLog/GDTrace.h"
#include <ctime>

CFileList::CFileList()
{
}

CFileList::~CFileList()
{
}

long CFileList::CreateList( const char* sTopPath, FileList& fileList, int nMode, bool bSort ) throw( CExceptionError )
{
	char sTmp[17] = {0};
	string sParam;
	sParam = sParam + "sTopPath[" + sTopPath
		+ "]nMode[" + CGDString::ltoa( nMode, 16 )
		+ "]";
	
	CGDTrace trace( LOG_L6, "CFileList:CreateList(%s)", sParam.c_str() );

	bool bFolder;

	try
	{
		bool bExist = CFileOpt::IsExist( sTopPath, &bFolder );
		if ( ! bExist && sTopPath && *sTopPath )
		{
			return 404; // Ŀ¼
		}

		if ( sTopPath && ( sTopPath[0] == '/' || sTopPath[1] == ':' ) )
		{
			if ( IsValidFile( sTopPath ) )		// ޸ֶͬųԲ add by liu_yfeng 2008-11-27
			{
				FileListNode fileListNode;
				fileListNode.name = sTopPath;

				if ( ! bFolder )
				{
					// ļ
					fileListNode.size = CFileOpt::GetFileSize( sTopPath );
					fileListNode.isfolder = false;
					
					fileList.push_back(fileListNode);
					//if ( nMode & FILE_OPTION_DIRECTORY )
					//{
					//	return -1; // ޷ļĿ¼
					//}
					return 0;
				}
				// Ŀ¼
				fileListNode.size = 0;
				fileListNode.isfolder = true;
				
				//if ( nMode & CFileList::FILE_OPTION_TOPPATH )
				{
					fileList.push_back(fileListNode);
				}
			}
			else
			{
				return 0;
			}
		}
		else
		{
			FileListNode fileListNode;
#ifdef WIN32
			// Windows ҵĵ
			// ȡϢ
			char drive[128] = {0};
			GetLogicalDriveStrings( sizeof(drive), drive );
			char * p = drive;
			for ( int i=0; *(p+1)==':'; i++ )
			{
				UINT uDriveType = GetDriveType(p);
				if ( ( uDriveType == DRIVE_FIXED || uDriveType == DRIVE_REMOVABLE )
					&& ( *p != 'A' ) )
				{
					fileListNode.name = p;
					fileListNode.size = 0;
					fileListNode.isfolder = false;
					fileList.push_back( fileListNode );
				}
				p = p + strlen(p) + 1;
			}
#else
			// Linux/UNIX /
			fileListNode.name = "/";
			fileListNode.size = 0;
			fileListNode.isfolder = false;
			fileList.push_back( fileListNode );
#endif
			return 0;
		}
	}
	catch ( CExceptionError & ex )
	{
		ex.AddErrText( "CFileList:CreateList(%s)", sParam.c_str() );
		throw ex;
	}

	long nRet = CreateNode( sTopPath, nMode, fileList);

	if ( 0 == nRet )
	{
		// 
		if ( bSort )
		{
			fileList.sort();
		}
	}
	else
	{
		fileList.clear();
	}

	return nRet;
}

//websiteIDжļǷϷ	
long CFileList::CreateList( const char* sTopPath, const char* sWebSiteID, FileList& fileList, int nMode, bool bSort ) throw ( CExceptionError )
{
	char sTmp[17] = {0};
	string sParam;
	sParam = sParam + "sTopPath[" + sTopPath
		+ "]nMode[" + CGDString::ltoa( nMode, 16 )
		+ "]";
	
	CGDTrace trace( LOG_L6, "CFileList:CreateList(%s)", sParam.c_str() );

	bool bFolder;

	try
	{
		bool bExist = CFileOpt::IsExist( sTopPath, &bFolder );
		if ( ! bExist && sTopPath && *sTopPath )
		{
			return 404; // Ŀ¼
		}

		if ( sTopPath && ( sTopPath[0] == '/' || sTopPath[1] == ':' ) )
		{
			if ( IsValidFile( sTopPath,sWebSiteID ) )		// ޸ֶͬųԲ add by liu_yfeng 2008-11-27
			{
				FileListNode fileListNode;
				fileListNode.name = sTopPath;

				if ( ! bFolder )
				{
					// ļ
					fileListNode.size = CFileOpt::GetFileSize( sTopPath );
					fileListNode.isfolder = false;
					
					fileList.push_back(fileListNode);
					//if ( nMode & FILE_OPTION_DIRECTORY )
					//{
					//	return -1; // ޷ļĿ¼
					//}
					return 0;
				}
				// Ŀ¼
				fileListNode.size = 0;
				fileListNode.isfolder = true;
				
				fileList.push_back(fileListNode);
			}
			else
			{
				return 0;
			}
		}
		else
		{
			FileListNode fileListNode;
#ifdef WIN32
			// Windows ҵĵ
			// ȡϢ
			char drive[128] = {0};
			GetLogicalDriveStrings( sizeof(drive), drive );
			char * p = drive;
			for ( int i=0; *(p+1)==':'; i++ )
			{
				UINT uDriveType = GetDriveType(p);
				if ( ( uDriveType == DRIVE_FIXED || uDriveType == DRIVE_REMOVABLE )
					&& ( *p != 'A' ) )
				{
					fileListNode.name = p;
					fileListNode.size = 0;
					fileListNode.isfolder = false;
					fileList.push_back( fileListNode );
				}
				p = p + strlen(p) + 1;
			}
#else
			// Linux/UNIX /
			fileListNode.name = "/";
			fileListNode.size = 0;
			fileListNode.isfolder = false;
			fileList.push_back( fileListNode );
#endif
			return 0;
		}
	}
	catch ( CExceptionError & ex )
	{
		ex.AddErrText( "CFileList:CreateList(%s)", sParam.c_str() );
		throw ex;
	}

	long nRet = CreateNode( sTopPath, sWebSiteID, nMode, fileList);

	if ( 0 == nRet )
	{
		// 
		if ( bSort )
		{
			fileList.sort();
		}
	}
	else
	{
		fileList.clear();
	}

	return nRet;
}

long CFileList::CreateNode( const char* sTopPath, int nMode, FileList& fileList)
{
#ifdef WIN32 // Windows

	WIN32_FIND_DATA FileData;
	HANDLE hSearch;
	
	char localPath[GD_MAX_PATH] = {0};
	char topPath[GD_MAX_PATH] = {0};
	
	strcpy( localPath, sTopPath );
	int localLen = strlen( localPath );
	if ( localPath[ localLen - 1 ] != C_PATHLASH )
	{
		strcat( localPath, S_PATHLASH );
	}
	
	sprintf( topPath, "%s*", localPath );
	
	hSearch = FindFirstFile( (LPCTSTR)topPath, &FileData );
	if ( hSearch == INVALID_HANDLE_VALUE )
	{
		// Ŀ¼ûļ
		ErrCode errCode = GetLastError();
		GDPrint( "FindFirstFile(%s) failed [%d][%s]-ɸĿ¼ûļ\n", topPath, errCode, strerror(errCode) );
		return 0;
	}
	
	char strCur[GD_MAX_PATH] = {0};

	// ǰĿ¼
	do
	{
		// ֻѡʱļ|Ŀ¼|ļĿ¼
		if ( ( (FileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY ) 
				&& ( (nMode & FILE_OPTION_DIRECTORY) == FILE_OPTION_DIRECTORY )
			|| ( (FileData.dwFileAttributes & FILE_OPTION_DIRECTORY) != FILE_ATTRIBUTE_DIRECTORY )
				&& ( (nMode & FILE_OPTION_FILE) == FILE_OPTION_FILE ) )
		{
			FileListNode son;
			memset(strCur, 0, sizeof(strCur));
			if ( strcmp(FileData.cFileName, ".") && strcmp(FileData.cFileName, "..") )
			{
				sprintf(strCur, "%s%s", localPath, FileData.cFileName);
				// Դ
				
				if ( !IsValidFile( strCur ) )
				{
					continue;
				}
				
				son.name = strCur; // FileData.cFileName

				if ( FileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
				{
					// Ŀ¼: ݹ鴦
					son.size = 0;
					son.isfolder = true;
					CDateTime::FileTimeToUnixTime( &FileData.ftLastWriteTime, son.time );

					fileList.push_back( son );

					if ( ( nMode & FILE_OPTION_LIST ) != FILE_OPTION_LIST )
					{
						CreateNode( strCur, nMode, fileList );
					}
				}
				else
				{
					// ļ: ¼С
					son.size = FileData.nFileSizeHigh << 32;
					son.size += FileData.nFileSizeLow;
					son.isfolder = false;
					CDateTime::FileTimeToUnixTime( &FileData.ftLastWriteTime, son.time );

					fileList.push_back( son );
				}
			}
		}//if

	} while( FindNextFile( hSearch, &FileData ) );
	
	FindClose(hSearch);

#else // Linux/UNIX

	DIR* dp = NULL;
	struct dirent *dirp = NULL;

	char topPath[GD_MAX_PATH] = {0};
	bool bFinished = false;
	
	strcpy( topPath, sTopPath );
	int localLen = strlen( topPath );
	if ( topPath[ localLen - 1 ] != C_PATHLASH )
	{
		strcat( topPath, S_PATHLASH );
	}
	
	dp = opendir(topPath);
	if( dp == NULL )
	{
		// Ŀ¼ûļ
		bFinished = true; 
	}
	else
	{
		// ???
		dirp = readdir(dp);
		if( dirp == NULL )
		{
			bFinished = true;
		}
	}
	
	char strCur[GD_MAX_PATH] = {0};

	// ǰĿ¼
	while ( ! bFinished )
	{
		if ( strcmp( dirp->d_name, "." ) && strcmp( dirp->d_name, ".." ) )
		{
			sprintf( strCur, "%s%s", topPath, dirp->d_name );
			
			// Դ
			if ( IsValidFile( strCur ) )
			{
				struct stat statbuf;
				memset( &statbuf, 0, sizeof(statbuf) );
				
				if ( lstat( strCur, &statbuf ) )
				{
					bFinished = true;
					continue;
				}
				
				// ֻѡʱļ|Ŀ¼|ļĿ¼
				if ( S_ISDIR(statbuf.st_mode) && ( (nMode & FILE_OPTION_DIRECTORY) == FILE_OPTION_DIRECTORY )
					|| S_ISREG(statbuf.st_mode) && ( (nMode & FILE_OPTION_FILE) == FILE_OPTION_FILE ) )
				{
					FileListNode son;
					
					son.name = strCur; // dirp->d_name
					
					if ( S_ISDIR(statbuf.st_mode) )
					{
						// Ŀ¼: ݹ鴦
						son.size = 0;
						son.isfolder = true;
						son.time = statbuf.st_mtime;
						
						fileList.push_back( son );
						
						if ( ( nMode & FILE_OPTION_LIST ) != FILE_OPTION_LIST )
						{
							CreateNode( strCur, nMode, fileList );
						}
					}
					else
					{
						// ļ: ¼С
						son.size = statbuf.st_size;
						son.isfolder = false;
						son.time = statbuf.st_mtime;
						
						fileList.push_back( son );
					}
				}
			}
		}
		
		dirp = readdir(dp);
		if ( dirp == NULL )
		{
			bFinished = true;
		}
	}//while
	
	closedir(dp);

#endif

	return 0;
}


long CFileList::CreateNode( const char* sTopPath, const char* sWebSiteID, int nMode, FileList& fileList)
{
#ifdef WIN32 // Windows

	WIN32_FIND_DATA FileData;
	HANDLE hSearch;
	
	char localPath[GD_MAX_PATH] = {0};
	char topPath[GD_MAX_PATH] = {0};
	
	strcpy( localPath, sTopPath );
	int localLen = strlen( localPath );
	if ( localPath[ localLen - 1 ] != C_PATHLASH )
	{
		strcat( localPath, S_PATHLASH );
	}
	
	sprintf( topPath, "%s*", localPath );
	
	hSearch = FindFirstFile( (LPCTSTR)topPath, &FileData );
	if ( hSearch == INVALID_HANDLE_VALUE )
	{
		// Ŀ¼ûļ
		ErrCode errCode = GetLastError();
		GDPrint( "FindFirstFile(%s) failed [%d][%s]-ɸĿ¼ûļ\n", topPath, errCode, strerror(errCode) );
		return 0;
	}
	
	char strCur[GD_MAX_PATH] = {0};

	// ǰĿ¼
	do
	{
		// ֻѡʱļ|Ŀ¼|ļĿ¼
		if ( ( (FileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY ) 
				&& ( (nMode & FILE_OPTION_DIRECTORY) == FILE_OPTION_DIRECTORY )
			|| ( (FileData.dwFileAttributes & FILE_OPTION_DIRECTORY) != FILE_ATTRIBUTE_DIRECTORY )
				&& ( (nMode & FILE_OPTION_FILE) == FILE_OPTION_FILE ) )
		{
			FileListNode son;
			memset(strCur, 0, sizeof(strCur));
			if ( strcmp(FileData.cFileName, ".") && strcmp(FileData.cFileName, "..") )
			{
				sprintf(strCur, "%s%s", localPath, FileData.cFileName);
				// Դ
				
				if ( !IsValidFile( strCur, sWebSiteID ) )
				{
					continue;
				}
				
				son.name = strCur; // FileData.cFileName

				if ( FileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
				{
					// Ŀ¼: ݹ鴦
					son.size = 0;
					son.isfolder = true;
					CDateTime::FileTimeToUnixTime( &FileData.ftLastWriteTime, son.time );

					fileList.push_back( son );

					if ( ( nMode & FILE_OPTION_LIST ) != FILE_OPTION_LIST )
					{
						CreateNode( strCur, sWebSiteID, nMode, fileList );
					}
				}
				else
				{
					// ļ: ¼С
					son.size = FileData.nFileSizeHigh << 32;
					son.size += FileData.nFileSizeLow;
					son.isfolder = false;
					CDateTime::FileTimeToUnixTime( &FileData.ftLastWriteTime, son.time );

					fileList.push_back( son );
				}
			}
		}//if

	} while( FindNextFile( hSearch, &FileData ) );
	
	FindClose(hSearch);

#else // Linux/UNIX

	DIR* dp = NULL;
	struct dirent *dirp = NULL;

	char topPath[GD_MAX_PATH] = {0};
	bool bFinished = false;
	
	strcpy( topPath, sTopPath );
	int localLen = strlen( topPath );
	if ( topPath[ localLen - 1 ] != C_PATHLASH )
	{
		strcat( topPath, S_PATHLASH );
	}
	
	dp = opendir(topPath);
	if( dp == NULL )
	{
		// Ŀ¼ûļ
		bFinished = true; 
	}
	else
	{
		// ???
		dirp = readdir(dp);
		if( dirp == NULL )
		{
			bFinished = true;
		}
	}
	
	char strCur[GD_MAX_PATH] = {0};

	// ǰĿ¼
	while ( ! bFinished )
	{
		if ( strcmp( dirp->d_name, "." ) && strcmp( dirp->d_name, ".." ) )
		{
			sprintf( strCur, "%s%s", topPath, dirp->d_name );
			
			// Դ
			if ( IsValidFile( strCur ,sWebSiteID ) )
			{
				struct stat statbuf;
				memset( &statbuf, 0, sizeof(statbuf) );
				
				if ( lstat( strCur, &statbuf ) )
				{
					bFinished = true;
					continue;
				}
				
				// ֻѡʱļ|Ŀ¼|ļĿ¼
				if ( S_ISDIR(statbuf.st_mode) && ( (nMode & FILE_OPTION_DIRECTORY) == FILE_OPTION_DIRECTORY )
					|| S_ISREG(statbuf.st_mode) && ( (nMode & FILE_OPTION_FILE) == FILE_OPTION_FILE ) )
				{
					FileListNode son;
					
					son.name = strCur; // dirp->d_name
					
					if ( S_ISDIR(statbuf.st_mode) )
					{
						// Ŀ¼: ݹ鴦
						son.size = 0;
						son.isfolder = true;
						son.time = statbuf.st_mtime;
						
						fileList.push_back( son );
						
						if ( ( nMode & FILE_OPTION_LIST ) != FILE_OPTION_LIST )
						{
							CreateNode( strCur, sWebSiteID, nMode, fileList );
						}
					}
					else
					{
						// ļ: ¼С
						son.size = statbuf.st_size;
						son.isfolder = false;
						son.time = statbuf.st_mtime;
						
						fileList.push_back( son );
					}
				}
			}
		}
		
		dirp = readdir(dp);
		if ( dirp == NULL )
		{
			bFinished = true;
		}
	}//while
	
	closedir(dp);

#endif

	return 0;
}

ULONGLONG CFileList::CalcList(FileList& fileList)
{
	ULONGLONG nSize = 0;

	FileList::iterator iter;
	for ( iter = fileList.begin(); iter != fileList.end(); ++iter )  
	{
		nSize += (*iter).size;
	}

	return nSize;
}
