// FileTranClient.cpp

#pragma warning( disable: 4786 )

#include "Base/AppOption.h"
#include "Base/NetTran.h"
#include "Base/RunInfor.h"
#include "Base/Tool.h"
#include "Base/Toolbox.h"
#include "Common/GDString.h"
#include "Common/GDFile.h"
#include "GDLog/GDTrace.h"
#include "FileTran/FileTranClient.h"
#include "GDLog/GDLog.h"
#include "GDLog/ThreadData.h"

std::map<string, string> CFileTranClient::m_basePathMap;
CThreadMutex CFileTranClient::m_mutex;

CFileTranClient::CFileTranClient( const char* sDstHostAddr, const char* sDstPort ) throw (CExceptionError)
{
	CGDTrace trace( LOG_L6, "CFileTranClient:CFileTranClient(%s,%s)", sDstHostAddr, sDstPort );
	Init(sDstHostAddr, sDstPort);
}

CFileTranClient::CFileTranClient()
{
	m_bInitialized = false;
}

CFileTranClient::~CFileTranClient()
{
}

//add by wu_bin 20090928 ӵĿպ
void CFileTranClient::Check() throw(CExceptionError)
{
	m_ref->Check();
}

long CFileTranClient::Init( const char* sDstHostAddr, const char* sDstPort ) throw (CExceptionError)
{
	CGDTrace trace( LOG_L6, "CFileTranClient:Init(%s,%s)", sDstHostAddr, sDstPort );
	
	m_sAddr = sDstHostAddr;
	m_sPort = sDstPort;

	try
	{
		CNetTran::GetFileTranObject( sDstHostAddr, sDstPort, m_ref );

		// WindowsԶȡĿ¼洢 by wang_jin at 2009-9-20 for Ϣ
        CORBA::String_var varBasePath;
        long nRet = m_ref->QueryBasePath( varBasePath.out() );
   		if( !CORBA::is_nil( m_ref ) )
		{
			CORBA::String_var varBasePath;
			long nRet = m_ref->QueryBasePath( varBasePath.out() );
			
			if ( ! nRet )
			{
				string sKey = sDstHostAddr;
				sKey.append("-");
				sKey.append(sDstPort);
				CThreadLock lock(m_mutex);
				std::map<string, string>::iterator itor = m_basePathMap.find( sKey );
				if ( itor == m_basePathMap.end() )
				{
					m_basePathMap.insert( make_pair( sKey, varBasePath.in() ) );
				}
				else
				{
					(*itor).second = varBasePath.in();
				}
			}
		}else
		{
			CGDLog::WriteErrLog( "CFileTranClient:Init-GetFileTranObject is NULL");
		}
	}
	catch ( CExceptionError & ex )
	{		
		ex.AddErrText( "CFileTranClient:Init(%s,%s)", sDstHostAddr, sDstPort );
		throw ex;
	}
	catch ( const CORBA::Exception& ex )
	{
		// ת쳣ʽ
		CExceptionError e;
		e.AddErrText( "%s", ex._to_string() );
		e.AddErrText( "CFileTranClient-Init(%s,%s)", sDstHostAddr, sDstPort );
		throw e;
	}
	catch ( ... )
	{
		CExceptionError e;
		e.AddErrText( "CFileTranClient-Init(%s,%s) catch(...)", sDstHostAddr, sDstPort );
		throw e;
	}

	m_bInitialized = true;

	return trace.SetRetValue(0);
}

long CFileTranClient::PushFile( const char* sSrcFileName, const char* sDstFileName ) throw (CExceptionError)
{
    std::string sParam;
	sParam = sParam 
		+  "sSrcFileName[" + sSrcFileName
		+ "]sDstFileName[" + sDstFileName
		+ "]";

	CGDTrace trace( LOG_L6, "CFileTranClient:PushFile(%s)", sParam.c_str() );

	long nRet = 0;

	// ״̬ṩһӻ
	if ( ! m_bInitialized )
	{
		try
		{
			nRet = Init( m_sAddr.c_str(), m_sPort.c_str() );
		}
		catch ( CExceptionError & ex )
		{		
			ex.AddErrText( "CFileTranClient:Init(%s,%s)", m_sAddr.c_str(), m_sPort.c_str() );
			throw ex;
		}
	}

	FILE* fp = CGDFile::open( sSrcFileName, "rb" );

	if ( fp )
	{
		long nLen = 0;
#ifdef WIN32
		fpos_t nOffset = (fpos_t)0;	// ʾļܴ4G
#else
		fpos_t nOffset = {0};
#endif

		OctetSeq buf;
		long nMaxBlogSize = CAppOption::Instance().GetOption("filetran.blocksize", 262144);
		
		if (nMaxBlogSize == 0)
			nMaxBlogSize = 262144;

		buf.length( nMaxBlogSize );
		do
		{
			// ״̬ṩһ
			if ( ! m_bInitialized )
			{
				try
				{
					nRet = Init( m_sAddr.c_str(), m_sPort.c_str() );
				}
				catch ( CExceptionError & ex )
				{		
					CGDFile::close( fp );
					ex.AddErrText( "CFileTranClient:Init(%s,%s)", m_sAddr.c_str(), m_sPort.c_str() );
					throw ex;
				}
			}

			nLen = fread( buf.get_buffer(), 1, nMaxBlogSize, fp );
			if ( nLen < nMaxBlogSize )
			{
				buf.length( nLen );
			}

			if ( nLen >= 0 ) // =0ļ
			{
#ifdef WIN32
				if ( nLen == 0 && 0 != nOffset )
#elif defined AIX
				if ( nLen == 0 && 0 != nOffset )
#elif defined SunOS
				if ( nLen == 0 && 0 != nOffset )
#elif defined HPUX
				if ( nLen == 0 && 0 != nOffset )
#elif defined HPUXIA
				if ( nLen == 0 && 0 != nOffset )
#else
				if ( nLen == 0 && 0 != nOffset.__pos )
#endif
				{
					break;
				}
				try
				{
					InforGuard::netCallData callData;
					CNetTran::InitNetCallData( callData ); // 
#ifdef WIN32
					nRet = m_ref->PutFileBlock( callData, sDstFileName, nOffset, buf, nLen );
#elif defined AIX
					nRet = m_ref->PutFileBlock( callData, sDstFileName, nOffset, buf, nLen );
#elif defined SunOS
					nRet = m_ref->PutFileBlock( callData, sDstFileName, nOffset, buf, nLen );
#elif defined HPUX
					nRet = m_ref->PutFileBlock( callData, sDstFileName, nOffset, buf, nLen );
#elif defined HPUXIA
					nRet = m_ref->PutFileBlock( callData, sDstFileName, nOffset, buf, nLen );
#else
					nRet = m_ref->PutFileBlock( callData, sDstFileName, nOffset.__pos, buf, nLen );
#endif
					CThreadData::GetThreadSerial( &callData.nRecSerial ); // 
				}
				catch( InforGuard::ExceptionError& ex )
				{
					CGDFile::close( fp );

					CExceptionError e;
					CNetTran::ConvertException(ex, e);
					e.AddErrText( "CFileTranClient:PushFile(%s)", sParam.c_str() );
					throw e;
				}
				catch( const CORBA::Exception& ex )
				{
					CGDFile::close( fp );
					// һCORBA쳣ֱΪ
					CNetTran::RemoveFileTranObject( m_sAddr.c_str(), m_sPort.c_str() );
					m_bInitialized = false;

					// ת쳣ʽ
					CExceptionError e;
					e.AddErrText( "%s", ex._to_string() );
					e.AddErrText( "CFileTranClient:PushFile(%s)", sParam.c_str() );
					throw e;
				}
				catch( ... )
				{
					CExceptionError e;
					e.AddErrText( "CFileTranClient:PushFile(%s)catch( ... )", sParam.c_str() );
					throw e;
				}

				fgetpos( fp, &nOffset );
				//nOffset += nLen;
			}//if
		} while( nLen > 0 && nRet == 0 );

		CGDFile::close( fp );
	}
	else
	{
		nRet = 0x1000 + 0x108;
	}

	return trace.SetRetValue( nRet );
}

long CFileTranClient::PullFile( const char* sSrcFileName, const char* sDstFileName ) throw (CExceptionError)
{
    std::string sParam;
	sParam = sParam 
		+ "sSrcFileName[" + sSrcFileName
		+ "]sDstFileName[" + sDstFileName
		+ "]";

	CGDTrace trace( LOG_L6, "CFileTranClient:PullFile(%s)", sParam.c_str() );

	long nRet = 0;

	if ( ! m_bInitialized )
	{
		try
		{
			Init( m_sAddr.c_str(), m_sPort.c_str() );
		}
		catch ( CExceptionError & ex )
		{		
			ex.AddErrText( "CFileTranClient:Init(%s,%s)", m_sAddr.c_str(), m_sPort.c_str() );
			throw ex;
		}
	}

	FILE* fp = CGDFile::open( sDstFileName, "wb" );
	if ( fp )
	{
		unsigned long nOffset = 0;
		long nMaxBlogSize = CAppOption::Instance().GetOption("filetran.blocksize", 262144);
		
		if (nMaxBlogSize == 0)
			nMaxBlogSize = 262144;

		long nLen = nMaxBlogSize;
		
		do
		{
			// ״̬ṩһ
			if ( ! m_bInitialized )
			{
				try
				{
					Init( m_sAddr.c_str(), m_sPort.c_str() );
				}
				catch ( CExceptionError & ex )
				{		
					CGDFile::close( fp );
					ex.AddErrText( "CFileTranClient:Init(%s,%s)", m_sAddr.c_str(), m_sPort.c_str() );
					throw ex;
				}
			}

			OctetSeq_var varBuf;
			try
			{
				InforGuard::netCallData callData;
				CNetTran::InitNetCallData( callData );
				nRet = m_ref->GetFileBlock( callData, sSrcFileName, nOffset, varBuf.out(), nLen );
				CThreadData::SetThreadSerial( callData.nRecSerial );
			}
			catch( InforGuard::ExceptionError& ex )
			{
				CGDFile::close( fp );
				
				CExceptionError e;
				CNetTran::ConvertException(ex, e);
				e.AddErrText( "CFileTranClient:PullFile(%s).", sParam.c_str() );
				throw e;
			}
			catch( const CORBA::Exception& ex )
			{
				CGDFile::close( fp );
				// һCORBA쳣ֱΪ
				CNetTran::RemoveFileTranObject( m_sAddr.c_str(), m_sPort.c_str() );
				m_bInitialized = false;

				// ת쳣ʽ
				CExceptionError e;
				e.AddErrText( "%s", ex._to_string() );
				e.AddErrText( "CFileTranClient:PullFile(%s)", sParam.c_str() );
				throw e;
			}
			catch( ... )
			{
				CExceptionError e;
				e.AddErrText( "CFileTranClient:PullFile(%s)catch( ... )", sParam.c_str() );
				throw e;
			}
			
			nOffset += nLen;
			
			nLen = fwrite( varBuf.ptr()->get_buffer(), 1, nLen, fp );
			
		} while( nLen == nMaxBlogSize && nRet == 0 );

		CGDFile::close( fp );

		return trace.SetRetValue( nRet );
	}
	else
	{
		return trace.SetRetValue( 0x1000 + 0x109 );
	}

	return trace.SetRetValue( nRet );
}

long CFileTranClient::MakePath( const char* sFileName, bool bTmp ) throw (CExceptionError)
{
    std::string sParam;
	sParam = sParam
		+  "sFileName[" + sFileName
		+ "]bTmp[" + (bTmp?"1":"0")
		+ "]";

	CGDTrace trace( LOG_L6, "CFileTranClient:MakePath(%s)", sParam.c_str() );

	long nRet = 0;

	try
	{
		InforGuard::netCallData callData;
		CNetTran::InitNetCallData( callData );
		nRet = m_ref->MakeDirectory( callData, sFileName, bTmp );
		CThreadData::SetThreadSerial( callData.nRecSerial );
	}
	catch( InforGuard::ExceptionError& ex )
	{
		CExceptionError e;
		CNetTran::ConvertException(ex, e);
		e.AddErrText( "CFileTranClient:MakePath(%s)", sParam.c_str() );
		throw e;
	}
	catch( const CORBA::Exception& ex )
	{
		// һCORBA쳣ֱΪ
		CNetTran::RemoveFileTranObject( m_sAddr.c_str(), m_sPort.c_str() );
		m_bInitialized = false;
		// ת쳣ʽ
		CExceptionError e;
		e.AddErrText( "%s", ex._to_string() );
		e.AddErrText( "CFileTranClient:MakePath(%s)", sParam.c_str() );
		throw e;
	}
	catch( ... )
	{
		CExceptionError e;
		e.AddErrText( "CFileTranClient:MakePath(%s)catch( ... )", sParam.c_str() );
		throw e;
	}

	return trace.SetRetValue( nRet );
}


long CFileTranClient::MoveFileIn(const char* srcfile, const char* dstfile, const char* delfile, bool bTopPath) throw(CExceptionError)
{
    std::string sParam;
	sParam = sParam
		+ "srcfile[" + srcfile
		+ "]dstfile[" + dstfile
		+ "]delfile[" + delfile
		+ "]bTopPath[" + ( bTopPath?"1":"0" )
		+ "]";

	CGDTrace trace( LOG_L6, "CFileTranClient:MoveFileIn(%s)", sParam.c_str() );

	long nRet = 0;

	try
	{
		InforGuard::netCallData callData;
		CNetTran::InitNetCallData( callData );

		nRet = m_ref->MoveFileIn( callData, srcfile, dstfile, delfile, bTopPath );

		CThreadData::SetThreadSerial( callData.nRecSerial );
	}
	catch( InforGuard::ExceptionError& ex )
	{
		CExceptionError e;
		CNetTran::ConvertException(ex, e);
		e.AddErrText( "CFileTranClient:MoveFileIn(%s)", sParam.c_str() );
		throw e;
	}
	catch( const CORBA::Exception& ex )
	{
		// һCORBA쳣ֱΪ
		CNetTran::RemoveFileTranObject( m_sAddr.c_str(), m_sPort.c_str() );
		m_bInitialized = false;
		// ת쳣ʽ
		CExceptionError e;
		e.AddErrText( "%s", ex._to_string() );
		e.AddErrText( "CFileTranClient:MoveFileIn(%s)", sParam.c_str() );
		throw e;
	}
	catch( ... )
	{
		CExceptionError e;
		e.AddErrText( "CFileTranClient:MoveFileIn(%s)catch( ... )", sParam.c_str() );
		throw e;
	}

	return trace.SetRetValue( nRet );
}

long CFileTranClient::MoveFileOut(const char* srcfile, const char* dstfile) throw(CExceptionError)
{
    std::string sParam;
	sParam = sParam 
		+ "srcfile=[" + srcfile
		+ "]dstfile=[" + dstfile
		+ "]";

	CGDTrace trace( LOG_L6, "CFileTranClient:MoveFileOut(%s)", sParam.c_str() );

	long nRet = 0;

	try
	{
		InforGuard::netCallData callData;
		CNetTran::InitNetCallData( callData );
		nRet = m_ref->MoveFileOut( callData, srcfile, dstfile );
		CThreadData::SetThreadSerial( callData.nRecSerial );
	}
	catch( InforGuard::ExceptionError& ex )
	{
		CExceptionError e;
		CNetTran::ConvertException(ex, e);
		e.AddErrText( "CFileTranClient:MoveFileOut(%s)", sParam.c_str() );
		throw e;
	}
	catch( const CORBA::Exception& ex )
	{
		// һCORBA쳣ֱΪ
		CNetTran::RemoveFileTranObject( m_sAddr.c_str(), m_sPort.c_str() );
		m_bInitialized = false;
		// ת쳣ʽ
		CExceptionError e;
		e.AddErrText( "%s", ex._to_string() );
		e.AddErrText( "CFileTranClient:MoveFileOut(%s)", sParam.c_str() );
		throw e;
	}
	catch( ... )
	{
		CExceptionError e;
		e.AddErrText( "CFileTranClient:MoveFileOut(%s)catch( ... )", sParam.c_str() );
		throw e;
	}

	return trace.SetRetValue( nRet );
}

long CFileTranClient::RenameFile(const char* oldfile, const char* newfile, long attr) throw(CExceptionError)
{
    std::string sParam;
	sParam = sParam
		+ "oldfile[" + oldfile
		+ "]newfile[" + newfile
		+ "]attr[" + CGDString::ltoa( attr )
		+ "]";

	CGDTrace trace( LOG_L6, "CFileTranClient:RenameFile(%s)", sParam.c_str() );

	long nRet = 0;

	try
	{
		InforGuard::netCallData callData;
		CNetTran::InitNetCallData( callData );
		nRet = m_ref->RenameFile( callData, oldfile, newfile, attr );
		CThreadData::SetThreadSerial( callData.nRecSerial );
	}
	catch( InforGuard::ExceptionError& ex )
	{
		CExceptionError e;
		CNetTran::ConvertException(ex, e);
		e.AddErrText( "CFileTranClient:RenameFile(%s)", sParam.c_str() );
		throw e;
	}
	catch( const CORBA::Exception& ex )
	{
		// һCORBA쳣ֱΪ
		CNetTran::RemoveFileTranObject( m_sAddr.c_str(), m_sPort.c_str() );
		m_bInitialized = false;
		// ת쳣ʽ
		CExceptionError e;
		CNetTran::ConvertException(ex, e);
		e.AddErrText( "CFileTranClient:RenameFile(%s)", sParam.c_str() );
		throw e;
	}
	catch( ... )
	{
		CExceptionError e;
		e.AddErrText( "CFileTranClient:RenameFile(%s)catch( ... )", sParam.c_str() );
		throw e;
	}

	return trace.SetRetValue( nRet );
}

long CFileTranClient::QueryBasePath(char* basepath, int maxsize) throw(CExceptionError)
{
    CGDTrace trace( LOG_L6, "CFileTranClient:QueryBasePath(%s)", basepath );
    
    if( basepath )
    {
        memset(basepath, 0, maxsize);
    }

    long nRet = 0;
    
	// by wang_jin at 2009-9-20 for Ϣ
	string sKey = m_sAddr;
	sKey.append("-");
	sKey.append(m_sPort);
	CThreadLock lock(m_mutex);
	std::map<string, string>::iterator itor = m_basePathMap.find( sKey );
	if ( itor != m_basePathMap.end() )
	{
		const string & path = (*itor).second;
		strncpy( basepath, path.c_str(), maxsize );
		return nRet;
	}

    try
    {
        CORBA::String_var varBasePath;
        nRet = m_ref->QueryBasePath( varBasePath.out() );

        if( !nRet && basepath )
        {
            strncpy( basepath, varBasePath.in(), maxsize );
        }
    }
    catch( InforGuard::ExceptionError& ex )
    {
        CExceptionError e;
        CNetTran::ConvertException(ex, e);
        e.AddErrText( "CFileTranClient:QueryBasePath()" );
        throw e;
    }
    catch( const CORBA::Exception& ex )
    {
 		// һCORBA쳣ֱΪ
		CNetTran::RemoveFileTranObject( m_sAddr.c_str(), m_sPort.c_str() );
		m_bInitialized = false;
		// ת쳣ʽ
		CExceptionError e;
        CNetTran::ConvertException(ex, e);
        e.AddErrText( "CFileTranClient:QueryBasePath()" );
        throw e;
    }
	catch( ... )
	{
		CExceptionError e;
		e.AddErrText( "CFileTranClient:QueryBasePath catch( ... )" );
		throw e;
	}
    
	return trace.SetRetValue( nRet );
}

std::string CFileTranClient::GetPeerTempPath()
{
	CGDTrace trace( LOG_L6, "CFileTranClient:GetPeerTempPath()" );
    std::string tempPath;
    try
    {
        char szBasePath[GD_MAX_PATH] = {0};
        QueryBasePath(szBasePath, sizeof(szBasePath));
        tempPath = CToolbox::GetGDTempPath(szBasePath);
    }
    catch ( CExceptionError & ex )
    {
        CGDLog::WriteErrLog( "1074:GetPeerTempPath() fail\n%s", ex.GetErrText().c_str() );
    }
    catch( ... )
    {
        CGDLog::WriteErrLog( "1075:GetPeerTempPath() fail exception" );
    }

     trace.SetRetText( tempPath.c_str() );
	 return tempPath;
}

std::string CFileTranClient::GetPeerDustPath()
{
	CGDTrace trace( LOG_L6, "CFileTranClient:GetPeerDustPath()" );
    std::string dustPath;
    try
    {
        char szBasePath[GD_MAX_PATH] = {0};
        QueryBasePath(szBasePath, sizeof(szBasePath));
        dustPath = CToolbox::GetGDDustPath(szBasePath);
    }
    catch ( CExceptionError & ex )
    {
        CGDLog::WriteErrLog( "1076:GetPeerDustPath() fail\n%s", ex.GetErrText().c_str() );
    }
    catch( ... )
    {
        CGDLog::WriteErrLog( "1077:GetPeerDustPath() fail exception" );
    }
    trace.SetRetText( dustPath.c_str() );
    return dustPath;
}

long CFileTranClient::isFileExist( const char *filePath, long &fileSize ) throw(CExceptionError) 
{
	CGDTrace trace( LOG_L6, "CFileTranClient:isFileExist(%s)", filePath);
	long nRet = 9999;//ĬϷһֵ
	
	try
    {
        nRet = m_ref->isFileExist( filePath, fileSize );
	}
    catch( InforGuard::ExceptionError& ex )
    {
        CExceptionError e;
        CNetTran::ConvertException(ex, e);
        e.AddErrText( "CFileTranClient:isFileExist()" );
        throw e;
    }
    catch( const CORBA::Exception& ex )
    {
 		// һCORBA쳣ֱΪ
		CNetTran::RemoveFileTranObject( m_sAddr.c_str(), m_sPort.c_str() );
		m_bInitialized = false;
		// ת쳣ʽ
		CExceptionError e;
        CNetTran::ConvertException(ex, e);
        e.AddErrText( "CFileTranClient:isFileExist()" );
        throw e;
    }
	catch( ... )
	{
		CExceptionError e;
		e.AddErrText( "CFileTranClient:isFileExist()catch( ... )");
		throw e;
	}
    
	return trace.SetRetValue( nRet );
}
