/******************************************************************************
*
* ļ              LogFile.cpp
* ģĿģ      д־ļ
* ߼ڣ        liu_yiFeng 2008-2-22
* 汾                1.0
* ȨϢ            CVICSE Middleware
* ļ¼            
*
******************************************************************************/

#pragma warning( disable: 4251 4786 )

#include "Common/Common.h"
#include "Common/GDString.h"
#include "FileOpt/FileOptDef.h"
#include "GDLock/GDLock.h"
#include "GDLog/LogFile.h"
#include "GDLog/TmpLog.h"
#include "GDLog/ThreadData.h"

#ifdef WIN32
#	include <windows.h>
#	define TMPLOGPATH "/tmp"
#else
#include <sys/stat.h>
#include <sys/types.h>
#	define TMPLOGPATH "C:"
#endif

#include <stdarg.h>
#include <time.h>

list< CLogFile* >	CLogFile::m_traceFile;
CThreadMutex		CLogFile::m_listmutex;

CLogFile::CLogFile( const char* sFilePath, const char* sFileName, bool bScroll, int nSize, int nCount )
:m_bScroll(bScroll)
{
	// ߳

	// ļ
	m_fp = NULL;

	// ļ·
	if ( sFilePath && *sFilePath )
	{
		strcpy( m_sFilePath, sFilePath );
	}
	else
	{
		const char* p = GetInforGuardPath();
		if ( p && *p )
		{
			sprintf( m_sFilePath, "%s%clog", p, C_PATHLASH );
		}
		else
		{
			// Error
			CLogFile::WriteLog( "GetInforGuardPath() failed" );
		}
	}

	// ļ
	if ( sFileName && *sFileName )
	{
		strcpy( m_sFileName, sFileName );
	}
	else
	{
		const char* pAppName = GetAppName();
		if ( pAppName && *pAppName )
		{
			strcpy( m_sFileName, pAppName );
		}
		else
		{
			// Error
			CLogFile::WriteLog( "GetAppName() failed" );
		}
	}

	// ļС
	if (  nSize <= 0 )
	{
		m_nFileSize = 1;
	}
	else
	{
		m_nFileSize = nSize;
	}

	// ļ
	if (  nCount <= 0 )
	{
		m_nFileCount = 9;
	}
	else
	{
		m_nFileCount = nCount;
	}
}

CLogFile::~CLogFile()
{
	if ( m_fp )
	{
		fclose( m_fp );
		m_fp = NULL;
	}
}

// ʼ־ļ
long CLogFile::Initialize( const char* sFilePath, const char* sFileName, int nFileSize, int nFileCount )
{
	strcpy( m_sFilePath, sFilePath );
	strcpy( m_sFileName, sFileName );
	m_nFileSize = nFileSize;
	m_nFileCount = nFileCount;

	m_sFullName = GetLogFilePath( m_sFilePath, m_sFileName, 1 );

	return 0;
}

// ļƹ
// static
void CLogFile::ScrollLogFileName( const string& sLogPath, const string& sLogName, int nLogCount )
{
	// ļ
	char szDstFileName[ GD_MAX_PATH ] = {0};
	sprintf( szDstFileName, "%s%c%s-%s_%d.log", sLogPath.c_str(), C_PATHLASH, GetAppName(), sLogName.c_str(), nLogCount );

	remove( szDstFileName );

	char szSrcFileName[ GD_MAX_PATH ] = {0};
	for ( int i=nLogCount - 1; i>0; i-- )
	{
		sprintf( szSrcFileName, "%s%c%s-%s_%d.log", sLogPath.c_str(), C_PATHLASH, GetAppName(), sLogName.c_str(), i );

#ifdef WIN32
		BOOL bRet = MoveFileA( szSrcFileName,szDstFileName );
		if ( ! bRet )
#else
		int nRet = rename( szSrcFileName,szDstFileName );
		if( nRet )
#endif
		{
			if( errno != 2 )
			{
				GDETrace( "ScrollLogFileName(%s,%s) err=[%d]", szSrcFileName,szDstFileName, errno );
			}
		}

		strcpy( szDstFileName, szSrcFileName );
	}
}

void CLogFile::ScrollLogFileName()
{
	ScrollLogFileName( CLogConfig::GetDefaultLogFilePath(), m_sFileName, m_nFileCount );
}

string CLogFile::GetLogFilePath( const char* sLogPath, const char* sLogName, int nIndex )
{
	string sName;
	if ( sLogPath && *sLogPath )
	{
		sName = sLogPath;
	}
	else
	{
		sName = CLogConfig::GetDefaultLogFilePath();
	}

	sName += S_PATHLASH;
	sName += GetAppName();
	sName += "-";

	if( sLogName && *sLogName )
	{
		sName += sLogName;
	}
	else
	{
		sName += "NoName";
	}
	
	sName += "_";
	sName += CGDString::ltoa(nIndex);
	sName += ".log";
	return sName;
}

CLogFile* CLogFile::GetLogFile( const char* sLogName, bool bShareFile )
{
	CLogConfig* pLogConfig = CLogConfig::GetLogConfig( sLogName );
	if ( NULL == pLogConfig )
	{
		return NULL; // ûã־ΪǣΪû
	}

	string sFilePath = CLogConfig::GetDefaultLogFilePath();

	CLogFile* pTraceFile = NULL;

	if( bShareFile )
	{
		CThreadLock lock( m_listmutex );

		list< CLogFile* >::iterator iter;
		for ( iter = m_traceFile.begin(); iter != m_traceFile.end(); ++iter )
		{
			CLogFile* cur = (CLogFile*) (*iter);

			if ( cur && !strcmp(cur->m_sFileName, pLogConfig->m_sLogName.c_str()) )
			{
				pTraceFile = cur;
				break;
			}
		}

		if( !pTraceFile )
		{
			pTraceFile = new CLogFile( pLogConfig->m_sLogPath.c_str(), sLogName, pLogConfig->m_bScroll, pLogConfig->m_nLogSize, pLogConfig->m_nLogCount );
			m_traceFile.push_back(pTraceFile);
		}
	}
	else
	{
		pTraceFile = new CLogFile( pLogConfig->m_sLogPath.c_str(), sLogName, pLogConfig->m_bScroll, pLogConfig->m_nLogSize, pLogConfig->m_nLogCount );
	}

	return pTraceFile;
}

// ֱдˮ־ĺ
bool CLogFile::WriteLog( const char* fmt, ... )
{
	va_list args;
	va_start( args, fmt );
	bool bRet = WriteLog2( fmt, args );
	va_end( args );
	return bRet;
}

// ʧдʱ־ĺ
bool CLogFile::WriteLog2( const char* fmt, va_list args )
{
	// ֯
	time_t curtime = time(NULL);
	struct tm* p = localtime( &curtime );

	char strdate[20] = {0};
	strftime( strdate, sizeof(strdate), "%m-%d %H:%M:%S", p );

	char prefix[260] = {0};
	sprintf( prefix, "%s.%4d [%-5d] ", strdate, 
#ifdef WIN32
		GetTickCount() % 10000, GetCurrentThreadId() );
#else
		clock() * 10000 / CLOCKS_PER_SEC, pthread_self() );
#endif

	bool bRet = WriteFile( prefix, fmt, args );

	return bRet;
}

// д־ļĺ
bool CLogFile::WriteFile( const char* prefix, const char* fmt, va_list args ) // Ҫ
{
	try
	{
		CThreadLock lock( m_mutex );

		if ( NULL == m_fp )
		{
			m_fp = OpenLogFile();
		}
		
		if ( m_fp )
		{
			fprintf( m_fp, "%s ", prefix );
			vfprintf( m_fp, fmt, args );
			fprintf( m_fp, "\n" );
			fflush( m_fp );

			// ƴС
			int nSize = ftell(m_fp);

			if ( nSize > ( m_nFileSize * 1024 * 1024 ) )
			{
				if ( m_bScroll )
				{
					fclose( m_fp );
					m_fp = NULL;

					ScrollLogFileName();
				}
				else
				{
					fclose( m_fp );
					m_fp = NULL;
					m_fp = OpenLogFile( "w" ); // д󸲸
					fclose( m_fp );
					m_fp = NULL;
				}
			}

			return true;
		}
		else
		{
			return false;
		}
	}
	catch ( ... )
	{
		return false;
	}
}

FILE * CLogFile::OpenLogFile( const char* sMode )
{
	if ( !m_fp && *m_sFilePath && *m_sFileName )
	{
		// ļ
		string sFileName = GetLogFilePath( m_sFilePath, m_sFileName, 1 );
		m_fp = fopen( sFileName.c_str(), sMode );
#ifndef WIN32
		chmod( sFileName.c_str(), 0666 ); // by wang_jin at 2009-11-10 for zhongqi
#endif
	}
	return m_fp;
}

CLogFile * CLogFile::InitLogFile( const char* sLogName )
{
	CLogConfig * pLogConfig = CLogConfig::GetLogConfig( sLogName );

	if ( pLogConfig )
	{
		return new CLogFile( pLogConfig->m_sLogPath.c_str(), sLogName, pLogConfig->m_bScroll, pLogConfig->m_nLogSize, pLogConfig->m_nLogCount );
	}

	return NULL;
}
