/***********************************************************************
*
* ļ              ThreadData.cpp
* ģĿģ		࣬ʵ߳ά
* ߼ڣ        wang_jin 2007-12-24
* 汾                1.0
* ȨϢ            CVICSE Middleware
* ļ¼            
*
***********************************************************************/

#pragma warning( disable: 4251 4786 )

#include "FileOpt/FileOptDef.h"
#include "GDLock/GDLock.h"
#include "GDLog/ThreadData.h"
#include "GDTraceEx.h"

#ifdef WIN32
#	include <windows.h>
#endif
#include <map>


CThreadMutex CThreadData::m_mutex;
map < THREADID, CThreadData* > CThreadData::m_threadData;
bool CThreadData::m_bEnable = true;

CThreadData::CThreadData()
{
	m_nIndent = 0;						// Ű
	m_nRecSerial = 0;					// ־
	m_nSerial = 0;						// ҵˮ
	m_nSubSerial = 0;					// ҵˮ
	m_nThreadId = CThreadData::GetCurrentThreadId();	// ߳ID
	m_pTraceFile = NULL;				// ־ļ

	if( m_bEnable )
	{
		AddThreadData( this );
	}
}

CThreadData::~CThreadData()
{
	if( m_bEnable )
	{
		RemoveThreadData( this );			// ڴ˴
	}
}

// 漰ʲҪ
THREADID CThreadData::GetCurrentThreadId()
{
#ifdef WIN32
	return ::GetCurrentThreadId();
#else
	return pthread_self();
#endif
}

// ʹҪ
void CThreadData::AddThreadData( CThreadData* pThreadData )
{
	if( !m_bEnable )
	{
		return;
	}

	try
	{
		CThreadLock lock(m_mutex);
#ifdef NOT_USE_STD_LIB
		m_threadData.insert( map<THREADID, CThreadData*>::value_type( pThreadData->m_nThreadId, pThreadData ) );
#else
		m_threadData.insert( make_pair( pThreadData->m_nThreadId, pThreadData ) );
#endif
	}
	catch ( ... )
	{
	}
}

// ʹҪ
void CThreadData::RemoveThreadData( CThreadData* pThreadData )
{
	if( !m_bEnable )
	{
		return;
	}

	try
	{
		CThreadLock lock(m_mutex);

		map< THREADID, CThreadData* >::iterator iter = m_threadData.find( pThreadData->m_nThreadId );

		if ( iter != m_threadData.end() )
		{
			m_threadData.erase( iter );
		}
	}
	catch ( ... )
	{
		GDETrace("CThreadData:RemoveThreadData(...)");
	}
}

// ʹҪ
CThreadData* CThreadData::GetCurrentThreadData()
{
	if( !m_bEnable )
	{
		return NULL;
	}

	try
	{
		THREADID threadId = GetCurrentThreadId();
		
		CThreadLock lock(m_mutex);

		map< THREADID, CThreadData* >::iterator iter = m_threadData.find( threadId );
		
		if ( iter != m_threadData.end() )
		{
			return (CThreadData*)(iter->second);
		}
	}
	catch ( ... )
	{
		GDETrace("CThreadData:GetCurrentThreadData(...)");
	}
	
	return NULL;
}

// ԼҶҪ
void CThreadData::SetCurrentThreadLogFile( const char* sLogName, bool bShareFile )
{
	if( !m_bEnable )
	{
		return;
	}

	// ȡõǰ̶߳
	CThreadData* pCurrentThreadData = GetCurrentThreadData();
	if( !pCurrentThreadData )
	{
		pCurrentThreadData = new CThreadData();
	}

	// õǰ߳־
	pCurrentThreadData->m_sLogName = sLogName;
	pCurrentThreadData->m_pTraceFile = CLogFile::GetLogFile(sLogName, bShareFile);
}

CLogFile * CThreadData::GetTraceFile()
{
	if( !m_bEnable )
	{
		return NULL;
	}

	return m_pTraceFile;
}

// ԼҶҪ
long CThreadData::SetThreadSerial( unsigned long nSerial, unsigned long nSubSerial, unsigned long nRecSerial )
{
	if( !m_bEnable )
	{
		return 0;
	}

	long nRet = -1;

	CThreadData* pThreadData = GetCurrentThreadData();

	if ( pThreadData )
	{
		if ( nSerial == 0xcccccccc ) nSerial = 0;
		if ( nSubSerial == 0xcccccccc ) nSubSerial = 0;
		if ( nRecSerial == 0xcccccccc ) nRecSerial = 0;

		pThreadData->m_nSerial = nSerial;
		pThreadData->m_nSubSerial = nSubSerial;
		pThreadData->m_nRecSerial = nRecSerial;

		nRet = 0;
	}

	return nRet;
}

// ԼҶҪ
long CThreadData::SetThreadSerial( unsigned long nRecSerial )
{
	if( !m_bEnable )
	{
		return 0;
	}

	long nRet = -1;

	CThreadData* pThreadData = GetCurrentThreadData();

	if ( pThreadData )
	{
		pThreadData->m_nRecSerial = nRecSerial;
		nRet = 0;
	}

	return nRet;
}

// ԼҶҪ
long CThreadData::GetThreadSerial( long* nRecSerial )
{
	if( !m_bEnable )
	{
		return 0;
	}

	long nRet = -1;

	CThreadData* pThreadData = GetCurrentThreadData();

	if ( pThreadData )
	{
		*nRecSerial = pThreadData->m_nRecSerial;
		nRet = 0;
	}

	return nRet;
}


// ʹҪ
CThreadData* CThreadData::SetThreadData( CThreadData* pThreadData )
{
	if( !m_bEnable )
	{
		return pThreadData;
	}

	try
	{
		CThreadLock lock(m_mutex);
	#ifdef NOT_USE_STD_LIB
		pair< map< THREADID, CThreadData* >::iterator, bool > ret = 
			m_threadData.insert( map<THREADID,CThreadData*>::value_type( pThreadData->m_nThreadId, pThreadData ) );
	#else
		pair< map< THREADID, CThreadData* >::iterator, bool > ret = 
			m_threadData.insert( make_pair( pThreadData->m_nThreadId, pThreadData ) );
	#endif
		if ( ! ret.second )
		{
			// ʧ
			ErrCode errCode = GetLastError();
		}
	}
	catch ( ... )
	{
	}
	
	return pThreadData;
}

void CThreadData::SetEnable(bool enable)
{
	m_bEnable = enable;
}
