//======================================================================
// 
// Disk.c
//
// Copyright (C) CVIC Software Engineering CO.,LTD.
//
// ̵İ
// 
//======================================================================


//#include <ntifs.h>
//#include <ntdddisk.h>
#include "Disk.h"


// б
PDEVICE_OBJECT      gLDriveDevices[26];
// Ӱ
int                 gLDriveMap[26];
int                 gLDriveGroup = 0;


// ʼ
VOID InitDisk()
{
	ULONG i = 0;

	for(i = 0; i < 26; i++ )        gLDriveMap[i] = 0;
}

// 
VOID ClearDisk()
{
	ULONG drive;

	for ( drive = 0; drive < 26; drive++ )
	{
		UnhookDrive( (char)('A'+drive ) );
	}
}

// Hook/Unhook a set of drives specified by user. Return the set 
// that is currently hooked.
ULONG HookDriveSet( IN ULONG DriveSet, IN PDRIVER_OBJECT DriverObject )
{
	ULONG drive, i;
	ULONG bit;

	//
	// Scan the drive table, looking for hits on the DriveSet bitmask
	//
	for ( drive = 0; drive < 26; ++drive )
	{
		bit = 1 << drive;

		// Are we suppoed to hook this drive?
		if ( bit & DriveSet )
		{

			// Try to hook drive 
			if ( ! HookDrive( (char)('A'+drive), DriverObject ) )
			{
				// Remove from drive set if can't be hooked
				DriveSet &= ~bit;
			}
			else
			{
				// hook drives in same drive group      
				for( i = 0; i < 26; i++ )
				{
					if( gLDriveMap[i] == gLDriveMap[ drive ] && !gLDriveDevices[i] )
					{
						DriveSet |= ( 1<<i );
						gLDriveDevices[i] = gLDriveDevices[drive];
					}
				}
			}

		}
		else
		{
			// Try to unhook drive 
			if ( ! UnhookDrive( (char)('A'+drive) ) )
			{
				// Unhook failed, leave the drive marked as hooked
				DriveSet |= bit;
			}
			else
			{
				// Unhook worked. Mark all drives in same group as
				// unhooked
				for( i = 0; i< 26; i++ )
				{
					if( gLDriveMap[i] == gLDriveMap[ drive ] && gLDriveDevices[i] )
					{
						DriveSet &= ~(1 << i); 
						gLDriveDevices[i] = NULL;
					}
				}
			}
		}
	}

	// Return set of drives currently hooked
	return DriveSet;
}

// 󶨴
BOOLEAN HookDrive( IN char Drive, IN PDRIVER_OBJECT DriverObject )
{
	IO_STATUS_BLOCK     ioStatus;
	HANDLE              ntFileHandle;   
	OBJECT_ATTRIBUTES   objectAttributes;
	PDEVICE_OBJECT      fileSysDevice;
	PDEVICE_OBJECT      hookDevice;
	UNICODE_STRING      fileNameUnicodeString;
	WCHAR               filename[] = L"\\DosDevices\\A:\\";
	NTSTATUS            ntStatus;
	ULONG               i;
	PFILE_OBJECT        fileObject;
	PHOOK_EXTENSION     hookExtension;

	// ת0-25
	if ( Drive >= 'a' && Drive <= 'z' )
	{
		Drive -= 'a';
	}
	else
	{
		Drive -= 'A';
	}

	// 26ĸ
	if ( (unsigned char) Drive >= 26 )
	{
		return FALSE;
	}

	// жǷ
	if ( gLDriveDevices[Drive] == NULL )
	{
		// תַ
		filename[12] = 'A'+Drive;

		// ǰȴ
		RtlInitUnicodeString( &fileNameUnicodeString, filename );
		InitializeObjectAttributes( &objectAttributes, &fileNameUnicodeString, 
									OBJ_CASE_INSENSITIVE, NULL, NULL );

		ntStatus = ZwCreateFile( &ntFileHandle, SYNCHRONIZE|FILE_ANY_ACCESS, 
								 &objectAttributes, &ioStatus, NULL, 0, FILE_SHARE_READ|FILE_SHARE_WRITE, 
								 FILE_OPEN, 
								 FILE_SYNCHRONOUS_IO_NONALERT|FILE_DIRECTORY_FILE, 
								 NULL, 0 );
		
		if( !NT_SUCCESS( ntStatus ) )
		{
		    DbgPrint(("GuardFileDriver: HookDrive-Could not open drive %c: %x\n", 'A'+Drive, ntStatus ));
			return FALSE;
		}

		DbgPrint(("GuardFileDriver: HookDrive-opened the root directory!!! handle: %x\n", ntFileHandle));   

		// Got the file handle, so now look-up the file-object it refers to
		
		ntStatus = ObReferenceObjectByHandle( ntFileHandle, FILE_READ_DATA, 
											  NULL, KernelMode, &fileObject, NULL );
		if( !NT_SUCCESS( ntStatus ))
		{
			DbgPrint(("GuardFileDriver: HookDrive-Could not get fileobject from handle: %c\n", 'A'+Drive ));
			ZwClose( ntFileHandle );
			return FALSE;
		}

		// Next, find out what device is associated with the file object by getting its related
		// device object
		fileSysDevice = IoGetRelatedDeviceObject( fileObject );

		if ( ! fileSysDevice )
		{
			DbgPrint(("GuardFileDriver: HookDrive-Could not get related device object: %c\n", 'A'+Drive ));
			ObDereferenceObject( fileObject );
			ZwClose( ntFileHandle );

			return FALSE;
		}

		// Check the device list to see if we've already attached to this particular device. 
		// This can happen when more than one drive letter is being handled by the same network
		// redirecter
		for( i = 0; i < 26; i++ )
		{
			if( gLDriveDevices[i] == fileSysDevice )
			{
				// If we're already watching it, associate this drive letter
				// with the others that are handled by the same network driver. This
				// enables us to intelligently update the hooking menus when the user
				// specifies that one of the group should not be watched -we mark all
				// of the related drives as unwatched as well
				ObDereferenceObject( fileObject );

				ZwClose( ntFileHandle );

				gLDriveMap[ Drive ]     = gLDriveMap[i];
				gLDriveDevices[ Drive ] = fileSysDevice;

				return TRUE;
			}
		}

		// The file system's device hasn't been hooked already, so make a hooking device
		//  object that will be attached to it.
		ntStatus = IoCreateDevice( DriverObject,
								   sizeof(HOOK_EXTENSION),
								   NULL,
								   fileSysDevice->DeviceType,
								   0,
								   FALSE,
								   &hookDevice );
		if ( !NT_SUCCESS(ntStatus) )
		{
			DbgPrint(("GuardFileDriver: HookDrive-failed to create associated device: %c\n", 'A'+Drive ));   
			ObDereferenceObject( fileObject );
			ZwClose( ntFileHandle );

			return FALSE;
		}

		// Clear the device's init flag as per NT DDK KB article on creating device 
		// objects from a dispatch routine
		hookDevice->Flags &= ~DO_DEVICE_INITIALIZING;

		// Setup the device extensions. The drive letter and file system object are stored
		// in the extension.
		hookExtension = hookDevice->DeviceExtension;
		hookExtension->LogicalDrive = 'A'+Drive;
		hookExtension->FileSystem   = fileSysDevice;

		// Finally, attach to the device. The second we're successfully attached, we may 
		// start receiving IRPs targetted at the device we've hooked.
		ntStatus = IoAttachDeviceByPointer( hookDevice, fileSysDevice );
		if ( !NT_SUCCESS(ntStatus) )
		{
			// Couldn' attach for some reason
			DbgPrint(("GuardFileDriver: HookDrive-Connect with Filesystem failed: %c (%x) =>%x\n", 
					 'A'+Drive, fileSysDevice, ntStatus ));

			// Derefence the object and get out
			ObDereferenceObject( fileObject );
			ZwClose( ntFileHandle );

			return FALSE;

		}
		else
		{
			// Make a new drive group for the device,l if it does not have one 
			// already
			DbgPrint(("GuardFileDriver: HookDrive-Successfully connected to Filesystem device %c\n", 'A'+Drive ));
			if( !gLDriveMap[ Drive ] )
			{
				gLDriveMap[ Drive ] = ++gLDriveGroup;
			}
		}

		// Close the file and update the hooked drive list by entering a
		// pointer to the hook device object in it.
		ObDereferenceObject( fileObject );

		ZwClose( ntFileHandle );

		gLDriveDevices[Drive] = hookDevice;
	}

	return TRUE;
}

// ̰
BOOLEAN UnhookDrive( IN char Drive )
{
	PHOOK_EXTENSION hookExt;

	// ĸת
	if ( Drive >= 'a' && Drive <= 'z' )
	{
		Drive -= 'a';
	}
	else
	{
		Drive -= 'A';
	}

	// If the drive has been hooked, unhook it and delete the hook
	// device object
	if ( gLDriveDevices[Drive] )
	{
		hookExt = gLDriveDevices[Drive]->DeviceExtension;
		IoDetachDevice( hookExt->FileSystem );
		IoDeleteDevice( gLDriveDevices[Drive] );
		gLDriveDevices[Drive] = NULL;
	}

	return TRUE;
}
