#include <stdio.h>
#include <string.h>
#include "es_stru.h"
#include "tinyxml.h"
#include "z.h"

#define   HttpVersion       "1.1"
#define   HttpEncoding      "GBK"
#define   HttpStandalone    ""

// ʼӦṹ
void InitEs(ESINFO *pEs)
{
    memset(pEs, 0, sizeof(ESINFO));

    pEs->iRegion    = -1;
    pEs->iCrmState  = -1;
    pEs->iBossState = -1;
    pEs->iResState  = -1;
    pEs->iResult    = -1;
    pEs->iSyncFlag  = -1;
    pEs->iAck       = -1;
    pEs->iEslists   = -1;
    pEs->iChannel   = -1;
    pEs->iMsgBlock  = -1;
}

// ӦϢб
void AddEslist(char *pBuf, const ESINFO *pEs)
{
    char szEle[1024];

    strcpy(pBuf, "<eslist>");

    if( pEs->cSeqNumber[0] )
    {
        sprintf(szEle, "<sequence_number>%s</sequence_number>", pEs->cSeqNumber);
        strcat(pBuf, szEle);
    }

    if( pEs->iRegion != -1 )
    {
        sprintf(szEle, "<region>%d</region>", pEs->iRegion);
        strcat(pBuf, szEle);
    }

    if( pEs->iChannel != -1 )
    {
        sprintf(szEle, "<channel>%s</channel>", pEs->cChannel);
        strcat(pBuf, szEle);
    }

    if( pEs->iCrmState != -1 )
    {
        sprintf(szEle, "<crmstatus>%d</crmstatus>", pEs->iCrmState);
        strcat(pBuf, szEle);
    }

    if( pEs->iBossState != -1 )
    {
        sprintf(szEle, "<bossstatus>%d</bossstatus>", pEs->iBossState);
        strcat(pBuf, szEle);
    }

    if( pEs->iResState != -1 )
    {
        sprintf(szEle, "<resstatus>%d</resstatus>", pEs->iResState);
        strcat(pBuf, szEle);
    }

    if( pEs->cStateTime[0] )
    {
        sprintf(szEle, "<esdate>%s</esdate>", pEs->cStateTime);
        strcat(pBuf, szEle);
    }

    if( pEs->cDoneTime[0] )
    {
        sprintf(szEle, "<donetime>%s</donetime>", pEs->cDoneTime);
        strcat(pBuf, szEle);
    }

    if( pEs->iResult != -1 )
    {
        sprintf(szEle, "<result>%d</result>", pEs->iResult);
        strcat(pBuf, szEle);
    }

    if( pEs->cNote[0] )
    {
        sprintf(szEle, "<note>%s</note>", pEs->cNote);
        strcat(pBuf, szEle);
    }

    if( pEs->iMsgBlock != -1 )
    {
        sprintf(szEle, "<msgblock>%d</msgblock>", pEs->iMsgBlock);
        strcat(pBuf, szEle);
    }

    if( pEs->cRepMsg[0] )
    {
        sprintf(szEle, "<msgcontent>%s</msgcontent>", pEs->cRepMsg);
        strcat(pBuf, szEle);
    }

    strcat(pBuf, "</eslist>");
}

// XML Ӧ
int MakeEsXml(const ESINFO *pEs, int cnt, char *pMsg, char *pError)
{
    // ʽ
    char szMsgFmt[2048];

    snprintf(szMsgFmt, sizeof(szMsgFmt),
        "<operation>"
            "<operation_name>%s</operation_name>"
            "<operation_code>%s</operation_code>"
            "<msg_version>0100</msg_version>"
            "<cont_version>0100</cont_version>"
            "<content>",
        pEs->cOpCode, pEs->cOpCode);

    char szEle[256];

    // ӽڵʽ
    if( pEs->iSyncFlag != -1 )
    {
        sprintf(szEle, "<syncflag>%d</syncflag>", pEs->iSyncFlag);
        strcat(szMsgFmt, szEle);
    }

    if( pEs->iAck != -1 )
    {
        sprintf(szEle, "<result>%d</result>", pEs->iAck);
        strcat(szMsgFmt, szEle);
    }

    if( pEs->iEslists != -1 )
    {
        strcpy(szEle, "<eslists></eslists>");
        strcat(szMsgFmt, szEle);
    }

    strcat(szMsgFmt, "</content></operation>");

    // ʽ
    TiXmlDocument Doc;

    TiXmlDeclaration *pDeclaration =
        new TiXmlDeclaration(HttpVersion, HttpEncoding, HttpStandalone);
    if( NULL != pDeclaration )
    {
        Doc.LinkEndChild(pDeclaration);
    }

    Doc.Parse(szMsgFmt);

    if( Doc.Error() )
    {
        snprintf(pError, 1024, "Doc.Error in [%4d] [%s][%d, %d] %s",
            Doc.ErrorId(), Doc.ErrorDesc(), Doc.ErrorRow(), Doc.ErrorCol(), Doc.Value());
        return -1;
    }

    if( pEs->iEslists != -1 )
    {
        TiXmlElement *pRootDoc = Doc.RootElement();
        TiXmlElement *pEslists = pRootDoc->FirstChildElement("content")->FirstChildElement("eslists");

        // ӦϢб
        char szBuf[2048] = {0};

        for( int i = 0; i < cnt; i++ )
        {
            AddEslist(szBuf, pEs + i);

            TiXmlDocument eDoc;
            eDoc.Parse(szBuf);

            if( eDoc.Error() )
            {
                snprintf(pError, 1024, "Doc.Error in [%4d] [%s][%d, %d] %s",
                    eDoc.ErrorId(), eDoc.ErrorDesc(), eDoc.ErrorRow(), eDoc.ErrorCol(), eDoc.Value());
                return -1;
            }
            pEslists->InsertEndChild( *(eDoc.RootElement()) );
        }
    }

    // pRootDoc->Print(pMsg, 0);
    TiXmlPrinter printer;
    Doc.Accept( &printer );
    strcpy(pMsg, printer.CStr());

    return 0;
}

const char * GetText(const TiXmlElement *pE)
{
    const TiXmlNode * pChildNode = pE->FirstChild();
    if( pChildNode )
    {
        const TiXmlText * pChildText = pChildNode->ToText();
        if( pChildText )
        {
            return pChildText->Value();
        }
    }
    return "";
}

// XML 
int ParseEsXml(const char *pMsg, ESINFO *pEs, int nES, char *pError)
{
    int  iSyncFlag = -1;
    char szOpCode[65];

    TiXmlElement *pE = NULL;

    TiXmlDocument Doc;
    Doc.Parse(pMsg);

    if( Doc.Error() )
    {
        snprintf(pError, 1024, "Doc.Error in [%4d] [%s][%d, %d] %s",
            Doc.ErrorId(), Doc.ErrorDesc(), Doc.ErrorRow(), Doc.ErrorCol(), Doc.Value());
        return -1;
    }

    TiXmlElement *pRootDoc = Doc.RootElement();

    // operation_code
    TiXmlElement *pCode = pRootDoc->FirstChildElement("operation_code");
    if( NULL == pCode )
    {
         snprintf(pError, 1024, "<operation_code>ڵΪ");
         return -1;
    }
    strcpy(szOpCode, GetText(pCode));

    // content
    TiXmlElement *pContent = pRootDoc->FirstChildElement("content");
    if( NULL == pContent )
    {
         snprintf(pError, 1024, "<content>ڵΪ");
         return -1;
    }

    // syncflag
    pE = pContent->FirstChildElement("syncflag");
    if( pE )
    {
        iSyncFlag = atoi(GetText(pE));
    }

    // eslists
    TiXmlElement *pEslists = pContent->FirstChildElement("eslists");
    if( NULL == pEslists )
    {
         snprintf(pError, 1024, "<eslists>ڵΪ");
         return -1;
    }

    int i = 0;
    TiXmlElement *pEslist = pEslists->FirstChildElement("eslist");

    for( ; pEslist; pEslist = pEslist->NextSiblingElement("eslist"), i++ )
    {
        // ṹĿ޶
        if( nES && i >= nES ) break;

        InitEs(pEs + i);

        strcpy(pEs[i].cOpCode, szOpCode);
        pEs[i].iSyncFlag = iSyncFlag;

        // sequence_number
        pE = pEslist->FirstChildElement("sequence_number");
        if( NULL != pE )
        {
            strncpy(pEs[i].cSeqNumber, GetText(pE), sizeof(pEs[i].cSeqNumber) - 1);
            pEs[i].cSeqNumber[sizeof(pEs[i].cSeqNumber) - 1] = 0;
        }

        // region
        pE = pEslist->FirstChildElement("region");
        if( NULL != pE )
        {
            pEs[i].iRegion = atoi(GetText(pE));
        }

        // channel
        pE = pEslist->FirstChildElement("channel");
        if( NULL != pE )
        {
            strncpy(pEs[i].cChannel, GetText(pE), sizeof(pEs[i].cChannel) - 1);
            pEs[i].cChannel[sizeof(pEs[i].cChannel) - 1] = 0;
        }

        // crmstatus
        pE = pEslist->FirstChildElement("crmstatus");
        if( NULL != pE )
        {
            pEs[i].iCrmState = atoi(GetText(pE));
        }

        // bossstatus
        pE = pEslist->FirstChildElement("bossstatus");
        if( NULL != pE )
        {
            pEs[i].iBossState = atoi(GetText(pE));
        }

        // resstatus
        pE = pEslist->FirstChildElement("resstatus");
        if( NULL != pE )
        {
            pEs[i].iResState = atoi(GetText(pE));
        }

        // esdate
        pE = pEslist->FirstChildElement("esdate");
        if( NULL != pE )
        {
            strncpy(pEs[i].cStateTime, GetText(pE), sizeof(pEs[i].cStateTime) - 1);
            pEs[i].cStateTime[sizeof(pEs[i].cStateTime) - 1] = 0;
        }

        // donetime
        pE = pEslist->FirstChildElement("donetime");
        if( NULL != pE )
        {
            strncpy(pEs[i].cDoneTime, GetText(pE), sizeof(pEs[i].cDoneTime) - 1);
            pEs[i].cDoneTime[sizeof(pEs[i].cDoneTime) - 1] = 0;
        }

        // result
        pE = pEslist->FirstChildElement("result");
        if( NULL != pE )
        {
            pEs[i].iResult = atoi(GetText(pE));
        }

        // msgblock
        pE = pEslist->FirstChildElement("msgblock");
        if( NULL != pE )
        {
            pEs[i].iMsgBlock = atoi(GetText(pE));
        }

        // msgcontent
        pE = pEslist->FirstChildElement("msgcontent");
        if( NULL != pE )
        {
            strncpy(pEs[i].cRepMsg, GetText(pE), sizeof(pEs[i].cRepMsg) - 1);
            pEs[i].cRepMsg[sizeof(pEs[i].cRepMsg) - 1] = 0;
        }
    }

    return i;
}
