/***********************************************************************
*
* ļ  Watcher.cpp
* ģĿģڻȡļ¼
* ߼ڣ  wang_jin 2008-04-11
* 汾 1.0
* ȨϢCVICSE Middleware
* ļ¼
*
***********************************************************************/
#pragma warning(disable:4251)

#include "Watcher.h"

#include "Common/Common.h"
#include "Common/ExceptionError.h"
#include "Common/ParamList.h"
#include "Common/DateTime.h"
#include "Common/GDTimer.h"
#include "Common/ConvertString.h"
#include "FileOpt/FileOpt.h"
#include "GDLog/GDLog.h"
#include "GDLog/GDTrace.h"
#include "GDLog/RunLog.h"
#include "GDLog/LogFile.h"
#include "GDLog/ThreadData.h"

#include <assert.h>
#include <process.h> // for _beginthreadex
#include <windows.h>
#include <algorithm> // for std::sort
#include <sstream>

using namespace std;


UINT CWatcher::m_dwDefaultFlags = 0;
bool CWatcher::CDirInfo::ReadDirectoryChanges()
{
	// ִ첽I/O
	// UINT nBytes = 0;
	unsigned long nBytes = 0;
	BOOL bRet = FALSE;
	
	bRet = ReadDirectoryChangesW(
		m_hDir, 
		m_evtBuf.buffer + m_evtBuf.offset, 
		MAX_BUFFER - m_evtBuf.offset, 
		m_bSubDir, 
		m_dwFlags, 
		&nBytes, 
		&m_ol, 
		NULL );
	
	if ( bRet )
	{
		return true;
	}
	else
	{
		CGDLog::WriteErrLog( "CWatcher:CompletionRoutine() ReadDirectoryChangesW 1 error[%d]", GetLastError() );
		return false;
	}
}


CWatcher::CWatcher()
{
	m_dwDefaultFlags = 
		FILE_NOTIFY_CHANGE_FILE_NAME |	// ļƱ仯
		FILE_NOTIFY_CHANGE_DIR_NAME |	// Ŀ¼Ʊ仯
	//	FILE_NOTIFY_CHANGE_ATTRIBUTES |	// ļԱ仯ֻ||鵵
	//	FILE_NOTIFY_CHANGE_SIZE |		// ļС仯
		FILE_NOTIFY_CHANGE_LAST_WRITE |	// ޸ʱ
	//	FILE_NOTIFY_CHANGE_LAST_ACCESS| // ʱ
	//	FILE_NOTIFY_CHANGE_CREATION |	// ¼
	//	FILE_NOTIFY_CHANGE_SECURITY |	// ȫԱ仯
		0;

	m_nMilliSeconds = 500; // ¼󣬹500Ϊ¼
}

void CWatcher::SetEventFlag( UINT nFlags )
{
	CGDTrace trace( LOG_L6, "CWatcher::SetEventFlag(%d)", nFlags );
	
	m_dwDefaultFlags = nFlags;
}

void CWatcher::SetEventDelay( int ms )
{
	CGDTrace trace( LOG_L6, "CWatcher::SetEventFlag(%d)", ms );
	
	if ( ms <= 0 ) ms = 100;
	else if ( ms > 30000 ) ms = 30000;

	m_nMilliSeconds = ms;
}

CWatcher::~CWatcher()
{
	if ( m_nThreads > 0 )
	{
		Uninitialize();
	}

	// ɾ
	map < string, CEventInfo* >::iterator iter;
	for ( iter = m_evtmap.begin(); iter != m_evtmap.end(); )
	{
		delete (CEventInfo*) ( *iter ).second;
		m_evtmap.erase( iter++ );
	}
	m_filter.clear();
}

// ʼʧ쳣һ˳
bool CWatcher::Initialize( const char* sParams ) throw ( CExceptionError )
{
	CGDTrace trace( LOG_L6, "CWatcher:Initialize(%s)", sParams );
	
	// Բ
	CParamList pl( sParams, "|" );
	if ( sParams && *sParams )
	{		
		this->SetEventDelay( pl.GetNumValue( "delay" ) );
	}
	SetEventFlag( m_dwDefaultFlags );

	// ̸߳(˫)
	SYSTEM_INFO sysInfo = {0};
	GetSystemInfo( &sysInfo );
	m_nThreads = sysInfo.dwNumberOfProcessors * 2;
	
	// ɶ˿
	m_hIOCPRead = CreateIoCompletionPort( INVALID_HANDLE_VALUE, NULL, 0, 0 );
	if ( NULL == m_hIOCPRead )
	{
		DWORD dwErr = GetLastError();
		assert(0);
		GDPrint( "CreateIoCompletionPort1 Failed: [%d]\n", GetLastError() );
		return trace.SetRetResult( false );
	}
	
	// ɶ˿
	m_hIOCPHold = CreateIoCompletionPort( INVALID_HANDLE_VALUE, NULL, 0, 0 );
	if ( NULL == m_hIOCPHold )
	{
		DWORD dwErr = GetLastError();
		assert(0);
		GDPrint( "CreateIoCompletionPort2 Failed: [%d]\n", GetLastError() );
		return trace.SetRetResult( false );
	}
	
	// ʼٽ
	InitializeCriticalSection( &m_cs );

#ifndef NDEBUG // ʼԶ̹()
	if (!pl.GetStringValue( "viewerip" ).empty())
	{
		if ( m_queue.init( pl.GetStringValue( "viewerip" ).c_str(), pl.GetNumValue( "viewerport" ), 0 ) )
		{
			GDPrint( "ʼɹ\n" );
		}
	}		
#endif

	// ¼
	m_hEvent = CreateEvent( NULL, FALSE, FALSE, NULL ); // ǰFALSE:ԶλFALSE:ʼź
	if ( NULL == m_hEvent )
	{
		DWORD dwErr = GetLastError();
		assert(0);
		GDPrint( "CreateEvent Failed: [%d]\n", dwErr );
		CloseHandle( m_hIOCPRead );
		CloseHandle( m_hIOCPHold );
		return trace.SetRetResult( false );
	}

	// ߳(ֻһ߳)
	UINT dwThreadId = 0;
	HANDLE hThread = (HANDLE)_beginthreadex( NULL, 0, CWatcher::FilterRoutine, this, 0, &dwThreadId );
	if ( NULL == hThread )
	{
		DWORD dwErr = GetLastError();
		assert(0);
		GDPrint( "CreateThread Failed: [%d]\n", dwErr );
		CloseHandle( m_hIOCPRead );
		CloseHandle( m_hIOCPHold );
		return trace.SetRetResult( false );
	}

	WaitForSingleObject( m_hEvent, INFINITE );
	CloseHandle( hThread );
	
	// ߳(һ߳)
	for ( int i=0; i<m_nThreads; i++ )
	{
		hThread = (HANDLE)_beginthreadex( NULL, 0, CWatcher::DequeRoutine, this, 0, &dwThreadId );
		if ( NULL == hThread )
		{
			DWORD dwErr = GetLastError();
			assert(0);
			GDPrint( "CreateThread Failed: [%d]\n", dwErr );
			CloseHandle( m_hIOCPRead );
			return trace.SetRetResult( false );
		}
		// ߳ȼΪ
		SetThreadPriority( hThread, 
			//THREAD_PRIORITY_TIME_CRITICAL // Time-criticalʵʱ
			//THREAD_PRIORITY_HIGHEST		// Highestߣ
			THREAD_PRIORITY_ABOVE_NORMAL  // Above normalWindows 98֧֣
			//THREAD_PRIORITY_NORMAL // Normal
			//THREAD_PRIORITY_BELOW_NORMAL  // Below normalWindows 98֧֣
			//THREAD_PRIORITY_LOWEST // Lowestͣ
			//THREAD_PRIORITY_IDLE // IdleУ
			);

		CloseHandle( hThread );
	}

	// ߳(߳)
	for ( int ii=0; ii<m_nThreads; ii++ )
	{
		hThread = (HANDLE)_beginthreadex( NULL, 0, CWatcher::CompletionRoutine, this, 0, NULL );
		if ( NULL == hThread )
		{
			DWORD dwErr = GetLastError();
			assert(0);
			GDPrint( "CreateThread Failed: [%d]\n", GetLastError() );
			CloseHandle( m_hIOCPRead );
			return trace.SetRetResult( false );
		}
		// ߳ȼΪ
		SetThreadPriority( hThread, 
			THREAD_PRIORITY_TIME_CRITICAL // Time-criticalʵʱ
			//THREAD_PRIORITY_HIGHEST		// Highestߣ
			//THREAD_PRIORITY_ABOVE_NORMAL  // Above normalWindows 98֧֣
			//THREAD_PRIORITY_NORMAL // Normal
			//THREAD_PRIORITY_BELOW_NORMAL  // Below normalWindows 98֧֣
			//THREAD_PRIORITY_LOWEST // Lowestͣ
			//THREAD_PRIORITY_IDLE // IdleУ
			);

		CloseHandle( hThread );
	}

	return trace.SetRetResult( true );
}

bool CWatcher::WatchDirectory( CMonitorInformation& moniInfo )
{
	CGDTrace trace( LOG_L6, "CWatcher:WatchDirectory(%s)", moniInfo.m_monitorPath.c_str() );
	
	// жǷѾ
	map< string, CDirInfo* >::iterator it = m_dirmap.find( moniInfo.m_sDirId );
	if ( it == m_dirmap.end() )
	{
		CDirInfo* pDirInfo = new CDirInfo( moniInfo, m_dwDefaultFlags );
		strcpy( pDirInfo->m_sDirId, moniInfo.m_sDirId.c_str() );			//ӼĿ¼ID
		if ( pDirInfo )
		{
			if ( pDirInfo->StartMonitor( m_hIOCPRead ) )
			{
				m_dirmap.insert( make_pair( moniInfo.m_sDirId, pDirInfo ) );
				return trace.SetRetResult( true );
			}

			delete pDirInfo;
			pDirInfo = NULL;
		}
	}
	else
	{
		// Ŀ¼
		CDirInfo* pDirInfo = it->second;
		pDirInfo->SetPolicy( moniInfo );
	}

	return trace.SetRetResult( false );
}

bool CWatcher::UnwatchDirectory( const char* sDirId )
{
	CGDTrace trace( LOG_L6, "CWatcher:UnwatchDirectory(%s)", sDirId );
	
	CDirInfo* pDirInfo = NULL;
	map < string, CDirInfo* >::iterator iter = m_dirmap.find( string(sDirId) );
	
	if ( iter != m_dirmap.end() )
	{
		pDirInfo = (CDirInfo*) iter->second;
		if ( pDirInfo->StopMonitor( m_hIOCPRead ) )
		{
			m_dirmap.erase( iter );
			delete pDirInfo;
			pDirInfo = NULL;
			return trace.SetRetResult( true );
		}
	}

	return trace.SetRetResult( false );
}

bool CWatcher::Uninitialize()
{
	// ɺtrace
	CGDTrace trace( LOG_L6, "CWatcher:Uninitialize()" );
	
	// ֪ͨ¼ȡ߳˳
	for ( int i=0; i<m_nThreads; i++ )
	{
		PostQueuedCompletionStatus( m_hIOCPRead, 0, 0, NULL );
	}

	// ֪ͨ¼߳˳
	for ( i=0; i<m_nThreads; i++ )
	{
		PostQueuedCompletionStatus( m_hIOCPHold, 0, 0, NULL );
	}

	// ֪ͨ¼߳˳
	m_bWorking = FALSE;
	::WaitForSingleObject( m_hEvent, INFINITE );
	::CloseHandle( m_hEvent );

	CloseHandle( m_hIOCPRead );
	m_hIOCPRead = NULL;

	CloseHandle( m_hIOCPHold );
	m_hIOCPHold = NULL;

	CDirInfo* pDirInfo = NULL;
	map < string, CDirInfo* >::iterator iter;
	for ( iter = m_dirmap.begin(); iter != m_dirmap.end(); ++iter )  
	{
		pDirInfo = (CDirInfo*) ( (*iter).second );
		if ( pDirInfo )
		{
			delete pDirInfo;
			pDirInfo = NULL;
		}
	}
	m_dirmap.clear();

	return trace.SetRetResult( true );
}

bool CWatcher::IsWatchingDirectory( const char* szDir )
{
	CGDTrace trace( LOG_L6, "CWatcher:IsWatchingDirectory(%s)", szDir );
	
	bool bRet = false;
	
	CDirInfo* pDirInfo = NULL;
	map < string, CDirInfo* >::iterator iter = m_dirmap.begin();
	while( iter != m_dirmap.end() )
	{
		pDirInfo = (CDirInfo*)iter->second;
		if ( ::_strnicmp(szDir, pDirInfo->m_szDir, strlen(pDirInfo->m_szDir)) == 0 )
		{
			bRet = true;
			break;
		}
		iter++;
	}
	
	return trace.SetRetResult(bRet);
}

bool CWatcher::IsWatched( const char* sDirId )
{
	CGDTrace trace( LOG_L6, "CWatcher:IsWatchingDirectory(%s)", sDirId );
	
	// жǷѾ
	if ( sDirId[1] == ':' )
	{
		return IsWatchingDirectory( sDirId ); // ͨĿ¼жǷ(MA˲֧ظ)
	}
	std::map<std::string, CDirInfo*>::iterator iter = m_dirmap.find( sDirId );
	if ( iter == m_dirmap.end() ) // ûнĿ¼ض
	{
		return trace.SetRetResult( false );
	}
	else
	{
		return trace.SetRetResult( true );
	}
}

bool CWatcher::IsWatchedByWebsiteId( const char * sWebsiteId )
{
	CGDTrace trace( LOG_L6, "CFileDiffer:IsWatchedByWebsiteId(%s)", sWebsiteId );

	// жǷѾ
	bool bRet = false;

	CDirInfo* pDirInfo = NULL;
	map < string, CDirInfo* >::iterator iter = m_dirmap.begin();
	while( iter != m_dirmap.end() )
	{
		pDirInfo = (CDirInfo*)iter->second;
		if( strcmp( sWebsiteId, pDirInfo->m_sDirId ) == 0 )
		{
			bRet = true;
			break;
		}
		iter++;
	}
	if ( !bRet ) // ûнĿ¼ض
	{
		return trace.SetRetResult( false );
	}
	else
	{
		return trace.SetRetResult( bRet );
	}
}

bool CWatcher::GetMonitorEvent( EVENT_S& evt )
{
	// ȡ¼
	CEventInfo ei;
	if ( ! m_queue.pop_front( ei ) )
	{
		return false;
	}
	// 
	evt.event_type = ei.m_nAction;
	evt.attr_flag = ei.m_nAttribute;
	memcpy( &evt.evt_time, &ei.m_time, sizeof(GDDateTime) );
	strcpy( evt.o_filepath, ei.m_szFileName );
	strcpy( evt.n_filepath, ei.m_newFileName );
	//Ŀ¼ID by wu_bin
	strcpy( evt.sDirId, ei.m_sDirId );
	
	//ɺtrace
	string sMoniInfo = evt.toString();
	
	CGDTrace trace( LOG_L6, "CWatcher:GetMonitorEvent(%s)", sMoniInfo.c_str() );
#ifndef NDEBUG
	if ( evt.event_type == MONITOR_EVENT_WATCHER_CREATE )
	{
		::InterlockedIncrement( &GnAddCount[5] );
	}
	else if ( evt.event_type == MONITOR_EVENT_WATCHER_DELETE )
	{
		::InterlockedIncrement( &GnRemoveCount[5] );
	}
	else if ( evt.event_type == MONITOR_EVENT_WATCHER_CHANGE )
	{
		::InterlockedIncrement( &GnChangeCount[5] );
	}
	else if ( evt.event_type == MONITOR_EVENT_WATCHER_RENAME )
	{
		::InterlockedIncrement( &GnRenameCount[5] );
	}
#endif
	return trace.SetRetResult( true );
}

void GetAnotherKey( string& sKey, const char* sOldFileName, const char* sNewFileName )
{
	int nFirstPos = sKey.find_last_of("\\");
	int nNameLen = sKey.length() - nFirstPos;
	int nNamePos = strlen(sOldFileName) - nNameLen;
	sKey.replace( nFirstPos, sKey.length() - 1, sNewFileName + nNamePos );
}

// Ψһ߳ʵ¼ϲעⱣmaplistԪһԣ
// ¼˵
// ! ʾϵͳϱ¼
// $ ʾϱ¼
// + ʾϲ¼
// - ʾҺԵ¼
// . ʾֱӺԵ¼(Ŀ¼޸)
UINT __stdcall CWatcher::FilterRoutine( LPVOID lpParam )
{
	// watch־
	CThreadData::SetCurrentThreadLogFile( "watch", true ); // ߳()־

	// trace־
	CGDTrace trace( LOG_L6, "CWatcher:FilterRoutine()" );
	
	CWatcher* pWatcher = (CWatcher*)lpParam;
	string sNewKey;
	std::pair< std::map< string, CEventInfo* >::iterator, bool > ret;

	if ( ! SetEvent( pWatcher->m_hEvent ) ) // set thread start event :ʱm_hEventΪ0
	{
		DWORD dwErr = GetLastError();
		assert( 0 ); // ͨ
		return -1; // 
	}

	// ˣǴӶж¼ɢбȴ¼ϲ˵ͬʱҪҪȴϲ¼¼

	map < string, CEventInfo* >::iterator mapiter;
	pWatcher->m_bWorking = TRUE;
	while ( pWatcher->m_bWorking )
	{
		DWORD dwNow = GetTickCount(); // ȡʱ䣬ںϲ¼ĩβ¼

		if ( ! pWatcher->m_deque.empty() )
		{
			// 
			EnterCriticalSection( &pWatcher->m_cs );
			// ȡ
			CEventInfo* pei = (CEventInfo*)pWatcher->m_deque.front();
			pWatcher->m_deque.pop_front();
			// 
			LeaveCriticalSection( &pWatcher->m_cs );
			assert( pei->m_pFileExclude );
			dwNow = pei->m_nTickCount; // ȡʱ䣬ںϲ¼ʱԱ⴦ӳٲƫ
#ifndef NDEBUG
			if ( pei->m_nAction == FILE_ACTION_ADDED )
			{
				::InterlockedIncrement( &GnAddCount[3] );
			}
			else if ( pei->m_nAction == FILE_ACTION_REMOVED )
			{
				::InterlockedIncrement( &GnRemoveCount[3] );
			}
			else if ( pei->m_nAction == FILE_ACTION_MODIFIED )
			{
				//::InterlockedIncrement( &GnChangeCount[3] );
			}
			else if ( pei->m_nAction == FILE_ACTION_RENAMED_OLD_NAME )
			{
				::InterlockedIncrement( &GnRenameCount[3] );
			}
#endif
			
			// ԭʼ¼
			CGDLog::WriteWatcherEvent( "![%x][%s][%s]", pei->m_nAction, pei->m_szFileName, pei->m_newFileName );

			// ж¼ͣԴ
			do
			{
				CEventInfo* prv = NULL;
				CEventInfo* prv2 = NULL;
				// ǰ¼
				mapiter = pWatcher->m_evtmap.find( pei->m_szKey );
				if ( pWatcher->m_evtmap.end() == mapiter )
				{
					if ( FILE_ACTION_RENAMED_OLD_NAME == pei->m_nAction )
					{
						sNewKey = pei->m_szKey;
						GetAnotherKey( sNewKey, pei->m_szFileName, pei->m_newFileName );
						mapiter = pWatcher->m_evtmap.find( sNewKey );
						if ( pWatcher->m_evtmap.end() != mapiter )
						{
							// ǰ¼
							prv2 = (CEventInfo*)(mapiter->second);
							if ( !prv2 )
							{
								CGDLog::WriteWatcherEvent("ERROR: NULL Pointer!!!");
								exit(-1);
							}
						}
						else
						{
							CEventInfo * nevt = new CEventInfo;
							nevt->m_nAction = FILE_ACTION_RENAMED_NEW_NAME;
							nevt->m_nAttribute = pei->m_nAttribute;
							strcpy( nevt->m_sDirId, pei->m_sDirId );
							nevt->m_nTickCount = pei->m_nTickCount;
							nevt->m_pFileExclude = pei->m_pFileExclude;
							strcpy( nevt->m_szFileName, pei->m_szFileName );
							strcpy( nevt->m_newFileName, pei->m_newFileName );					
							strcpy( nevt->m_szKey, sNewKey.c_str() );
							// ˶Уȴܵĺϲ
							ret = pWatcher->m_evtmap.insert( make_pair( string(nevt->m_szKey), nevt ) );
							if( ret.second )
							{
								pWatcher->m_filter.push_back( nevt ); // ļ
							}
							else
							{
								CGDLog::WriteErrLog( "2102:ϲ¼дʧ!" );
								assert(0);
							}
						}
					}
				}
				else
				{
					// ǰ¼
					prv = (CEventInfo*)(mapiter->second);
					if ( !prv )
					{
						CGDLog::WriteWatcherEvent("ERROR: NULL Pointer!!!");
						exit(-1);
					}
				}
			
				// жļĿ¼
				try
				{
					if ( FILE_ACTION_RENAMED_OLD_NAME == pei->m_nAction )
					{
						if ( CFileOpt::IsDirectory( pei->m_newFileName ) )
						{
							pei->m_nAttribute = FILE_ATTRIBUTE_DIRECTORY;
						}
						else
						{
							pei->m_nAttribute = FILE_ATTRIBUTE_ARCHIVE;
						}
					}
					else if ( FILE_ACTION_REMOVED != pei->m_nAction )
					{
						try
						{
							if ( CFileOpt::IsDirectory( pei->m_szFileName ) )
							{
								pei->m_nAttribute = FILE_ATTRIBUTE_DIRECTORY;
							}
							else
							{
								pei->m_nAttribute = FILE_ATTRIBUTE_ARCHIVE;
							}
						}
						catch ( CExceptionError & ex )
						{
							// ͨǴĿ¼޸¼ʱĿ¼ѱɾ¼
							break;
						}
						catch ( ... )
						{
							CGDLog::WriteErrLog( "¼жʧ(...)" );		
						}
					}

					if ( FILE_ATTRIBUTE_DIRECTORY == pei->m_nAttribute )
					{					
						if( FILE_ACTION_MODIFIED == pei->m_nAction )
						{
							CGDLog::WriteWatcherEvent( ".[%x][%s][%s]", pei->m_nAction, pei->m_szFileName, pei->m_newFileName );
							delete pei;
							pei = NULL;
							break;
						}
					}
				}
				catch ( CExceptionError & ex )
				{
					ex.AddErrText("CWatcher:FilterRoutine()");
					CGDLog::WriteErrLog( "¼жʧ\n%s", ex.GetErrText().c_str() );				
				}
				catch ( ... )
				{
					CGDLog::WriteErrLog( "¼жʧ(...)" );		
				}
					
				if ( NULL == prv && NULL == prv2 )
				{
					// ¼-˶Уȴܵĺϲ
					ret = pWatcher->m_evtmap.insert( make_pair( string( pei->m_szKey ), pei ) );
					if( ret.second )
					{
						pWatcher->m_filter.push_back( pei ); // ¼
					}
					else
					{
						CGDLog::WriteErrLog( "2202:ϲ¼дʧ!" );
						mapiter = pWatcher->m_evtmap.find( pei->m_szKey );
						if ( pWatcher->m_evtmap.end() != mapiter )
						{
							assert(0);
						}
					}
					break;
				}				
				switch ( pei->m_nAction )
				{
				case FILE_ACTION_ADDED:
					switch ( prv->m_nAction )
					{
					case FILE_ACTION_ADDED:
						// ½+½=߼ңǰ¼ǰ¼ͷŵ¼
						CGDLog::WriteWatcherEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL;
						break;
					case FILE_ACTION_REMOVED:
						// ɾ+½=ϲΪ޸ģǰ¼ͷŵǰ¼
						CGDLog::WriteWatcherEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_MODIFIED, pei->m_szFileName );
						prv->m_nAction = FILE_ACTION_MODIFIED;
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL;
						break;
					case FILE_ACTION_MODIFIED:
						// ޸+½=߼ңǰ¼ǰ¼ͷŵ¼
						CGDLog::WriteWatcherEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
						prv->m_nAction = pei->m_nAction;
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL;
						break;
					case FILE_ACTION_RENAMED_OLD_NAME:	//test1
						// old->new+½old=Ϊ½new+޸oldǰ¼õǰ¼ѭ
						CGDLog::WriteWatcherEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s]+[%x][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_MODIFIED, pei->m_szFileName, FILE_ACTION_ADDED, prv->m_newFileName );						
						prv->m_nAction = FILE_ACTION_MODIFIED;
						{
							sNewKey = pei->m_szKey;
							GetAnotherKey( sNewKey, pei->m_szFileName, prv->m_newFileName );
							strcpy( pei->m_szKey, sNewKey.c_str() );
							strcpy( pei->m_szFileName, prv->m_newFileName );
						}
						prv->m_newFileName[0] = '\0';
						pei->m_nAction = FILE_ACTION_ADDED;
						continue;
					case FILE_ACTION_RENAMED_NEW_NAME:
						// old->new+½new=߼ңǰ¼ͷǰ¼ѭ
						CGDLog::WriteWatcherEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
						pWatcher->m_evtmap.erase( prv->m_szKey );
						pWatcher->m_filter.remove( prv );
						delete prv;
						prv = NULL;
						continue;
					default: // ǰ¼ʲôǣͷǰ¼ѭ
						assert(0);
						pWatcher->m_evtmap.erase( prv->m_szKey );
						pWatcher->m_filter.remove( prv );
						delete prv;
						prv = NULL;								
						continue;
					}				
					break;
				case FILE_ACTION_REMOVED:
					switch ( prv->m_nAction )
					{
					case FILE_ACTION_ADDED:
						// ½+ɾ=ͷǰ¼ͷŵǰ¼
						CGDLog::WriteWatcherEvent( "+[%x][%s][%s]+[%x][%s][%s]=0", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName  );
						pWatcher->m_evtmap.erase( prv->m_szKey );
						pWatcher->m_filter.remove( prv );
						delete prv;
						prv = NULL;
						delete pei;
						pei = NULL;
						break;
					case FILE_ACTION_REMOVED:
						// ɾ+ɾ=߼ңǰ¼ǰ¼ͷŵǰ¼
						CGDLog::WriteWatcherEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL;
						break;
					case FILE_ACTION_MODIFIED:
						// ޸+ɾ=ɾǰ¼ǰ¼ͷŵǰ¼
						CGDLog::WriteWatcherEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
						prv->m_nAction = pei->m_nAction;
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL;
						break ;
					case FILE_ACTION_RENAMED_OLD_NAME://test2
						// old->new+ɾold(ϲ)=߼ңǰ¼ͷǰ¼
						CGDLog::WriteWatcherEvent( "[%x][%s][%s]+[%x][%s][%s]=[%x][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_REMOVED, pei->m_szFileName );
						pWatcher->m_evtmap.erase( prv->m_szKey );
						pWatcher->m_filter.remove( prv );
						delete prv;
						prv = NULL;
						break;
					case FILE_ACTION_RENAMED_NEW_NAME://test3
						// old->new+ɾnew=ϲΪɾoldǰ¼ͷŵǰ¼
						CGDLog::WriteWatcherEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_REMOVED, prv->m_szFileName );
						pWatcher->m_evtmap.erase( prv->m_szKey );
						pWatcher->m_filter.remove( prv );
						sNewKey = pei->m_szKey;
						GetAnotherKey( sNewKey,pei->m_szFileName, prv->m_szFileName );
						strcpy( pei->m_szKey, sNewKey.c_str() );
						strcpy( pei->m_szFileName, prv->m_szFileName );
						delete prv;
						prv = NULL;
						continue;
					default: // ǰ¼ʲôǣͷǰ¼ѭ
						assert(0);
						pWatcher->m_evtmap.erase( prv->m_szKey );
						pWatcher->m_filter.remove( prv );
						delete prv;
						prv = NULL;								
						continue;
					}
					break;
				case FILE_ACTION_MODIFIED:
					switch ( prv->m_nAction )
					{
					case FILE_ACTION_ADDED:
						// ½+޸=½ǰ¼ͷŵǰ¼
						CGDLog::WriteWatcherEvent( "+[%x][%s][%s]+[%x][%s][%s]=[1]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName  );
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL;
						break;
					case FILE_ACTION_REMOVED:
						// ɾ+޸=߼ңǰ¼ǰ¼ͷŵǰ¼
						CGDLog::WriteWatcherEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_szKey );
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL;
						break;
					case FILE_ACTION_MODIFIED:
						// ޸+޸=޸ģԵǰ¼ǰ¼ͷŵǰ¼
						CGDLog::WriteWatcherEvent( "+[%x][%s][%s]+[%x][%s][%s]=[3]", prv->m_nAction, prv->m_szFileName, prv->m_szKey, pei->m_nAction, pei->m_szFileName, pei->m_newFileName  );
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL;
						break;
					case FILE_ACTION_RENAMED_OLD_NAME:
						// old->new+޸old=߼ңǰ¼ǰ¼ͷŵǰ¼
						CGDLog::WriteWatcherEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
						prv->m_nTickCount = pei->m_nTickCount;
						delete pei;
						pei = NULL;
						break;
					case FILE_ACTION_RENAMED_NEW_NAME://test4
						// old->new+޸new=Ϊɾold+½newǰ¼õǰ¼
						CGDLog::WriteWatcherEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s]+[%x][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_REMOVED, prv->m_szFileName, FILE_ACTION_ADDED, pei->m_szFileName );
						//add by dai_hyan
						prv->m_nAction = FILE_ACTION_ADDED;
						{
							sNewKey = pei->m_szKey;
							GetAnotherKey( sNewKey, pei->m_szFileName, prv->m_szFileName );
							strcpy( pei->m_szKey, sNewKey.c_str() );
						}
						pei->m_nAction = FILE_ACTION_REMOVED;
						strcpy(pei->m_szFileName, prv->m_szFileName);
						strcpy(prv->m_szFileName, prv->m_newFileName);
						prv->m_newFileName[0] = '\0';
						continue;
					default: // ǰ¼ʲôǣͷǰ¼ѭ
						assert(0);
						pWatcher->m_evtmap.erase( prv->m_szKey );
						pWatcher->m_filter.remove( prv );
						delete prv;
						prv = NULL;								
						continue;
					}
					break;	
				case FILE_ACTION_RENAMED_OLD_NAME:
					if( prv )//ֻоļǰ¼
					{
						switch( prv->m_nAction )
						{
						case FILE_ACTION_ADDED://ǰ½old test5
							// ½old+old->new=ϲΪ½newͷǰ¼õǰ¼ѭɾnewϲ
							CGDLog::WriteWatcherEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_ADDED, pei->m_newFileName );
							pWatcher->m_evtmap.erase( prv->m_szKey );
							pWatcher->m_filter.remove( prv );
							delete prv;
							prv = NULL;
							pei->m_nAction = FILE_ACTION_ADDED;
							sNewKey = pei->m_szKey;
							GetAnotherKey( sNewKey, pei->m_szFileName, pei->m_newFileName );
							strcpy( pei->m_szKey, sNewKey.c_str() );
							strcpy( pei->m_szFileName, pei->m_newFileName );
							pei->m_newFileName[0] = '\0';							
							continue;
						case FILE_ACTION_REMOVED://ǰɾold
							// ɾold+old->new=߼ңǰ¼ǰ¼ͷŵǰ¼
							CGDLog::WriteWatcherEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
							prv->m_nTickCount = pei->m_nTickCount;
							delete pei;
							pei = NULL;
							break;
						case FILE_ACTION_MODIFIED://ǰ޸old test6
							// ޸old+old->new=ɾold+½newǰ¼õǰ¼ѭɾnewϲ
							CGDLog::WriteWatcherEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s]+[%x][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_REMOVED, pei->m_szFileName, FILE_ACTION_ADDED, pei->m_newFileName );
							prv->m_nAction = FILE_ACTION_REMOVED;
							prv->m_nTickCount = pei->m_nTickCount;
							pei->m_nAction = FILE_ACTION_ADDED;
							//add by dai_hyan 2009-8-24
							sNewKey = pei->m_szKey;
							GetAnotherKey( sNewKey, pei->m_szFileName, pei->m_newFileName );
							strcpy( pei->m_szKey, sNewKey.c_str() );
							strcpy( pei->m_szFileName, pei->m_newFileName );
							pei->m_newFileName[0] = '\0';				
							continue;
						case FILE_ACTION_RENAMED_OLD_NAME://ǰļ
							// old->new1+old->new2=߼ңǰ¼ͷǰ¼ѭ
							CGDLog::WriteWatcherEvent( "-[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName );
							pWatcher->m_evtmap.erase( prv->m_szKey );
							pWatcher->m_filter.remove( prv );
							delete prv;
							prv = NULL;
							continue;
						case FILE_ACTION_RENAMED_NEW_NAME://ǰļ
							// old->new1+new1->new=ϲΪold->newͷǰ¼õǰ¼ѭ
							CGDLog::WriteWatcherEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s][%s]", prv->m_nAction, prv->m_szFileName, prv->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_RENAMED_OLD_NAME, prv->m_szFileName, pei->m_newFileName );
							strcpy(pei->m_szFileName, prv->m_szFileName);
							pWatcher->m_evtmap.erase( prv->m_szKey );
							pWatcher->m_filter.remove( prv );
							delete prv;
							prv = NULL;								
							continue;
						default: // ǰ¼ʲôǣͷǰ¼ѭ
							assert(0);
							pWatcher->m_evtmap.erase( prv->m_szKey );
							pWatcher->m_filter.remove( prv );
							delete prv;
							prv = NULL;								
							continue;
						}							
					}
					else if( prv2 )//ֻļǰ¼                                                                                          
					{
						switch( prv2->m_nAction )
						{
						case FILE_ACTION_ADDED://ǰ½
							// ½new+old->new=߼ңǰ¼ͷǰ¼ѭ
							CGDLog::WriteWatcherEvent( "-[%x][%s][%s]", prv2->m_nAction, prv2->m_szFileName, prv2->m_newFileName );
							pWatcher->m_evtmap.erase( prv2->m_szKey );
							pWatcher->m_filter.remove( prv2 );
							delete prv2;
							prv2 = NULL;
							continue;
						case FILE_ACTION_REMOVED://ǰɾ test7
							// ɾnew+old->new=ɾold+޸newǰ¼ɾoldõǰ¼޸newѭ
							CGDLog::WriteWatcherEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s][%x][%s]", prv2->m_nAction, prv2->m_szFileName, prv2->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_REMOVED, pei->m_szFileName, FILE_ACTION_MODIFIED, pei->m_newFileName );
							prv2->m_nAction = FILE_ACTION_MODIFIED;
							prv2->m_nTickCount = pei->m_nTickCount;
							pei->m_nAction = FILE_ACTION_REMOVED;
							pei->m_newFileName[0] = '\0';
							continue;
						case FILE_ACTION_MODIFIED://ǰ޸
							// ޸new+old->new=߼ңǰ¼ͷǰ¼ѭ
							CGDLog::WriteWatcherEvent( "-[%x][%s][%s]", prv2->m_nAction, prv2->m_szFileName, prv2->m_newFileName );
							pWatcher->m_evtmap.erase( prv2->m_szKey );
							pWatcher->m_filter.remove( prv2 );
							delete prv2;
							prv2 = NULL;
							continue;
						case FILE_ACTION_RENAMED_OLD_NAME://ǰļ test8
							// old->new1+old2->old=½new1+޸oldǰ¼õǰ¼ѭ
							CGDLog::WriteWatcherEvent( "+[%x][%s][%s]+[%x][%s][%s]=[%x][%s]+[%x][%s]", prv2->m_nAction, prv2->m_szFileName, prv2->m_newFileName, pei->m_nAction, pei->m_szFileName, pei->m_newFileName, FILE_ACTION_ADDED, prv2->m_newFileName, FILE_ACTION_MODIFIED, pei->m_newFileName );
							prv2->m_nAction = FILE_ACTION_MODIFIED;
							prv2->m_nTickCount = pei->m_nTickCount;
							pei->m_nAction = FILE_ACTION_ADDED;
							//add by dai_hyan 2009-8-24
							sNewKey = prv2->m_szKey;
							GetAnotherKey( sNewKey, prv2->m_szFileName, prv2->m_newFileName );
							strcpy( pei->m_szKey, sNewKey.c_str() );
							strcpy(pei->m_szFileName, prv2->m_newFileName);
							prv2->m_newFileName[0] = '\0';
							pei->m_newFileName[0] = '\0';
							continue;
						case FILE_ACTION_RENAMED_NEW_NAME://ǰļ
							// old1->new+old2->new=߼ңǰ¼ͷǰ¼ѭ
							CGDLog::WriteWatcherEvent( "-[%x][%s][%s]", prv2->m_nAction, prv2->m_szFileName, prv2->m_newFileName );
							pWatcher->m_evtmap.erase( prv2->m_szKey );
							pWatcher->m_filter.remove( prv2 );
							delete prv2;
							prv2 = NULL;
							continue;
						default: // ǰ¼ʲôǣͷǰ¼ѭ
							assert(0);
							pWatcher->m_evtmap.erase( prv2->m_szKey );
							pWatcher->m_filter.remove( prv2 );
							delete prv2;
							prv2 = NULL;								
							continue;
						}							
					}
					else // ¾ļ¼
					{
						// ¼ӵǰ¼ǰͳһʵ̲ᵽ
					}
					break;
				default:
					// δ֪¼
					break;
				}//switch
			} while(pei);
		}//if
		else
		{
			// Ϣ
			GDMINSLEEP();
		}
		
		static DWORD dwLastTime = GetTickCount();
		DWORD dwCurrTime = GetTickCount();
		if ( dwLastTime + 300 <= dwCurrTime ) // 300һ¼
		{
			dwLastTime = dwCurrTime;
			
			// ˶
			CEventInfo* pei = NULL;
			while ( ! pWatcher->m_filter.empty() )
			{
				pei = pWatcher->m_filter.front();
				// жʱ
				if ( pei->m_nTickCount + pWatcher->m_nMilliSeconds > dwNow )
				{
					// ʱ䲻
					break;
				}
				// ж½дǷ
				if ( FILE_ACTION_ADDED == pei->m_nAction || FILE_ACTION_MODIFIED == pei->m_nAction )
				{
					// ռļ
					//HANDLE hHandle = CreateFile( pei->m_szFileName,GENERIC_WRITE,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
					HANDLE hHandle = CreateFile( pei->m_szFileName, GENERIC_WRITE, FILE_SHARE_READ, 0, OPEN_EXISTING, 0, 0 );
					DWORD dwErr = GetLastError();
					if ( INVALID_HANDLE_VALUE != hHandle )
					{
						CloseHandle( hHandle );
					}
					if ( 32 == dwErr ) // "ʹøļ޷"
					{
						// ½ļռáд
						pWatcher->m_filter.pop_front();
						pei->m_nTickCount = GetTickCount();
						pWatcher->m_filter.push_back( pei ); // ¼ĩβ
						continue;
					}
				}
				// ʱˣ¼
				pWatcher->m_filter.pop_front();
				pWatcher->m_evtmap.erase( pei->m_szKey ); // ˴쳣by wang_jin at 2009-05-23
				// ж¼½Ŀ¼½ļɾĿ¼/ļ޸Ŀ¼޸ļĿ¼/ļ
				switch ( pei->m_nAction )
				{
				case FILE_ACTION_ADDED:// 0x00000001
					pei->m_nAction = MONITOR_EVENT_WATCHER_CREATE;
					break;
				case FILE_ACTION_REMOVED://  0x00000002
					pei->m_nAction = MONITOR_EVENT_WATCHER_DELETE;
					break;
				case FILE_ACTION_MODIFIED:// 0x00000003
					pei->m_nAction = MONITOR_EVENT_WATCHER_CHANGE;
					break;
				case FILE_ACTION_RENAMED_OLD_NAME://  0x00000004
					pei->m_nAction = MONITOR_EVENT_WATCHER_RENAME;
					break;
				case FILE_ACTION_RENAMED_NEW_NAME://  0x00000005
					delete pei;
					pei = NULL;
					break;
				default:
					CGDTrace::GDTrace( LOG_INF, "δ֪¼[%d]", pei->m_nAction );
					assert(0);
					break;
				}
				if ( pei )
				{
					// ־				
					CGDLog::WriteWatcherEvent( "$[%x][%s][%s]", pei->m_nAction, pei->m_szFileName, pei->m_newFileName );
					// Ӧò
					try
					{
						bool bExclude = pei->m_pFileExclude->IsExcludeEvent( (eventType)pei->m_nAction, pei->m_szFileName );
						if ( bExclude && pei->m_nAction == MONITOR_EVENT_WATCHER_RENAME )
						{
							bExclude = pei->m_pFileExclude->IsExcludeEvent( (eventType)pei->m_nAction, pei->m_newFileName );			
						}					
						if ( !bExclude )
						{							
							pWatcher->m_queue.push_back( *pei );
#ifndef NDEBUG
							if ( pei->m_nAction == MONITOR_EVENT_WATCHER_CREATE )
							{
								::InterlockedIncrement( &GnAddCount[4] );
							}
							else if ( pei->m_nAction == MONITOR_EVENT_WATCHER_DELETE )
							{
								::InterlockedIncrement( &GnRemoveCount[4] );
							}
							else if ( pei->m_nAction == MONITOR_EVENT_WATCHER_CHANGE )
							{
								::InterlockedIncrement( &GnChangeCount[4] );
							}
							else if ( pei->m_nAction == MONITOR_EVENT_WATCHER_RENAME )
							{
								::InterlockedIncrement( &GnRenameCount[4] );
							}
#endif
						}
					}
					catch ( CExceptionError )
					{
						//
					}
					catch ( ... )
					{
						//
					}
					// ɾ¼
					delete pei;
					pei = NULL;
				}
			}//for
		}//if
	}//while

	SetEvent( pWatcher->m_hEvent );

	return 0;
}

UINT __stdcall CWatcher::DequeRoutine( LPVOID lpParam )
{
	CWatcher* pWatcher = (CWatcher*)lpParam;

	DWORD dwBytesTransfered;
	DWORD dwCompletionKey;
	OVERLAPPED* pOverlapped;

	// һIOCPϢ
	while ( GetQueuedCompletionStatus( pWatcher->m_hIOCPHold, &dwBytesTransfered, &dwCompletionKey, &pOverlapped, INFINITE ) )
	{
		if (pOverlapped == ((OVERLAPPED*) ((__int64) -1)) || 0 == dwCompletionKey )
		{
			break; // ˳
		}
		else
		{
			WPARAM request = (WPARAM) dwCompletionKey;
			
			// Ϣ
			CEventInfo * pei = (CEventInfo*)request;
			assert( pei->m_pFileExclude );
#ifndef NDEBUG
			if ( pei->m_nAction == FILE_ACTION_ADDED )
			{
				::InterlockedIncrement( &GnAddCount[2] );
			}
			else if ( pei->m_nAction == FILE_ACTION_REMOVED )
			{
				::InterlockedIncrement( &GnRemoveCount[2] );
			}
			else if ( pei->m_nAction == FILE_ACTION_MODIFIED )
			{
				::InterlockedIncrement( &GnChangeCount[2] );
			}
			else if ( pei->m_nAction == FILE_ACTION_RENAMED_OLD_NAME )
			{
				::InterlockedIncrement( &GnRenameCount[2] );
			}
#endif
			// 
			EnterCriticalSection(&pWatcher->m_cs);
			// 
			pWatcher->m_deque.push_back( pei );
			// 
			LeaveCriticalSection(&pWatcher->m_cs);
		}
	}

	return 0;
}


// ߳()˳(мͣزӰ)
UINT __stdcall CWatcher::CompletionRoutine( LPVOID lpParam )
{
	// watch־
	CThreadData::SetCurrentThreadLogFile( "watch", true ); // ߳()־

	CGDTrace trace( LOG_L6, "CWatcher:CompletionRoutine(%s)", "" );

	CWatcher* pWatcher = (CWatcher*)lpParam;

	BOOL bRet = FALSE;
	DWORD dwBytes = 0;
	CDirInfo* pDirInfo = NULL;
	LPOVERLAPPED pOL = NULL;

	CGDTimer watchTimer(30);  //üʱ
	CGDLog::WriteThreadState("watch started"); //wu_binд߳״̬
	do
	{
		if (watchTimer.IsTimeout())
		{
			CGDLog::WriteThreadState("watch loop running(%d)...", watchTimer.GetMaxRunCount()); //wu_binд߳״̬
		}
#ifdef NDEBUG
#	define USE_TRY
#endif

#ifdef USE_TRY
		try
#endif
		{
			bRet = GetQueuedCompletionStatus( pWatcher->m_hIOCPRead, 
				&dwBytes, (ULONG*)&pDirInfo, &pOL, INFINITE );

			if ( !bRet )
			{
				// The io completion request failed...
				// probably because the handle to the directory that
				// was used in a call to ReadDirectoryChangesW has been closed.
				//
				// Calling pDirInfo->CloseDirectoryHandle() will cause GetQueuedCompletionStatus() to return false.
				DWORD dwErr = GetLastError();
				//assert( 0 ); // ˳ʱᵽ֪Ϊʲô?
				continue;
			}

			if ( pDirInfo ) // ?? This is NULL if "PostQueuedCompletionStatus(m_hIOCPRead,0,0,NULL);" be called.
			{
				switch ( pDirInfo->m_status )
				{
				case CWatcher::CDirInfo::STATUS_STARTING:
					pDirInfo->m_status = CWatcher::CDirInfo::STATUS_STARTED;
					break;
				case CWatcher::CDirInfo::STATUS_STOPPING:
					pDirInfo->m_status = CWatcher::CDirInfo::STATUS_STOPED;
					break;
				case CWatcher::CDirInfo::STATUS_CLOSING:
					pDirInfo->m_status = CWatcher::CDirInfo::STATUS_CLOSED;
					break;
				case CWatcher::CDirInfo::STATUS_NORMAL:
					// ¼
					if ( pDirInfo->m_evtBuf.offset )
					{
						dwBytes += pDirInfo->m_evtBuf.offset;
						pDirInfo->m_evtBuf.offset = 0;
					}
					ParseEvent( pWatcher, pDirInfo, dwBytes );
					pDirInfo->ReadDirectoryChanges(); // ǰﲻ
					break;
				default:
					cout << "δ֪״̬" << pDirInfo->m_status << endl;
					break;
				}//switch(status)
			}//if(pDirInfo)
		}
#ifdef USE_TRY
		catch ( ... )
		{
			CGDLog::WriteErrLog( "1082:CWatcher:CompletionRoutine() catch ..." );
		}
#endif
	} while ( pDirInfo  );

	CGDLog::WriteThreadState("watch exited");//wu_binд߳״̬
	return trace.SetRetValue( 0 );
}
/*		
#ifndef NDEBUG
{
	static FILE * fp1 = fopen("c:\\tmp\\evt_1.log","a");
	static int nCount1 = 0;
	if ( fp1 )
	{
		fprintf(fp1, "%d: [%d] Buffer Len:[%d]\n", ++nCount1, GetTickCount(), dwBytes );
		fflush(fp1);
	}
}
#endif
*/
//	// 
//	if ( InterlockedCompareExchange( &pDirInfo->m_nParsing, 1, 0 ) == 0 )
//	{
		// 
//		InterlockedExchange( &pDirInfo->m_nParsing, 0 );
//	}
