// MsgTran_impl.cpp: implementation of the MsgTran_impl class.

#include <STAR/CORBA.h>

#include "Base/App.h"
#include "Base/NetTran.h"
#include "Base/RunInfor.h"
#include "Base/TaskControl.h"
#include "Base/ThreadProcMsg.h"
#include "Base/Tool.h"
#include "Common/ExceptionError.h"
#include "GDLog/GDLog.h"
#include "GDLog/GDTrace.h"
#include "Groundwork/NumberFormatter.h"
#include "MsgTran/MsgTran_impl.h"


using namespace InforGuard;

CExtQueue <MsgBlockStruct2> MsgTran_impl::m_recvMsgQue;

MsgTran_impl::MsgTran_impl()
{

}

MsgTran_impl::~MsgTran_impl()
{

}

//ӷϢ
CORBA::Long MsgTran_impl::GetMessage(const InforGuard::HostChain& seqIpPort,
									 InforGuard::MsgBlock&  mb)
									 throw(InforGuard::ExceptionError,CORBA::SystemException)
{
	CGDTrace trace( LOG_L9, "MsgTran_impl:GetMessage( InforGuard::HostChain& seqIpPort, InforGuard::MsgBlock_out mb )" ); 

	// Ϣ
	switch ( mb.mid )
	{
	case MID_MC_FETCH_SERVICE_INFO:
		{
			mb.pl.length( 2 );
			mb.pl[0] = CORBA::string_dup( GetAppName() );
#if defined WIN32
			mb.pl[1] = CORBA::string_dup( "Win32" );
#elif defined Linux
			mb.pl[1] = CORBA::string_dup( "Linux" );
#else
			mb.pl[1] = CORBA::string_dup( "UNIX" );
#endif
			break;
		}
	case MID_MC_FETCH_WEBSITE_STATUS:
		{
			long taskType = 0;
			
			mb.pl.length(1);
			CApp::Instance().TaskControl().GetWebSiteTask(mb.pl[0].in(), taskType);
			mb.pl[0] = CORBA::string_dup( InforGuard::Groundwork::NumberFormatter::format(taskType).c_str() );
			
			break;
		}
	default:
		break;
	}

	return 0;
}

//Ϣ
CORBA::Long MsgTran_impl::PutMessage(const InforGuard::HostChain& seqIpPort,
									 const InforGuard::MsgBlock& mb)
									 throw(InforGuard::ExceptionError,CORBA::SystemException)
{
	string sParam;
	sParam = sParam
		+ "mb[" + CTool::MB2String( mb )
		+ "]seqIpPort[" + CNetTran::HostChainToString(seqIpPort)
		+ "]";

	CGDTrace trace( LOG_L6, "MsgTran_impl:PutMessage(%s)", sParam.c_str() ); 

	int nRet = 0;

	string sToIp;
	string sToPort;
	
	try
	{
		// ת
		if (CNetTran::IsTransmit( seqIpPort, sToIp, sToPort ))
		{
			MsgTran_var ref;
			CNetTran::GetMsgTranObject( sToIp.c_str(), sToPort.c_str(), ref );
			return ref->PutMessage( seqIpPort, mb );
		}
		
		// Ϣݽṹת
		MsgBlockStruct2 mb2;
		//mb2.mb = mb;
		//mb2.hosts = seqIpPort;
		mb2.mb.mid = mb.mid;
		mb2.mb.level = mb.level;
		mb2.mb.serial = mb.serial;
		mb2.mb.src = CORBA::string_dup( mb.src.in() );
		mb2.mb.dst = CORBA::string_dup( mb.dst.in() );
		int nCount = mb.pl.length();
		mb2.mb.pl.length( nCount );
		int i = 0;
		for ( i=0; i<nCount; i++ )
		{
			mb2.mb.pl[i] = CORBA::string_dup( mb.pl[i] );
		}
		nCount = seqIpPort.length();
		mb2.hosts.length( nCount );
		for ( i=0; i<nCount; i++ )
		{
            InforGuard::netIpPort host;
            host.sIp = CORBA::string_dup( seqIpPort[i].sIp.in() );
            host.sPort = CORBA::string_dup( seqIpPort[i].sPort.in() );
			mb2.hosts[i] = host;
		}
		
		if ( mb.level )
		{
			//CApp::Instance().RunInfor().SendToMc( INF, "(*gpDealMsgCallback)(%d)", mb.mid );
			nRet = CApp::Instance().MessageHandler().HandleMessage(mb2);
		}
		else
		{
			// Ϣд(첽)
			// ̻߳
            CGDTrace::GDTrace(LOG_DBG, "MsgTran_impl::PutMessage put msg to queue");
			m_recvMsgQue.Add(mb2);
		}
	}
	catch ( CExceptionError & ex )
	{		
		ex.AddErrText( "MsgTran_impl:PutMessage(%s)", sParam.c_str() );
		// ת쳣ʽ
		//InforGuard::ExceptionError e;
		//CNetTran::ConvertException(ex, e);
		//e._raise();
		CGDLog::WriteErrLog( "1150:MsgTran_impl:PutMessage\n%s", ex.GetErrText().c_str() );
	}
	catch( CORBA::Exception& ex)
	{
		ex._raise();
	}

	return trace.SetRetValue( nRet );
}

