// NetDirectoryClient.cpp: implementation of the CNetDirectoryClient class.
//
//////////////////////////////////////////////////////////////////////

#pragma warning( disable: 4786 )

#include "Base/FileListEx.h"
#include "Base/NetTran.h"
#include "Base/Tool.h"
#include "Common/GDString.h"
#include "GDLog/GDLog.h"
#include "GDLog/GDTrace.h"
#include "GDLog/ThreadData.h"
#include "NetDirectory/NetDirectoryClient.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CNetDirectoryClient::CNetDirectoryClient()
{

}

CNetDirectoryClient::~CNetDirectoryClient()
{

}

long CNetDirectoryClient::Init( const char* sDstHostAddr, const char* sDstPort ) throw (CExceptionError)
{
	string sParam;
	sParam = sParam
		+ "sDstHostAddr[" + sDstHostAddr
		+ "]sDstPort[" + sDstPort
		+ "]";
	
	CGDTrace trace( LOG_L6, "CNetDirectoryClient:Init(%s)", sParam.c_str() );

	m_sAddr = sDstHostAddr;
	m_sPort = sDstPort;

	try
	{
		CNetTran::GetNetDirectoryObject( sDstHostAddr, sDstPort, m_ref );
	}
	catch ( CExceptionError & ex )
	{		
		ex.AddErrText( "CNetDirectoryClient:Init(%s)", sParam.c_str() );
		throw ex;
	}
	catch( const CORBA::Exception& ex )
	{
		CExceptionError e;
		e.m_nErrCode = 0x1000 + 0x800;
		e.AddErrText( "CNetDirectoryClient:Init(%s). %s", sParam.c_str(), ex._to_string() );
		throw e;
	}

	return trace.SetRetValue( 0 );
}

long CNetDirectoryClient::GetList(const InforGuard::HostChain& seqIpPort, const char* remotefoldername, int mode, FileList &filelist) throw (CExceptionError)
{
	string sParam;
	sParam = sParam
		+ "seqIpPort[" + CNetTran::HostChainToString(seqIpPort)
		+ "]remotefoldername[" + remotefoldername
		+ "]mode[" + CGDString::ltoa( mode )
		+ "]";

	CGDTrace trace( LOG_L6, "CNetDirectoryClient:GetList(%s)", sParam.c_str() );

	if ( mode == 0 )
	{
		mode = CFileList::FILE_OPTION_FILE | CFileList::FILE_OPTION_DIRECTORY;
	}

	NetFileList_var varList;

	long nRet = 0;
	try
	{
		InforGuard::netCallData callData;
		CNetTran::InitNetCallData( callData );
		nRet = m_ref->GetList( callData, seqIpPort, remotefoldername, mode, varList.out());
		CThreadData::GetThreadSerial( (long*)&callData.nRecSerial );
	}
	catch ( CExceptionError & ex )
	{
		ex.AddErrText( "CNetDirectoryClient:GetList(%s)", sParam.c_str() );
		throw ex;
	}
	catch( InforGuard::ExceptionError& ex )
	{
		CExceptionError e;
		CNetTran::ConvertException(ex, e);
		e.AddErrText( "CNetDirectoryClient:GetList(%s)", sParam.c_str() );
		throw e;
	}
	catch( const CORBA::Exception& ex )
	{
		// һCORBA쳣ֱΪ
		CNetTran::RemoveNetDirectoryObject( m_sAddr.c_str(), m_sPort.c_str() );
		m_bInitialized = false;
		// ת쳣ʽ
		CExceptionError e;
		e.AddErrText( "%s", ex._to_string() );
		e.AddErrText( "CNetDirectoryClient:GetList(%s)", sParam.c_str() );
		throw e;
	}

	if ( nRet == 0 )
	{
		//תΪvector洢б
		long nCount = (*varList.ptr()).length();

		for ( int i = 0; i < nCount; i++ )
		{
			FileListNode node;
			node.name = (*varList.ptr())[i].name;
			node.size = (*varList.ptr())[i].size;
			node.time = (*varList.ptr())[i].time;
			node.isfolder = (*varList.ptr())[i].isfolder;
			filelist.push_back( node );
		}
	}

	return trace.SetRetValue( nRet );
}
