// NetWaterMarkClient.cpp

#pragma warning( disable: 4786 )

#include "Base/NetTran.h"
#include "Base/Tool.h"
#include "GDLog/GDLog.h"
#include "GDLog/GDTrace.h"
#include "GDLog/ThreadData.h"
#include "NetWaterMark/NetWaterMarkClient.h"

#ifdef WIN32
#	include <time.h>
#endif

using namespace std;


CNetWaterMarkClient::CNetWaterMarkClient()
{
}

CNetWaterMarkClient::~CNetWaterMarkClient()
{
}

long CNetWaterMarkClient::Init( const char* sDstHostAddr, const char* sDstPort ) throw (CExceptionError)
{	
	string sParam;
	sParam = sParam 
		+ "sDstHostAddr[" + sDstHostAddr
		+ "]sDstPort[" + sDstPort
		+ "]";

	CGDTrace trace( LOG_L6, "CNetWaterMarkClient:Init(%s)", sParam.c_str() );
	
	m_sAddr = sDstHostAddr;
	m_sPort = sDstPort;

	try
	{
		CNetTran::GetNetWaterMarkObject( sDstHostAddr, sDstPort, m_ref );
	}
	catch ( CExceptionError & ex )
	{		
		ex.AddErrText( "CNetWaterMarkClient:Init(%s)", sParam.c_str() );
		throw ex;
	}
	catch( const CORBA::Exception& ex )
	{
		CExceptionError e;
		e.m_nErrCode = 0x1000 + 0x800;
		e.AddErrText( "CNetWaterMarkClient:Init(%s). %s", sParam.c_str(), ex._to_string() );
		throw e;
	}

	return trace.SetRetValue( 0 );
}


long CNetWaterMarkClient::Set(const char* sFilePath, const char* sWm) throw (CExceptionError)
{
	string sParam;
	sParam = sParam
		+ "sFilePath[" + sFilePath
		+ "]";

	CGDTrace trace( LOG_L6, "CNetWaterMarkClient:Set(%s)", sParam.c_str() ); // [2008/06/20 13:20]

	long nRet = 0;
	CORBA::String_var sTmp = sWm;
	
	try
	{
		InforGuard::netCallData callData;
		CNetTran::InitNetCallData( callData );

		nRet = m_ref->Set( callData, sFilePath, sTmp.in() );

		CThreadData::GetThreadSerial( (long*)&callData.nRecSerial );
	}
	catch( InforGuard::ExceptionError& ex )
	{
		CExceptionError e;
		CNetTran::ConvertException(ex, e);
		e.AddErrText( "CNetWaterMarkClient:Set(%s).", sParam.c_str() );
		throw e;
	}
	catch( const CORBA::Exception& ex )
	{
		// һCORBA쳣ֱΪ
		CNetTran::RemoveNetWaterMarkObject( m_sAddr.c_str(), m_sPort.c_str() );
		m_bInitialized = false;
		// ת쳣ʽ
		CExceptionError e;
		e.AddErrText( "%s", ex._to_string() );
		e.AddErrText( "CNetWaterMarkClient:Set(%s)", sParam.c_str() );
		throw e;
	}

	return trace.SetRetValue( nRet );
}

long CNetWaterMarkClient::Get(const char* sFilePath, char* sWm) throw (CExceptionError)
{
	string sParam;
	sParam = sParam
		+ "sFilePath[" + sFilePath
		+ "]";

	CGDTrace trace( LOG_L6, "CNetWaterMarkClient:Get(%s)", sParam.c_str() );

	long nRet = 0;
	CORBA::String_var sTmp;
	
	try
	{
		InforGuard::netCallData callData;
		CNetTran::InitNetCallData( callData );
		nRet = m_ref->Get( callData, sFilePath, sTmp.out() );
		CThreadData::GetThreadSerial( (long*)&callData.nRecSerial );
	}
	catch( InforGuard::ExceptionError& ex )
	{
		CExceptionError e;
		CNetTran::ConvertException(ex, e);
		e.AddErrText( "CNetWaterMarkClient:Get(%s).", sParam.c_str() );
		throw e;
	}
	catch( const CORBA::Exception& ex )
	{
		// һCORBA쳣ֱΪ
		CNetTran::RemoveNetWaterMarkObject( m_sAddr.c_str(), m_sPort.c_str() );
		m_bInitialized = false;
		// ת쳣ʽ
		CExceptionError e;
		e.AddErrText( "%s", ex._to_string() );
		e.AddErrText( "CNetWaterMarkClient:Get(%s)", sParam.c_str() );
		throw e;
	}
	
	::strcpy(sWm, sTmp.in());

	return trace.SetRetValue( nRet );
}

long CNetWaterMarkClient::Add( const char* sFilePath ) throw (CExceptionError)
{
	string sParam;
	sParam = sParam
		+ "sFilePath[" + sFilePath
		+ "]";

	CGDTrace trace( LOG_L6, "CNetWaterMarkClient:Add(%s)", sParam.c_str() );

	long nRet = 0;
	
	try
	{
		InforGuard::netCallData callData;
		CNetTran::InitNetCallData( callData );
		nRet = m_ref->Add( callData, sFilePath );
		CThreadData::GetThreadSerial( (long*)&callData.nRecSerial );
	}
	catch( InforGuard::ExceptionError& ex )
	{
		CExceptionError e;
		CNetTran::ConvertException(ex, e);
		e.AddErrText( "CNetWaterMarkClient:Add(%s).", sParam.c_str() );
		throw e;
	}
	catch( const CORBA::Exception& ex )
	{
		// һCORBA쳣ֱΪ
		CNetTran::RemoveNetWaterMarkObject( m_sAddr.c_str(), m_sPort.c_str() );
		m_bInitialized = false;
		// ת쳣ʽ
		CExceptionError e;
		e.AddErrText( "%s", ex._to_string() );
		e.AddErrText( "CNetWaterMarkClient:Add(%s)", sParam.c_str() );
		throw e;
	}
	
	return trace.SetRetValue( nRet );
}

long CNetWaterMarkClient::Remove(const char* sFilePath) throw (CExceptionError)
{
	string sParam;
	sParam = sParam
		+ "sFilePath[" + sFilePath
		+ "]";

	CGDTrace trace( LOG_L6, "CNetWaterMarkClient:Remove(%s)", sParam.c_str() );

	long nRet = 0;
	
	try
	{
		InforGuard::netCallData callData;
		CNetTran::InitNetCallData( callData );
		nRet = m_ref->Remove( callData, sFilePath );
		CThreadData::GetThreadSerial( (long*)&callData.nRecSerial );
	}
	catch( InforGuard::ExceptionError& ex )
	{
		CExceptionError e;
		CNetTran::ConvertException(ex, e);
		e.AddErrText( "CNetWaterMarkClient:Remove(%s).", sParam.c_str() );
		throw e;
	}
	catch( const CORBA::Exception& ex )
	{
		// һCORBA쳣ֱΪ
		CNetTran::RemoveNetWaterMarkObject( m_sAddr.c_str(), m_sPort.c_str() );
		m_bInitialized = false;
		// ת쳣ʽ
		CExceptionError e;
		e.AddErrText( "%s", ex._to_string() );
		e.AddErrText( "CNetWaterMarkClient:Remove(%s)", sParam.c_str() );
		throw e;
	}
	
	return trace.SetRetValue( nRet );
}

long CNetWaterMarkClient::Rename( const char* sOldFilePath, const char* sNewFilePath ) throw (CExceptionError)
{
	string sParam;
	sParam = sParam
		+ "sOldFilePath[" + sOldFilePath
		+ "]sNewFilePath[" + sNewFilePath
		+ "]";

	CGDTrace trace( LOG_L6, "CNetWaterMarkClient:Rename(%s)", sParam.c_str() );

	long nRet = 0;
	
	try
	{
		InforGuard::netCallData callData;
		CNetTran::InitNetCallData( callData );
		nRet = m_ref->Rename( callData, sOldFilePath, sNewFilePath );
		CThreadData::GetThreadSerial( (long*)&callData.nRecSerial );
	}
	catch( InforGuard::ExceptionError& ex )
	{
		CExceptionError e;
		CNetTran::ConvertException(ex, e);
		e.AddErrText( "CNetWaterMarkClient:Rename(%s).", sParam.c_str() );
		throw e;
	}
	catch( const CORBA::Exception& ex )
	{
		// һCORBA쳣ֱΪ
		CNetTran::RemoveNetWaterMarkObject( m_sAddr.c_str(), m_sPort.c_str() );
		m_bInitialized = false;
		// ת쳣ʽ
		CExceptionError e;
		e.AddErrText( "%s", ex._to_string() );
		e.AddErrText( "CNetWaterMarkClient:Rename(%s)", sParam.c_str() );
		throw e;
	}
	
	return trace.SetRetValue( nRet );
}

long CNetWaterMarkClient::Verify(const char* sFileName,
								 long nFileSize,
								 const char* sWm,
								 bool bSimpleSync) 
								 throw (CExceptionError)
{
	string sParam;
	sParam = sParam
		+ "sFileName[" + sFileName
		+ "]sWm[" + sWm
		+ "]";

	CGDTrace trace( LOG_L6, "CNetWaterMarkClient:Rename(%s)", sParam.c_str() );

	long nRet = 0;
	
	InforGuard::netCallData callData;
	CNetTran::InitNetCallData( callData );
	try
	{
		nRet = m_ref->Verify( callData, sFileName, nFileSize, sWm, bSimpleSync );
	}
	catch ( InforGuard::ExceptionError& ex )
	{
		CExceptionError e;
		CNetTran::ConvertException(ex, e);
		e.AddErrText( "CNetWaterMarkClient:Verify(%s).", sParam.c_str() );
		throw e;
	}
	catch ( const CORBA::Exception& ex )
	{
		// һCORBA쳣ֱΪ
		CNetTran::RemoveNetWaterMarkObject( m_sAddr.c_str(), m_sPort.c_str() );
		m_bInitialized = false;
		// ת쳣ʽ
		CExceptionError e;
		e.AddErrText( "%s", ex._to_string() );
		e.AddErrText( "CNetWaterMarkClient:Verify(%s)", sParam.c_str() );
		throw e;
	}
	CThreadData::GetThreadSerial( (long*)&callData.nRecSerial );
	
	return trace.SetRetValue( nRet );
}
