                                                      // NetWaterMark_impl.cpp

#include <STAR/CORBA.h>

#include "Base/App.h"
#include "Base/FileListEx.h"
#include "Base/RunInfor.h"
#include "Base/Tool.h"
#include "Base/NetTran.h"
#include "Base/FileTranQueue.h"
#include "FileOpt/FileOpt.h"
#include "GDLog/GDLog.h"
#include "GDLog/GDTrace.h"
#include "GDLog/ThreadData.h"
#include "WaterMark/WaterMarkDB.h"
#include "NetWaterMark/NetWaterMark_impl.h"

using namespace InforGuard;

/* by wang_jin at 2008-9-30
ˮӡҪĿ˽Ŀ¼ת
*/

void NetWaterMark_impl::Check() throw(InforGuard::ExceptionError)
{
	return; // ʲôҲ·
}

CORBA::Long NetWaterMark_impl::Set(InforGuard::netCallData& callData, 
								   const char* pFile,
								   const char* wm)
								   throw(InforGuard::ExceptionError)
{
	CThreadData threadData;
	CThreadData::SetCurrentThreadLogFile( "wmserver", true );
	CThreadData::SetThreadSerial( callData.nSerial, callData.nSubSerial, callData.nRecSerial );

	//<< Ŀ¼ת(˷ֱΪWindowsͷWindows)
	char file[ GD_MAX_PATH ] = {0};
	strcpy( file, pFile );
	char * p = file;
	while ( p && *p )
	{
		if ( *p == '\\' || *p == '/' )
		{
			*p = C_PATHLASH;
		}
		p++;
	}
	//>>

	string sParam;
	sParam = sParam
		+ "file[" + file
		+ "]wm[" + wm
		+ "]";

	CGDTrace trace( LOG_L6, "NetWaterMark_impl:Set(%s)", sParam.c_str() );

	CORBA::Long  nRet = 0;

	try
	{
		nRet = CApp::Instance().WmDb().Set( file, wm );

		CThreadData::GetThreadSerial( (long*)&callData.nRecSerial );
	}
	catch ( CExceptionError & ex )
	{
		ex.AddErrText( "NetWaterMark_impl:Set(%s)", sParam.c_str() );
		InforGuard::ExceptionError e;
		CNetTran::ConvertException(ex, e);
		e._raise();
	}

	return nRet;
}

CORBA::Long NetWaterMark_impl::Get(InforGuard::netCallData& callData, 
								   const char* pFilePath,
								   CORBA::String_out wm)
								   throw(InforGuard::ExceptionError)
{
	CThreadData threadData;
	CThreadData::SetCurrentThreadLogFile( "wmserver", true );
	CThreadData::SetThreadSerial( callData.nSerial, callData.nSubSerial, callData.nRecSerial );

	//<< Ŀ¼ת(վֱΪWindowsͷWindows)
	char filePath[ GD_MAX_PATH ] = {0};
	strcpy( filePath, pFilePath );
	char * p = filePath;
	while ( p && *p )
	{
		if ( *p == '\\' || *p == '/' )
		{
			*p = C_PATHLASH;
		}
		p++;
	}
	//>>

	string sParam;
	sParam = sParam
		+ "filePath[" + filePath
		+ "]";

	CGDTrace trace( LOG_L6, "NetWaterMark_impl:Get(%s)", sParam.c_str() );

	CORBA::Long  nRet = 0;

	try
	{
		wm = CORBA::string_alloc( WMARK_LEN );

		nRet = CApp::Instance().WmDb().Get( filePath, wm.ptr() );

		CThreadData::GetThreadSerial( (long*)&callData.nRecSerial );
	}
	catch ( CExceptionError & ex )
	{
		CORBA::string_free( wm );
		ex.AddErrText( "NetWaterMark_impl:Get(%s)", sParam.c_str() );
		InforGuard::ExceptionError e;
		CNetTran::ConvertException(ex, e);
		e._raise();
	}

	return nRet;
}

CORBA::Long NetWaterMark_impl::Add(InforGuard::netCallData& callData, 
									  const char* pFilePath)
									  throw(InforGuard::ExceptionError)
{
	CThreadData threadData;
	CThreadData::SetCurrentThreadLogFile( "wmserver", true );
	CThreadData::SetThreadSerial( callData.nSerial, callData.nSubSerial, callData.nRecSerial );

	//<< Ŀ¼ת(վֱΪWindowsͷWindows)
	char filePath[ GD_MAX_PATH ] = {0};
	strcpy( filePath, pFilePath );
	char * p = filePath;
	while ( p && *p )
	{
		if ( *p == '\\' || *p == '/' )
		{
			*p = C_PATHLASH;
		}
		p++;
	}
	}

	return nRet;
}

CORBA::Long NetWaterMark_impl::Remove(InforGuard::netCallData& callData, 
									  const char* pFilePath)
									  throw(InforGuard::ExceptionError)
{
	CThreadData threadData;
	CThreadData::SetCurrentThreadLogFile( "wmserver", true );
	CThreadData::SetThreadSerial( callData.nSerial, callData.nSubSerial, callData.nRecSerial );

	//<< Ŀ¼ת(վֱΪWindowsͷWindows)
	char filePath[ GD_MAX_PATH ] = {0};
	strcpy( filePath, pFilePath );
	char * p = filePath;
	while ( p && *p )
	{
		if ( *p == '\\' || *p == '/' )
		{
			*p = C_PATHLASH;
		}
		p++;
	}
	//>>

	string sParam;
	sParam = sParam
		+ "filePath[" + filePath
		+ "]";

	CGDTrace trace( LOG_L6, "NetWaterMark_impl:Remove(%s)", sParam.c_str() );

	CORBA::Long  nRet = 0;

	try
	{
		nRet = CApp::Instance().WmDb().Remove( filePath );

		CThreadData::GetThreadSerial( (long*)&callData.nRecSerial );
	}
	catch ( CExceptionError & ex )
	{
		ex.AddErrText( "NetWaterMark_impl:Remove(%s)", sParam.c_str() );
		InforGuard::ExceptionError e;
		CNetTran::ConvertException(ex, e);
		e._raise();
	}

	return nRet;
}

CORBA::Long NetWaterMark_impl::Rename(InforGuard::netCallData& callData, 
									  const char* pOldfilename,
									  const char* pNewfilename)
									  throw(InforGuard::ExceptionError)
{
	CThreadData threadData;
	CThreadData::SetCurrentThreadLogFile( "wmserver", true );
	CThreadData::SetThreadSerial( callData.nSerial, callData.nSubSerial, callData.nRecSerial );

	//<< Ŀ¼ת(վֱΪWindowsͷWindows)
	char oldfilename[ GD_MAX_PATH ] = {0};
	strcpy( oldfilename, pOldfilename );
	char * p = oldfilename;
	while ( p && *p )
	{
		if ( *p == '\\' || *p == '/' )
		{
			*p = C_PATHLASH;
		}
		p++;
	}
	char newfilename[ GD_MAX_PATH ] = {0};
	strcpy( newfilename, pNewfilename );
	p = newfilename;
	while ( p && *p )
	{
		if ( *p == '\\' || *p == '/' )
		{
			*p = C_PATHLASH;
		}
		p++;
	}
	//>>

	string sParam;
	sParam = sParam
		+ "oldfilename[" + oldfilename
		+ "]newfilename[" + newfilename
		+ "]";

	CGDTrace trace( LOG_L6, "NetWaterMark_impl:Rename(%s)", sParam.c_str() );

	CORBA::Long  nRet = 0;

	try
	{
		nRet = CApp::Instance().WmDb().Rename( oldfilename, newfilename );

		CThreadData::GetThreadSerial( (long*)&callData.nRecSerial );
	}
	catch ( CExceptionError & ex )
	{
		ex.AddErrText( "NetWaterMark_impl:Rename(%s)", sParam.c_str() );
		InforGuard::ExceptionError e;
		CNetTran::ConvertException(ex, e);
		e._raise();
	}

	return nRet;
}

/*
[2007-10-12]֤
طҪ֤ͬ
*/
CORBA::Long NetWaterMark_impl::Verify(InforGuard::netCallData& callData, 
									  const char* pFilePath,
									  CORBA::Long nFileSize,
									  const char* sOriWm,
									  CORBA::Boolean bSimpleSync)
									  throw(InforGuard::ExceptionError)
{
	CThreadData threadData;
	CThreadData::SetCurrentThreadLogFile( "wmserver", true );
	CThreadData::SetThreadSerial( callData.nSerial, callData.nSubSerial, callData.nRecSerial );

	CGDTrace trace( LOG_L6, "NetWaterMark_impl:Verify(%s,%d,%s)", pFilePath, nFileSize, sOriWm );

	//<< Ŀ¼ת(վֱΪWindowsͷWindows)
	char filePath[ GD_MAX_PATH ] = {0};
	strcpy( filePath, pFilePath );
	char * p = filePath;
	while ( p && *p )
	{
		if ( *p == '\\' || *p == '/' )
		{
			*p = C_PATHLASH;
		}
		p++;
	}
	//>>

	int nRet = 0;
	// ж֤Դ(nFileSizeΪ-1ʾ󣬷ʾͬ)
	if ( -1 == nFileSize )
	{
		// ֤
		try
		{
			char sBakWm[ WMARK_LEN ] = {0};
			// жļĿ¼Ƿ

			if ( ! CFileOpt::IsExist( filePath ) )
			{
				// ļڣһļͲ(ˮӡ)ļǷɾ(ˮӡ)
				// жˮӡ
				try
				{
					nRet = CApp::Instance().WmDb().Get( filePath, sBakWm );
				}
				catch ( CExceptionError & ex )
				{
					ex.AddErrText( "NetWaterMark_impl::Verify(%s,%d,%s)", pFilePath, nFileSize, sOriWm );
					// ˮӡʧˣǲҪ׳쳣
					CGDTrace::GDTrace( LOG_ERR, "֤ļ[%s]ˮӡ쳣\n%s", 
						filePath, ex.GetErrText().c_str() );
					// Թ׳쳣ѽ
					return 403;
				}
				if ( nRet == 0 )
				{
					// ˮӡļǷɾָ
					//CResume::PutResumeTask( filePath );
					std::vector<RemotePath> remotePaths;
					CAppConfig::Instance().GetRemoteInfoByLocalFileName(filePath, remotePaths);

					if (remotePaths.size() > 0)
					{
						FileTranInfo info;
						info.sIp = remotePaths.at(0).ip;
						info.sPort = remotePaths.at(0).port;
						info.sTopPath = filePath;
						info.nHandleCount = 0;
						info.bIsPushFile = false;
						CApp::Instance().FileTranQueue().AddOneTaskIntoQue(info);
						// Թ֤󣬷ʧ
						return 403;
					}
					else
					{
						return 0;	// liu_yfeng ӣΪ
					}
				}
				else if ( nRet == -1 )
				{
					// ˮӡļ
					// Թ֤󣬷֤ɹ
					return 0;
				}
				else
				{
					// ˮӡʧ
					CGDTrace::GDTrace( LOG_ERR, "֤ļ[%s]ˮӡʧ[%d]", 
						filePath, nRet );
					// Թ׳쳣ѽ
					return 403;
				}
			}
			else
			{
				// ļ
				// жǷ񱻱ļ
				// liu_yfengɾGuardPolicy
				if ( !CAppConfig::Instance().IsValidFile( filePath ) )
				{
					return 0;
				}
				/*
				CGuardPolicy policy;
				CFilePolicy::GetFilePolicy( POLICY_UPDATE, filePath, policy );
				if ( !policy.IsValidFile( filePath ) )
				{
					// Ǳļ
					return 0;
				}
				*/
				else
				{
					// ļ
					// ɵǰˮӡ
					char sNewWm[ WMARK_LEN ] = {0};
					try
					{
						nRet = CWaterMarkDB::BuildWaterMark( filePath, sNewWm );
					}
					catch ( CExceptionError & ex )
					{
						ex.AddErrText( "NetWaterMark_impl::Verify(%s,%d,%s)", pFilePath, nFileSize, sOriWm );
						CGDTrace::GDTrace( LOG_ERR, "֤ļ[%s]ˮӡ쳣\n%s", 
							filePath, ex.GetErrText().c_str() );
						return 403;
					}
					if ( nRet )
					{
						CGDTrace::GDTrace( LOG_ERR, "֤ļ[%s]ˮӡʧ[%d]", filePath, nRet ); 
						return 403;
					}

					// ȡԭʼˮӡ
					try
					{
						nRet = CApp::Instance().WmDb().Get( filePath, sBakWm );
					}
					catch ( CExceptionError & ex )
					{
						ex.AddErrText( "NetWaterMark_impl::Verify(%s,%d,%s)", pFilePath, nFileSize, sOriWm );
						CGDTrace::GDTrace( LOG_ERR, "֤ļ[%s]ˮӡ쳣\n%s", 
							filePath, ex.GetErrText().c_str() );
						return 403;
					}

					// Ƚϵǰˮӡԭʼˮӡ
					if ( memcmp( sBakWm, sNewWm, WMARK_LEN ) == 0 )
					{
						// һ
						return 0;
					}
					else
					{
						// ˮӡһ£ָ
						//CResume::PutResumeTask( filePath );

						std::vector<RemotePath> remotePaths;
						CAppConfig::Instance().GetRemoteInfoByLocalFileName(filePath, remotePaths);

						FileTranInfo info;
						info.sIp = remotePaths.at(0).ip;
						info.sPort = remotePaths.at(0).port;
						info.sTopPath = filePath;
						info.nHandleCount = 0;
						info.bIsPushFile = false;
						CApp::Instance().FileTranQueue().AddOneTaskIntoQue(info);
						// Թ֤󣬷ʧ
						return 403;
					}
				}
			}
		}
		catch ( CExceptionError & ex )
		{
			CGDLog::WriteErrLog( "1051:NetWaterMark_impl::Verify(%s,%d,%s)\n%s", pFilePath, nFileSize, sOriWm, ex.GetErrText().c_str() );

			ex.AddErrText( "NetWaterMark_impl::Verify(%s,%d,%s)\n%s", pFilePath, nFileSize, sOriWm, ex.GetErrText().c_str() );
			CGDTrace::GDTrace( LOG_ERR, "֤ļ[%s]ˮӡ쳣\n%s", 
				filePath, ex.GetErrText().c_str() );
			return 500;
		}
	}
	else
	{
		// ֤ͬ
		// ͬࣨܣ
		//		ֻͬԷһµļ
		//			򵥣ֻҶԷûеļ
		//		ȫͬļ
		// ͬࣨҵ񣩣
		//		
		//		£ͬ
		// жļĿ¼Ƿ

		try
		{
			if ( ! CFileOpt::IsExist( filePath ) )
			{
				// ļڣ
				return 404;
			}
			else
			{
				// ļ
				// жǷͬ
				if ( bSimpleSync )
				{
					// ͬļڣ贫
					return 0;
				}
				else
				{
					// ȷͬ
					// ȱȽļС(Сͬ򲻱رȽˮӡ)
					if ( CFileOpt::GetFileSize( filePath ) != nFileSize )
					{
						// ļ
						return 403;
					}

					// С֤ͬˮӡ
					char sNewWm[WMARK_LEN] = {0};

					try
					{
						nRet = CWaterMarkDB::BuildWaterMark( filePath, sNewWm );
					}
					catch ( CExceptionError & ex )
					{
						ex.AddErrText( "Get(%s)", filePath );
						// ת쳣ʽ
						InforGuard::ExceptionError e;
						e.nErrCode = ex.m_nErrCode;
						int nCount = ex.m_vErrText.size();
						e.vErrText.length( nCount );
						for ( int i=0; i<nCount; i++ )
						{
							e.vErrText[i] = CORBA::string_dup( ex.m_vErrText[i].c_str() );
						}
						e._raise();
					}
					if ( nRet )
					{
						// ҪôɹҪô쳣
						InforGuard::ExceptionError e;
						e.nErrCode = 0x1000 + 0x404;
						e.vErrText.length( 1 );
						char szTmp[512] = {0};
						sprintf( szTmp, "ȷ֤ͬļ[%s]ˮӡʧ[%d]", filePath, nRet ); 
						e.vErrText[0] = CORBA::string_dup( szTmp );
						e._raise();
					}

					// Ƚˮӡ
					if ( memcmp( sOriWm, sNewWm, WMARK_LEN ) == 0)
					{
						// ˮӡ
						return 0;
					}
					else
					{
						// ˮӡ
						// ļ
						return 403;
					}
				}
			}
		}
		catch ( CExceptionError & ex )
		{
			CGDLog::WriteErrLog( "1052:NetWaterMark_impl::Verify(%s,%d,%s)\n%s", pFilePath, nFileSize, sOriWm, ex.GetErrText().c_str() );
			ex.AddErrText( "NetWaterMark_impl::Verify(%s,%d,%s)", pFilePath, nFileSize, sOriWm );
			// ת쳣ʽ
			InforGuard::ExceptionError e;
			CNetTran::ConvertException(ex, e);
			e._raise();
		}
	}

    return -1;
}

CORBA::Long NetWaterMark_impl::VerifyForFilter(const char* pFilePath)
										 throw(InforGuard::ExceptionError)
{

	//<< Ŀ¼ת(վֱΪWindowsͷWindows)
	char filePath[ GD_MAX_PATH ] = {0};
	strcpy( filePath, pFilePath );
	char * p = filePath;
	while ( p && *p )
	{
		if ( *p == '\\' || *p == '/' )
		{
			*p = C_PATHLASH;
		}
		p++;
	}
	//>>

	CORBA::Long  nResult = 0;
	/*
	CORBA::Long  nResult = VERIFYRESULT_SUCCESS;
	CORBA::Long  nRet = VERIFYRESULT_SUCCESS;

	string sFiles = files;
	string filePath;
	if (sFiles.length() <= 0)
	{
		return nResult;
	}

	string::size_type bi, ei;
	
	bi = sFiles.find_first_not_of("|");
	while (bi != string::npos)
	{
		ei = sFiles.find_first_of("|", bi);
		if (ei == string::npos)
		{
			ei = sFiles.length();
		}
		filePath = sFiles.substr(bi, ei - bi);

		nRet = VerifySingleFileForFilter(filePath.c_str());
		if (nRet)
		{
			nResult = nRet;
			gpLog->WriteInfo( ERR, "֤ʧ[%d], ļ:%s", nResult, filePath.c_str());
			GDPrint("֤ʧ[%d]: %s\r\n", nResult, filePath.c_str());
		}
		else
		{
			GDPrint(": %s\r\n", filePath.c_str());
		}

		bi = sFiles.find_first_not_of("|", ei);
	}
	*/
	return nResult;
}


CORBA::Long NetWaterMark_impl::RequestResume(const char* pFilePath) throw(InforGuard::ExceptionError)
{

	//<< Ŀ¼ת(վֱΪWindowsͷWindows)
	char filePath[ GD_MAX_PATH ] = {0};
	strcpy( filePath, pFilePath );
	char * p = filePath;
	while ( p && *p )
	{
		if ( *p == '\\' || *p == '/' )
		{
			*p = C_PATHLASH;
		}
		p++;
	}
	//>>

	//͵ָ
	//CResume::PutResumeTask( filePath );
	std::vector<RemotePath> remotePaths;
	CAppConfig::Instance().GetRemoteInfoByLocalFileName(filePath, remotePaths);

	FileTranInfo info;
	info.sIp = remotePaths.at(0).ip;
	info.sPort = remotePaths.at(0).port;
	info.sTopPath = filePath;
	info.nHandleCount = 0;
	info.bIsPushFile = false;
	CApp::Instance().FileTranQueue().AddOneTaskIntoQue(info);
	return 0;
}

