// 紫ʵ

#include "Base/App.h"
#include "Base/AppConfig.h"
#include "Base/NetTran.h"
#include "Base/NetTran.h"
#include "Base/Tool.h"
#include "Common/AppError.h"
#include "Common/Common.h"
#include "Common/DateTime.h"
#include "GDLog/GDLog.h"
#include "GDLog/GDTrace.h"
#include "GDLog/ThreadData.h"

#ifdef WIN32
#	include <windows.h>
#	include <process.h>
#endif

#include "MsgTran/MsgTran_impl.h"
#include "NetConfig/NetConfig_impl.h"
#include "NetDirectory/NetDirectory_impl.h"
#include "NetWaterMark/NetWaterMark_impl.h"


using namespace InforGuard;

CORBA::ORB_var CNetTran::m_orb;
bool CNetTran::m_bUseMultiORB;
CORBA::ORB_var CNetTran::m_orb_netdir[];
CORBA::ORB_var CNetTran::m_orb_filetran[];
CORBA::ORB_var CNetTran::m_orb_netwatermark[];

map<string, FileTran_var> CNetTran::m_mapFileTran;
map<string, MsgTran_var> CNetTran::m_mapMsgTran;
map<string, NetDirectory_var> CNetTran::m_mapNetDir;
map<string, NetWaterMark_var> CNetTran::m_mapNetWM;

CThreadMutex CNetTran::m_mutex_netdir;
CThreadMutex CNetTran::m_mutex_filetran;
CThreadMutex CNetTran::m_mutex_netwatermark;
CThreadMutex CNetTran::m_mutex_msgtran;

CNetTran::CNetTran()
{
}

CNetTran::~CNetTran()
{
}

void CNetTran::InitORB( const char* szIp, const char* szPort, const char* sThreads ) throw (CExceptionError)
{
	CGDTrace trace( LOG_L6, "CNetTran:InitORB(%s,%s,%s)", szIp, szPort, sThreads );

	// ʼORB
#ifdef WIN32	// ÿÿ߳
	char * argv[] = { "", "-OAhost", "", "-OAport", "", "-OAthread_per_request" }; // thread_per_client
#else			// ̳߳
	char * argv[] = { "", "-OAhost", "", "-OAport", "", "-OAthread_pool", "" };
	argv[6] = (char*)sThreads;
#endif
	argv[4] = (char*)szPort;
	argv[2] = (char*)szIp;
	int argc = sizeof(argv) / sizeof(argv[0]);

	m_bUseMultiORB = false; // ļгʼ

	try
	{
		// ΨһORB
		m_orb = CORBA::ORB_init( argc, argv );

		if ( m_bUseMultiORB )
		{
			int n = 1;
			char * v[] = {""};
			char orbName[10] = {0};
			// ͻڶORB
			for ( int i=0; i<MAX_ORB_COUNT; i++ )
			{
				sprintf( orbName, "netdir%d", i );
				m_orb_netdir[i] = CORBA::ORB_init( n, v, orbName );
				sprintf( orbName, "filetran%d", i );
				m_orb_filetran[i] = CORBA::ORB_init( n, v, orbName );
				sprintf( orbName, "netwatermark%d", i );
				m_orb_netwatermark[i] = CORBA::ORB_init( n, v, orbName );
			}
		}
		int ms = CAppOption::Instance().GetOption( "orb.timeout", 1000*15 ); // ͻORBʱĬ15
		if ( ! ResetOrbTimeoutValue( m_orb, ms ) )
		{
			CGDLog::WriteErrLog( "1032:ResetOrbTimeoutValue(%d) return false", ms );
		}
	}
	catch( const CORBA::Exception& ex )
	{
		// ת쳣ʽ
		CExceptionError e;
		e.m_nErrCode = CAppError::GD_APP_ERR_EXCEPTION_CORBA;
		e.AddErrText( "ʼORBʧ" );
		e.AddErrText( "********\n%s********", ex._to_string() );
		e.AddErrText( "CNetTran:InitORB(%s,%s,%s)", szIp, szPort, sThreads );
		throw e;
	}
}

void CNetTran::ORBRun() throw (CExceptionError) // 
{
	CGDTrace trace( LOG_L6, "CNetTran:ORBRun()" );

	try
	{
		// 1 servant - 
		FileTran_impl* fileTranImpl = new FileTran_impl();
		MsgTran_impl* msgTranImpl = new MsgTran_impl();
		NetConfig_impl* netConfigImpl = new NetConfig_impl();
		NetDirectory_impl* netDirectoryImpl = new NetDirectory_impl();
		NetWaterMark_impl* netWaterMarkImpl = new NetWaterMark_impl();

		// 2 id - ʶ
		PortableServer::ObjectId_var objIdFileTran     = PortableServer::string_to_ObjectId("FileTran");
		PortableServer::ObjectId_var objIdMsgTran      = PortableServer::string_to_ObjectId("MsgTran");
		PortableServer::ObjectId_var objIdNetConfig    = PortableServer::string_to_ObjectId("NetConfig");
		PortableServer::ObjectId_var objIdNetDirectory = PortableServer::string_to_ObjectId("NetDirectory");
		PortableServer::ObjectId_var objIdNetWaterMark = PortableServer::string_to_ObjectId("NetWaterMark");
		
		// 3 ȡRootPOAüManageròPOAManager
		CORBA::Object_var poaObj = m_orb -> resolve_initial_references("RootPOA");
		if( CORBA::is_nil(poaObj) )
		{
			CExceptionError ex;
			ex.m_nErrCode = CAppError::GD_APP_ERR_CORBA_ISNULL;
			ex.AddErrText( "ʧ" );
			ex.AddErrText( "CNetTran:ORBRun()" );
			throw ex;
		}
		PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(poaObj);
		if( CORBA::is_nil(rootPOA) )
		{
			CExceptionError ex;
			ex.m_nErrCode = CAppError::GD_APP_ERR_CORBA_ISNULL;
			ex.AddErrText( "תʧ" );
			ex.AddErrText( "CNetTran:ORBRun()" );
			throw ex;
		}
		PortableServer::POAManager_var manager = rootPOA -> the_POAManager();
		if( CORBA::is_nil(manager) )
		{
			CExceptionError ex;
			ex.m_nErrCode = CAppError::GD_APP_ERR_CORBA_ISNULL;
			ex.AddErrText( "ȡʧ" );
			ex.AddErrText( "CNetTran:ORBRun()" );
			throw ex;
		}

		// 4 POA
		CORBA::PolicyList policies; 
		policies.length (2); 
		policies[0] = rootPOA->create_lifespan_policy(PortableServer::PERSISTENT);
		policies[1] = rootPOA->create_id_assignment_policy(PortableServer::USER_ID);
		

		PortableServer::POA_var poa = rootPOA->create_POA("GD_POA", manager, policies);
		if( CORBA::is_nil(poa) )
		{
			CExceptionError ex;
			ex.m_nErrCode = CAppError::GD_APP_ERR_CORBA_ISNULL;
			ex.AddErrText( "POAʧ" );
			ex.AddErrText( "CNetTran:ORBRun()" );
			throw ex;
		}
		
		poa->activate_object_with_id(objIdFileTran, fileTranImpl);  
		poa->activate_object_with_id(objIdMsgTran, msgTranImpl); 
		poa->activate_object_with_id(objIdNetConfig, netConfigImpl);  
		poa->activate_object_with_id(objIdNetDirectory, netDirectoryImpl);  
		poa->activate_object_with_id(objIdNetWaterMark, netWaterMarkImpl);  
		
		manager->activate();
		
		// 5 ORB
		m_orb->run(); // 
	}
	catch ( CORBA::Exception &ei )
	{
		CExceptionError ex;
		ex.m_nErrCode = CAppError::GD_APP_ERR_EXCEPTION_CORBA;
		ex.AddErrText( "********\n%s********", ei._to_string() );
		ex.AddErrText( "CNetTran:ORBRun()" );
		throw ex;
	}
	catch (...)
	{
		CExceptionError ex;
		ex.m_nErrCode = CAppError::GD_APP_ERR_EXCEPTION;
		ex.AddErrText("δ֪쳣");
		ex.AddErrText("CNetTran:ORBRun()");
		throw ex;
	}
}

bool CNetTran::ResetOrbTimeoutValue(CORBA::ORB_ptr orb,CORBA::ULong val)
{
	CGDTrace trace( LOG_L6, "CNetTran:ResetOrbTimeoutValue()" );

	CORBA::ULong gl_timeout_val=val;
	CORBA::PolicyManager_var gl_pc1;
	CORBA::PolicyList gl_pl;
	CORBA::Any gl_any;
	
	try
	{
		CORBA::Object_var pcObj = orb -> resolve_initial_references("ORBPolicyManager");
		gl_pc1 = CORBA::PolicyManager::_narrow(pcObj);
        if( CORBA::is_nil(gl_pc1) )
        {
            return trace.SetRetResult( false );
        }
		gl_pl.length(2);
	}
	catch(...)
	{
		return trace.SetRetResult( false );
	}

	try
	{
		gl_any <<= gl_timeout_val;
		gl_pl[0]=orb->create_policy(STAR::ORB_TIMEOUT_POLICY_ID, gl_any);
		gl_pl[1]=orb->create_policy(STAR::ORB_CON_TIMEOUT_POLICY_ID, gl_any);
		gl_pc1->set_policy_overrides(gl_pl, CORBA::SET_OVERRIDE);
	}
	catch(...)
	{
		return trace.SetRetResult( false );
	}

	return trace.SetRetResult( true );
}

string CNetTran::AssembleMapKey(const char* sIp, const char* sPort)
{
	CGDTrace trace( LOG_L6, "CNetTran:AssembleMapKey(%s,%s)", sIp,sPort );
	string sKey;
	
	sKey = sIp;
	sKey += "_";
	sKey += sPort;
	trace.SetRetText( sKey.c_str() );
	return sKey;
}

CORBA::Object_var CNetTran::GetTran(CORBA::ORB_ptr orb, const char* sIp, const char* sPort, const char* sServantName)
	throw (CExceptionError)
{
	string sParam;
	sParam = sParam 
		+ "sIp[" + sIp
		+ "]sPort[" + sPort
		+ "]sServantName[" + sServantName
		+ "]";

	CGDTrace trace( LOG_L6, "CNetTran:GetTran(%s)", sParam.c_str() );

	CORBA::Object_var objRef;

	try
	{
		if ( ( sIp == NULL || sIp[0] == '\0' )
			|| ( sPort == NULL || sPort[0] == '\0' )
			|| ( sServantName == NULL || sServantName[0] == '\0' ) )
		{
			CExceptionError ex;			
			ex.AddErrText( "Զ̶Ч[%s]", sParam.c_str() );
			throw ex;
		}

		// ȡԶ̷
		char szStr[256] = {0};
#ifdef NDEBUG // by wang_jin at 2009-3-21 for debug
        CORBA::String_var keyStr = ConstructKeyStr(sServantName);
		string sKeyStr = keyStr.in();
#else // by wang_jin at 2009-05-23
		string sKeyStr = "%AB%AC%AB0%5FRootPOA%00GD%5FPOA%00%00";
		sKeyStr += sServantName;
#endif

        CGDTrace::GDTrace( LOG_DBG, "ĿKey(%s --> %s)", sServantName, sKeyStr.c_str() ); 

		sprintf( szStr, "corbaloc:iiop:1.2@%s:%s/%s", sIp, sPort, sKeyStr.c_str() );
		objRef = orb->string_to_object( szStr );

		if ( CORBA::is_nil( objRef ) )
		{
			// "Cannot produce object reference from ref"
			CExceptionError ex;
			ex.AddErrText( "ȡԶ̶ʧ[%s]", szStr );
			throw ex;
		}
	}
	catch ( CExceptionError & ex )
	{
		// ת쳣
		ex.AddErrText( "CNetTran:GetTran(%s)", sParam.c_str() );
		throw ex;
	}
	catch( const InforGuard::ExceptionError& ei )
	{
		// ת쳣
		CExceptionError ex;
		CNetTran::ConvertException(ei, ex);
		ex.AddErrText( "CNetTran:GetTran(%s)", sParam.c_str() );
		throw ex;
	}
	catch( const CORBA::Exception& ei )
	{
		// ת쳣
		CExceptionError ex;
		ex.AddErrText( ei._to_string() );
		ex.AddErrText( "CNetTran:GetTran(%s)", sParam.c_str() );
		throw ex;
	}

	return objRef._retn();
}

/*
 * GetXXXObject()ʧܣϲҵ񲢲ʧԭֻҪָ
 * ڴˣֻҪͨ쳣֪ʧܼʧԭ򼴿
 */
void CNetTran::GetFileTranObject( const char* sIp, const char* sPort, FileTran_var& obj ) throw (CExceptionError)
{
	CGDTrace trace( LOG_L6, "CNetTran:GetFileTranObject(%s,%s)", sIp,sPort );
	CThreadLock lock( m_mutex_filetran );

	map<string, FileTran_var>::iterator iter;
	iter = m_mapFileTran.find( AssembleMapKey(sIp, sPort) );

	if ( iter == m_mapFileTran.end() ) // δȡԶ˶
	{
		try
		{
			int nThreadIndex = 0;
			CThreadData* pData = CThreadData::GetCurrentThreadData();
			if( pData )
			{
				nThreadIndex = pData->m_nThreadIndex;
			}

			if ( !m_bUseMultiORB || nThreadIndex == 0 )
			{
				obj = InforGuard::FileTran::_narrow( CNetTran::GetTran( m_orb, sIp, sPort, "FileTran") );
			}
			else
			{
				obj = InforGuard::FileTran::_narrow( CNetTran::GetTran( m_orb_filetran[nThreadIndex - 1], sIp, sPort, "FileTran") );
			}

            if( CORBA::is_nil(obj) )
            {
                CExceptionError ex;
                ex.AddErrText( "CNetTran:GetFileTranObject obj is null");
			    throw ex;
            }

			// ӳ
			m_mapFileTran.insert( make_pair(AssembleMapKey(sIp, sPort), obj) );
		}
		catch ( CExceptionError & ex )
		{
			// ת쳣
			ex.AddErrText( "CNetTran:GetFileTranObject(%s,%s)", sIp, sPort );
			throw ex;
		}
		catch( const InforGuard::ExceptionError& ei )
		{
			// ת쳣
			CExceptionError ex;
			CNetTran::ConvertException(ei, ex);
			ex.AddErrText( "CNetTran:GetFileTranObject(%s,%s)", sIp, sPort );
			throw ex;
		}
		catch( const CORBA::Exception& ei )
		{
			// ת쳣
			CExceptionError ex;
			ex.AddErrText( ei._to_string() );
			ex.AddErrText( "CNetTran:GetFileTranObject(%s,%s)", sIp, sPort );
			throw ex;
		}
	}
	else
	{
		obj = iter->second;
	}
}

void CNetTran::GetMsgTranObject( const char* sIp, const char* sPort, MsgTran_var& obj )
{
	CGDTrace trace( LOG_L6, "CNetTran:GetMsgTranObjec(%s,%s)", sIp,sPort );
	
	CThreadLock lock( m_mutex_msgtran );

	map<string, MsgTran_var>::iterator iter;
	iter = m_mapMsgTran.find( AssembleMapKey(sIp, sPort) );

	if ( iter == m_mapMsgTran.end() ) // δȡԶ˶
	{
		try
		{
			obj = InforGuard::MsgTran::_narrow( CNetTran::GetTran(m_orb, sIp, sPort, "MsgTran") );
            if( CORBA::is_nil(obj) )
            {
                CExceptionError ex;
                ex.AddErrText( "CNetTran:GetMsgTranObjec obj is null" );
                throw ex;
            }
			// ӳ
			m_mapMsgTran.insert( make_pair(AssembleMapKey(sIp, sPort), obj) );
		}
		catch ( CExceptionError & ex )
		{
			// ת쳣
			ex.AddErrText( "CNetTran:GetMsgTranObject(%s,%s)", sIp, sPort );
			throw ex;
		}
		catch( const InforGuard::ExceptionError& ei )
		{
			// ת쳣
			CExceptionError ex;
			CNetTran::ConvertException(ei, ex);
			ex.AddErrText( "CNetTran:GetMsgTranObject(%s,%s)", sIp, sPort );
			throw ex;
		}
		catch( const CORBA::Exception& ei )
		{
			// ת쳣
			CExceptionError ex;
			ex.AddErrText( ei._to_string() );
			ex.AddErrText( "CNetTran:GetMsgTranObject(%s,%s)", sIp, sPort );
			throw ex;
		}
	}
	else
	{
		obj = iter->second;
	}
}

void CNetTran::GetNetDirectoryObject( const char* sIp, const char* sPort, NetDirectory_var& obj )
{
	CGDTrace trace( LOG_L6, "CNetTran:GetNetDirectoryObject(%s,%s)", sIp,sPort );
	CThreadLock lock( m_mutex_netdir );

	map<string, NetDirectory_var>::iterator iter;
	iter = m_mapNetDir.find( AssembleMapKey(sIp, sPort) );

	if ( iter == m_mapNetDir.end() ) // δȡԶ˶
	{
		try
		{
			int nThreadIndex = 0;
			CThreadData* pData = CThreadData::GetCurrentThreadData();
			if( pData )
			{
				nThreadIndex = pData->m_nThreadIndex;
			}

			if ( !m_bUseMultiORB || nThreadIndex == 0 )
			{
				obj = InforGuard::NetDirectory::_narrow( CNetTran::GetTran(m_orb, sIp, sPort, "NetDirectory") );
			}
			else
			{
				obj = InforGuard::NetDirectory::_narrow( CNetTran::GetTran( m_orb_netdir[nThreadIndex - 1], sIp, sPort, "NetDirectory") );
			}

            if( CORBA::is_nil(obj) )
            {
                CExceptionError ex;
                ex.AddErrText( "CNetTran:GetNetDirectoryObject obj is null" );
                throw ex;
            }

			// ӳ
			m_mapNetDir.insert( make_pair(AssembleMapKey(sIp, sPort), obj) );
		}
		catch ( CExceptionError & ex )
		{
			// ת쳣
			ex.AddErrText( "CNetTran:GetNetDirectoryObject(%s,%s)", sIp, sPort );
			throw ex;
		}
		catch( const InforGuard::ExceptionError& ei )
		{
			// ת쳣
			CExceptionError ex;
			CNetTran::ConvertException(ei, ex);
			ex.AddErrText( "CNetTran:GetNetDirectoryObject(%s,%s)", sIp, sPort );
			throw ex;
		}
		catch( const CORBA::Exception& ei )
		{
			// ת쳣
			CExceptionError ex;
			ex.AddErrText( ei._to_string() );
			ex.AddErrText( "CNetTran:GetNetDirectoryObject(%s,%s)", sIp, sPort );
			throw ex;
		}
	}
	else
	{
		obj = iter->second;
	}
}

void CNetTran::GetNetWaterMarkObject( const char* sIp, const char* sPort, NetWaterMark_var& obj )
{
	CGDTrace trace( LOG_L6, "CNetTran:GetNetWaterMarkObject(%s,%s)", sIp,sPort );
	CThreadLock lock( m_mutex_netwatermark );

	map<string, NetWaterMark_var>::iterator iter;
	iter = m_mapNetWM.find( AssembleMapKey(sIp, sPort) );

	if ( iter == m_mapNetWM.end() ) // δȡԶ˶
	{
		try
		{
			int nThreadIndex = 0;
			CThreadData* pData = CThreadData::GetCurrentThreadData();
			if( pData )
			{
				nThreadIndex = pData->m_nThreadIndex;
			}

			if ( !m_bUseMultiORB || nThreadIndex == 0 )
			{
				obj = InforGuard::NetWaterMark::_narrow( CNetTran::GetTran( m_orb, sIp, sPort, "NetWaterMark") );
			}
			else
			{
				obj = InforGuard::NetWaterMark::_narrow( CNetTran::GetTran( m_orb_netwatermark[nThreadIndex - 1], sIp, sPort, "NetWaterMark") );
			}
            if( CORBA::is_nil(obj) )
            {
                CExceptionError ex;
                ex.AddErrText( "CNetTran:GetNetWaterMarkObject obj is null" );
                throw ex;
            }

			// ӳ
			m_mapNetWM.insert( make_pair(AssembleMapKey(sIp, sPort), obj) );
		}
		catch ( CExceptionError & ex )
		{
			// ת쳣
			ex.AddErrText( "CNetTran:GetNetWaterMarkObject(%s,%s)", sIp, sPort );
			throw ex;
		}
		catch( const InforGuard::ExceptionError& ei )
		{
			// ת쳣
			CExceptionError ex;
			CNetTran::ConvertException(ei, ex);
			ex.AddErrText( "CNetTran:GetNetWaterMarkObject(%s,%s)", sIp, sPort );
			throw ex;
		}
		catch( const CORBA::Exception& ei )
		{
			// ת쳣
			CExceptionError ex;
			ex.AddErrText( ei._to_string() );
			ex.AddErrText( "CNetTran:GetNetWaterMarkObject(%s,%s)", sIp, sPort );
			throw ex;
		}
	}
	else
	{
		obj = iter->second;
	}
}

void CNetTran::RemoveFileTranObject( const char* sIp, const char* sPort )
{
	CGDTrace trace( LOG_L6, "CNetTran:RemoveFileTranObject(%s,%s)", sIp,sPort );

	CThreadLock lock( m_mutex_filetran );

	map<string, FileTran_var>::iterator iter;
	iter = m_mapFileTran.find( AssembleMapKey(sIp, sPort) );

	if ( iter != m_mapFileTran.end() )
	{
		m_mapFileTran.erase( iter );
	}
}

void CNetTran::RemoveMsgTranObject( const char* sIp, const char* sPort )
{
	CGDTrace trace( LOG_L6, "CNetTran:RemoveMsgTranObject(%s,%s)", sIp,sPort );

	CThreadLock lock( m_mutex_msgtran );

	map<string, MsgTran_var>::iterator iter;
	iter = m_mapMsgTran.find( AssembleMapKey(sIp, sPort) );

	if ( iter != m_mapMsgTran.end() )
	{
		m_mapMsgTran.erase( iter );
	}
}

void CNetTran::RemoveNetDirectoryObject( const char* sIp, const char* sPort )
{
	CGDTrace trace( LOG_L6, "CNetTran:RemoveNetDirectoryObject(%s,%s)", sIp,sPort );
	CThreadLock lock( m_mutex_netdir );

	map<string, NetDirectory_var>::iterator iter;
	iter = m_mapNetDir.find( AssembleMapKey(sIp, sPort) );

	if ( iter != m_mapNetDir.end() )
	{
		m_mapNetDir.erase( iter );
	}
}

void CNetTran::RemoveNetWaterMarkObject( const char* sIp, const char* sPort )
{
	CGDTrace trace( LOG_L6, "CNetTran:RemoveNetWaterMarkObject(%s,%s)", sIp,sPort );
	CThreadLock lock( m_mutex_netwatermark );

	map<string, NetWaterMark_var>::iterator iter;
	iter = m_mapNetWM.find( AssembleMapKey(sIp, sPort) );

	if ( iter != m_mapNetWM.end() )
	{
		m_mapNetWM.erase( iter );
	}
}

long CNetTran::GetNextIpPort( const InforGuard::HostChain& seqIpPort, string& sIp, string& sPort )
{
	CGDTrace trace( LOG_L6, "CNetTran:GetNextIpPort(%s,%s)", sIp.c_str(),sPort.c_str() );

	string sLocalIp = CAppConfig::Instance().GetLocalIp();
	string sLocalPort = CAppConfig::Instance().GetLocalPort();
	string sToIp;
	string sToPort;
	int nCount = seqIpPort.length();
	if ( nCount <= 0 )
	{
		return 1;
	}

	sToIp = seqIpPort[nCount-1].sIp.in();
	sToPort = seqIpPort[nCount-1].sPort.in();
	if ( sToIp == sLocalIp && sToPort == sLocalPort )	//ĿԼ
	{
		return 1;
	}
	//Ŀ겻Լ
	for( int i=0; i< nCount - 1; i++ )
	{
		sToIp = seqIpPort[i].sIp.in();
		sToPort = seqIpPort[i].sPort.in();
		if ( sToIp == sLocalIp && sToPort == sLocalPort )
		{
			sIp = seqIpPort[i+1].sIp.in();
			sPort = seqIpPort[i+1].sPort.in();
			return 0;
		}
	}
	return -1;//return GD_APP_ERR_NETTRAN_NO_IP;
}

bool CNetTran::IsTransmit(const InforGuard::HostChain& seqIpPort, string& sToIp, string &sToPort)
{
	CGDTrace trace( LOG_L6, "CNetTran:IsTransmit(%s,%s)", sToIp.c_str(),sToPort.c_str() );

	long nRet = -1;

	nRet = GetNextIpPort( seqIpPort, sToIp, sToPort );
	if ( nRet == 0 )		//ת
	{
		CGDTrace::GDTrace(LOG_DBG, "CNetTran::IsTransmit return true; Transmit!");
		return trace.SetRetResult( true );	
	}

	return trace.SetRetResult( false );
}

void CNetTran::ConvertException(const InforGuard::ExceptionError& ei, CExceptionError& ex)
{
	CGDTrace trace( LOG_L6, "CNetTran:ConvertException()" );
	// ת쳣ʽ
	ex.m_nErrCode = ei.nErrCode;
	int nCount = ei.vErrText.length();
	for ( int i=0; i<nCount; i++ )
	{
		ex.m_vErrText.push_back( ei.vErrText[i] );
	}
}

void CNetTran::ConvertException(const CExceptionError& ei, InforGuard::ExceptionError& ex)
{
	CGDTrace trace( LOG_L6, "CNetTran:ConvertException()" );
	// ת쳣ʽ
	ex.nErrCode = ei.m_nErrCode;
	int nCount = ei.m_vErrText.size();
	ex.vErrText.length(nCount);
	for ( int i=0; i<nCount; i++ )
	{
		ex.vErrText[i] = CORBA::string_dup(ei.m_vErrText[i].c_str());
	}
}

void CNetTran::ConvertException(const CORBA::Exception& ei, CExceptionError &ex)
{
	// ת쳣ʽ
	ex.AddErrText( ei._to_string() );
}

void CNetTran::ReverseHostChain(InforGuard::HostChain& srcSeqIpPort, InforGuard::HostChain& dstSeqIpPort)
{
	CGDTrace trace( LOG_L6, "CNetTran:ReverseHostChain()" );
	long nSrcCount = srcSeqIpPort.length();

	dstSeqIpPort.length(nSrcCount);

	for (long i = 0; i < nSrcCount; i++)
	{
		dstSeqIpPort[nSrcCount - i - 1].sIp = srcSeqIpPort[i].sIp;
		dstSeqIpPort[nSrcCount - i - 1].sPort = srcSeqIpPort[i].sPort;
	}
}

string CNetTran::HostChainToString(const InforGuard::HostChain& hostchain)
{
	CGDTrace trace( LOG_L6, "CNetTran:HostChainToString()" );
	
	long nCount = hostchain.length();
	string sChain;
	
	for (long i = 0; i < nCount; i++)
	{
		sChain += "IP[";
		sChain += hostchain[i].sIp;
		sChain += "]PORT[";
		sChain += hostchain[i].sPort;
		sChain += "]";
	}
	trace.SetRetText( sChain.c_str() );
	return sChain;
}

void CNetTran::InitNetCallData( InforGuard::netCallData& callData )
{
	//CGDTrace trace( LOG_L6, "CNetTran:InitNetCallData()" ); // Ҫ

	CThreadData* pThreadData = CThreadData::GetCurrentThreadData();

	if( pThreadData )
	{
		callData.nThreadId  = pThreadData->m_nThreadId;
		callData.nSerial    = pThreadData->m_nSerial;
		callData.nSubSerial = pThreadData->m_nSubSerial;
		callData.nRecSerial = pThreadData->m_nRecSerial;
		callData.nIndent	= pThreadData->m_nIndent;
	}
	else
	{
		callData.nThreadId  = CThreadData::GetCurrentThreadId();
		callData.nSerial    = 0;
		callData.nSubSerial = 0;
		callData.nRecSerial = 0;
		callData.nIndent	= 0;
	}

	callData.sIp        = CORBA::string_dup( CAppConfig::Instance().GetLocalIp().c_str() );
	callData.sPort      = CORBA::string_dup( CAppConfig::Instance().GetLocalPort().c_str() );
}

void CNetTran::shutdown_orb()
{
	CGDTrace trace( LOG_L6, "CNetTran:shutdown_orb()" );
	try
	{
		if ( ! CORBA::is_nil(CNetTran::m_orb) )
		{
			try
			{
				CNetTran::m_orb->shutdown(true);
				CNetTran::m_orb->destroy();
				CNetTran::m_orb = NULL;
			}
			catch(const CORBA::Exception& ex)
			{
				GDPrint( "%s\n", ex._to_string() );
			}
		}
	}
	catch (...)
	{
		trace.SetRetText("catch(...)");
	}
}

char CNetTran::EncodeHex(int i)
{
    if( (i < 0) || (i >= 16) )
    {
        return 0;
    }
    
    if(i >= 0 && i <= 9)
    {
        return i + '0';
    }
    
    return ((i - 10) + 'A');
}

char* CNetTran::keyToStr(const ATF::ObjectKey& key)
{
    CORBA::ULong klen = key.length();
    CORBA::ULong len = klen + 1;
    
    char* strBuf = CORBA::string_alloc(len * 3);
    CORBA::String_var keyStr(strBuf);
    
    CORBA::ULong wr;
    CORBA::ULong rd;
    
    for(wr = 0, rd = 0; rd < klen; rd++, wr++)
    {
        unsigned char s = key[rd];
        if(isalpha(s) || isdigit(s))
        {
            strBuf[wr] = s;
        }
        else
        {
            strBuf[wr] = '%';
            strBuf[++wr] = EncodeHex((s >> 4) & 0xF);
            strBuf[++wr] = EncodeHex(s & 0xF);
        }
    }
    
    strBuf[wr]='\0';
    return keyStr._retn();
}

char* CNetTran::ConstructKeyStr(const char* name)
{
    PortableServer::ObjectId_var oid = PortableServer::string_to_ObjectId(name);
    
    STAR::ObjectKeyData data;
    data.persistent = true;
    data.createTime = 0;
    data.oid = oid;
    data.poaID.length(2);
    data.poaID[0] = CORBA::string_dup("_RootPOA");
    data.poaID[1] = CORBA::string_dup("GD_POA");
        
    ATF::ObjectKey_var oct = STAR::CreateObjectKey(data);
    
	return keyToStr(oct);
}
