/***********************************************************************
*
* ļ  ProtectLinux.cpp
* ģĿģ File Protectection Module
* ߼ڣ  zhang_quan at 2010-02-03
* 汾 1.0
* ȨϢCVICSE Middleware
* ļ¼ zhang_quan at 2010-02-03
*
***********************************************************************/

#include "ProtectLinux.h"

#include "Common/Common.h"
#include "Common/ExceptionError.h"
#include "Common/ParamList.h"
#include "Common/DateTime.h"
#include "Common/GDTimer.h"
#include "Common/ConvertString.h"
#include "FileOpt/FileOpt.h"
#include "GDLog/GDLog.h"
#include "GDLog/GDTrace.h"
#include "GDLog/RunLog.h"
#include "GDLog/LogFile.h"
//#include "GDLog/ThreadData.h"

#include <assert.h>
#ifdef WIN32
#	include <process.h> // for _beginthreadex
#	include <windows.h>
#	include <algorithm> // for std::sort
#	include <sstream>
#else
#	define InterlockedIncrement atomic_inc
#endif

#	include <pthread.h> // added by zhangquan for threadPriorConfig function
#	include <sched.h>
#	include <iostream>


using namespace std;

std::map<std::string, CProtect::CDirInfo*> CProtect::m_dirmap;

CProtect::CProtect()
{
	Initialize("");
}


CProtect::~CProtect()
{
	if ( isInited )
	{
		Uninitialize();
	}
}

// ʼʧ쳣һ˳
bool CProtect::Initialize( const char * sParams )
{
	CGDTrace trace( LOG_L6, "CProtect:Initialize(%s)", sParams );
	isInited = true;
	return trace.SetRetResult( true );
}

bool CProtect::WatchDirectory( CMonitorInformation& moniInfo )
{
	CGDTrace trace( LOG_L6, "CProtect:WatchDirectory(%s)", moniInfo.m_monitorPath.c_str() );
	
	// жǷѾ
	map< string, CDirInfo* >::iterator it = m_dirmap.find( moniInfo.m_sDirId );
	if ( it == m_dirmap.end() )
	{
		CDirInfo* pDirInfo = new CDirInfo( moniInfo );
		strcpy( pDirInfo->m_szDirId, moniInfo.m_sDirId.c_str() );			//ӼĿ¼ID
		if ( pDirInfo )
		{
			if ( pDirInfo->StartMonitor(moniInfo) )
			{
				m_dirmap.insert( make_pair( moniInfo.m_sDirId, pDirInfo ) );
				return trace.SetRetResult( true );
			}
		}
	}
	else
	{
		// Ŀ¼
		CDirInfo* pDirInfo = it->second;
		return (pDirInfo->refreshExcludeRules( moniInfo ) == 0) ? true : false;
	}
	
	return trace.SetRetResult( false );
}

bool CProtect::UnwatchDirectory( const char * sDirId )
{
	CGDTrace trace( LOG_L6, "CProtect:UnwatchDirectory(%s)", sDirId );

	CDirInfo* pDirInfo = NULL;
	map < string, CDirInfo* >::iterator iter = m_dirmap.find( string(sDirId) );
	
	if ( iter != m_dirmap.end() )
	{
		pDirInfo = (CDirInfo*) iter->second;
		if ( pDirInfo->StopMonitor() )
		{
			m_dirmap.erase( iter );
			delete pDirInfo;
			pDirInfo = NULL;
			return trace.SetRetResult( true );
		}
	}
	
	return trace.SetRetResult( false );
}

bool CProtect::Uninitialize()
{
	// ɺtrace
	CGDTrace trace( LOG_L6, "CProtect:Uninitialize()" );
	
	// ֪ͨ¼߳˳
	m_bWorking = false;
	
	CDirInfo* pDirInfo = NULL;
	map < string, CDirInfo* >::iterator iter;
	for ( iter = m_dirmap.begin(); iter != m_dirmap.end(); ++iter )  
	{
		pDirInfo = (CDirInfo*) ( (*iter).second );
		if ( pDirInfo )
		{
			delete pDirInfo;
			pDirInfo = NULL;
		}
	}
	m_dirmap.clear();
	
	return trace.SetRetResult( true );
}

bool CProtect::IsWatchingDirectory( const char * szDir )
{
	CGDTrace trace( LOG_L6, "CProtect:IsWatchingDirectory(%s)", szDir );
	
	bool bRet = false;
	
	CDirInfo* pDirInfo = NULL;
	map < string, CDirInfo* >::iterator iter = m_dirmap.begin();
	while( iter != m_dirmap.end() )
	{
		pDirInfo = (CDirInfo*)iter->second;
		trace.GDTrace( LOG_L6, "IsWatchingDirectory([%s<-->%s][%s]", szDir, pDirInfo->m_szDir, pDirInfo->m_szDirId );
		if( strncmp(szDir, pDirInfo->m_szDir, strlen(pDirInfo->m_szDir)) == 0 )
		{
			bRet = true;
			break;
		}
		iter++;
	}
	
	return trace.SetRetResult(bRet);
}

bool CProtect::IsWatched( const char* sDirId )
{
    CGDTrace trace( LOG_L6, "CProtect:IsWatched(%s)", sDirId );
	
    // жǷѾ
    if ( sDirId[0] == '/' || sDirId[1] == ':' )
    {
        return trace.SetRetResult( IsWatchingDirectory( sDirId ) ); // ͨĿ¼жǷ(MA˲֧ظ)
    }
    std::map<std::string, CDirInfo*>::iterator iter = m_dirmap.find( sDirId );
    if ( iter == m_dirmap.end() ) // ûнĿ¼ض
    {
        return trace.SetRetResult( false );
    }
    else
    {
        return trace.SetRetResult( true );
    }
}

bool CProtect::IsWatchedByWebsiteId( const char * sWebsiteId )
{
    CGDTrace trace( LOG_L6, "CFileDiffer:IsWatchedByWebsiteId(%s)", sWebsiteId );
	
    // жǷѾ
    bool bRet = false;
	
    CDirInfo* pDirInfo = NULL;
    map < string, CDirInfo* >::iterator iter = m_dirmap.begin();
    while( iter != m_dirmap.end() )
    {
        pDirInfo = (CDirInfo*)iter->second;
        if( strcmp( sWebsiteId, pDirInfo->m_szDirId ) == 0 )
        {
            bRet = true;
            break;
        }
        iter++;
    }
    if ( !bRet ) // ûнĿ¼ض
    {
        return trace.SetRetResult( false );
    }
    else
    {
        return trace.SetRetResult( bRet );
    }
}

