

#ifndef QMP_PORT_H_
#define QMP_PORT_H_

#if !defined(__SPICE_CLIENT_H_INSIDE__) && !defined(SPICE_COMPILATION)
#warning "Only <spice-client.h> can be included directly"
#endif

#include <glib-object.h>
#include "channel-port.h"

G_BEGIN_DECLS

#define SPICE_TYPE_QMP_PORT            (spice_qmp_port_get_type ())
#define SPICE_QMP_PORT(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SPICE_TYPE_QMP_PORT, SpiceQmpPort))
#define SPICE_QMP_PORT_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), SPICE_TYPE_QMP_PORT, SpiceQmpPortClass))
#define SPICE_IS_QMP_PORT(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SPICE_TYPE_QMP_PORT))
#define SPICE_IS_QMP_PORT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SPICE_TYPE_QMP_PORT))
#define SPICE_QMP_PORT_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), SPICE_TYPE_QMP_PORT, SpiceQmpPortClass))


typedef struct _SpiceQmpPort SpiceQmpPort;
typedef struct _SpiceQmpPortClass SpiceQmpPortClass;


typedef enum SpiceQmpPortVmAction {
    SPICE_QMP_PORT_VM_ACTION_QUIT,
    SPICE_QMP_PORT_VM_ACTION_RESET,
    SPICE_QMP_PORT_VM_ACTION_POWER_DOWN,
    SPICE_QMP_PORT_VM_ACTION_PAUSE,
    SPICE_QMP_PORT_VM_ACTION_CONTINUE,

    SPICE_QMP_PORT_VM_ACTION_LAST,
} SpiceQmpPortVmAction;


typedef struct _SpiceQmpStatus {
    
    gint ref;

    
    gint version;

    gboolean running;
    gboolean singlestep;
    gchar *status;
} SpiceQmpStatus;

GType spice_qmp_port_get_type(void);

SpiceQmpPort *spice_qmp_port_get(SpicePortChannel *channel);

void spice_qmp_port_vm_action_async(SpiceQmpPort *self,
                                    SpiceQmpPortVmAction action,
                                    GCancellable *cancellable,
                                    GAsyncReadyCallback callback,
                                    gpointer user_data);

gboolean spice_qmp_port_vm_action_finish(SpiceQmpPort *self,
                                         GAsyncResult *result,
                                         GError **error);

GType spice_qmp_status_get_type(void);

SpiceQmpStatus *spice_qmp_status_ref(SpiceQmpStatus *status);
void spice_qmp_status_unref(SpiceQmpStatus *status);

void spice_qmp_port_query_status_async(SpiceQmpPort *self,
                                       GCancellable *cancellable,
                                       GAsyncReadyCallback callback,
                                       gpointer user_data);

SpiceQmpStatus *spice_qmp_port_query_status_finish(SpiceQmpPort *self,
                                                   GAsyncResult *result,
                                                   GError **error);

G_END_DECLS

#endif 

