/***********************************************************************
*
* ļ        Validator.h
* ģĿģִļ֤
* ߼ڣ  wang_jin 2008-05-11
* 汾          1.0
* ȨϢ      CVICSE Middleware
* ļ¼  
*
***********************************************************************/

#ifndef Validator_H
#define Validator_H


#include "Common/GDString.h"
#include "Common/DateTime.h"
#include "FileOpt/FileOpt.h"
#include "GDChecker/GDCheckerDef.h"
#include "GDLock/GDLock.h"
#include "GDQue/MessageQueue.h"
#include "GDQue/SharedBuffer.h"
#include "PcreRex/PcreRex.h"

#include <list>
#include <string>
#include <vector>
#include <ctime>

#ifndef WIN32
#	include <sys/stat.h>
#endif


#ifndef NOT_USE_STD_LIB
using namespace std;
#endif

const string SQLRULES_NAME = "sqlrules.xml";

class CSQLValidator
{
private:
	typedef struct SQLRules
	{
		vector<PcreRex*> exUrls;
		vector<PcreRex*> inRules;
		vector<PcreRex*> exRules;
	}SQLRULES;

public:
	CSQLValidator();
	~CSQLValidator();

public:
	bool Initialize();
	bool UnInitialize();
	bool Check( char cType, const char* sHost, const char* method, const char* url, char* pQueryString );
	const char* GetErrCode() { return m_nErrCode.c_str(); }
	const char* GetErrPage() { return m_sErrPage.c_str(); }
	const char* GetErrMessage() { return m_sErrMessage.c_str(); }

private:
	bool LoadErrCode( const char* sCfgFile );
	void ReleaseRules( SQLRules& rules );
	bool ReadRules( const char* name, SQLRules& rules );
	bool IsRulesChange();
	time_t GetLastTime();

private:
	bool m_bInitialized;
	CThreadMutex m_mutex;
	CMessageQueue m_msgQue;
	SQLRules		m_sqlRules;
	string		m_SqlRulePath;
	time_t		m_sqlRuleLastTime;
	time_t		m_sqlReadCycle;
	bool		m_bUseCheck;

	string m_nErrCode;
	string m_sErrMessage;
	string m_sErrPage;

};


inline time_t CSQLValidator::GetLastTime()
{
#ifdef WIN32
	WIN32_FILE_ATTRIBUTE_DATA fad;
	if ( GetFileAttributesEx(( m_SqlRulePath + S_PATHLASH + SQLRULES_NAME ).c_str(), GetFileExInfoStandard, &fad) == 0 )
	{
		return 0;
	}
	else
	{
		time_t	timeTemp;
		CDateTime::FileTimeToUnixTime( &(fad.ftLastWriteTime), timeTemp );
		return timeTemp;
	}
#else
	struct stat st;
	if ( stat(( m_SqlRulePath + S_PATHLASH + SQLRULES_NAME ).c_str(), &st) == 0 )
	{
		return st.st_mtime;
	}
	else
	{
		return 0;
	}
#endif
}


#endif


