// IISSQLFILTER6.CPP - Implementation file for your Internet Server
//    IISSqlFilter

#include "stdafx.h"
#include "IISSqlFilter6.h"
#include "IISSqlCommon.h"

#define DEFAULT_TMPLOG_FILENAME "gd-iis-sql.log"
#define TIME_LENGTH 32

bool bLog = true;

void TmpLog( bool bForce, const char* fmt, ... )
{
	if ( bLog || bForce )
	{
		char tmplog_path[ MAX_PATH ] = {0};
		FILE *tmp_logfile = NULL;
		
#ifndef _WIN32
		sprintf( tmplog_path, "/tmp/%s", DEFAULT_TMPLOG_FILENAME );
		tmp_logfile = fopen(tmplog_path, "a+");
#else
		sprintf( tmplog_path, "C:\\tmp\\%s", DEFAULT_TMPLOG_FILENAME );
		tmp_logfile = fopen( tmplog_path, "a+" );
#endif
		
		if( tmp_logfile )
		{
			char time_string[ TIME_LENGTH ] = {0};
			time_t nowtime = time( NULL );
			
			va_list args;
			va_start( args, fmt );
			
			// Format is month day hour:minute:second (24 time) 
			strftime( time_string, TIME_LENGTH, "%b %d %H:%M:%S", localtime(&nowtime) );
			
#if defined _WIN32
			fprintf( tmp_logfile, "%s [%u] - ", time_string, GetCurrentThreadId() );
#else
			fprintf( tmp_logfile, "%s [%u] - ", time_string, pthread_self() );
#endif
			
			vfprintf( tmp_logfile, fmt, args );
			va_end(args);
			
			fprintf( tmp_logfile, "\n" );
			fclose(tmp_logfile);
			tmp_logfile = NULL;
		}
	}
}

///////////////////////////////////////////////////////////////////////
// The one and only CWinApp object
// NOTE: You may remove this object if you alter your project to no
// longer use MFC in a DLL.

CWinApp theApp;



///////////////////////////////////////////////////////////////////////
// The one and only CIISSqlFilter object

CIISSqlFilter theFilter;


///////////////////////////////////////////////////////////////////////
// CIISSqlFilter implementation

CIISSqlFilter::CIISSqlFilter()
{
	m_nCount = 9;

	TmpLog( true, "IISע" );
}

CIISSqlFilter::~CIISSqlFilter()
{
	TmpLog( true, "IISעر" );
}

BOOL CIISSqlFilter::GetFilterVersion(PHTTP_FILTER_VERSION pVer)
{
	// Call default implementation for initialization
	CHttpFilter::GetFilterVersion(pVer);

	// Clear the flags set by base class
	pVer->dwFlags &= ~SF_NOTIFY_ORDER_MASK;

	//liu_yfeng 2009-03-11
	pVer->dwFlags |= SF_NOTIFY_ORDER_LOW | SF_NOTIFY_SECURE_PORT | SF_NOTIFY_NONSECURE_PORT
			 | SF_NOTIFY_PREPROC_HEADERS | SF_NOTIFY_READ_RAW_DATA | SF_NOTIFY_URL_MAP;

	// Load description string
	TCHAR sz[SF_MAX_FILTER_DESC_LEN+1];
	ISAPIVERIFY(::LoadString(AfxGetResourceHandle(),
			IDS_FILTER, sz, SF_MAX_FILTER_DESC_LEN));
	_tcscpy(pVer->lpszFilterDesc, sz);
	return TRUE;
}

// SQL
BOOL CIISSqlFilter::InforGuardSqlCheck( CHttpFilterContext* pCtxt, const char* method, char* szQueryString, const char* url )
{
	// ȡѯϢǳؼ
	// ڷǳؼ϶ڲͼֹ
	BOOL bRet = TRUE;
	
	// ύ
	char sHost[128] = {0};
	char sAddr[32] = {0};
	DWORD dwLen = sizeof(sHost);
	pCtxt->GetServerVariable( "REMOTE_HOST", sHost, &dwLen );
	
	dwLen = sizeof(sAddr);
	pCtxt->GetServerVariable( "REMOTE_ADDR", sAddr, &dwLen );
	if ( !m_checker.Check( 'I', sHost, method, url, szQueryString ) )
	{
		// ֹ 
		// ¼ֹ־
		TmpLog( false, "NO:[%s][%s][%s][%s]", sHost, method, url, szQueryString );
		bRet = FALSE;
	}
	else
	{
		TmpLog( false, "OK:[%s][%s][%s][%s]", sHost, method, url, szQueryString );
	}

	return bRet;
}

DWORD CIISSqlFilter::OnUrlMap(CHttpFilterContext* pCtxt,
	PHTTP_FILTER_URL_MAP pMapInfo)
{
	try
	{
		if(pCtxt!=NULL && pMapInfo!=NULL)
		{
			if ( m_nCount > 0 )
			{
				m_nCount --;
				if ( m_nCount == 0 )
				{
					bLog = false;
				}
			}
			
			// ȡύʽ
			DWORD nErr = 0;
			char method[32] = "UNKNOWN";
			DWORD nMethodLength = sizeof(method);
			if ( ! pCtxt->GetServerVariable( "REQUEST_METHOD", method, &nMethodLength ) )
			{
				nErr = GetLastError();
				// ¼ʧ־
				TmpLog( false, "[%s]GetServerVariable(REQUEST_METHOD) failed. [%d][%s]", pMapInfo->pszURL, nErr, strerror(nErr) );
			}

			BOOL bOK = FALSE;
			
			// ȡύ
			char szQueryString[512] = {0};
			DWORD nQueryStringLength = 0;
			int get = pCtxt->GetServerVariable( "QUERY_STRING", szQueryString, &nQueryStringLength );
			if ( ! pCtxt->GetServerVariable( "QUERY_STRING", szQueryString, &nQueryStringLength ) )
			{
				nErr = GetLastError();
				if ( ERROR_INSUFFICIENT_BUFFER == nErr ) // ̫С
				{
					char* pQueryString = new char[ nQueryStringLength + 1 ];
					// »ȡύ
					if ( ! pCtxt->GetServerVariable( "QUERY_STRING", pQueryString, &nQueryStringLength ) )
					{
						nErr = GetLastError();
						// ¼ʧ־
						TmpLog( false, "[%s][%s]GetServerVariable(QUERY_STRING) fail [%d][%s]", pMapInfo->pszURL, method, nErr, strerror(nErr) );
					}
					else
					{
						bOK = InforGuardSqlCheck( pCtxt, method, szQueryString, pMapInfo->pszPhysicalPath );
					}
					delete []pQueryString;
				}
				else
				{
					// ¼ʧ־
					TmpLog( false, "[%s][%s]GetServerVariable(QUERY_STRING) err [%d][%s]", pMapInfo->pszURL, method, nErr, strerror(nErr) );
				}
			}
			else
			{
				// ύ
				bOK = InforGuardSqlCheck( pCtxt, method, szQueryString, pMapInfo->pszPhysicalPath );
			}	

			if ( ! bOK )
			{
				return SF_STATUS_REQ_FINISHED_KEEP_CONN;
			}
			//>>**** InforGuard Code ****
		}
	}
	catch ( ... )
	{
		TmpLog( true, "OnUrlMap(%s) exception(...)", pMapInfo->pszPhysicalPath );
	}

	return CHttpFilter::OnUrlMap(pCtxt, pMapInfo);
}

DWORD CIISSqlFilter::OnReadRawData(CHttpFilterContext* pCtxt, PHTTP_FILTER_RAW_DATA pRawData)
{
	try
	{
		if(pCtxt!=NULL && pRawData!=NULL)
		{
			if ( m_nCount > 0 )
			{
				m_nCount --;
				if ( m_nCount == 0 )
				{
					bLog = false;
				}
			}

			char method[32] = "UNKNOWN";
			DWORD nMethodLength = sizeof(method);
			if ( ! pCtxt->GetServerVariable( "REQUEST_METHOD", method, &nMethodLength ) )
			{
				DWORD nErr = GetLastError();
				// ¼ʧ־
				TmpLog( false, "[%s]GetServerVariable(REQUEST_METHOD) failed. [%d][%s]", "OnReadRawData", nErr, strerror(nErr) );
			}

			CHAR szServerName[256] = {0};
			ULONG nSize = sizeof(szServerName);
			pCtxt->GetServerVariable( "SERVER_NAME", szServerName, &nSize );

			// λPOSTʼλ
			CString Raw = CStringHelper::ConvertBinaryData((BYTE*)pRawData->pvInData,pRawData->cbInData,' ');

			CString strh;				//for headers
			CString strd;				//for post data

			PrepareRawData(pCtxt,Raw,strh,strd);

			BOOL bOK = TRUE;
			bOK = InforGuardSqlCheck(pCtxt, method, strh.GetBuffer(0), szServerName);
			if ( bOK )
			{
				bOK = InforGuardSqlCheck(pCtxt, method, strd.GetBuffer(0), szServerName);
			}

			if ( !bOK )
			{
				return SF_STATUS_REQ_FINISHED_KEEP_CONN;
			}
		}
	}
	catch ( ... )
	{
		TmpLog( true, "OnReadRawData(%s) exception(...)", pRawData->pvInData );
	}
	//>>**** InforGuard Code ****

	return SF_STATUS_REQ_NEXT_NOTIFICATION;
}

//liu_yfeng 2009-03-11
#define CR '\r'
#define LF '\n'
#define CRLF "\r\n"
#define CRLFCRLF "\r\n\r\n"
#define CONTEXT_NEWREQUEST			NULL
#define CONTEXT_HEADERSRECEIVED		(VOID *)1
void CIISSqlFilter::PrepareRawData(CHttpFilterContext* pCtxt,CString& RawData, CString& Headers, CString& Data)
{
	if(pCtxt->m_pFC->pFilterContext == CONTEXT_NEWREQUEST){	//new request / not all headers received?

		int maxheader = RawData.Find(CRLFCRLF);	//maximum size header
	
		if(maxheader!=-1){						//if end of headers are found (if not, raw data is not complete!)
			Headers = RawData.Left(maxheader);			//only headers
			//ignore the request line (!= header)
			int starth;
			starth = Headers.Find(CRLF);
			if (starth==-1){
				Headers = "";						//no headers (only request line)!
			}else{
				Headers = Headers.Right(Headers.GetLength()-starth-2);
			}
			Data = RawData.Right(RawData.GetLength()-maxheader-4);//only look at the body
			//mark request (all headers received)
			pCtxt->m_pFC->pFilterContext = CONTEXT_HEADERSRECEIVED;
		}else{
			//logger.Append("WARNING: Could not determine end of headers, will process all data as headers");
			Headers = RawData;
			//if double crlf was between events, process this as data as well
			if(RawData.Left(1)==CR || RawData.Left(1)==LF){
				//logger.Append("WARNING: Raw data starts with <CR> or <LF>, so this is probably data part and not headers. Will process it as data as well!");
				Data = RawData;
			}
		}

	}else{	//is additional data
		Data = RawData;
	}	
}

//liu_yfeng 2009-03-11
#define MAX_HEADER_SIZE	96000
#define MIN_HEADER_SIZE 4128
CString CIISSqlFilter::GetHeader(CHttpFilterContext* pCtxt, PHTTP_FILTER_PREPROC_HEADERS pHeaders, char* h)
{
	DWORD bufsize = MIN_HEADER_SIZE;
	char buf[MIN_HEADER_SIZE]="";
    
	//get header value with MIN_HEADER_SIZE (average minimum header size)
	if(pHeaders->GetHeader(pCtxt->m_pFC,h,buf,&bufsize)){
		for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
			if(buf[i]=='\0') buf[i]=' ';
		}
		buf[bufsize-1]='\0';			//it should be null terminated already, but make sure it is. You never know!
		return CString(buf);
	}else{
		switch(GetLastError()){
			case ERROR_INSUFFICIENT_BUFFER:{	//if the buffer was too small, try again with value set by web server in bufsize
				CString tmp;				
				char* buf2 = new char[bufsize];
				if(buf2==NULL){
					tmp = "Error retrieving header";
				}else{
					if(pHeaders->GetHeader(pCtxt->m_pFC,h,buf2,&bufsize)){
						for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
							if(buf2[i]=='\0') buf2[i]=' ';
						}
						buf2[bufsize-1]='\0';	//it should be null terminated already, but make sure it is. You never know!
						tmp=buf2;
					}else{
						tmp="Error retrieving header"; //some error occured
					}
					delete[] buf2;
				}
				return tmp;
				break;
			}
			case ERROR_MORE_DATA: //buffer was too small, but no size set in bufsize, try again with MAX_HEADER_SIZE
				bufsize = MAX_HEADER_SIZE;
				char buf2[MAX_HEADER_SIZE];
				if(pHeaders->GetHeader(pCtxt->m_pFC,h,buf2,&bufsize)){
					for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
						if(buf2[i]=='\0') buf2[i]=' ';
					}
					buf2[bufsize-1]='\0';	//it should be null terminated already, but make sure it is. You never know!
					return CString(buf2);
				}else{
					return "Error retrieving header"; //some error occured
				}
				break;
			default:
				return ""; //header is not present or some other error occured
				break;
		}
	}
}

//liu_yfeng 2009-03-11
CString CIISSqlFilter::GetServerVariable(CHttpFilterContext* pCtxt,char* sv)
{
	DWORD bufsize = MIN_HEADER_SIZE;
	char buf[MIN_HEADER_SIZE]="";
    
	//get header value with MIN_HEADER_SIZE (average minimum header size)
	if(pCtxt->GetServerVariable(sv,buf,&bufsize)){
		for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
			if(buf[i]=='\0') buf[i]=' ';
		}
		buf[bufsize-1]='\0';			//it should be null terminated already, but make sure it is. You never know!
		return CString(buf);
	}else{
		switch(GetLastError()){
			case ERROR_INSUFFICIENT_BUFFER:{	//if the buffer was too small, try again with value set by web server in bufsize
				CString tmp;				
				char* buf2 = new char[bufsize];
				if(buf2==NULL){
					tmp = "Error retrieving server variable";
				}else{
					if(pCtxt->GetServerVariable(sv,buf2,&bufsize)){
						for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
							if(buf2[i]=='\0') buf2[i]=' ';
						}
						buf2[bufsize-1]='\0';	//it should be null terminated already, but make sure it is. You never know!
						tmp=buf2;
					}else{
						tmp="Error retrieving server variable"; //some error occured
					}
					delete[] buf2;
				}
				return tmp;
				break;
			}
			case ERROR_MORE_DATA: //buffer was too small, but no size set in bufsize, try again with MAX_HEADER_SIZE
				bufsize = MAX_HEADER_SIZE;
				char buf2[MAX_HEADER_SIZE];
				if(pCtxt->GetServerVariable(sv,buf2,&bufsize)){
					for(DWORD i=0;i<bufsize-1;i++){	//replace all embedded nulls with space
						if(buf2[i]=='\0') buf2[i]=' ';
					}
					buf2[bufsize-1]='\0';	//it should be null terminated already, but make sure it is. You never know!
					return CString(buf2);
				}else{
					return "Error retrieving server variable"; //some error occured
				}
				break;
			default:
				return ""; //variable is not present or some other error occured
				break;
		}
	}
}

//liu_yfeng 2009-03-11
DWORD CIISSqlFilter::OnPreprocHeaders(CHttpFilterContext* pCtxt, PHTTP_FILTER_PREPROC_HEADERS pHeaders) 
{
	try
	{
		if(pCtxt!=NULL && pHeaders!=NULL)
		{
			if ( m_nCount > 0 )
			{
				m_nCount --;
				if ( m_nCount == 0 )
				{
					bLog = false;
				}
			}

			CHAR szServerName[256] = {0};
			ULONG nSize = sizeof(szServerName);
			pCtxt->GetServerVariable( "SERVER_NAME", szServerName, &nSize );

			BOOL bOK = TRUE;
			CURL RawUrl(GetHeader(pCtxt,pHeaders,"url"));
			bOK = InforGuardSqlCheck(pCtxt, "URL", RawUrl.Querystring.GetBuffer(0), szServerName);

			if ( bOK )
			{
				CString cookie = GetHeader(pCtxt,pHeaders,"Cookie:");
				bOK = InforGuardSqlCheck(pCtxt, "COOKIE", cookie.GetBuffer(0), szServerName);
			}

			if ( bOK )
			{
				CString allRaw = GetServerVariable(pCtxt, "ALL_RAW");
				bOK = InforGuardSqlCheck(pCtxt, "ALLRAW", allRaw.GetBuffer(0), szServerName);
			}
			if ( bOK )
			{
				CString referer = GetHeader(pCtxt,pHeaders,"Referer:");
				CURL RefUrl(referer);
				CString URLDecoded = CURL::URLEncodedToASCII(RefUrl.Url,true);
				bOK = InforGuardSqlCheck(pCtxt, "REFERER", URLDecoded.GetBuffer(0), szServerName);
			}
			if ( !bOK )
			{
				return SF_STATUS_REQ_FINISHED_KEEP_CONN;
			}
		}
	}
	catch ( ... )
	{
		TmpLog( true, "OnPreprocHeaders() exception(...)" );
	}
	return CHttpFilter::OnPreprocHeaders(pCtxt, pHeaders);
}


// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CIISSqlFilter, CHttpFilter)
	//{{AFX_MSG_MAP(CIISSqlFilter)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

///////////////////////////////////////////////////////////////////////
// If your extension will not use MFC, you'll need this code to make
// sure the extension objects can find the resource handle for the
// module.  If you convert your extension to not be dependent on MFC,
// remove the comments arounn the following AfxGetResourceHandle()
// and DllMain() functions, as well as the g_hInstance global.

/****

static HINSTANCE g_hInstance;

HINSTANCE AFXISAPI AfxGetResourceHandle()
{
	return g_hInstance;
}

BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ulReason,
					LPVOID lpReserved)
{
	if (ulReason == DLL_PROCESS_ATTACH)
	{
		g_hInstance = hInst;
	}

	return TRUE;
}

****/
