//
// NumberFormatter.cpp
//


#include "Groundwork/NumberFormatter.h"
#include <cstdio>
#include <cctype>


#if defined(_MSC_VER)
	#define I64_FMT "I64"
#elif defined(__APPLE__) 
	#define I64_FMT "q"
#else
	#define I64_FMT "ll"
#endif


namespace InforGuard {
namespace Groundwork {


string NumberFormatter::format(int value)
{
	char buffer[64] = {0};
	sprintf(buffer, "%d", value);
	return string(buffer);
}


string NumberFormatter::format(int value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%*d", width, value);
	return string(buffer);
}


string NumberFormatter::format0(int value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%0*d", width, value);
	return string(buffer);
}


string NumberFormatter::formatHex(int value)
{
	char buffer[64] = {0};
	sprintf(buffer, "%X", value);
	return string(buffer);
}


string NumberFormatter::formatHex(int value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%0*X", width, value);
	return string(buffer);
}


string NumberFormatter::format(unsigned value)
{
	char buffer[64] = {0};
	sprintf(buffer, "%u", value);
	return string(buffer);
}


string NumberFormatter::format(unsigned value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%*u", width, value);
	return string(buffer);
}


string NumberFormatter::format0(unsigned int value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%0*u", width, value);
	return string(buffer);
}


string NumberFormatter::formatHex(unsigned value)
{
	char buffer[64] = {0};
	sprintf(buffer, "%X", value);
	return string(buffer);
}


string NumberFormatter::formatHex(unsigned value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%0*X", width, value);
	return string(buffer);
}


string NumberFormatter::format(long value)
{
	char buffer[64] = {0};
	sprintf(buffer, "%ld", value);
	return string(buffer);
}


string NumberFormatter::format(long value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%*ld", width, value);
	return string(buffer);
}


string NumberFormatter::format0(long value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%0*ld", width, value);
	return string(buffer);
}


string NumberFormatter::formatHex(long value)
{
	char buffer[64] = {0};
	sprintf(buffer, "%lX", value);
	return string(buffer);
}


string NumberFormatter::formatHex(long value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%0*lX", width, value);
	return string(buffer);
}


string NumberFormatter::format(unsigned long value)
{
	char buffer[64] = {0};
	sprintf(buffer, "%lu", value);
	return string(buffer);
}


string NumberFormatter::format(unsigned long value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%*lu", width, value);
	return string(buffer);
}


string NumberFormatter::format0(unsigned long value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%0*lu", width, value);
	return string(buffer);
}


string NumberFormatter::formatHex(unsigned long value)
{
	char buffer[64] = {0};
	sprintf(buffer, "%lX", value);
	return string(buffer);
}


string NumberFormatter::formatHex(unsigned long value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%0*lX", width, value);
	return string(buffer);
}


#if defined(GROUNDWORK_HAVE_INT64) && !defined(GROUNDWORK_LONG_IS_64_BIT)


string NumberFormatter::format(Int64 value)
{
	char buffer[64] = {0};
	sprintf(buffer, "%"I64_FMT"d", value);
	return string(buffer);
}


string NumberFormatter::format(Int64 value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%*"I64_FMT"d", width, value);
	return string(buffer);
}


string NumberFormatter::format0(Int64 value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%0*"I64_FMT"d", width, value);
	return string(buffer);
}


string NumberFormatter::formatHex(Int64 value)
{
	char buffer[64] = {0};
	sprintf(buffer, "%"I64_FMT"X", value);
	return string(buffer);
}


string NumberFormatter::formatHex(Int64 value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%0*"I64_FMT"X", width, value);
	return string(buffer);
}


string NumberFormatter::format(UInt64 value)
{
	char buffer[64] = {0};
	sprintf(buffer, "%"I64_FMT"u", value);
	return string(buffer);
}


string NumberFormatter::format(UInt64 value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%*"I64_FMT"u", width, value);
	return string(buffer);
}


string NumberFormatter::format0(UInt64 value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%0*"I64_FMT"u", width, value);
	return string(buffer);
}


string NumberFormatter::formatHex(UInt64 value)
{
	char buffer[64] = {0};
	sprintf(buffer, "%"I64_FMT"X", value);
	return string(buffer);
}


string NumberFormatter::formatHex(UInt64 value, int width)
{
	groundwork_assert (width > 0 && width < 64);

	char buffer[64] = {0};
	sprintf(buffer, "%0*"I64_FMT"X", width, value);
	return string(buffer);
}


#endif // defined(GROUNDWORK_HAVE_INT64) && !defined(GROUNDWORK_LONG_IS_64_BIT)


string NumberFormatter::format(float value)
{
	char buffer[64] = {0};
	sprintf(buffer, "%.*g", 10, (double) value);
	return string(buffer);
}


string NumberFormatter::format(double value)
{
	char buffer[64] = {0};
	sprintf(buffer, "%.*g", 20, value);
	return string(buffer);
}


string NumberFormatter::format(double value, int precision)
{
	groundwork_assert (precision >= 0 && precision < 32);

	char buffer[64] = {0};
	sprintf(buffer, "%.*f", precision, value);
	return string(buffer);
}


string NumberFormatter::format(double value, int width, int precision)
{
	groundwork_assert (width > 0 && width < 64 && precision >= 0 && precision < width);

	char buffer[64] = {0};
	sprintf(buffer, "%*.*f", width, precision, value);
	return string(buffer);
}


string NumberFormatter::format(const void* ptr)
{
	char buffer[24] = {0};
#if defined(GROUNDWORK_PTR_IS_64_BIT)
	sprintf(buffer, "%016"I64_FMT"X", (UIntPtr) ptr);
#else
	sprintf(buffer, "%08lX", (UIntPtr) ptr);
#endif
	return string(buffer);
}


} // namespace Groundwork
}	// namespace InforGuard
