//
// UTF8Encoding.cpp
//


#include "Groundwork/UTF8Encoding.h"
#ifdef GROUNDWORK_OS_FAMILY_WINDOWS
#include <windows.h>
#endif 


namespace InforGuard {
namespace Groundwork {


UTF8Encoding::UTF8Encoding()
{
	_len = 0;
	_utf8 = NULL;
}


UTF8Encoding::~UTF8Encoding()
{
	if ( NULL != _utf8 )
	{
		delete _utf8;
		_utf8 = NULL;
	}
}


int UTF8Encoding::setText( const wchar_t* szSrc )
{
	_len = wcslen(szSrc);
	if ( NULL != _utf8 )
	{
		delete _utf8;
		_utf8 = NULL;
	}
	_utf8 = new char[_len];
	convert( szSrc, _utf8 );
	return _len;
}


int UTF8Encoding::getLength()
{
	return _len;
}


int UTF8Encoding::getText( char* szDst )
{
	if ( NULL == szDst )
	{
		return 0;
	}
	int len = strlen( szDst );
	int minLen = len < _len ? len : _len;
	strncpy( szDst, _utf8, minLen );
	return minLen;
}


int UTF8Encoding::convert( const wchar_t* szSrc, char* szDst )
{
#ifdef GROUNDWORK_OS_FAMILY_WINDOWS
	WORD ch;
	
	BYTE bt1, bt2, bt3, bt4, bt5, bt6;
	
	int n, nMax = wcslen(szSrc);
	
	char sTemp[260] = {0};
	int nPos = 0;
	
	for (n = 0; n < nMax; ++n)
	{
		ch = (WORD)szSrc[n];
		
		if (ch == '=')
		{
			szDst[nPos++] = char( ch );
		}
		else if (ch < 128)
		{
			szDst[nPos++] = char( szSrc[n] );
		}
		else if (ch <= 2047)
		{
			bt1 = (BYTE)(192 + (ch / 64));
			bt2 = (BYTE)(128 + (ch % 64));
			
			szDst[nPos++] = bt1;
			szDst[nPos++] = bt2;
		}
		else if (ch <= 65535)
		{
			bt1 = (BYTE)(224 + (ch / 4096));
			bt2 = (BYTE)(128 + ((ch / 64) % 64));
			bt3 = (BYTE)(128 + (ch % 64));
			
			szDst[nPos++] = bt1;
			szDst[nPos++] = bt2;
			szDst[nPos++] = bt3;
		}
		else if (ch <= 2097151)
		{
			bt1 = (BYTE)(240 + (ch / 262144));
			bt2 = (BYTE)(128 + ((ch / 4096) % 64));
			bt3 = (BYTE)(128 + ((ch / 64) % 64));
			bt4 = (BYTE)(128 + (ch % 64));
			
			szDst[nPos++] = bt1;
			szDst[nPos++] = bt2;
			szDst[nPos++] = bt3;
			szDst[nPos++] = bt4;
		}
		else if (ch <=67108863)
		{
			bt1 = (BYTE)(248 + (ch / 16777216));
			bt2 = (BYTE)(128 + ((ch / 262144) % 64));
			bt3 = (BYTE)(128 + ((ch / 4096) % 64));
			bt4 = (BYTE)(128 + ((ch / 64) % 64));
			bt5 = (BYTE)(128 + (ch % 64));
			
			szDst[nPos++] = bt1;
			szDst[nPos++] = bt2;
			szDst[nPos++] = bt3;
			szDst[nPos++] = bt4;
			szDst[nPos++] = bt5;
		}
		else if (ch <=2147483647)
		{
			bt1 = (BYTE)(252 + (ch / 1073741824));
			bt2 = (BYTE)(128 + ((ch / 16777216) % 64));
			bt3 = (BYTE)(128 + ((ch / 262144) % 64));
			bt4 = (BYTE)(128 + ((ch / 4096) % 64));
			bt5 = (BYTE)(128 + ((ch / 64) % 64));
			bt6 = (BYTE)(128 + (ch % 64));
			
			szDst[nPos++] = bt1;
			szDst[nPos++] = bt2;
			szDst[nPos++] = bt3;
			szDst[nPos++] = bt4;
			szDst[nPos++] = bt5;
			szDst[nPos++] = bt6;
		}
		
	}
	return nMax;
#endif
	return 0;
}


} // namespace Groundwork
}	// namespace InforGuard
