
#include "WmDb.h"
#include <afxwin.h>
#include <afxcmn.h>

#include <db_cxx.h>
#include <string>
#include "WaterMark/WmdbType.h"

using namespace std;

#pragma comment( lib, "libdb44" )

struct RECORD_S 
{
	char sWm[WMARK_LEN];
};


CWmDb::CWmDb()
: m_pDb(new Db(0, 0))
, m_pListCtrl(NULL)
, m_bIsOpen(false)
, m_nId(0)
{
}

CWmDb::~CWmDb()
{
	Close();
	delete m_pDb;
}

int CWmDb::EncodeToUTF8(const wchar_t* szSrc, char* szDst)
{
#ifdef WIN32
	WORD ch;
	
	BYTE bt1, bt2, bt3, bt4, bt5, bt6;
	
	int n, nMax = wcslen(szSrc);
	
	char sTemp[GD_MAX_PATH] = {0};
	int nPos = 0;
	
	for (n = 0; n < nMax; n++)
	{
		ch = (WORD)szSrc[n];
		
		if (ch == '=')
		{
			szDst[nPos++] = ch;
		}
		else if (ch < 128)
		{
			szDst[nPos++] = szSrc[n];
		}
		else if (ch <= 2047)
		{
			bt1 = (BYTE)(192 + (ch / 64));
			bt2 = (BYTE)(128 + (ch % 64));
			
			szDst[nPos++] = bt1;
			szDst[nPos++] = bt2;
		}
		else if (ch <= 65535)
		{
			bt1 = (BYTE)(224 + (ch / 4096));
			bt2 = (BYTE)(128 + ((ch / 64) % 64));
			bt3 = (BYTE)(128 + (ch % 64));
			
			szDst[nPos++] = bt1;
			szDst[nPos++] = bt2;
			szDst[nPos++] = bt3;
		}
		else if (ch <= 2097151)
		{
			bt1 = (BYTE)(240 + (ch / 262144));
			bt2 = (BYTE)(128 + ((ch / 4096) % 64));
			bt3 = (BYTE)(128 + ((ch / 64) % 64));
			bt4 = (BYTE)(128 + (ch % 64));
			
			szDst[nPos++] = bt1;
			szDst[nPos++] = bt2;
			szDst[nPos++] = bt3;
			szDst[nPos++] = bt4;
		}
		else if (ch <=67108863)
		{
			bt1 = (BYTE)(248 + (ch / 16777216));
			bt2 = (BYTE)(128 + ((ch / 262144) % 64));
			bt3 = (BYTE)(128 + ((ch / 4096) % 64));
			bt4 = (BYTE)(128 + ((ch / 64) % 64));
			bt5 = (BYTE)(128 + (ch % 64));
			
			szDst[nPos++] = bt1;
			szDst[nPos++] = bt2;
			szDst[nPos++] = bt3;
			szDst[nPos++] = bt4;
			szDst[nPos++] = bt5;
		}
		else if (ch <=2147483647)
		{
			bt1 = (BYTE)(252 + (ch / 1073741824));
			bt2 = (BYTE)(128 + ((ch / 16777216) % 64));
			bt3 = (BYTE)(128 + ((ch / 262144) % 64));
			bt4 = (BYTE)(128 + ((ch / 4096) % 64));
			bt5 = (BYTE)(128 + ((ch / 64) % 64));
			bt6 = (BYTE)(128 + (ch % 64));
			
			szDst[nPos++] = bt1;
			szDst[nPos++] = bt2;
			szDst[nPos++] = bt3;
			szDst[nPos++] = bt4;
			szDst[nPos++] = bt5;
			szDst[nPos++] = bt6;
		}
		
	}
#endif
	return 0;
}

long CWmDb::Init(const char* sDbPath, const char* sDbName, CListCtrl* pListCtrl)
{
	char sRealDbPath[ 256 ] = {0};

	m_pListCtrl = pListCtrl;

	// sDbPathҪתΪutf-8֧·
	wchar_t uniName[ 256 ] = {0};
	char convName[ 256 ] = {0};
	
	MultiByteToWideChar( CP_ACP, 0, 
						sDbPath, strlen(sDbPath)+1, 
						uniName, sizeof(uniName)/sizeof(wchar_t) );
	
	if( EncodeToUTF8(uniName, convName) )
	{
		return -1;
	}

	::strcpy(sRealDbPath, convName);

    ::strcpy(m_sDbName, sRealDbPath);
	::strcat(m_sDbName, "\\");
	::strcat(m_sDbName, sDbName);
	
	return 0;
}

long CWmDb::Open()
{
	long nRet = 0;
	// m_db.set_error_stream(&std::cerr);
	// m_db.set_errpfx("WaterMarkDB");
	m_pDb->set_pagesize(1024);		/* Page size: 1K. */
	m_pDb->set_cachesize(0, 32 * 1024, 0);
	
	nRet = m_pDb->open(NULL, m_sDbName, NULL, DB_BTREE, DB_CREATE, 0664);
	if (nRet)
	{
		CString strError;
		strError.Format("CWaterMarkDB::Open(). m_db.open return %d", nRet);
		::AfxMessageBox(strError);
	}

	m_bIsOpen = true;

	return nRet;
}

long CWmDb::Close()
{
	if (m_bIsOpen)
	{
		m_pDb->close(0);
	}
	
	return 0;
}

long CWmDb::ShowDb()
{
	ASSERT(m_pListCtrl != NULL);
	m_pListCtrl->DeleteAllItems();

	long nRet = 0;
	Dbc *cursorp = NULL;
	char sPath[256] = {0};
	struct RECORD_S record;

	try 
	{
		Dbt key;
		Dbt data;
		m_pDb->cursor(NULL, &cursorp, 0);
		while (cursorp->get(&key, &data, DB_NEXT) == 0) 
		{
			strcpy(sPath, (char*)key.get_data());
			::memcpy(&record, data.get_data(), sizeof(RECORD_S));

			ShowToUI(record.sWm, sPath);
		}
	}
	catch (DbException &dbe) 
	{	
		if (cursorp != NULL)
			cursorp->close();
		::AfxMessageBox(dbe.what());
		return -1;
	}

	if (cursorp != NULL)
		cursorp->close();

	return nRet;
}

long CWmDb::ShowToUI(const char* sWm, const char* sPath)
{
	CString strId;
	strId.Format("%d", m_nId + 1);

	m_pListCtrl->InsertItem(m_nId, strId);
	m_pListCtrl->SetItemText(m_nId, 0, strId);
	m_pListCtrl->SetItemText(m_nId, 1, sWm);
	m_pListCtrl->SetItemText(m_nId, 2, (strlen(sWm) == 0) ? "" : "-");
	m_pListCtrl->SetItemText(m_nId, 3, sPath);

	//m_listUserInfo.SetBkColor(RGB(255,255,255));
	//m_listUserInfo.SetTextColor(RGB(50,50,50));
	//m_listUserInfo.SetTextBkColor(RGB(255,255,255));

	m_nId++;
	return 0;
}


