/***********************************************************************
*
* ļ FileTreeComper.cpp
* ģĿģ Ŀ¼ļȽ
* ߼ڣ wang_jin 2009-05-15
* 汾 1.0
* ȨϢ CVICSE Middleware
* ļ¼ 
*
***********************************************************************/

#include "EventQueue.h"
#include "FileTreeWalker.h"
#include "FileTreeSaver.h"
#include "FileTreeComper.h"
#include "GDLog/GDLog.h"
#include "GDLog/GDTrace.h"
#include "GDLog/RunLog.h"
#include "GDLog/LogFile.h"
#include "GDLog/ThreadData.h"
#include "FileOpt/FileOptDef.h"
#include "FileOpt/FileOpt.h"
#include "Base/FileOptEx.h"
#include "Common/DateTime.h"
#include <time.h>
#include <assert.h>
#ifndef WIN32
#	include <unistd.h>
#endif

namespace InforGuard{
namespace FileScanner{

CFileTreeComper::CFileTreeComper( const string & sTopPath, const string & sSavePath, CFileExclude * pFileExclude )
:m_pFileTreeCompFunc(NULL), m_pFilePathItemRead(NULL), m_pFileExclude(pFileExclude)
{
	m_sTopPath = sTopPath;
	m_sSavePath = sSavePath;
#ifdef WIN32
	InitializeCriticalSection( &m_cs );
#else
	pthread_mutex_init( &m_mutex, NULL );
#endif
}

CFileTreeComper::~CFileTreeComper()
{
#ifdef WIN32
	DeleteCriticalSection( &m_cs );
#else
	pthread_mutex_destroy( &m_mutex );
#endif
}

void CFileTreeComper::Lock()
{
#ifdef WIN32
	EnterCriticalSection( &m_cs );
#else
	pthread_mutex_lock( &m_mutex );
#endif
}

void CFileTreeComper::Unlock()
{
#ifdef WIN32
	LeaveCriticalSection( &m_cs );
#else
	pthread_mutex_unlock( &m_mutex );
#endif
}

int CFileTreeComper::FileTreeFuncWrite( const FilePathItem * pFilePathItem, void * pvoid )
{
	CGDTrace trace( LOG_L6, "CFileTreeComper:FileTreeFuncWrite(%s)", pFilePathItem->sName.c_str() );
	
	CFileTreeSaver* pFileTreeWriter = (CFileTreeSaver*)( ( (CFileTreeComper*) pvoid )->m_pFileTreeWriter );
	return pFileTreeWriter->WriteFileItem( pFilePathItem );
}

string CFileTreeComper::GetHashCodeFileName( const char* sFilename, const char* sPrefix, const char* sSuffix )
{
	CGDTrace trace( LOG_L6, "CFileTreeComper:GetHashCodeFileName(%s)", sFilename );
	int n = strlen( sFilename );
	int h = 0;
	for ( int i = 0; i < n; i++ )
	{
		h = 31*h + sFilename[i];
	}
	char sBuf[32] = {0};
	sprintf( sBuf, "%s%08x%s", sPrefix, h, sSuffix );
	
	return sBuf;
}

int CFileTreeComper::FileTreeFuncCompare( const FilePathItem * pFilePathItemNew, void * pvoid )
{
	CGDTrace trace( LOG_L6, "CFileTreeComper:FileTreeFuncCompare(pFilePathItemNew is %s)", pFilePathItemNew->sName.c_str() );
	
	CFileTreeComper* pFileTreeComper = (CFileTreeComper*)pvoid;

	CFileTreeSaver* pFileTreeReader = (CFileTreeSaver*)pFileTreeComper->m_pFileTreeReader;
	CFileTreeSaver* pFileTreeWriter = (CFileTreeSaver*)pFileTreeComper->m_pFileTreeWriter;
	pFileTreeWriter->WriteFileItem( pFilePathItemNew );

	FilePathItem * pFilePathItemOld = NULL;

	// Ƚ
	do
	{
		if ( NULL == pFileTreeComper->m_pFilePathItemRead )
		{
			pFileTreeComper->m_pFilePathItemRead = pFileTreeReader->ReadNextFileItem();
		}
		pFilePathItemOld = pFileTreeComper->m_pFilePathItemRead;

		FilePathEvent * pResult = NULL;

		if ( NULL == pFilePathItemOld )
		{
			// ½
			pResult = new FilePathEvent;
			pResult->nActionType = 1;
			pResult->sFileName = pFilePathItemNew->sName;
			pResult->nFileType = pFilePathItemNew->nFileType;
			pResult->tLastTime = pFilePathItemNew->tLastTime;
			pResult->nFileSize = pFilePathItemNew->nFileSize;
			EVENT_S event = {0};
			event.event_type = MONITOR_EVENT_SCANNER_CREATE;
			// ȡ۸ʱ
			CDateTime::GetTime(event.evt_time);
			GDDateTime tam_time;
			time_t tTamTime = CFileOpt::GetFileTime( event.o_filepath, &tam_time, 'c' );
			event.tam_time = tam_time;
			strcpy( event.o_filepath, pFileTreeComper->m_sTopPath.c_str() );
			strcat( event.o_filepath, pFilePathItemNew->sName.c_str() );
			if (!pFileTreeComper->m_pFileExclude->IsExcludeEvent(MONITOR_EVENT_SCANNER_CREATE,event.o_filepath) )
			{
				CGDLog::WriteScannerEvent( "[%x][%s]", event.event_type, event.o_filepath );
				CEventQueue::Add( event );
			}
		}
		else
		{
			// ȽϲԣļȽϱͬĿ¼Ŀ¼Ƚϱͬ
			int nRet = 0;
			if ( pFilePathItemNew->nFileType > pFilePathItemOld->nFileType )
			{
				nRet = -1;
			}
			else if ( pFilePathItemNew->nFileType < pFilePathItemOld->nFileType )
			{
				nRet = 1;
			}
			else
			{
				if ( pFilePathItemNew->nDepth != pFilePathItemOld->nDepth )
				{
					nRet = pFilePathItemNew->nDepth - pFilePathItemOld->nDepth;
				}
				else
				{
					nRet = strcmp( pFilePathItemNew->sName.c_str(), pFilePathItemOld->sName.c_str() );
				}
			}
			switch ( nRet )
			{
			case 0: // ͬȽϴСʱ
				if ( (pFilePathItemNew->nFileSize != pFilePathItemOld->nFileSize) 
					|| (pFilePathItemNew->tLastTime != pFilePathItemOld->tLastTime) )
				{
					// ͬ޸
					pResult = new FilePathEvent;
					pResult->nActionType = 3;
					pResult->sFileName = pFilePathItemNew->sName;
					pResult->nFileType = pFilePathItemNew->nFileType;
					pResult->tLastTime = pFilePathItemNew->tLastTime;
					pResult->nFileSize = pFilePathItemNew->nFileSize;
					if( pResult->nFileType != FILE_ATTRIBUTE_DIRECTORY )
					{
						EVENT_S event = {0};
						event.event_type = MONITOR_EVENT_SCANNER_CHANGE;
						// ȡ۸ʱ
						CDateTime::GetTime(event.evt_time);
						GDDateTime tam_time;
						time_t tTamTime = CFileOpt::GetFileTime( event.o_filepath, &tam_time, 'm' );
						event.tam_time = tam_time;
						strcpy( event.o_filepath, pFileTreeComper->m_sTopPath.c_str() );
						strcat( event.o_filepath, pFilePathItemNew->sName.c_str() );
						if (!pFileTreeComper->m_pFileExclude->IsExcludeEvent(MONITOR_EVENT_SCANNER_CHANGE,event.o_filepath) )
						{
						CGDLog::WriteScannerEvent( "[%x][%s]", event.event_type, event.o_filepath );
						CEventQueue::Add( event );
						}
					}
				}
				pFileTreeComper->m_pFilePathItemRead = pFilePathItemOld = NULL; // ȽҪ¶ȡ-Old&New
				break;
			case 1: // ɾ
				pResult = new FilePathEvent;
				pResult->nActionType = 2;
				pResult->sFileName = pFilePathItemOld->sName;
				pResult->nFileType = pFilePathItemOld->nFileType;
				pResult->tLastTime = pFilePathItemOld->tLastTime;
				pResult->nFileSize = pFilePathItemOld->nFileSize;
				{
				EVENT_S event = {0};
				event.event_type = MONITOR_EVENT_SCANNER_DELETE;
				// ȡ۸ʱ
				CDateTime::GetTime(event.evt_time);
				CDateTime::GetTime(event.tam_time);
				strcpy( event.o_filepath, pFileTreeComper->m_sTopPath.c_str() );
				strcat( event.o_filepath, pFilePathItemOld->sName.c_str() );
				if (!pFileTreeComper->m_pFileExclude->IsExcludeEvent(MONITOR_EVENT_SCANNER_DELETE,event.o_filepath) )
				{
					CGDLog::WriteScannerEvent( "[%x][%s]", event.event_type, event.o_filepath );
					CEventQueue::Add( event );
				}
				}
				pFileTreeComper->m_pFilePathItemRead = NULL; // ȽҪ¶ȡ
				break;
			case -1: // ½
				pResult = new FilePathEvent;
				pResult->nActionType = 1;
				pResult->sFileName = pFilePathItemNew->sName;
				pResult->nFileType = pFilePathItemNew->nFileType;
				pResult->tLastTime = pFilePathItemNew->tLastTime;
				pResult->nFileSize = pFilePathItemNew->nFileSize;
				{
				EVENT_S event = {0};
				event.event_type = MONITOR_EVENT_SCANNER_CREATE;
				// ȡ۸ʱ
				CDateTime::GetTime(event.evt_time);
				GDDateTime tam_time;
				time_t tTamTime = CFileOpt::GetFileTime( event.o_filepath, &tam_time, 'c' );
				event.tam_time = tam_time;
				strcpy( event.o_filepath, pFileTreeComper->m_sTopPath.c_str() );
				strcat( event.o_filepath, pFilePathItemNew->sName.c_str() );
				if (!pFileTreeComper->m_pFileExclude->IsExcludeEvent(MONITOR_EVENT_SCANNER_CREATE,event.o_filepath) )
				{
					CGDLog::WriteScannerEvent( "[%x][%s]", event.event_type, event.o_filepath );
					CEventQueue::Add( event );
				}
				}
				pFilePathItemOld = NULL; // ȽҪ¶ȡ
				break;
			}
		}
		if ( pResult )
		{
			// Ҫ
			pFileTreeComper->Lock();
			pFileTreeComper->m_filedeque.push_back( pResult );
			pFileTreeComper->Unlock();
			if ( pFileTreeComper->m_pFileTreeCompFunc )
			{
				(*pFileTreeComper->m_pFileTreeCompFunc)( pResult, pFileTreeComper->m_param );
			}
		}
	} while ( pFilePathItemOld );

	return 0;
}

int CFileTreeComper::FileTreeCompare() 
{
	CGDTrace trace( LOG_L6, "CFileTreeComper:FileTreeCompare()" );
	
	int nRet = 0;

	string sFilePathOld = ::GetInforGuardPath();
	sFilePathOld.append(S_PATHLASH);
	sFilePathOld.append("dbhome");
	sFilePathOld.append(S_PATHLASH);
	string sFilePathNew = sFilePathOld;

	sFilePathOld.append( GetHashCodeFileName( m_sTopPath.c_str(), "Old", ".dat" ) );
	sFilePathNew.append( GetHashCodeFileName( m_sTopPath.c_str(), "New", ".dat" ) );
	// add at 2009.11.10
	FILE *fp = fopen(sFilePathOld.c_str(),"a+");
	if( fp != NULL )
	{
		fclose(fp);
	}
	string chOldFile="chmod 666 ";
	chOldFile +=sFilePathOld;
	system( chOldFile.c_str() );
	FILE *fp1 = fopen(sFilePathNew.c_str(),"w");
	if( fp1 != NULL )
	{
		fclose(fp1);
	}
	string chNewFile="chmod 666 ";
	chNewFile +=sFilePathNew;
	system( chNewFile.c_str() );
	
	// add by zhangtao Ŀ¼Ϊʱһͬʧܵ
	CFileTreeSaver fileTreeReader( sFilePathOld.c_str() );
	CFileTreeSaver fileTreeWriter( sFilePathNew.c_str() );
	m_pFileTreeReader = &fileTreeReader;
	m_pFileTreeWriter = &fileTreeWriter;
	fileTreeReader.IsReadable();
	CFileTreeWalker fileTreeWalker( m_sTopPath.c_str(), m_sSavePath.c_str() );
	// ǲۺֱӽбȽϵ add at 2009.11.10
	nRet = fileTreeWalker.FileTreeWalk( FileTreeFuncCompare, this );
	if ( nRet == 404 ) // Ŀ¼
	{
		// ɾĿ¼
		FilePathEvent* pResult = new FilePathEvent;
		pResult->nActionType = 2;
		pResult->sFileName = m_sTopPath;
		pResult->nFileType = FILE_ATTRIBUTE_DIRECTORY;
		pResult->tLastTime = 0;
		pResult->nFileSize = 0;
		
		// Ҫ
		Lock();
		m_filedeque.push_back( pResult );
		Unlock();
	}
	else if ( nRet == 0 )
	{
		// ĩβҪɾԪ
		if ( NULL == m_pFilePathItemRead )
		{
			m_pFilePathItemRead = fileTreeReader.ReadNextFileItem();
		}
		FilePathItem * pFilePathItemOld = m_pFilePathItemRead;
		while ( pFilePathItemOld )
		{
			// ɾ
			FilePathEvent* pResult = new FilePathEvent;
			pResult->nActionType = 2;
			pResult->sFileName = pFilePathItemOld->sName;
			pResult->nFileType = pFilePathItemOld->nFileType;
			pResult->tLastTime = pFilePathItemOld->tLastTime;
			pResult->nFileSize = pFilePathItemOld->nFileSize;
			{
			EVENT_S event = {0};
			event.event_type = MONITOR_EVENT_SCANNER_DELETE;
			// ȡ۸ʱ
			CDateTime::GetTime(event.evt_time);
			CDateTime::GetTime(event.tam_time);
			strcpy( event.o_filepath, m_sTopPath.c_str() );
			strcat( event.o_filepath, pFilePathItemOld->sName.c_str() );
			if (!m_pFileExclude->IsExcludeEvent(MONITOR_EVENT_SCANNER_DELETE,event.o_filepath) )
			{
				CGDLog::WriteScannerEvent( "[%x][%s]", event.event_type, event.o_filepath );
				CEventQueue::Add( event );
			}
			}//add by zhangtao һļļеɾҪӵ¼
			// Ҫ
			Lock();
			m_filedeque.push_back( pResult );
			Unlock();
			if ( m_pFileTreeCompFunc )
			{
				(*m_pFileTreeCompFunc)( pResult, m_param );
			}
			pFilePathItemOld = fileTreeReader.ReadNextFileItem();
		}
		fileTreeReader.Close();
		nRet = fileTreeWriter.WriteEnd();
	}//else.if

	// ļ
#ifdef WIN32
	if( ! DeleteFile( sFilePathOld.c_str() ) )//ɾOldļ
	{
		ErrCode errCode = GetLastError();
		GDPrint( "DeleteFile(%s) failed[%d][%s]\n", sFilePathOld.c_str(), errCode, strerror(errCode) );
	}

	if ( ! MoveFile( sFilePathNew.c_str(), sFilePathOld.c_str() ) )
	{
		ErrCode errCode = GetLastError();
		GDPrint( "MoveFile(%s,%s) failed[%d][%s]\n", sFilePathNew.c_str(), sFilePathOld.c_str(), errCode, strerror(errCode) );
	}
#else
	unlink( sFilePathOld.c_str() );
	rename( sFilePathNew.c_str(), sFilePathOld.c_str() );
#endif
	return nRet;
}

int CFileTreeComper::GetFilePathEvent( FilePathEvent& filePathEvent )
{
	CGDTrace trace( LOG_L6, "CFileTreeComper::GetFilePathEvent(%s)", "void" );
	int bRet = 404; // û
	// Ҫ
	Lock();
	if ( ! m_filedeque.empty() )
	{
		FilePathEvent * pFilePathEvent = m_filedeque.front();
		m_filedeque.pop_front();
		filePathEvent.nActionType = pFilePathEvent->nActionType;
		filePathEvent.nFileSize = pFilePathEvent->nFileSize;
		filePathEvent.nFileType = pFilePathEvent->nFileType;
		filePathEvent.sFileName = pFilePathEvent->sFileName;
		filePathEvent.tLastTime = pFilePathEvent->tLastTime;
		delete pFilePathEvent;
		pFilePathEvent = NULL;
		bRet = 0;
	}
	Unlock();
	return bRet;
}

}	// namespace FileDiffer
}	// namespace InforGuard
