// UrlParse.cpp

#pragma warning(disable:4786)

#include "FileOpt/FileOptDef.h"
#include "GDLog/GDTrace.h"
#include "UrlParse/UrlParse.h"

#ifdef WIN32
#	include <windows.h>
#else
#	include <sys/types.h>
#	include <sys/stat.h>
#	include <errno.h>
#endif

#include <algorithm>
#ifndef NOT_USE_STD_LIB
#	include <fstream>
#	include <iostream>
//#	include <sstream>
#else
#	include <fstream.h>
#	include <iostream.h>
#endif

class ToLower //ToLowerǸ
{
public:
	char operator()(char val)
	{
		return tolower(val);
	}
};


template <class S>
S& replaceInPlace(S& str, const typename S::value_type* from, const typename S::value_type* to, typename S::size_type start = 0)
{
	S result;
	typename S::size_type pos = 0;
	typename S::size_type fromLen = strlen(from);
	result.append(str, 0, start);
	do
	{
		pos = str.find(from, start);
		if (pos != S::npos)
		{
			result.append(str, start, pos - start);
			result.append(to);
			start = pos + fromLen;
		}
		else result.append(str, start, str.size() - start);
	}
	while (pos != S::npos);
	str.swap(result);
	return str;
}

template <class S>
S trim(const S& str)
	/// Returns a copy of str with all leading and
	/// trailing whitespace removed.
{
	int first = 0;
	int last  = int(str.size()) - 1;
	
	while (first <= last && isspace(str[first])) ++first;
	while (last >= first && isspace(str[last])) --last;

	return S(str, first, last - first + 1);
}

#ifdef WIN32
const char* CUrlParse::PATHSLASH = "\\";
const char* CUrlParse::PATHREVERSESLASH = "/";
#else
const char* CUrlParse::PATHSLASH = "/";
const char* CUrlParse::PATHREVERSESLASH = "\\";
#endif

CUrlParse::CUrlParse(const char* sFilePath, const char* sRootPath, PageType pageType, bool bToLower) :
	_filePath(sFilePath),
	_rootPath(sRootPath),
	_pageType(pageType),
	_bToLower(bToLower)

{
#ifdef WIN32
	_bToLower = true; // Win32ƽ̨תСдСдУ
#endif
	InitPath();
}

CUrlParse::~CUrlParse()
{
}

CUrlParse::FileAttr CUrlParse::GetFileType(const char* strFilePath)
{
	CGDTrace trace( LOG_L6, "CUrlParse:GetFileType(%s)", strFilePath );
#ifdef WIN32

	DWORD fattri = GetFileAttributes(strFilePath);

	if(fattri == 0xFFFFFFFF)
	{
		return FILEATTR_EXCEPT;
	}

	if(fattri & FILE_ATTRIBUTE_DIRECTORY)
	{
		return FILEATTR_DIR;
	}

	return FILEATTR_FILE;

#else

	struct stat buf;
	int iRet = stat(strFilePath, &buf);
	if(iRet)
	{
		if( errno == ENOENT )
		{
			return FILEATTR_EXCEPT;
		}
	}

	if( S_ISDIR(buf.st_mode) )
	{
		return FILEATTR_DIR;
	}

	return FILEATTR_FILE;

#endif
}

void CUrlParse::InitPath()
{
	replaceInPlace(_filePath, PATHREVERSESLASH, PATHSLASH); // ???
}

string CUrlParse::GetParentPath( const string& fileName )
{
	CGDTrace trace( LOG_L6, "CUrlParse:GetParentPath(%s)", fileName.c_str() );
	string str = fileName.substr( 0, fileName.find_last_of(PATHSLASH) );// Ŀ¼󲻼'\\''/'
	trace.SetRetText( str.c_str() );
	return str; 
}

/*
Ҹʽ
<!--#include file="inc/Conn.asp" -->

˼·:
ȸ<!--#includeҵincludeУ
ݵȺȷǰincludeؼֺͺincludeļ
ҵincludeļݹ

д
<!--
#include file="inc/Conn.asp"
-->
޷ҳ

Ŀ¼include޷
<!--#include virtual="admin/Admin_IsLogin.asp" --> ?
*/
void CUrlParse::ParseAspInclude( const char* fileName )
{
	CGDTrace trace( LOG_L6, "CUrlParse:ParseAspInclude(%s)", fileName );

	ifstream ifs(fileName);

	if ( ifs.good() )
	{
		string line;
		string parentPath = GetParentPath( fileName );

		while ( getline( ifs, line ) )
		{
			if (_bToLower)
			{
				transform( line.begin(), line.end(), line.begin(), ToLower() );
				// ڴСдƽ̨ϣֱӵļ by wang_jin at 2009-11-30
			}

			string::size_type index1 = string::npos;
			if ( ( index1 = line.find( "#include" ) ) != string::npos ) // inlcude
			{
				string::size_type index2 = string::npos;
				if ( ( index2 = line.find( "\"", index1 ) ) != string::npos )
				{
					string::size_type index3 = string::npos;
					if ( ( index3 = line.find( ".asp", index2 ) ) != string::npos 
						|| ( index3 = line.find( ".htm", index2 ) ) != string::npos 
						|| ( index3 = line.find( ".html", index2 ) ) != string::npos 
						|| ( index3 = line.find( ".inc", index2 ) ) != string::npos 
						|| ( index3 = line.find( ".jsp", index2 ) ) != string::npos 
						|| ( index3 = line.find( ".php", index2 ) ) != string::npos )
					{
						string::size_type index4 = string::npos;
						if ( ( index4 = line.find( "\"", index3 ) ) != string::npos )
						{
							string sIncludeFile = line.substr( index2 + 1, index4 - index2 - 1 ); 
							CGDTrace::GDTrace( LOG_L9, "ParseAspInclude() [%s]", sIncludeFile.c_str() );

							if ( sIncludeFile[0] == '/' )
							{
								// Ӧ÷Ŀ¼װļ·
								sIncludeFile = _rootPath + sIncludeFile;
							}
							else if ( sIncludeFile[0] == '.' )
							{
								if ( sIncludeFile[1] == '/' )
								{
									// ɵǰ·װļ·
									sIncludeFile = parentPath + S_PATHLASH + sIncludeFile.substr(2);
								}
								else if ( sIncludeFile[1] == '.' &&  sIncludeFile[2] == '/' )
								{
									// ȥ·
									char sParentPath[ GD_MAX_PATH ] = {0};
									int nPathLen = strlen( sParentPath );
									const char* p = sIncludeFile.c_str();
									strcpy( sParentPath, parentPath.c_str() );
									while ( memcmp( p, "../", 3 ) == 0 )
									{
										for ( int i=nPathLen; i>0; i-- )
										{
											if ( sParentPath[i-1] == '/' || sParentPath[i-1] == '\\' )
											{
												sParentPath[i-1] = '\0';
												nPathLen = strlen( sParentPath );
												break;
											}
										}
										p += 3;
									}
									strcat( sParentPath, S_PATHLASH );
									strcat( sParentPath, p );
									sIncludeFile = sParentPath;
								}
							}
							else
							{
								// ɵǰ·װļ·
								sIncludeFile = parentPath + S_PATHLASH + sIncludeFile;
							}
							CGDTrace::GDTrace( LOG_DBG, "ParseAspInclude() [%s]", sIncludeFile.c_str() );

							// תЧ·
							int nLen = sIncludeFile.length();
							for ( int i=0; i<nLen; i++ )
							{
								if ( sIncludeFile[i] == '\\' || sIncludeFile[i] == '/' )
								{
									sIncludeFile[i] = C_PATHLASH;
								}
							}
							// by wang_jin at 2009-11-29 for "includedhead.asp?location=service"
							string::size_type nPos = sIncludeFile.find_first_of('?');
							if ( string::npos != nPos )
							{
								sIncludeFile = sIncludeFile.substr( 0, nPos );
							}

							ParseAspInclude( sIncludeFile.c_str() );
						}
					}
				}
			}
		}
		ifs.close();
	}

	_filelist.push_back( fileName );
}

/*
JSPű
*/
void CUrlParse::ParseJspInclude(const char* fileName)
{
	CGDTrace trace( LOG_L6, "CUrlParse:ParseJspInclude(%s)", fileName );

	ifstream ifs( fileName );

	if ( ifs.good() )
	{
		string line;
		string parentPath = GetParentPath( fileName );

		while ( getline( ifs, line ) )
		{
			if (_bToLower)
			{
				transform( line.begin(), line.end(), line.begin(), ToLower() );
				// ڴСдƽ̨ϣֱӵļ by wang_jin at 2009-11-30
			}

			string::size_type index1 = string::npos;
			if ( ( index1 = line.find( "include" ) ) != string::npos || // inlcude
				( index1 = line.find( "forward" ) ) != string::npos ) // forward
			{
				string::size_type index2 = string::npos;
				if ( ( index2 = line.find( "\"", index1 ) ) != string::npos )
				{
					string::size_type index3 = string::npos;
					if ( ( index3 = line.find( ".asp", index2 ) ) != string::npos
						|| ( index3 = line.find( ".htm", index2 ) ) != string::npos
						|| ( index3 = line.find( ".html", index2 ) ) != string::npos
						|| ( index3 = line.find( ".inc", index2 ) ) != string::npos
						|| ( index3 = line.find( ".jsp", index2 ) ) != string::npos 
						|| ( index3 = line.find( ".php", index2 ) ) != string::npos )
					{
						string::size_type index4 = string::npos;
						if ( ( index4 = line.find( "\"", index3 ) ) != string::npos )
						{
							string sIncludeFile = line.substr( index2 + 1, index4 - index2 - 1 ); 
							//CGDTrace::GDTrace( LOG_DBG, "ParseJspInclude() [%s]", sIncludeFile.c_str() );

							if ( sIncludeFile[0] == '/' )
							{
								// Ӧ÷Ŀ¼װļ·
								sIncludeFile = _rootPath + sIncludeFile;
							}
							else
							{
								// ɵǰ·װļ·
								sIncludeFile = parentPath + S_PATHLASH + sIncludeFile;
							}
							// by wang_jin at 2009-11-29 for GZYD-MH "includedhead.jsp?location=service"
							string::size_type nPos = sIncludeFile.find_first_of('?');
							if ( string::npos != nPos )
							{
								sIncludeFile = sIncludeFile.substr( 0, nPos );
							}

							CGDTrace::GDTrace( LOG_DBG, "ParseJspInclude() [%s]", sIncludeFile.c_str() );
							ParseJspInclude( sIncludeFile.c_str() );
						}
					}
				}
			}
		}
		ifs.close();
	}

	_filelist.push_back( fileName );
}

/*
Ҹʽ
<?include("config.php");?>
<?php require("check_access.php");?>

˼·:
ȸinclude(require(ҵincludeУ
ҵȡмĴ
ҵincludeļݹ


require_once "config.php";
޷ҳ
*/
void CUrlParse::ParsePhpInclude(const char* fileName)
{
	CGDTrace trace( LOG_L6, "CUrlParse:ParsePhpInclude(%s)", fileName );

	ifstream ifs(fileName);

	if (ifs.good())
	{
		string line;
		string includestr = "include(";
		string requirestr = "require(";

		string parentPath = GetParentPath( fileName );

		while (getline(ifs, line))
		{
			if (_bToLower)
			{
				transform( line.begin(), line.end(), line.begin(), ToLower() );
				// ڴСдƽ̨ϣֱӵļ by wang_jin at 2009-11-30
			}

			if (line.find(includestr) != string::npos ||
				line.find(requirestr) != string::npos) // inlcude
			{
				string::size_type leftparPos;
				string::size_type rightparPos;

				leftparPos = line.find_first_of("(");
				rightparPos = line.find_last_of(")");

				if ( leftparPos != string::npos &&
					rightparPos != string::npos)
				{
					string sIncludeFile = line.substr(leftparPos + 1, rightparPos - leftparPos - 1); // = Ĳ

					replaceInPlace(sIncludeFile, "\"", " ");
					replaceInPlace(sIncludeFile, "\'", " ");

					sIncludeFile = trim(sIncludeFile);

					if ( sIncludeFile[0] == '/' )
					{
						sIncludeFile = _rootPath + sIncludeFile; 
					}
					else
					{
						sIncludeFile = parentPath + S_PATHLASH + sIncludeFile;
					}

					replaceInPlace(sIncludeFile, PATHREVERSESLASH, PATHSLASH);
					// by wang_jin at 2009-11-29 for "includedhead.asp?location=service"
					string::size_type nPos = sIncludeFile.find_first_of('?');
					if ( string::npos != nPos )
					{
						sIncludeFile = sIncludeFile.substr( 0, nPos );
					}

					ParsePhpInclude(sIncludeFile.c_str()); 
				}
			}
		}
	}
	_filelist.push_back( fileName );
}

/*
Ҹʽ
	<!--#include file="chinareinsurance/index.htm"-->
	<!--#include file="external.htm"-->
	<!--#include virtual="/includes/navbar.txt"-->  // "virtual" for html root
	<!--#flastmod virtual="index.html"-->

˼·:
ȸ#includeҵУҵȡмĴ
ҵincludeļݹ
*/
void CUrlParse::ParseSHtmlInclude(const char* fileName)
{
	CGDTrace trace( LOG_L6, "CUrlParse:ParseSHTMLInclude(%s)", fileName );

	ifstream ifs(fileName);

	if ( ifs.good() )
	{
		string line;
		///////////////////
#ifndef NODEBUG
		int line_NO = 0;
#endif
		string parentPath = GetParentPath( fileName );//ͷȡõһбܵбܵ·
		
		while ( getline( ifs, line ) )
		{
			if (_bToLower)
			{
				transform( line.begin(), line.end(), line.begin(), ToLower() );
				// ڴСдƽ̨ϣֱӵļ by wang_jin at 2009-11-30
			}
#ifndef NODEBUG
			line_NO++;
#endif
			int lineSize = line.size();
			if(lineSize < 10)//<!--# + file or virtual >9
				continue;
			
			line = trim(line);
#ifndef NODEBUG
			lineSize = line.size();
#endif
			string::size_type indexPos = 0;
			
			if ((indexPos = line.compare(indexPos, 5, "<!--#")) != 0)//
				continue;//û"<!--"
			
			string::size_type indexFro = 0;
			string::size_type indexEnd = 0;

			string s = "";//ȡļ
			indexFro = line.find_first_of("\"", indexPos+4);
			indexEnd = line.find_last_of("\"", lineSize - 1);
			if ((indexFro != string::npos) && (indexEnd != string::npos))
			{
				s = line.substr(indexFro+1, indexEnd - indexFro -1);//ڶǳ  
			}
			else 
			{
				continue;
			}

			string sIncludeFile = "";
			if ((indexFro = line.find("file", indexPos+4)) != string::npos)//ҵfileʱΪ·
			{
				sIncludeFile = parentPath + S_PATHLASH + s;
			}
			else if (indexFro = line.find("virtual", indexPos+6) != string::npos)//ҵvirtualʱserverRootʼ
			{
				if ( s[0] == '/' )
				{
					sIncludeFile = _rootPath + s; 
				}
				else
				{
					sIncludeFile = parentPath + S_PATHLASH + s;
				}
			}
			replaceInPlace(sIncludeFile, "/", PATHSLASH);//滻/ΪPATHSLASH

			//ҪǶͣݹãֱӷŵ
			indexPos = sIncludeFile.find_last_of('.');
			if(sIncludeFile.find(".shtml", indexPos) != string::npos ||
			   sIncludeFile.find(".shtm",  indexPos) != string::npos ||
			   sIncludeFile.find(".stm",   indexPos) != string::npos )
			{
				ParseSHtmlInclude(sIncludeFile.c_str());
			}
			else
			{
				_filelist.push_back( sIncludeFile );
			}
		}//while
	}
	ifs.close();			

	_filelist.push_back( fileName );
}

void CUrlParse::Parse()
{
	CGDTrace trace( LOG_L6, "CUrlParse:Parse(%s)", _filePath.c_str() );

	switch (_pageType)
	{
	case ASP_PAGE:
		ParseAspInclude(_filePath.c_str());
		break;
	case JSP_PAGE:
		ParseJspInclude(_filePath.c_str());
		break;
	case PHP_PAGE:
		ParsePhpInclude(_filePath.c_str());
		break;
	case SHTML_PAGE:
		ParseSHtmlInclude(_filePath.c_str());
		break;
	case ASP_AND_PHP_PAGE:
		ParseAspInclude(_filePath.c_str());
		ParsePhpInclude(_filePath.c_str());
		break;
	default:
		return;
	}
}
