#include "articlepage.h"
#include "imagebutton.h"
#include <QLabel>
#include <QPalette>
#include <QFont>
#include <QListWidget>
#include <QIcon>
#include <QPixmap>
#include <QDebug>
#include <windows.h>

ArticlePage* ArticlePage::articleInstance = NULL;

ArticlePage::ArticlePage(QObject *parent) : QObject(parent)
{
    middle = Middle::getMiddleInstance();
    articleSubPage = ArticleSubPage::getArticleSubPageInstance();
    videoPage = VideoPage::getVideoPageInstance();
}

ArticlePage* ArticlePage::getArticleInstance()
{
    if (articleInstance == NULL) {
        articleInstance = new ArticlePage();
    }

    return articleInstance;
}

void ArticlePage::setWidget(QWidget *article, QStackedWidget *stack, QWidget *next, QWidget *nextVideo)
{
    articlePage = article;
    stackWidget = stack;
    nextPage = next;
    nextVideoPage = nextVideo;

    titleText = new QLabel(articlePage);
    titleText->hide();
    btnBack = new ImageButton(articlePage);
    btnBack->hide();
    topImage = new QLabel(articlePage);
    topImage->hide();
    listView = new QListView(articlePage);
    listView->hide();

    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
    connect(listView,SIGNAL(clicked(QModelIndex)),this,SLOT(itemClicked(QModelIndex)));
}

void ArticlePage::setArticlePage(QWidget *parent, QList<QString> info, QString title_, QListView *listViewP, QString type)
{
    parentPage = parent;
    listViewParent = listViewP;
    stop = false;
    type_ = type;

    setBackGround(title_, type);
    setBackButton();
    setListView(info[0]);
}

void ArticlePage::setBackGround(QString title, QString type)
{
    QFont font;
    QPalette pa;
    QString topImageFile;

    pa.setColor(QPalette::WindowText,QColor(251,151,5));
    font.setFamily("宋体");
    font.setPixelSize(35);
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);

    articlePage->setStyleSheet("QWidget#articlePage{image:url(:/nianji/shouye/bj_01.jpg)}");

    titleText->setText(title);
    titleText->setAlignment(Qt::AlignCenter);
    titleText->setFont(font);
    titleText->setGeometry(0,115,1000,50);
    titleText->show();

    topImage->setGeometry(387,34,253,57);
    if (type == "dbxz")
        topImageFile = "image:url(:/nianji/shouye/dbxz_top.png)";
    if (type == "xbxz")
        topImageFile = "image:url(:/nianji/shouye/xbxz_top.png)";
    if (type == "mbkt")
        topImageFile = "image:url(:/mbkt/mbkt_top.png)";

    topImage->setStyleSheet(topImageFile);
    topImage->show();
}

void ArticlePage::setListView(QString code)
{
    QList<QVariant> lectureList;
    QStringList lectureStringList;
    QStringListModel *lectureListModel;

    lectureList = middle->getLectureList(code);
    qDebug() << "lectureList:" << lectureList;
    if (lectureList.count() == 0)
        return;

    info.clear();
    for (int i=0; i<lectureList.count(); i++) {
        if (stop)
            return;
        QVariant tmp = lectureList[i];
        QList<QString> info_tmp;
        lectureStringList.insert(i,tmp.toMap()["name"].toString());
        info_tmp.append(tmp.toMap()["flag"].toString());
        info_tmp.append(tmp.toMap()["video"].toString());
        info_tmp.append(tmp.toMap()["video_size"].toString());
        info_tmp.append(tmp.toMap()["lecture_code"].toString());
        info.append(info_tmp);
    }

    lectureListModel = new QStringListModel(lectureStringList);
    listView->setModel(lectureListModel);
    listView->setGeometry(387,200,400,400);
    listView->setSpacing(4);
//    listView->setFlow(QListView::PositionAtCenter);
    listView->setCursor(QCursor(Qt::PointingHandCursor));
    listView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    listView->verticalScrollBar()->setStyleSheet("QScrollBar:vertical{width:6px;background:transparent;background-color:rgb(248, 248, 248);margin:0px,0px,0px,0px;padding-top:10px;padding-bottom:10px;}"
                       "QScrollBar::handle:vertical{width:6px;background:lightgray ;border-radius:5px;min-height:20px;}"
                       "QScrollBar::handle:vertical:hover{width:6px;background:gray;border-radius:5px;min-height:20px;}"
                       "QScrollBar::add-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/down.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/up.png);subcontrol-position:top;}"
                       "QScrollBar::add-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/down_mouseDown.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/up_mouseDown.png);subcontrol-position:top;}"
                       "QScrollBar::add-page:vertical,QScrollBar::sub-page:vertical{background:transparent;border-radius:5px;}");
    listView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    // 去边框
    listView->setFrameStyle(0);
    // 设置样式
    listView->setStyleSheet("QListView{font-size: 25px; font-family:宋体}"
                            "QListView::item:hover:active{background-color: rgb(251,151,5)}"
                            "QListView::item:hover{background-color: rgb(251,151,5)}"
                             "QListView{background-color:transparent}");
    listView->show();
}

void ArticlePage::setBackButton()
{
    btnBack->setGeometry(855,520,109,101);
    btnBack->show();
    QPixmap backP1(":/nianji/yiji/fh_03.png");
    QPixmap backP2(":/nianji/yiji/fh_04.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void ArticlePage::onBackClick()
{
    stop = true;
    listViewParent->blockSignals(false);
    listViewParent->clearSelection();
    titleText->clear();
    QStringListModel *listModel = new QStringListModel(QStringList{});
    listView->setModel(listModel);
    stackWidget->setCurrentWidget(parentPage);
}

void ArticlePage::itemClicked(QModelIndex index)
{
//    listView->blockSignals(true);
    QString title = index.data().toString();
    int i = index.row();
    QString flag, video;
    int videoSize;

    flag = info[i][0];
    video = info[i][1];
    videoSize = info[i][2].toInt();

    if (flag =="1") {
        videoPage->setVideoPage("image:url(:/xzzs/bj_11.jpg)", "",
                                "", "image:url(:/xzzs/sjqp.png)",
                                video, videoSize, NULL, articlePage, "dxpy");

        stackWidget->setCurrentWidget(nextVideoPage);
    } else {
        articleSubPage->setArticleSubPage(articlePage, info[i], title, listView, type_);
        stackWidget->setCurrentWidget(nextPage);
    }
}
