#include "download.h"
#include "ui_download.h"
#include <QDebug>
#include <QMessageBox>
#include <QProcess>

Download::Download(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::Download)
{
    ui->setupUi(this);
    setFixedSize(700,275);
    setWindowFlags(Qt::FramelessWindowHint);

    middle = Middle::getMiddleInstance();
    tool = Tool::getToolInstance();

    ui->progressBar->hide();
    manager = new QNetworkAccessManager(this);
    type = "db";
    startDownload();
}

Download::~Download()
{
    delete ui;
}

void Download::closeEvent(QCloseEvent *event)
{
    Q_UNUSED(event);
    stopRequest();
}

void Download::startDownload()
{
    QString currentVer, newVer;
    if (type == "client") {
        currentVer = middle->getCurrentVersion();
        newVer = middle->getNewVersion();

        if (middle->verCap(newVer, currentVer) == 1) {
            url = middle->getClientUrl();
            QFileInfo info(url.path());
            fileName = info.fileName();

            if (fileName.isEmpty())
                fileName = "client.exe";
        } else {
            return;
        }
    } else if (type == "db") {
        currentVer = middle->getCurrentDBVersion();
        newVer = middle->getNewDBVersion();

        qDebug() << "newVer currentVer" << newVer << currentVer;
        if (middle->verCap(newVer, currentVer) == 1) {
            url = middle->getdbUrl();
            QFileInfo info(url.path());
            fileName = info.fileName();

            if (fileName.isEmpty())
                fileName = "data.db";
        } else {
            type = "client";
            startDownload();
            return;
        }
    }

    QString path = QCoreApplication::applicationDirPath();
    filePath = path + "/" + fileName;
    qDebug() << "保存文件路径" << filePath;
    file = new QFile(filePath);
    if (!file->open(QIODevice::WriteOnly)) {
        qDebug() << "open file error";
        delete file;
        file = 0;
        return;
    }

    startRequest(url);
    ui->progressBar->setValue(0);
    ui->progressBar->show();
}

void Download::startRequest(QUrl url)
{
    reply = manager->get(QNetworkRequest(url));
    connect(reply,SIGNAL(finished()),this,SLOT(httpFinished()));
    connect(reply,SIGNAL(readyRead()),this,SLOT(httpReadyRead()));
    connect(reply,SIGNAL(downloadProgress(qint64,qint64)), this,SLOT(updateDataReadProgress(qint64,qint64)));
}

void Download::httpFinished()
{
    ui->progressBar->hide();
    file->flush();
    file->close();
    reply->deleteLater();
    reply = 0;
    delete file;
    file = 0;

//    this->close();
    if (type == "client")
        installClient();
    if (type == "db") {
        type = "client";
        middle->setCurrentDBVersion(middle->getNewDBVersion());
        startDownload();
    }
}

void Download::httpReadyRead()
{
    if (file) {
        file->write(reply->readAll());
    }
}

void Download::updateDataReadProgress(qint64 bytesRead, qint64 totalBytes)
{
    ui->progressBar->setMaximum(totalBytes);
    ui->progressBar->setValue(bytesRead);
}

void Download::installClient()
{
//    QMessageBox::StandardButton rb = QMessageBox::question(NULL, "更新", "下载完成是否安装?", QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes);
//    if(rb == QMessageBox::Yes)
//    {
        qDebug() << "客户端安装包路径" << filePath;
        QString client = filePath.replace("/", "\\\\");
        qDebug() << client;
        QProcess::startDetached(client,QStringList());
//    }
}

void Download::stopRequest()
{
    if (reply) {
        disconnect(reply,SIGNAL(finished()),this,SLOT(httpFinished()));
        disconnect(reply,SIGNAL(readyRead()),this,SLOT(httpReadyRead()));
        disconnect(reply,SIGNAL(downloadProgress(qint64,qint64)), this,SLOT(updateDataReadProgress(qint64,qint64)));
        reply->abort();
        reply->deleteLater();
        reply = NULL;
    }
}
