#include "flashpage.h"
#include <QDebug>
#include <QApplication>
#include <QDesktopWidget>

FlashPage* FlashPage::flashPageInstance = NULL;

FlashPage::FlashPage(QObject *parent) : QObject(parent)
{
    middle = middle->getMiddleInstance();
    isFullScreen = false;
    clicked_num = 0;
    timer = new QTimer(this);

    connect(timer,SIGNAL(timeout()),this,SLOT(timerOut()));
}

FlashPage* FlashPage::getFlashPageInstance()
{
    if (flashPageInstance == NULL)
        flashPageInstance = new FlashPage();

    return flashPageInstance;
}

void FlashPage::setWidget(QWidget *flashP, QStackedWidget *stack, QWidget *root)
{
    flashPage = flashP;
    stackWidget = stack;
    rootWidget = root;

    BgLabel = new QLabel(flashPage);
    btnBack = new ImageButton(flashPage);

    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
}

void FlashPage::setFlashPage(QString bgImage, QString flashUrl, int flashSize, ImageButton *btnP,
                             QWidget *parent, QAxWidget *flashP)
{
    btnParent = btnP;
    parentPage = parent;
    flashP_ = flashP;

    // 设置背景图片
    BgLabel->setGeometry(0,0,1000,640);
    BgLabel->setStyleSheet(bgImage);

    // 设置flash
    flash = new QAxWidget(flashPage);
    flash->installEventFilter(this);
    flash->setGeometry(174,116,640,465);
    bool ret = flash->setControl(QString::fromUtf8("{d27cdb6e-ae6d-11cf-96b8-444553540000}"));

    if (!ret)
        middle->errorMBox("未检测到Adobe Flash Player,请下载安装");
    else {
        flash->dynamicCall("LoadMovie(long,string)",0,middle->getVideoPath("ggxz", flashUrl, flashSize));
//        flash->dynamicCall("LoadMovie(long,string)",0,"http://res.cguoguo.net/jfx-web/attachment/aggxz/flash4/11447.swf");

        middle->download(flashUrl, flashSize);
        flash->dynamicCall("Play()");
        flash->show();
    }

    // 设置返回按钮
    setBackButton();
}

void FlashPage::setBackButton()
{
    btnBack->setGeometry(855,540,109,101);
    QPixmap backP1(":/nianji/yiji/fh_03.png");
    QPixmap backP2(":/nianji/yiji/fh_04.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void FlashPage::onBackClick()
{
    if (btnParent)
        btnParent->blockSignals(false);

    if (flash) {
        flash->dynamicCall("Rewind()");
        delete flash;
        flash = NULL;
    }

    if (flashP_)
        flashP_->dynamicCall("Play()");

    stackWidget->setCurrentWidget(parentPage);
}

bool FlashPage::eventFilter(QObject *obj, QEvent *event)
{
    if(obj==flash)
    {
        if (event->type()==QEvent::MouseButtonDblClick)           //判断类型
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton)	//判断左键
            {
                if (timer->isActive())
                    clicked_num--;
               onDoubleClick();
            }
        } else if (event->type()==QEvent::MouseButtonPress)           //判断类型
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton)	//判断左键
            {
                clicked_num++;
                timer->start(500);
            }
        }
    }

    return false;
}

void FlashPage::resizeFlash()
{
    if (flash)
        flash->setGeometry(stackWidget->x()+174,stackWidget->y()+116,640,465);
}

void FlashPage::onDoubleClick()
{
    if (isFullScreen) {
        isFullScreen = false;
        emit showNormal();

        flash->setParent(rootWidget,0);
        flash->setWindowFlags(Qt::FramelessWindowHint);
        flash->setGeometry(stackWidget->x()+174,stackWidget->y()+116,640,465);
        flash->show();
    } else {
        isFullScreen = true;
        emit fullScreen();
        qDebug() << "flash 全屏" << rootWidget->width() << QApplication::desktop()->height();
        flash->setParent(0,0);
        flash->showFullScreen();
    }
}

void FlashPage::onClick()
{

    bool flashPlaying = flash->dynamicCall("Playing").toBool();
    if (flashPlaying)
        flash->dynamicCall("StopPlay()");
    else
        flash->dynamicCall("Play()");
}

void FlashPage::timerOut()
{
    if (clicked_num == 1)
        onClick();

    clicked_num = 0;
    if (timer->isActive())
        timer->stop();
}
