#include "imagebutton.h"
#include <QDebug>

ImageButton::ImageButton(QWidget *parent)
    : QWidget(parent),m_normalShowImg(NULL),m_pressShowImg(NULL),
    nButtonStatus(IMAGE_BUTTON_UP),fArc(0),m_pTimer(NULL)
{
    m_pTimer=new QTimer (this);
    connect(m_pTimer,SIGNAL(timeout()),this,SLOT(onClickedOutTime()));
}
//长按超时
void ImageButton::onClickedOutTime()
{
    m_pTimer->stop();
    isClickedSendEmit = true;
    emit longClicked();
}
/**
设置普通图片路劲,图片不存在将返回false
*/
bool ImageButton::setNormalImage(QString filePath)
{
    bool isFileExits = QFile::exists(filePath);
    if (!isFileExits)
    {
        return false;
    }
    if (m_normalShowImg == NULL)
    {
        m_normalShowImg = new QPixmap();
    }
    else
    {
        if (m_normalShowImg != NULL)
        {
            delete m_normalShowImg;
            m_normalShowImg = new QPixmap();
        }
    }
    isFileExits = m_normalShowImg->load(filePath);
    update();
    return isFileExits;
}
/**
设置按下去图片路径,图片不存在将返回false
*/
bool ImageButton::setPressImage(QString filePath)
{
    bool isFileExits =QFile::exists(filePath);
    if (!isFileExits)
    {
        return false;
    }
    if (m_pressShowImg == NULL)
    {
        m_pressShowImg = new QPixmap();
    }
    else
    {
        if (m_pressShowImg != NULL)
        {
            delete m_pressShowImg;
            m_pressShowImg = new QPixmap();
        }
    }
    return m_pressShowImg->load(filePath);
}
/**
设置普通图片,图片设置失败将返回false
*/
bool ImageButton::setNormalImage(QPixmap &image)
{
    if (m_normalShowImg == NULL)
    {
        m_normalShowImg = new QPixmap(image);
    }
    else
    {
        if (m_normalShowImg != NULL)
        {
            delete m_normalShowImg;
            m_normalShowImg = new QPixmap(image);
        }
    }
    return true;
}
/**
设置按下图片,图片设置失败将返回false
*/
bool ImageButton::setPressImage(QPixmap &image)
{
    if (m_pressShowImg == NULL)
    {
        m_pressShowImg = new QPixmap(image);
    }
    else
    {
        if (m_pressShowImg != NULL)
        {
            delete m_pressShowImg;
            m_pressShowImg = new QPixmap(image);
        }
    }
    return true;
}
//设置按钮弧度
void ImageButton::setImageArc(float arc)
{
    fArc = arc;
}
void ImageButton::mousePressEvent(QMouseEvent *event)
{
    isClickedSendEmit = false;
    m_pTimer->start(CLICKED_OUT_TIME);
    nButtonStatus = IMAGE_BUTTON_DWON;
    update();
    QWidget::mousePressEvent(event);
}
void ImageButton::mouseMoveEvent(QMouseEvent *event)
{
    nButtonStatus = IMAGE_BUTTON_MOVE;
    update();
    QWidget::mouseMoveEvent(event);
}
//双击事件
void ImageButton::mouseDoubleClickEvent(QMouseEvent *event)
{
    emit doubleClicked();
    isClickedSendEmit=true;
    QWidget::mouseDoubleClickEvent(event);
}
void ImageButton::mouseReleaseEvent(QMouseEvent *event)
{
    m_pTimer->stop();
    nButtonStatus = IMAGE_BUTTON_UP;
    update();
    //如果没有发送点击事件就发送
    if (!isClickedSendEmit)
    {
        emit clicked();
    }
    QWidget::mouseReleaseEvent(event);
}

void ImageButton::enterEvent(QEvent *event)
{
   nButtonStatus = IMAGE_BUTTON_ENTER;
   update();
   QWidget::enterEvent(event);
}

void ImageButton::leaveEvent(QEvent *event)
{
    nButtonStatus = IMAGE_BUTTON_LEAVE;
    update();
    QWidget::leaveEvent(event);
}

void ImageButton::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    drawImageButton(painter);
    painter.setPen(QColor(Qt::white));
    QWidget::paintEvent(event);
}

ImageButton::~ImageButton()
{

}

void ImageButton::drawImageButton(QPainter &painter)
{
    QPixmap nowImage(size());
    if(nButtonStatus == IMAGE_BUTTON_UP)
    {
        if (m_normalShowImg != NULL)
        {
            nowImage =m_normalShowImg->scaled(width(),height());
        }
    }
    else if (nButtonStatus == IMAGE_BUTTON_DWON)
    {
        if (m_pressShowImg != NULL)
        {
            nowImage =m_pressShowImg->scaled(width(),height());
        }
    }
    else if(nButtonStatus == IMAGE_BUTTON_MOVE)
    {
        if (m_pressShowImg != NULL)
        {
            nowImage =m_pressShowImg->scaled(width(),height());
        }
    }
    else if (nButtonStatus == IMAGE_BUTTON_ENTER)
    {
        if (m_pressShowImg != NULL)
        {
            nowImage =m_pressShowImg->scaled(width(),height());
        }
    }
    else if (nButtonStatus == IMAGE_BUTTON_LEAVE)
    {
        if (m_normalShowImg != NULL)
        {
            nowImage =m_normalShowImg->scaled(width(),height());
        }
    }

    painter.setPen(Qt::NoPen);
    painter.setRenderHint(QPainter::Antialiasing); // 反锯齿;
    nowImage=QPixmapToRound(nowImage,fArc);
    painter.drawPixmap(0,0,nowImage);
}

//图片圆角算法
QPixmap ImageButton::QPixmapToRound(const QPixmap & img, int radius)
{
    if (img.isNull())
    {
        return QPixmap();
    }
    QSize size(img.size());
    QBitmap mask(size);
    QPainter painter(&mask);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.setRenderHint(QPainter::SmoothPixmapTransform);
    painter.fillRect(mask.rect(), Qt::white);
    painter.setBrush(QColor(0, 0, 0));
    painter.drawRoundedRect(mask.rect(), radius, radius);
    QPixmap image = img;// .scaled(size);
    image.setMask(mask);
    return image;
}

void ImageButton::keyPressEvent(QKeyEvent *event)
{
    if (event->key() == Qt::Key_Return)
        emit enterPressed();
}
