#include "itemwidget.h"
#include "ui_itemwidget.h"
#include <QDebug>
#include <QFont>

ItemWidget::ItemWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::Form)
{
    ui->setupUi(this);
    rectLabel = new QLabel(this);
    titleLabel = new QLabel(this);
    listWidget = new QListWidget(this);
    sztxWord = new QListWidget(this);
    sztxWord->hide();
    sztxGroupWord = new QListWidget(this);
    sztxGroupWord->hide();
    sztxIdiom = new QListWidget(this);
    sztxIdiom->hide();

    szzbSub = SzzbSubPage::getSzzbSubInstance();
    middle = Middle::getMiddleInstance();
}

void ItemWidget::setWidget(int width, int height, QStringList list, QString type_, QString grade_, QString level_,
                           QString index, QWidget *parent, QWidget *next, QStackedWidget *stack)
{
    this->setFixedSize(width, height);
    this->setStyleSheet("background-color:transparent");



    type = type_;
    grade = grade_;
    level = level_;
    parentPage = parent;
    nextPage = next;
    stackWidget = stack;

    rectLabel->setGeometry(1,1,width-10,height-10);
    rectLabel->setStyleSheet("border-width: 0px; border-style: solid; border-radius:8px;"
                             "background-color:white");

    titleLabel->setGeometry(37, 8, 50, 50);
    titleLabel->setText(index);
    titleLabel->setStyleSheet("font-size: 25px; color:rgb(74,179,234); font-family:宋体");

    listWidget->clear();
    listWidget->setGeometry(25,35,width-60,height-60);
    listWidget->setFocusPolicy(Qt::NoFocus);
    //设置QListWidget中单元项的间距
    listWidget->setSpacing(15);
    //设置QListWidget的显示模式
    listWidget->setViewMode(QListView::IconMode);
    //设置QListWidget中单元项的图片大小
    listWidget->setIconSize(QSize(93,36));
    //设置自动适应布局调整（Adjust适应，Fixed不适应），默认不适应
    listWidget->setResizeMode(QListWidget::Adjust);
    //设置不能移动
    listWidget->setMovement(QListWidget::Static);
    //去掉边框
    listWidget->setFrameStyle(0);
    // 去掉滚动条
    listWidget->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    listWidget->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    //去掉选中的背景框
    listWidget->setSelectionMode(QAbstractItemView::NoSelection);
    listWidget->setCursor(QCursor(Qt::PointingHandCursor));

    listWidget->setStyleSheet("QListWidget{font-size: 20px; font-family:HaiCi Dict}"
                               "QListWidget::item:hover{background-color: rgb(251,151,5);border-color:rgb(251,151,5)}"
                               "QListWidget::item:hover{color: white}"
                               "QListWidget::item{border-width: 1px; border-style: solid; border-radius:8px; border-color: rgb(73, 127, 191);}"
                               "QListWidget{background-color:transparent}");

    for (int i=0; i<list.count(); i++) {
        QListWidgetItem *item = new QListWidgetItem;
        item->setText(list[i]);
        item->setSizeHint(QSize(93,36));
        listWidget->addItem(item);
    }

    connect(listWidget, SIGNAL(clicked(QModelIndex)), this, SLOT(onItemClicked(QModelIndex)));
}

void ItemWidget::setSztxItemWidget(int width, int height, QList<QVariant> list, QString type)
{
    QFont font;
    QLabel *label = new QLabel(this);

    this->setFixedSize(width, height);
    this->setStyleSheet("background-color:transparent");

    label->setGeometry(0,0,width,height);
    label->setStyleSheet("border-width: 0px;"
                         "background-color:white");

    font.setFamily("华文楷体");
    font.setPixelSize(40);
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);
    font.setBold(true);

    if (type == "word") {
//        sztxWord = new QListWidget(this);
        sztxWord->show();
        sztxWord->raise();
        sztxWord->setFont(font);
        sztxWord->setSelectionMode(QAbstractItemView::MultiSelection);
        sztxWord->setGeometry(0,0,width,height);
        //设置QListWidget中单元项的间距
        sztxWord->setSpacing(15);
        //设置QListWidget的显示模式
        sztxWord->setViewMode(QListView::IconMode);
        //设置QListWidget中单元项的图片大小
//        sztxWord->setIconSize(QSize(93,36));
        //设置自动适应布局调整（Adjust适应，Fixed不适应），默认不适应
        sztxWord->setResizeMode(QListWidget::Adjust);
        //设置不能移动
        sztxWord->setMovement(QListWidget::Static);
        //去掉边框
        sztxWord->setFrameStyle(0);
        // 去掉滚动条
        sztxWord->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        sztxWord->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        sztxWord->setCursor(QCursor(Qt::PointingHandCursor));

        sztxWord->setStyleSheet("QListWidget::item:hover{background-color: rgb(251,151,5);border-color:rgb(251,151,5)}"
                                   "QListWidget::item:selected{background:lightgray;}"
                                   "QListWidget::item{border-width: 1px; border-style: solid; border-radius:8px; border-color: rgb(255,0,0);}"
                                   "QListWidget{background-color:transparent}");

        foreach (QVariant word, list) {
            QListWidgetItem *item = new QListWidgetItem;
            item->setData(Qt::WhatsThisRole, middle->getBaseUrl()+word.toMap()["listen"].toString());
            item->setText(word.toMap()["zi"].toString());
            item->setSizeHint(QSize(100,65));
            sztxWord->addItem(item);
        }
    }

    if (type == "groupword") {
//        sztxGroupWord = new QListWidget(this);
        sztxGroupWord->show();
        sztxGroupWord->raise();
        sztxGroupWord->setFont(font);
        sztxGroupWord->setSelectionMode(QAbstractItemView::MultiSelection);
        sztxGroupWord->setGeometry(0,0,width,height);
        //设置QListWidget中单元项的间距
        sztxGroupWord->setSpacing(15);
        //设置QListWidget的显示模式
        sztxGroupWord->setViewMode(QListView::IconMode);
        //设置自动适应布局调整（Adjust适应，Fixed不适应），默认不适应
        sztxGroupWord->setResizeMode(QListWidget::Adjust);
        //设置不能移动
        sztxGroupWord->setMovement(QListWidget::Static);
        //去掉边框
        sztxGroupWord->setFrameStyle(0);
        // 去掉滚动条
        sztxGroupWord->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        sztxGroupWord->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        sztxGroupWord->setCursor(QCursor(Qt::PointingHandCursor));

        sztxGroupWord->setStyleSheet( "QListWidget::item:hover{background-color: rgb(251,151,5);border-color:rgb(251,151,5)}"
                                   "QListWidget::item:selected{background:lightgray;}"
                                   "QListWidget::item{border-width: 1px; border-style: solid; border-radius:8px; border-color: rgb(255,0,0);}"
                                   "QListWidget{background-color:transparent}");

        foreach (QVariant word, list) {
            QListWidgetItem *item = new QListWidgetItem;
            item->setData(Qt::WhatsThisRole, middle->getBaseUrl()+word.toMap()["listen"].toString());
            item->setText(word.toMap()["zi"].toString());
            item->setSizeHint(QSize(150,65));
            sztxGroupWord->addItem(item);
        }
    }

    if (type == "idiom") {
//        sztxIdiom = new QListWidget(this);
        sztxIdiom->show();
        sztxIdiom->raise();
        sztxIdiom->setFont(font);
        sztxIdiom->setSelectionMode(QAbstractItemView::MultiSelection);
        sztxIdiom->setGeometry(0,0,width,height);
        //设置QListWidget中单元项的间距
        sztxIdiom->setSpacing(15);
        //设置QListWidget的显示模式
        sztxIdiom->setViewMode(QListView::IconMode);
        //设置自动适应布局调整（Adjust适应，Fixed不适应），默认不适应
        sztxIdiom->setResizeMode(QListWidget::Adjust);
        //设置不能移动
        sztxIdiom->setMovement(QListWidget::Static);
        //去掉边框
        sztxIdiom->setFrameStyle(0);
        // 去掉滚动条
        sztxIdiom->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        sztxIdiom->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        sztxIdiom->setCursor(QCursor(Qt::PointingHandCursor));

        sztxIdiom->setStyleSheet("QListWidget::item:hover{background-color: rgb(251,151,5);border-color:rgb(251,151,5)}"
                                   "QListWidget::item:selected{background:lightgray;}"
                                   "QListWidget::item{border-width: 1px; border-style: solid; border-radius:8px; border-color: rgb(255,0,0);}"
                                   "QListWidget{background-color:transparent}");

        foreach (QVariant word, list) {
            QListWidgetItem *item = new QListWidgetItem;
            item->setData(Qt::WhatsThisRole, middle->getBaseUrl()+word.toMap()["listen"].toString());
            item->setText(word.toMap()["idiom"].toString());
            item->setSizeHint(QSize(210,65));
            sztxIdiom->addItem(item);
        }
    }
}

void ItemWidget::onItemClicked(QModelIndex index)
{
    QString str = index.data().toString();
    qDebug() << str << type;
    listWidget->blockSignals(true);
    stackWidget->setCurrentWidget(nextPage);
    szzbSub->setSzzbSubPage(parentPage, listWidget, str, type, grade, level);
}

void ItemWidget::selectAllItems()
{
    qDebug() << "select all items";
    sztxWord->selectAll();
    sztxGroupWord->selectAll();
    sztxIdiom->selectAll();
}

void ItemWidget::clearAllItems()
{
    sztxWord->clearSelection();
    sztxGroupWord->clearSelection();
    sztxIdiom->clearSelection();
}

QList<QListWidgetItem*> ItemWidget::getSelectedItems()
{
    QList<QListWidgetItem*> list, wordList, groupWordList, idiomList;

    wordList = sztxWord->selectedItems();
    groupWordList = sztxGroupWord->selectedItems();
    idiomList = sztxIdiom->selectedItems();

    list.append(wordList);
    list.append(groupWordList);
    list.append(idiomList);

    return list;
}

ItemWidget::~ItemWidget()
{
    delete ui;
}
