#include "login.h"
#include "imagebutton.h"
#include "middle.h"
#include "loading.h"
#include "download.h"
#include <QLabel>
#include <QFont>
#include <QPixmap>

Login::Login(QObject *parent) : QObject(parent)
{
    middle = Middle::getMiddleInstance();
    welcome = WelcomePage::getWelcomeInstance();
    tool = Tool::getToolInstance();
}

void Login::setWidget(QWidget *loginPage, QStackedWidget *stack, QWidget *welcomePage, QMenuBar *bar)
{
    widget = loginPage;
    welPage = welcomePage;
    stackWidget = stack;
    menuBar = bar;
}

void Login::setLoginPage()
{
    widget->setStyleSheet("QWidget#loginPage{image:url(:/login/bj_20.jpg)}");

    QLabel *codeLabel = new QLabel(widget);
    codeLabel->setStyleSheet("image:url(:/login/ico_11.png)");
    codeLabel->setGeometry(337,266,47,47);

    codeLine = new QLineEdit(widget);
    codeLine->setGeometry(384,266,272,47);
    codeLine->setFont(QFont( "Timers" , 16 , QFont::Bold));
    codeLine->setFocusPolicy(Qt::ClickFocus);
    codeLine->setPlaceholderText("请输入编码");

    QLabel *serialLabel = new QLabel(widget);
    serialLabel->setStyleSheet("image:url(:/login/ico_12.png)");
    serialLabel->setGeometry(337,350,47,47);

    serialLine = new QLineEdit(widget);
    serialLine->setGeometry(383,350,272,47);
    serialLine->setFont(QFont( "Timers" , 16, QFont::Bold));
    serialLine->setFocusPolicy(Qt::ClickFocus);
    serialLine->setPlaceholderText("请输入序列号");

    // 登录按钮
    ImageButton *loginButton = new ImageButton(widget);
    loginButton->setGeometry(337,437,317,47);
    loginButton->setMinimumSize(317,47);
    QPixmap loginP1(":/login/login.jpg");
    QPixmap loginP2(":/login/login.jpg");
    loginButton->setNormalImage(loginP1);
    loginButton->setPressImage(loginP2);
    loginButton->setCursor(QCursor(Qt::PointingHandCursor));

    QObject::connect(loginButton, SIGNAL(clicked()), this, SLOT(onLoginClick()));
//    QObject::connect(loginButton, SIGNAL())
}


void Login::setLoginInfo()
{
    QString school_number = tool->readIni("SchoolNumber");
    QString serial = tool->readIni("Serial");
    if (school_number.length()>0 && serial.length()>0) {
        codeLine->setText(school_number);
        serialLine->setText(serial);
        bAutoLogin = true;
    }
    else
    {
         bAutoLogin = false;
    }

}

void Login::onLoginClick()
{
    Loading *loading = new Loading();       //创建加载窗口


    QString msg;
    if (codeLine->text().length() == 0 or serialLine->text().length() == 0) {
        middle->errorMBox("编码或序列号不能为空!");
        return;
    }
    loading->show();                        //显示加载窗口
    RestAPI *restAPI = RestAPI::getRestApiInstance();
    QByteArray replay = restAPI->loginAPIServer("apiuser","123456");
    if(replay.isNull()) {
        bLoginSuccess = false;                          //登录失败标记
        loading->hide();
        middle->errorMBox("无法访问服务器，请检查本地网络！");
        return;
    }
    int ret = middle->login(msg);
    if (ret != 0 && ret != 2)
        ret = middle->enrollSchool(codeLine->text(), serialLine->text(), msg);
    qDebug() << "注册返回结果" << ret;
    if (ret == 0) {
//        menuBar->show();
        tool->writeIni("SchoolNumber", codeLine->text());
        tool->writeIni("Serial", serialLine->text());
        tool->writeIni("Flag", "1");
        middle->loginServer();                  //登录服务器
        welcome->setWelcomePage();
        stackWidget->setCurrentWidget(welPage);
        bLoginSuccess = true;                          //登录成功标记
        loading->hide();
        Download * down = new Download();
    } else if (ret == -99) {
        bLoginSuccess = false;                          //登录失败标记
        loading->hide();
        middle->errorMBox("无法访问服务器，请检查本地网络！");
        return;
    } else if (ret == 2){ // 账号被禁用
        tool->writeIni("SchoolNumber", "");
        tool->writeIni("Serial", "");
        tool->writeIni("Flag", "0");
        bLoginSuccess = false;
        loading->hide();
        middle->errorMBox(msg);
    } else{
        bLoginSuccess = false;                          //登录失败标记
        loading->hide();
        middle->errorMBox(msg);
    }

    delete loading;                        //删除加载窗口


}
