#include "querypage.h"
#include <QDebug>

QueryPage* QueryPage::queryInstance = NULL;

QueryPage::QueryPage(QWidget *parent) : QWidget(parent)
{
    middle = Middle::getMiddleInstance();
    querySubPage = QuerySubPage::getQuerySubInstance();
}

QueryPage* QueryPage::getQueryInstance()
{
    if (queryInstance == NULL)
        queryInstance = new QueryPage();

    return queryInstance;
}

void QueryPage::setWidget(QWidget *query, QStackedWidget *stack, QWidget *next)
{
    queryPage = query;
    nextPage = next;
    stackWidget = stack;

    // 页面元素初始化
//    bgLabel = new QLabel(queryPage);
    line = new QLineEdit(queryPage);
    line->hide();
    btnQuery = new ImageButton(queryPage);
    btnBack = new ImageButton(queryPage);

    connect(line, SIGNAL(returnPressed()), this, SLOT(onEnterClick()));
    connect(btnQuery, SIGNAL(clicked()), this, SLOT(onEnterClick()));
    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
}

void QueryPage::setQueryPage(QWidget *parent, ImageButton *btnP)
{
    parentPage = parent;
    btnParent = btnP;

    setBackground();
    setButtons();
}

void QueryPage::setBackground()
{
    queryPage->setStyleSheet("QWidget#queryPage{image:url(:/chaxun/bj_12.jpg)}");
//    bgLabel->setGeometry(0,0,1000,700);
//    bgLabel->lower();
//    QPixmap pix;
//    pix.loadFromData(middle->getImageData2("http://www.cguoguo.org.cn/jfx-web/attachment/aggxz/bjpic/bj_12.jpg"));
//    bgLabel->setPixmap(pix);

    line->setGeometry(330,242,220,54);
    line->show();
    line->setStyleSheet("border-width: 4px; border-style: solid; border-radius:8px; border-color: rgb(255, 170, 0); font-size: 20px; font-family:宋体");
    line->setPlaceholderText("支持多字查询");
}

void QueryPage::setButtons()
{
    // 查询
    btnQuery->setGeometry(566,242,109,54);
    QPixmap queryP1(":/chaxun/cx_01.png");
    QPixmap queryP2(":/chaxun/cx_01.png");
    btnQuery->setNormalImage(queryP1);
    btnQuery->setPressImage(queryP2);
    btnQuery->setCursor(QCursor(Qt::PointingHandCursor));
    btnQuery->setFocus();

    btnBack->setGeometry(581,393,109,101);
    QPixmap backP1(":/chaxun/fh_04.png");
    QPixmap backP2(":/chaxun/fh_03.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void QueryPage::onEnterClick()
{
    QString words = line->text();
    QList< QList<QString> > wordList;

    words.remove(QRegExp("\\s"));
    if (words.length() == 0) {
        middle->errorMBox("请至少输入一个汉字");
        return;
    }

    wordList = this->getWordList(words);

    if (wordList.count() == 0) {
        middle->errorMBox("未查询到字: " + words);
        return;
    }

    btnQuery->blockSignals(true);
    stackWidget->setCurrentWidget(nextPage);
    querySubPage->setQuerySubPage(queryPage, btnQuery, wordList);

    line->clear();
    line->clearFocus();
}

void QueryPage::onBackClick()
{
    line->clear();
    line->clearFocus();
    btnParent->blockSignals(false);
    stackWidget->setCurrentWidget(parentPage);
}

QList< QList<QString> > QueryPage::getWordList(QString words)
{
    QList< QList<QString> > wordList;

    for (int i=0; i<words.length(); i++) {
        QList<QString> info;
        QString word = QString(words[i]);
        QList<QVariant> wordInfo = middle->getWordInfo(word);
        if (wordInfo.count() == 0)
            continue;
        info.append(wordInfo[0].toMap()["img_url"].toString());
        info.append(word);

        wordList.append(info);
    }

    return wordList;
}
