#include "queryplaypage.h"
#include <QAbstractItemView>
#include <QtDebug>

QueryPlayPage* QueryPlayPage::queryPlayInstance = NULL;

QueryPlayPage::QueryPlayPage(QWidget *parent) : QWidget(parent)
{
    middle = Middle::getMiddleInstance();
    videoPage = VideoPage::getVideoPageInstance();
}

QueryPlayPage* QueryPlayPage::getQueryPlayPageInstance()
{
    if (queryPlayInstance == NULL)
        queryPlayInstance = new QueryPlayPage();

    return queryPlayInstance;
}

void QueryPlayPage::setWidget(QWidget *queryPlay, QStackedWidget *stack, QWidget *next)
{
    queryPlayPage = queryPlay;
    stackWidget = stack;
    nextPage = next;
    type = "bs";

    // 页面元素初始化
    image = new QLabel(queryPlayPage);
    imageList = new QListWidget(queryPlayPage);
    imageList->hide();
    btnOk = new ImageButton(queryPlayPage);
    btnBack = new ImageButton(queryPlayPage);
    checkBox = new QCheckBox(queryPlayPage);
    title = new QLabel(queryPlayPage);

    bssfButton = new ImageButton(queryPlayPage);
    mbsfButton = new ImageButton(queryPlayPage);
    gbsfButton = new ImageButton(queryPlayPage);

    connect(btnOk, SIGNAL(clicked()), this, SLOT(onOkClick()));
    connect(checkBox, SIGNAL(clicked(bool)), this, SLOT(onCheckBoxClick(bool)));
    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
    connect(bssfButton, SIGNAL(clicked()), this, SLOT(bssfClick()));
    connect(gbsfButton, SIGNAL(clicked()), this, SLOT(gbsfClick()));
    connect(mbsfButton, SIGNAL(clicked()), this, SLOT(mbsfClick()));
}

void QueryPlayPage::setQueryPlayPage(QWidget *parent, ImageButton *btnP, QList< QList<QString> > wordList)
{
    parentPage = parent;
    btnParent = btnP;

    setBackground();
    setListView(wordList);
    setButtons();
}

void QueryPlayPage::setBackground()
{
    QFont fontCheck, fontTitle;
    QPalette pa;

    pa.setColor(QPalette::WindowText,QColor(251,151,4));

    fontCheck.setFamily("楷体");
    fontCheck.setPixelSize(25);
    fontCheck.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontCheck.setBold(true);

    fontTitle.setFamily("方正新舒体简体");
    fontTitle.setPixelSize(24);
    fontTitle.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontTitle.setBold(true);

    queryPlayPage->setStyleSheet("QWidget#queryPlayPage{image:url(:/chaxun/bj_17.jpg)}");

    image->setGeometry(50,130,907,535);
    image->setStyleSheet("image:url(:/chaxun/xhbf_bg.png)");

    checkBox->setGeometry(797,168,100,50);
    checkBox->setText("全选");
    checkBox->setFont(fontCheck);

    title->setGeometry(400, 155,200,60);
    title->setFont(fontTitle);
    title->setPalette(pa);
    title->setText("请选择播放内容");
}

void QueryPlayPage::setListView(QList< QList<QString> > wordList)
{
    imageList->clear();
    imageList->setGeometry(130,221,685,310);
    imageList->setSelectionMode(QAbstractItemView::MultiSelection);
    imageList->show();
    //设置QListWidget的显示模式
    imageList->setViewMode(QListView::IconMode);
    //设置QListWidget中单元项的图片大小
    imageList->setIconSize(QSize(80,80));
    //设置QListWidget中单元项的间距
    imageList->setSpacing(12);
    //设置自动适应布局调整（Adjust适应，Fixed不适应），默认不适应
    imageList->setResizeMode(QListWidget::Adjust);
    //设置不能移动
    imageList->setMovement(QListWidget::Static);
    //去掉边框
    imageList->setFrameStyle(0);
    //设置鼠标样式
    imageList->setCursor(QCursor(Qt::PointingHandCursor));

    for (int j=0; j<wordList.count(); j++) {
        //定义QListWidgetItem对象
        QListWidgetItem *imageItem = new QListWidgetItem;
        //为单元项设置属性
        QPixmap image;
        //TODO
        // 添加BaseUrl
//        image.loadFromData(middle->getImageData(wordList[j][0]));
        QString imageFile = middle->getImageFile(wordList[j][0], "/zi/");
        if (imageFile.length() > 0)
            image.load(imageFile);
        else
            image.loadFromData(middle->getImageData(wordList[j][0]));
        imageItem->setIcon(QIcon(image));
        imageItem->setData(Qt::WhatsThisRole, wordList[j][1]);
        //重新设置单元项图片的宽度和高度
        imageItem->setSizeHint(QSize(80,80));
        //将单元项添加到QListWidget中
        imageList->addItem(imageItem);
    }
}

void QueryPlayPage::setButtons()
{
    // 板书示范
    bssfButton->setGeometry(200,540,166,48);
    bssfButton->setMinimumSize(120,35);
    QPixmap bssfP1(":/nianji/erji/nj_bssf01.png");
    QPixmap bssfP2(":/nianji/erji/nj_bssf02.png");
    bssfButton->setNormalImage(bssfP1);
    bssfButton->setPressImage(bssfP2);
    bssfButton->setImageArc(10);
    bssfButton->setCursor(QCursor(Qt::PointingHandCursor));

    // 钢笔示范
    gbsfButton->setGeometry(400,540,166,48);
    gbsfButton->setMinimumSize(120,35);
    QPixmap gbsfP1(":/nianji/erji/nj_gbsf01.png");
    QPixmap gbsfP2(":/nianji/erji/nj_gbsf02.png");
    gbsfButton->setNormalImage(gbsfP1);
    gbsfButton->setPressImage(gbsfP2);
    gbsfButton->setImageArc(10);
    gbsfButton->setCursor(QCursor(Qt::PointingHandCursor));

    // 毛笔示范
    mbsfButton->setGeometry(600,540,166,48);
    mbsfButton->setMinimumSize(120,35);
    QPixmap mbsfP1(":/nianji/erji/nj_mbsf01.png");
    QPixmap mbsfP2(":/nianji/erji/nj_mbsf02.png");
    mbsfButton->setNormalImage(mbsfP1);
    mbsfButton->setPressImage(mbsfP2);
    mbsfButton->setImageArc(10);
    mbsfButton->setCursor(QCursor(Qt::PointingHandCursor));

    // 确定
    btnOk->setGeometry(865,410,109,101);
    QPixmap okP1(":/nianji/tingxie/sztx_qd.png");
    QPixmap okP2(":/nianji/tingxie/sztx_qd.png");
    btnOk->setNormalImage(okP1);
    btnOk->setPressImage(okP2);
    btnOk->setCursor(QCursor(Qt::PointingHandCursor));

    // 取消
    btnBack->setGeometry(852,525,109,101);
    QPixmap backP1(":/nianji/tingxie/sztx_qx.png");
    QPixmap backP2(":/nianji/tingxie/sztx_qx.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

QMap<QString ,int> QueryPlayPage::getVideoMaps(QList<QString> words)
{
	QMap<QString ,int> videoMap;

    if (words.count() == 0)
        return videoMap;

    foreach (QString word, words) {
        QList<QVariant> wordInfo = middle->getWordInfo(word);
        if (wordInfo.count() == 0)
            continue;
        if (type == "bs") {
            videoMap.insert(wordInfo[0].toMap()["bs_video_url"].toString(),
                    wordInfo[0].toMap()["bs_video_size"].toInt());
        }
        if (type == "gb") {
            videoMap.insert(wordInfo[0].toMap()["gb_video_url"].toString(),
                    wordInfo[0].toMap()["gb_video_size"].toInt());
        }
        if (type == "mb") {
            videoMap.insert(wordInfo[0].toMap()["mb_video_url"].toString(),
                    wordInfo[0].toMap()["mb_video_size"].toInt());
        }
    }

    return videoMap;
}

void QueryPlayPage::onBackClick()
{
    btnParent->blockSignals(false);
    stackWidget->setCurrentWidget(parentPage);
}

void QueryPlayPage::onOkClick()
{
    QList<QString> words;
    QMap<QString ,int> videoMap;

    btnParent->blockSignals(false);

    QList<QListWidgetItem*> list = imageList->selectedItems();
    foreach (QListWidgetItem *item, list) {
        qDebug() << __FUNCTION__  << item->data(Qt::WhatsThisRole).toString();
        words.append(item->data(Qt::WhatsThisRole).toString());
    }

    if (words.length() == 0) {
        middle->errorMBox("请至少选择一个字");
        return;
    }
    qDebug() << "words:" << words << "type:" << type;
	videoMap = getVideoMaps(words);
    videoPage->setVideoPage("image:url(:/xzzs/bj_11.jpg)", "",
                            "", "image:url(:/xzzs/sjqp.png)",
                            videoMap, btnOk, queryPlayPage, "ggxz");

	stackWidget->setCurrentWidget(nextPage);
}

void QueryPlayPage::onCheckBoxClick(bool check)
{
    qDebug() << __FUNCTION__  << check;
    if (check) {
        imageList->selectAll();
    } else {
        imageList->clearSelection();
    }
}

void QueryPlayPage::bssfClick()
{
    type = "bs";
}

void QueryPlayPage::gbsfClick()
{
    type = "gb";
}

void QueryPlayPage::mbsfClick()
{
    type = "mb";
}
