#include "restapi.h"
#include <QCoreApplication>
#include <QString>
#include <QtNetwork/QNetworkAccessManager>
#include <QtNetwork/QNetworkCookie> //单个cookie
#include <QtNetwork/QNetworkCookieJar> //储存cookie
#include <QtNetwork/QNetworkRequest>
#include <QtNetwork/QNetworkReply>
#include <QTimer>
#include <iostream>
#include <QDebug>
#include <QEventLoop>

RestAPI* RestAPI::restApiInstance = NULL;

RestAPI::RestAPI(QObject *parent) : QObject(parent)
{
    middle = Middle::getMiddleInstance();
}

RestAPI* RestAPI::getRestApiInstance()
{
    if (restApiInstance == NULL) {
        QString url = Tool::readXml("BaseUrl") + "/admin/";
        restApiInstance = new RestAPI(url) ;
    }
    return restApiInstance;
}

RestAPI::RestAPI(QString url)
{
    m_BaseURL = url;//构造对象时初始化根路径URL
    m_NetManager = new QNetworkAccessManager;
    m_CookieJar = new QNetworkCookieJar(this);
    m_QTimer = new QTimer(this);
//    connect( m_QTimer,SIGNAL(timeout()), this, SLOT(slot_SendHeartBeat()) );
}

QByteArray RestAPI::loginAPIServer(QString user, QString passwd)
{
    QString url = m_BaseURL + "login?username=" + user + "&password=" + passwd;
    m_NetManager->setCookieJar(m_CookieJar);

    QByteArray reply = getReply(url);
    QList<QNetworkCookie>  cookies = m_NetManager->cookieJar()->cookiesForUrl(QUrl(url));
    qDebug() << __FUNCTION__ << "COOKIES for" << url << cookies;

    return reply;
}

QByteArray RestAPI::enrollSchool(QString school_number, QString serial, QString hw_info, QString version)
{
    QString url = m_BaseURL + "clientrecord/enroll?" +
            "&school_number=" + school_number + "&serial=" + serial +
            "&hw_info=" + hw_info + "&version=" + version;
    QByteArray reply = getReply(url);

    //处理注册信息
    //result是错误码
    //0：注册成功；1001：学校不存在；1002：序列号不存在；1003：序列号已满；1004：超时；1005：失败
//    int ret = tool->jsonParserString(reply, "errorCode").toInt();


    return reply;
}

int RestAPI::denrollSchool(QString school_number, QString serial, QString hw_info)
{
    QString url = m_BaseURL + "clientrecord/denroll?" +
            "&school_number=" + school_number + "&serial=" + serial + "&hw_info=" + hw_info;
    QByteArray reply = getReply(url);

    //处理注销信息
    //result是错误码
    //0：注销成功；1001：学校不存在；1002：序列号不存在；1003：序列号已满；1004：超时；1005：失败
    int ret = tool->jsonParserString(reply, "errorCode").toInt();


    return ret;
}

QByteArray RestAPI::login(QString school_number, QString serial, QString hw_info, QString version)
{
    QString url = m_BaseURL + "loginrecord/login?" +
            "&school_number=" + school_number + "&serial=" + serial +
            "&hw_info=" + hw_info + "&version=" + version;
    QByteArray reply = getReply(url);

    //处理注销信息
    //result是错误码
    //0：登录成功；1001：学校不存在；1002：序列号不存在；1003：序列号已满；1004：超时；1005：失败
    int ret = tool->jsonParserString(reply, "errorCode").toInt();
    qDebug() << __FUNCTION__ <<"@@@@Login ret= "<<ret;
    if(ret ==0)
    {
        m_SchoolID = school_number;
        m_Serial = serial;
        m_HWInfo = hw_info;
        qDebug() << __FUNCTION__ <<"@@@@Start timer for sending heart beat";
        m_QTimer->start(300000);//五分钟
    }

    return reply;
}

int RestAPI::logout()
{
    return logoutImpl(m_SchoolID, m_Serial, m_HWInfo);
}

int RestAPI::logoutImpl(QString school_number, QString serial, QString hw_info)
{
    qDebug() << __FUNCTION__ <<"@@@@RestAPI::logoutImpl";
    //先停止心跳Timer
    m_QTimer->stop();

    QString url = m_BaseURL + "loginrecord/logout?" +
            "&school_number=" + school_number + "&serial=" + serial + "&hw_info=" + hw_info;
    QByteArray reply = getReply(url);

    //处理注销信息
    //result是错误码
    //0：退出成功；1001：学校不存在；1002：序列号不存在；1003：序列号已满；1004：超时；1005：失败
    int ret = tool->jsonParserString(reply, "errorCode").toInt();

    return ret;
}

int RestAPI::sendHeartBeat(QString school_number, QString serial, QString hw_info)
{
    QString url = m_BaseURL + "loginrecord/heartbeat?" +
            "&school_number=" + school_number + "&serial=" + serial + "&hw_info=" + hw_info;
    QByteArray reply = getReply(url);

    //处理注销信息
    //result是错误码
    //0：退出成功；1001：学校不存在；1002：序列号不存在；1003：序列号已满；1004：超时；1005：失败
    int ret = tool->jsonParserString(reply, "result").toInt();


    return ret;
}

QByteArray RestAPI::getLatestVersion()
{
    QString url = m_BaseURL + "upgrade/jsonList";
    QByteArray ret = getReply(url);

    return ret;
}

QByteArray RestAPI::getCourseList(QString type)
{
    QString url = m_BaseURL + "module/catalogList?module_code=" + type;
    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getLectureList(QString code)
{
    QString url = m_BaseURL + "module/lectureList?catalog_code=" + code;
    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getLectureWordList(QString code)
{
    QString url = m_BaseURL + "module/lectureWordList?lecture_code=" + code;
    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getCourseWordList(QString course_code)
{
    QString url = m_BaseURL + "course_word/jsonList?" + "course_code=" + course_code;
    QByteArray reply = getReply(url);

    //处理课程生字信息


    return reply;
}

QByteArray RestAPI::getXieZiZiShiList(QString code)
{
    QString url = m_BaseURL + "xzzs/jsonList?code=" + code;
    QByteArray reply = getReply(url);

    //处理写字姿势信息


    return reply;
}

QByteArray RestAPI::getBuShouList(QString group_code)
{
    QString url = m_BaseURL + "bushou/jsonList?" + "group_code=" + group_code;
    QByteArray reply = getReply(url);

    //处理部首信息


    return reply;
}

QByteArray RestAPI::getStrokesList(QString group_code)
{
    QString url = m_BaseURL + "strokes/jsonList?group_code=" + group_code;
    QByteArray reply = getReply(url);

    //处理笔画信息


    return reply;
}

QByteArray RestAPI::getPoemList(QString grade)
{
    QString url = m_BaseURL + "poem/jsonList?" + "grade=" + grade;
    QByteArray reply = getReply(url);

    //处理古诗信息


    return reply;
}

QByteArray RestAPI::getPoemItem(QString poem_id)
{
    QString url = m_BaseURL + "poemitem/jsonList?" + "poem_id=" + poem_id;
    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getHardPenList()
{
    QString url = m_BaseURL + "hardpen/jsonList";
    QByteArray reply = getReply(url);

    //处理硬笔信息


    return reply;
}

QByteArray RestAPI::getCalligraphyList(QString grade, QString level, QString type)
{
    QString url = m_BaseURL + "calligraphy/jsonList?" + "grade=" + grade + "&level="+level + "&type="+type;
    QByteArray reply = getReply(url);

    //处理书法课程信息


    return reply;
}

QByteArray RestAPI::getCalligraphyLessonList(QString calligraphy_id)
{
    QString url = m_BaseURL + "calligraphylesson/jsonList?" + "calligraphy_id=" + calligraphy_id;
    QByteArray reply = getReply(url);

    //处理书法节次课程信息


    return reply;
}

QByteArray RestAPI::getCalligraphyLessonktlxList(QString calligraphy_id)
{
    QString url = m_BaseURL + "calligraphylessonktlx/jsonList?" + "calligraphy_id=" + calligraphy_id;
    QByteArray reply = getReply(url);

    //处理书法节次课程信息


    return reply;
}

QByteArray RestAPI::getWordInfo(QString word)
{
    QString url = m_BaseURL + "word/getWord?" + "zi=" + word;
    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getCalligraphyWord(QString word)
{
    QString url = m_BaseURL + "calligraphy_word/getCalligraphyWord?" + "zi=" + word;
    QByteArray reply = getReply(url);

    return reply;
}

//------------------------- 生字表 ------------------------------------------
QByteArray RestAPI::getPinyinIndexList(QString version, QString grade, QString level)
{
    QString url;
    if (grade.length()>0 && level.length()>0) {
        url = m_BaseURL + "word/getPinyinIndexList?" + "version=" + version + "&grade=" + grade + "&level="+level;
    } else {
        url = m_BaseURL + "word/getPinyinIndexList?" + "version=" + version;
    }

    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getBushouIndexList(QString version, QString grade, QString level)
{
    QString url;
    if (grade.length()>0 && level.length()>0) {
        url = m_BaseURL + "word/getBushouIndexList?" + "version=" + version + "&grade=" + grade + "&level="+level;
    } else {
        url = m_BaseURL + "word/getBushouIndexList?" + "version=" + version;
    }

    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getPinyinList(QString version, QString grade, QString level, QString index)
{
    QString url;
    if (grade.length()>0 && level.length()>0) {
        url = m_BaseURL + "word/getPinyinList?" + "version=" + version + "&grade=" + grade + "&level="+level + "&index="+index;
    } else {
        url = m_BaseURL + "word/getPinyinList?" + "version=" + version + "&index="+index;
    }

    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getBushouList(QString version, QString grade, QString level, QString index)
{
    QString url;
    if (grade.length()>0 && level.length()>0) {
        url = m_BaseURL + "word/getBushouList?" + "version=" + version + "&grade=" + grade + "&level="+level + "&bs_bihua="+index;
    } else {
        url = m_BaseURL + "word/getBushouList?" + "version=" + version + "&bs_bihua="+index;
    }

    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getZiBihuaIndexList(QString version, QString grade, QString level, QString pinyin)
{
    QString url;
    if (grade.length()>0 && level.length()>0) {
        url = m_BaseURL + "word/getZiBihuaIndexList?" + "version=" + version + "&grade=" + grade + "&level="+level + "&wd_pinyin="+pinyin;
    } else {
        url = m_BaseURL + "word/getZiBihuaIndexList?" + "version=" + version + "&wd_pinyin="+pinyin;
    }

    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getWbBihuaIndexList(QString version, QString grade, QString level, QString bushou)
{
    QString url;
    if (grade.length()>0 && level.length()>0) {
        url = m_BaseURL + "word/getWbBihuaIndexList?" + "version=" + version + "&grade=" + grade + "&level="+level + "&bushou="+bushou;
    } else {
        url = m_BaseURL + "word/getWbBihuaIndexList?" + "version=" + version + "&bushou="+bushou;
    }

    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getWordByPinyinBihua(QString version, QString grade, QString level, QString pinyin, QString bihua)
{
    QString url;
    if (grade.length()>0 && level.length()>0) {
        url = m_BaseURL + "word/getWord?" + "version=" + version + "&grade=" + grade + "&level="+level +
                "&wd_pinyin="+pinyin + "&zi_bihua="+bihua;
    } else {
        url = m_BaseURL + "word/getWord?" + "version=" + version + "&wd_pinyin="+pinyin + "&zi_bihua="+bihua;
    }

    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getWordByBushouBihua(QString version, QString grade, QString level, QString bushou, QString bihua)
{
    QString url;
    if (grade.length()>0 && level.length()>0) {
        url = m_BaseURL + "word/getWord?" + "version=" + version + "&grade=" + grade + "&level="+level +
                "&bushou="+bushou + "&bw_bihua="+bihua;
    } else {
        url = m_BaseURL + "word/getWord?" + "version=" + version + "&bushou="+bushou + "&bw_bihua="+bihua;
    }

    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getWordByPinyin(QString version, QString grade, QString level, QString pinyin)
{
    QString url;
    if (grade.length()>0 && level.length()>0) {
        url = m_BaseURL + "word/getWord?" + "version=" + version + "&grade=" + grade + "&level="+level + "&wd_pinyin="+pinyin;
    } else {
        url = m_BaseURL + "word/getWord?" + "version=" + version + "&wd_pinyin="+pinyin;
    }

    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getWordByBushou(QString version, QString grade, QString level, QString bushou)
{
    QString url;
    if (grade.length()>0 && level.length()>0) {
        url = m_BaseURL + "word/getWord?" + "version=" + version + "&grade=" + grade + "&level="+level + "&bushou="+bushou;
    } else {
        url = m_BaseURL + "word/getWord?" + "version=" + version + "&bushou="+bushou;
    }

    QByteArray reply = getReply(url);

    return reply;
}
//------------------------- 生字表 ------------------------------------------

QByteArray RestAPI::getWord(QString version, QString grade, QString level)
{
    QString url = m_BaseURL + "word/getCacheList?" + "version=" + version + "&grade=" + grade + "&level="+level;
    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getCourse(QString version, QString grade, QString level)
{
    QString url = m_BaseURL + "course/getCacheList?" + "version=" + version + "&grade=" + grade + "&level="+level;
    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getCalligraphylesson(QString grade, QString level, QString type)
{
    QString url = m_BaseURL + "calligraphylesson/getCacheList?" + "&grade=" + grade + "&level="+level + "&type="+type;
    QByteArray reply = getReply(url);

    return reply;
}


QByteArray RestAPI::getXzzs()
{
    QString url = m_BaseURL + "xzzs/getCacheList";
    QByteArray reply = getReply(url);

    return reply;
}


QByteArray RestAPI::getBushou()
{
    QString url = m_BaseURL + "bushou/getCacheList";
    QByteArray reply = getReply(url);

    return reply;
}


QByteArray RestAPI::getPoemitem(QString grade)
{
    QString url = m_BaseURL + "poemitem/getCacheList?" + "&grade=" + grade;
    QByteArray reply = getReply(url);

    return reply;
}


QByteArray RestAPI::getOption(QString key)
{
    QString url = m_BaseURL + "option/jsonKey?" + "key=" + key;
    QByteArray ret = getReply(url);

    return ret;
}

QString RestAPI::getBaseUrl()
{
    return m_BaseURL;
}

QByteArray RestAPI::getVersionList()
{
    QString url = m_BaseURL + "getversion";
    QByteArray ret = getReply(url);
    qDebug() << __FUNCTION__ << ret;

    return ret;
}

QByteArray RestAPI::getSwjzInfo(QString word)
{
//    QByteArray tmp = "{\"swjz\":[{\"id\":\"id\", \"zi\":\"府\", \"pinyin\":\"fǔ\", "
//                     "\"bigpicurl\":\"http://www.cguoguo.org.cn/klxz/res/word/common/jbbh/shu/cls.png\", "
//                     "\"explain\":\"注释双方都阿斯蒂芬啊\", \"zhuanpicurl\":\"http://www.cguoguo.org.cn/klxz/res/word/common/jbbh/shu/cls.png\", "
//                     "\"icpicurl\":\"http://www.cguoguo.org.cn/klxz/res/word/common/jbbh/shu/dxs.png\", "
//                     "\"kaiurl\":\"http://www.cguoguo.org.cn/klxz/res/word/common/jbbh/shu/cls.png\", "
//                     "\"caopicurl\":\"http://www.cguoguo.org.cn/klxz/res/word/common/jbbh/shu/dxs.png\", "
//                     "\"xingpicurl\":\"http://www.cguoguo.org.cn/klxz/res/word/common/jbbh/shu/cls.png\"}]}";
//    return tmp;
    QString url = m_BaseURL + "swjz/jsonKey?" + "word=" + word;
    QByteArray reply = getReply(url);

    return reply;
}

QByteArray RestAPI::getReply(QString url)
{
    QTimer timer;
    timer.setInterval(10000);  // 设置超时时间 10 秒
//    timer.setSingleShot(true);  // 单次触发

    QNetworkRequest request;
    QByteArray line;
    QEventLoop eventLoop;
    request.setUrl(QUrl(url));
    qDebug() << __FUNCTION__<<request.url();
    request.setHeader(QNetworkRequest::ContentTypeHeader,"application/json");
    QNetworkReply* reply = m_NetManager->get(request);//非阻塞
    //使用EventLoop进行循环实现调用同步
    QObject::connect(m_NetManager, SIGNAL(finished(QNetworkReply*)), &eventLoop, SLOT(quit()));
    QObject::connect(&timer, SIGNAL(timeout()), &eventLoop, SLOT(quit()));
    timer.start();
    eventLoop.exec();
//    line=reply->readAll();

    if (timer.isActive()) {  // 处理响应
        timer.stop();
        if (reply->error() != QNetworkReply::NoError) {
            // 错误处理
            qDebug() << __FUNCTION__ << "Error String : " << reply->errorString();
        } else {
            line=reply->readAll();
        }
    } else {  // 处理超时
        disconnect(m_NetManager, SIGNAL(finished(QNetworkReply*)), &eventLoop, SLOT(quit()));
        reply->abort();
        reply->deleteLater();
        qDebug() << __FUNCTION__ << "Timeout";
    }

    return line;
}

RestAPI::~RestAPI()
{
    if(m_QTimer != NULL)
        delete m_QTimer;
    if(m_CookieJar != NULL)
        delete m_CookieJar;
    if(m_NetManager != NULL)
        delete m_NetManager;
    if(restApiInstance != NULL)
        delete restApiInstance;
}

int RestAPI::slot_SendHeartBeat()
{
    qDebug() << __FUNCTION__<<"@@@@Timer timeout";
    this->sendHeartBeat(m_SchoolID, m_Serial, m_HWInfo);
    return 0;
}

