#ifndef RESTAPI_H
#define RESTAPI_H

#include <QObject>
#include "tool.h"
#include "middle.h"

class QByteArray;
class QNetworkAccessManager;
class QNetworkCookieJar;
class QTimer;
class Middle;

class RestAPI : public QObject
{
    Q_OBJECT
public:
    explicit RestAPI(QObject *parent = 0);
    RestAPI(QString url);

    static RestAPI* getRestApiInstance();

    /*api授权
     * user:api的用户名默认apiuser
     * passwd:api的密码默认123456
     */
    QByteArray loginAPIServer(QString user, QString passwd);

    /*注册
    * school_number:学校ID
    * serial:序列号
    * hw_info:客户端MAC地址
     */
    QByteArray enrollSchool(QString school_number, QString serial, QString hw_info, QString version);

    /*注销
    * school_number:学校ID
    * serial:序列号
    * hw_info:客户端MAC地址
     */
    int denrollSchool(QString school_number, QString serial, QString hw_info);

    /*登录
    * school_number:学校ID
    * serial:序列号
    * hw_info:客户端MAC地址
     */
    QByteArray login(QString school_number, QString serial, QString hw_info, QString version);

    /*退出
    * school_number:学校ID
    * serial:序列号
    * hw_info:客户端MAC地址
     */
    int logoutImpl(QString school_number, QString serial, QString hw_info);
    int logout();

    // 获取最新版本
    QByteArray getLatestVersion();

    /*发送客户端心跳
    * school_number:学校ID
    * serial:序列号
    * hw_info:客户端MAC地址
     */
    int sendHeartBeat(QString school_number, QString serial, QString hw_info);

    /*获取课表
     * version:版本
     *grade:年级
     * level：上下册
    */
    QByteArray getCourseList(QString type);

    /*获取培训版节次列表
     * code: 讲次code
    */
    QByteArray getLectureList(QString code);

    /*获取培训版节次文字列表
     * code: 节次code
    */
    QByteArray getLectureWordList(QString code);

    /*获取课程里的生字
     *course_code:课程ID
    */
    QByteArray getCourseWordList(QString course_code);

    //获取写字姿势信息列表
    QByteArray getXieZiZiShiList(QString code);

    /*获取部首列表
     * group_code:字头、字底、包围、左偏旁、右偏旁
    */
    QByteArray getBuShouList(QString group_code);

    //获取基本笔画列表
    QByteArray getStrokesList(QString group_code);

    /*获取古诗书写列表
     *grade:年级
    */
    QByteArray getPoemList(QString grade);

    /*获取古诗书内容列表
     *poem_id: code
    */
    QByteArray getPoemItem(QString poem_id);

    //获取硬笔作品列表
    QByteArray getHardPenList();

    /*获取书法课堂讲次列表
     *grade:年级
     * level：上下册
    */
    QByteArray getCalligraphyList(QString grade, QString level, QString type);

    /*获取书法课堂节次列表
     *calligraphy_id:讲次ID
    */
    QByteArray getCalligraphyLessonList(QString calligraphy_id);

    /*获取书法课堂第三节详细内容
     *calligraphy_id:讲次ID
    */
    QByteArray getCalligraphyLessonktlxList(QString calligraphy_id);

    /*获取指定字的详细信息
     *word:指定的汉字
    */
    QByteArray getWordInfo(QString word);

    /*获取指定字的五体书法
     *word:指定的汉字
    */
    QByteArray getCalligraphyWord(QString word);

    //------------------------- 生字表 ------------------------------------------
    /*获取拼音的Index列表 (A,B,C,M,L,N...)
     * version:版本
     * grade:年级
     * level:上下册
    */
    QByteArray getPinyinIndexList(QString version, QString grade, QString level);

    /*获取部首的笔画列表 (1,2,3,6,7,8...)
     * version:版本
     * grade:年级
     * level:上下册
    */
    QByteArray getBushouIndexList(QString version, QString grade, QString level);

    /*根据拼音的Index获取对应的拼音列表 (查询index为 A 的所有拼音)
     * version:版本
     * grade:年级
     * level:上下册
     * index:拼音首字母
    */
    QByteArray getPinyinList(QString version, QString grade, QString level, QString index);

    /*根据部首的笔画数获取对应的部首列表 (查询笔画数为 2 画 的所有部首)
     * version:版本
     * grade:年级
     * level:上下册
     * index:部首笔画数
    */
    QByteArray getBushouList(QString version, QString grade, QString level, QString index);

    /*获取该拼音对应的生字表中总笔画数的列表 (查询拼音 zhang 对应的所有生字，返回这些字包含的笔画数的列表)
     * version:版本
     * grade:年级
     * level:上下册
     * pinyin:拼音
    */
    QByteArray getZiBihuaIndexList(QString version, QString grade, QString level, QString pinyin);

    /*获取该部首对应的生字表中外部笔画数的列表(同上)
     * version:版本
     * grade:年级
     * level:上下册
    */
    QByteArray getWbBihuaIndexList(QString version, QString grade, QString level, QString bushou);

    /*根据拼音和总笔画数获取生字列表
     * version:版本
     * grade:年级
     * level:上下册
    */
    QByteArray getWordByPinyinBihua(QString version, QString grade, QString level, QString pinyin, QString bihua);

    /*根据部首和外部笔画数获取生字列表
     * version:版本
     * grade:年级
     * level:上下册
    */
    QByteArray getWordByBushouBihua(QString version, QString grade, QString level, QString bushou, QString bihua);

    /*根据拼音和总笔画数获取生字列表
     * version:版本
     * grade:年级
     * level:上下册
    */
    QByteArray getWordByPinyin(QString version, QString grade, QString level, QString pinyin);

    /*根据部首和外部笔画数获取生字列表
     * version:版本
     * grade:年级
     * level:上下册
    */
    QByteArray getWordByBushou(QString version, QString grade, QString level, QString bushou);
    //------------------------- 生字表 ----------------------------------------------------------

    /*
     * 根据key值获取书写要领, 欢迎页图片, 欢迎文字, 客服电话
     * klxz_writegist: 书写要领
     * klxz_welcome: 欢迎页图片
     * klxz_welcometext: 欢迎文字
     * klxz_service_telephone: 客服电话
    */

    /*获取字的视频列表
     * version:版本
     * grade:年级
     * level:上下册
    */
    QByteArray getWord(QString version, QString grade, QString level);


    /*获取全文跟读的视频列表
     * version:版本
     * grade:年级
     * level:上下册
    */
    QByteArray getCourse(QString version, QString grade, QString level);


    /*获取书法课堂的视频列表
     * grade:年级
     * level:上下册
     * type: 用笔类型
    */
    QByteArray getCalligraphylesson(QString grade, QString level, QString type);


    /*获取写字姿势的视频列表
     *
    */
    QByteArray getXzzs();

    /*获取部首的视频列表
     *
    */
    QByteArray getBushou();

    /*获取书写章法的视频列表
     * grade:年级
    */
    QByteArray getPoemitem(QString grade);

    /*获取配置信息 欢迎页图片 文字 电话等
     * key: 关键字
    */
    QByteArray getOption(QString key);

    //获取根url
    QString getBaseUrl();

    //获取版本列表
    QByteArray getVersionList();

    /*通用函数
    * 负责调用REST API
    */
    QByteArray getReply(QString url);

    /*获取说文解字的详细信息
     *word:指定的汉字
    */
    QByteArray getSwjzInfo(QString word);

    ~RestAPI();
private:

    QString m_BaseURL;
    QNetworkAccessManager * m_NetManager;
    QNetworkCookieJar * m_CookieJar;
    QTimer * m_QTimer;
    QString m_SchoolID;
    QString m_Serial;
    QString m_HWInfo;

    static RestAPI * restApiInstance;
    Tool* tool = Tool::getToolInstance();
    Middle *middle = NULL;

//signals:

public slots:
    int slot_SendHeartBeat();
};



#endif // RESTAPI_H
