#include "sfkt2page.h"
#include <QDebug>
#include <QStringList>
#include <QStringListModel>
#include <QScrollBar>

Sfkt2Page* Sfkt2Page::sfkt2Instance = NULL;

Sfkt2Page::Sfkt2Page(QObject *parent) : QObject(parent)
{
    middle = Middle::getMiddleInstance();
    sfkt3 = Sfkt3Page::getSfkt3Instance();
}

Sfkt2Page* Sfkt2Page::getSfkt2Instance()
{
    if (sfkt2Instance == NULL)
        sfkt2Instance = new Sfkt2Page();

    return sfkt2Instance;
}

void Sfkt2Page::setWidget(QWidget *sfkt2, QStackedWidget *stack, QWidget *next)
{
    sfkt2Page = sfkt2;
    nextPage = next;
    stackWidget = stack;

    // 窗口组件初始化
    bottomImage = new QLabel(sfkt2Page);
    leftText = new QLabel(sfkt2Page);
    ybImage = new QLabel(sfkt2Page);
    mbImage = new QLabel(sfkt2Page);
    ybView = new QListView(sfkt2Page);
    ybView->hide();
    mbView = new QListView(sfkt2Page);
    mbView->hide();
    btnBack = new ImageButton(sfkt2Page);

    connect(ybView,SIGNAL(clicked(QModelIndex)),this,SLOT(ybItemClicked(QModelIndex)));
    connect(mbView,SIGNAL(clicked(QModelIndex)),this,SLOT(mbItemClicked(QModelIndex)));
    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
}

void Sfkt2Page::setSfkt2Page(QWidget *parent, QString gradeNum, QString grade_, QString levelNum, QString level_, ImageButton *btnP)
{
    parentPage = parent;
    btnParent = btnP;
    grade = grade_;
    level = level_;
    stop = false;

    setBackGround();
    setText(grade_, level_);
    setButtons();
    setListView(gradeNum, levelNum);
}

void Sfkt2Page::setBackGround()
{
    sfkt2Page->setStyleSheet("QWidget#sfkt2Page{image:url(:/sfkt/erji/bj_15.jpg)}");

    bottomImage->setGeometry(179,574,620,50);
    bottomImage->setStyleSheet("image:url(:/sfkt/erji/szkt_gg.png)");

    ybImage->setGeometry(224,70,124,39);
    ybImage->setStyleSheet("image:url(:/sfkt/erji/szkt_ybz.png)");
    ybImage->hide();

    mbImage->setGeometry(552,70,124,39);
    mbImage->setStyleSheet("image:url(:/sfkt/erji/szkt_mbz.png)");
    mbImage->hide();
}

void Sfkt2Page::setText(QString grade, QString level)
{
    QString text = grade + level;
    QFont font;
    QPalette pa;

    pa.setColor(QPalette::WindowText, QColor(0,47,124));
    font.setFamily("宋体");
    font.setPixelSize(45);
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);
    font.setBold(true);

    leftText->setGeometry(99,80,50,300);
    leftText->setText(text.split("", QString::SkipEmptyParts).join("\n"));
    leftText->setFont(font);
    leftText->setPalette(pa);
}

void Sfkt2Page::setListView(QString gradeNum, QString levelNum)
{
    QList<QVariant> ybList, mbList;
    QStringList ybListModel, mbListModel;
    QStringListModel *ybModel, *mbModel;

    ybInfo.clear();
    mbInfo.clear();
    ybList = middle->getCalligraphyList(gradeNum, levelNum, "0");
    for (int i=0; i<ybList.count(); i++) {
        if (stop)
            return;
        QVariant tmp = ybList[i];
        ybListModel.insert(i,"第"+tmp.toMap()["number"].toString()+"周: " + tmp.toMap()["title"].toString());
        ybInfo.append(tmp.toMap()["code"].toString());
    }

    mbList = middle->getCalligraphyList(gradeNum, levelNum, "1");
    for (int i=0; i<mbList.count(); i++) {
        if (stop)
            return;
        QVariant tmp = mbList[i];
        mbListModel.insert(i,"第"+tmp.toMap()["number"].toString()+"周: " + tmp.toMap()["title"].toString());
        mbInfo.append(tmp.toMap()["code"].toString());
    }

    if (ybList.count() == 0)
        ybImage->hide();
    else
        ybImage->show();

    if (mbList.count() == 0)
        mbImage->hide();
    else
        mbImage->show();

    // 硬笔课
    ybModel = new QStringListModel(ybListModel);
    ybView->setModel(ybModel);
    ybView->setGeometry(224,130,250,430);
    ybView->show();
    ybView->setSpacing(4);
    ybView->setCursor(QCursor(Qt::PointingHandCursor));
    ybView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    // 去掉滚动条
    ybView->verticalScrollBar()->setStyleSheet("QScrollBar:vertical{width:6px;background:transparent;background-color:rgb(248, 248, 248);margin:0px,0px,0px,0px;padding-top:10px;padding-bottom:10px;}"
                       "QScrollBar::handle:vertical{width:6px;background:lightgray ;border-radius:5px;min-height:20px;}"
                       "QScrollBar::handle:vertical:hover{width:6px;background:gray;border-radius:5px;min-height:20px;}"
                       "QScrollBar::add-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/down.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/up.png);subcontrol-position:top;}"
                       "QScrollBar::add-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/down_mouseDown.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/up_mouseDown.png);subcontrol-position:top;}"
                       "QScrollBar::add-page:vertical,QScrollBar::sub-page:vertical{background:transparent;border-radius:5px;}");
//    ybView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    ybView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    // 去边框
    ybView->setFrameStyle(0);
    // 设置样式
    ybView->setStyleSheet("QListView{font-size: 18px; font-family:宋体}"
                          "QListView::item:selected:!active{background-color: transparent}"
                          "QListView::item:hover:active{background-color: rgb(251,151,5)}"
                          "QListView::item:hover{background-color: rgb(251,151,5)}"
                          "QListView{background-color:transparent}");

    // 毛笔课
    mbModel = new QStringListModel(mbListModel);
    mbView->setModel(mbModel);
    mbView->setGeometry(552,130,250,430);
    mbView->show();
    mbView->setSpacing(4);
    mbView->setCursor(QCursor(Qt::PointingHandCursor));
    mbView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    // 去掉滚动条
    mbView->verticalScrollBar()->setStyleSheet("QScrollBar:vertical{width:6px;background:transparent;background-color:rgb(248, 248, 248);margin:0px,0px,0px,0px;padding-top:10px;padding-bottom:10px;}"
                       "QScrollBar::handle:vertical{width:6px;background:lightgray ;border-radius:5px;min-height:20px;}"
                       "QScrollBar::handle:vertical:hover{width:6px;background:gray;border-radius:5px;min-height:20px;}"
                       "QScrollBar::add-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/down.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/up.png);subcontrol-position:top;}"
                       "QScrollBar::add-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/down_mouseDown.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/up_mouseDown.png);subcontrol-position:top;}"
                       "QScrollBar::add-page:vertical,QScrollBar::sub-page:vertical{background:transparent;border-radius:5px;}");
//    mbView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    mbView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    // 去边框
    mbView->setFrameStyle(0);
    // 设置样式
    mbView->setStyleSheet("QListView{font-size: 18px; font-family:宋体}"
                          "QListView::item:selected:!active{background-color: transparent}"
                          "QListView::item:hover:active{background-color: rgb(251,151,5)}"
                          "QListView::item:hover{background-color: rgb(251,151,5)}"
                          "QListView{background-color:transparent}");
}

void Sfkt2Page::setButtons()
{
    // 返回
    btnBack->setGeometry(837,503,109,110);
    QPixmap backP1(":/sfkt/fh_10.png");
    QPixmap backP2(":/sfkt/fh_10.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void Sfkt2Page::ybItemClicked(QModelIndex index)
{
    ybView->blockSignals(true);
    mbView->blockSignals(true);
    int i = index.row();
    QString title = index.data().toString();
//    qDebug() << title << ybInfo[i];

    stackWidget->setCurrentWidget(nextPage);
    sfkt3->setSfkt3Page(sfkt2Page, ybInfo[i], ybView, mbView, title, grade + level);
}

void Sfkt2Page::mbItemClicked(QModelIndex index)
{
    int i = index.row();
    QString title = index.data().toString();
//    qDebug() << title << mbInfo[i];

    stackWidget->setCurrentWidget(nextPage);
    sfkt3->setSfkt3Page(sfkt2Page, mbInfo[i], ybView, mbView, title, grade + level);
}

void Sfkt2Page::onBackClick()
{
    stop = true;
    btnParent->blockSignals(false);
    stackWidget->setCurrentWidget(parentPage);
    QStringListModel *listModel = new QStringListModel(QStringList{});
    mbView->setModel(listModel);
    ybView->setModel(listModel);
}
