#include "sfkt3page.h"
#include "sfktvideopage.h"
#include <QStringList>
#include <QStringListModel>
#include <QDebug>
#include <QScrollBar>

Sfkt3Page* Sfkt3Page::sfkt3Instance = NULL;

Sfkt3Page::Sfkt3Page(QObject *parent) : QObject(parent)
{
    middle = Middle::getMiddleInstance();
    sfktVideo = SfktVideoPage::getSfktVideoInstance();
}

Sfkt3Page* Sfkt3Page::getSfkt3Instance()
{
    if (sfkt3Instance == NULL)
        sfkt3Instance = new Sfkt3Page();

    return sfkt3Instance;
}

void Sfkt3Page::setWidget(QWidget *sfkt3, QStackedWidget *stack, QWidget *next)
{
    sfkt3Page = sfkt3;
    nextPage = next;
    stackWidget = stack;

    bottomImage = new QLabel(sfkt3Page);
    btnBack = new ImageButton(sfkt3Page);
    listView = new QListView(sfkt3Page);
    listView->hide();
    titleLabel = new QLabel(sfkt3Page);
    leftLabel = new QLabel(sfkt3Page);

    connect(listView,SIGNAL(clicked(QModelIndex)),this,SLOT(onItemClicked(QModelIndex)));
    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
}

void Sfkt3Page::setSfkt3Page(QWidget *parent, QString code, QListView *ybParentView, QListView *mbParentView, QString title, QString leftText)
{
    parentPage = parent;
    ybView = ybParentView;
    mbView = mbParentView;
    stop = false;

    setBackGround();
    setText(title, leftText);
    setButtons();
    setListView(code);
}

void Sfkt3Page::setBackGround()
{
    sfkt3Page->setStyleSheet("QWidget#sfkt3Page{image:url(:/sfkt/erji/bj_15.jpg)}");

    bottomImage->setGeometry(179,574,620,50);
    bottomImage->setStyleSheet("image:url(:/sfkt/erji/szkt_gg.png)");
}

void Sfkt3Page::setText(QString title, QString leftText)
{
    QFont fontLeft, fontTitle;
    QPalette paLeft, paTitle;

    paTitle.setColor(QPalette::WindowText,QColor(251,151,5));
    fontTitle.setFamily("黑体");
    fontTitle.setPixelSize(40);
    fontTitle.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontTitle.setBold(true);

    paLeft.setColor(QPalette::WindowText,QColor(0,47,124));
    fontLeft.setFamily("宋体");
    fontLeft.setPixelSize(45);
    fontLeft.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontLeft.setBold(true);

    titleLabel->setText(title);
    titleLabel->setAlignment(Qt::AlignCenter);
    titleLabel->setFont(fontTitle);
    titleLabel->setPalette(paTitle);
    titleLabel->setGeometry(0,65,1000,50);

    leftLabel->setGeometry(99,80,50,300);
    leftLabel->setText(leftText.split("", QString::SkipEmptyParts).join("\n"));
    leftLabel->setFont(fontLeft);
    leftLabel->setPalette(paLeft);
}

void Sfkt3Page::setListView(QString code)
{
    QList<QVariant> lessonList;
    QStringList lessonStringList;
    QStringListModel *lessonListModel;

    videoList.clear();
    lessonList = middle->getCalligraphyLessonList(code);
    for (int i=0; i<lessonList.count(); i++) {
        if (stop)
            return;
        QVariant tmp = lessonList[i];
        lessonStringList.insert(i,"第"+tmp.toMap()["number"].toString()+"节  " +
                tmp.toMap()["title"].toString() +
                "  ("+tmp.toMap()["times"].toString()+"分钟)");
        videoList.append(tmp.toMap()["video_url"].toString() + "," + tmp.toMap()["videosize"].toString());
        if (i == 2)
            thirdCode = tmp.toMap()["calligraphy_id"].toString();
    }

    lessonListModel = new QStringListModel(lessonStringList);
    listView->setModel(lessonListModel);
    listView->setGeometry(335,172,500,400);
    listView->show();
    listView->setSpacing(4);
    listView->setCursor(QCursor(Qt::PointingHandCursor));
    listView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    // 去掉滚动条
    listView->verticalScrollBar()->setStyleSheet("QScrollBar:vertical{width:6px;background:transparent;background-color:rgb(248, 248, 248);margin:0px,0px,0px,0px;padding-top:10px;padding-bottom:10px;}"
                       "QScrollBar::handle:vertical{width:6px;background:lightgray ;border-radius:5px;min-height:20px;}"
                       "QScrollBar::handle:vertical:hover{width:6px;background:gray;border-radius:5px;min-height:20px;}"
                       "QScrollBar::add-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/down.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/up.png);subcontrol-position:top;}"
                       "QScrollBar::add-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/down_mouseDown.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/up_mouseDown.png);subcontrol-position:top;}"
                       "QScrollBar::add-page:vertical,QScrollBar::sub-page:vertical{background:transparent;border-radius:5px;}");
//    listView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    listView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    // 去边框
    listView->setFrameStyle(0);
    // 设置样式
    listView->setStyleSheet("QListView{font-size: 30px; font-family:宋体}"
                            "QListView::item:hover:active{background-color: rgb(251,151,5)}"
                            "QListView::item:hover{background-color: rgb(251,151,5)}"
                             "QListView{background-color:transparent}");

}

void Sfkt3Page::setButtons()
{
    // 返回
    btnBack->setGeometry(837,503,109,110);
    QPixmap backP1(":/sfkt/fh_10.png");
    QPixmap backP2(":/sfkt/fh_10.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void Sfkt3Page::onItemClicked(QModelIndex index)
{
    QStringList list = videoList[index.row()].split(",");
    listView->blockSignals(true);
    qDebug() << "sfkt" << __FUNCTION__ <<index.data().toString() << videoList[index.row()] << thirdCode;

    sfktVideo->setVideoPage(sfkt3Page, listView, list[0], list[1].toInt(), index.row(), thirdCode);
    stackWidget->setCurrentWidget(nextPage);
}

void Sfkt3Page::onBackClick()
{
    stop = true;
    ybView->blockSignals(false);
    mbView->blockSignals(false);
    ybView->clearSelection();
    mbView->clearSelection();
    stackWidget->setCurrentWidget(parentPage);
    QStringListModel *listModel = new QStringListModel(QStringList{});
    listView->setModel(listModel);
}
