#include "sfktvideopage.h"
#include "imagebutton.h"
#include <QLabel>
#include <QPixmap>
#include <QDebug>
#include <QApplication>
#include <QDesktopWidget>
#include <QFont>
#include <QScrollBar>
#include "windows.h"

SfktVideoPage* SfktVideoPage::sfktVideoInstance = NULL;

SfktVideoPage::SfktVideoPage(QWidget *parent) : QWidget(parent)
{
    isFullScreen = false;
    middle = Middle::getMiddleInstance();
    video = new Video();
//    connect(video, SIGNAL(endReached()), this, SLOT(onVideoEndReached()));
}

SfktVideoPage* SfktVideoPage::getSfktVideoInstance()
{
    if (sfktVideoInstance == NULL)
        sfktVideoInstance = new SfktVideoPage();

    return sfktVideoInstance;
}

void SfktVideoPage::setWidget(QWidget *sfktVideo, QStackedWidget *stack, QWidget *root,
                              MyLabel *video, QSlider *slider, QLabel *timeLabel, QLabel *lenLabel,
                              QSlider *_volSlider, ImageButton *play, ImageButton *vol, ImageButton *full, QLabel *ctlRect)
{
    sfktVideoPage = sfktVideo;
    stackWidget = stack;

    rootWidget = root;
    videoLabel = video;

    BgLabel = new QLabel(sfktVideoPage);
    btnBack = new ImageButton(sfktVideoPage);
    imageList = new QListWidget(sfktVideoPage);
    imageList->hide();

    posSlider = slider;
    timer = new QTimer();
    videoCtlTimer = new QTimer();
    curTimeLabel = timeLabel;
    lengthLabel = lenLabel;

    posSlider->installEventFilter(this);

    videoBgLabel = new QLabel(sfktVideoPage);
    volSlider = _volSlider;
    btnPlay = play;
    btnVol = vol;
    btnFull = full;
    videoCtlRect = ctlRect;
    volSlider->installEventFilter(this);

    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
    connect(videoLabel, SIGNAL(LabelDoubleClicked()), this, SLOT(onVideoDoubleClick()));
//    connect(videoLabel, SIGNAL(LabelClicked()), this, SLOT(onVideoClick()));
    connect(videoLabel, SIGNAL(LabelMouseMove()), this, SLOT(onMouseMove()));

    connect(posSlider, SIGNAL(sliderMoved(int)), this, SLOT(changePosition(int)));
//    connect(posSlider, SIGNAL(sliderPressed()), this, SLOT(onPressed()));
    connect(timer, SIGNAL(timeout()), this, SLOT(updateInfo()));
    connect(videoCtlTimer, SIGNAL(timeout()), this, SLOT(hideVideoCtl()));

    connect(btnPlay, SIGNAL(clicked()), this, SLOT(onVideoClick()));
    connect(btnFull, SIGNAL(clicked()), this, SLOT(onVideoDoubleClick()));
    connect(btnVol, SIGNAL(clicked()), this, SLOT(onVolClick()));
    connect(volSlider, SIGNAL(sliderMoved(int)), this, SLOT(onVolSliderMoved(int)));
    connect(imageList,SIGNAL(clicked(QModelIndex)),this,SLOT(itemClicked(QModelIndex)));
}
void SfktVideoPage::setVideoPage(QWidget *parent, QListView *listViewP, QString videoUrl,
                                 int videoSize, int num, QString code)
{
    QFont font;
    QFont fontVideo;
    QPalette paVideo;

    font.setFamily("黑体");
    font.setPixelSize(15);

    QPalette pa;
    pa.setColor(QPalette::WindowText,QColor(211,211,211));

    fontVideo.setFamily("楷体");
    fontVideo.setPixelSize(30);
    fontVideo.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontVideo.setBold(true);
    paVideo.setColor(QPalette::WindowText,QColor(2,173,237));

    parentPage = parent;
    listViewParent = listViewP;
    stop  = false;
    videoStop = false;
    // 设置背景图片
    BgLabel->setGeometry(0,0,1000,640);
    BgLabel->setStyleSheet("image:url(:/sfkt/erji/bj_15.jpg)");

    initVideoCtl();
    showVideoCtl();

    videoLabel->show();
//    videoLabel->setStyleSheet("background-color:red");
    // videoLabel->setGeometry(stackWidget->x()+149,stackWidget->y()+83,700,378);
    videoLabel->setGeometry(stackWidget->x()+180,stackWidget->y()+115,640,360);
    videoLabel->setFont(fontVideo);
    videoLabel->setPalette(paVideo);
    videoLabel->setAlignment(Qt::AlignCenter);
    videoLabel->clear();
    if(num != 2) {
        if (videoUrl == "0") {
            videoLabel->setText("视频正在努力制作中，敬请期待！");
        }
    //    video->loadVlc("c:\\1周.mp4", (void*)videoLabel->winId());
        QString videoUrl_ = middle->getVideoPath("ggxz", videoUrl, videoSize);
        qDebug() << "sfktvideopage" << __FUNCTION__ << "videoUrl:" << videoUrl_;
        video->loadVlc(videoUrl_, (void*)videoLabel->winId());
        video->setVolume(middle->getVolume());
        video->play();

        setVideoCtl();
        initVolCtl();
        middle->download(videoUrl, videoSize);
    }

    setViewList(num, code);

    // 设置返回按钮
    setBackButton();
}

void SfktVideoPage::setBackButton()
{
    // 返回
    btnBack->setGeometry(837,503,109,110);
    QPixmap backP1(":/sfkt/fh_10.png");
    QPixmap backP2(":/sfkt/fh_10.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void SfktVideoPage::onBackClick()
{
    stop = true;
    timer->stop();
    listViewParent->blockSignals(false);
    listViewParent->clearSelection();
    video->stop();
    videoLabel->hide();
    stackWidget->setCurrentWidget(parentPage);
    hideVideoCtl();
    if (videoCtlTimer->isActive()) {
        videoCtlTimer->stop();
    }
    imageList->clear();
    lengthLabel->clear();
    curTimeLabel->clear();
}

void SfktVideoPage::onVideoDoubleClick()
{
    qDebug() << __FUNCTION__ << "video double click";
    if (isFullScreen) {
        isFullScreen = false;
        emit showNormal();

        videoLabel->setGeometry(stackWidget->x()+180,stackWidget->y()+115,640,360);
        videoLabel->show();

        showVideoCtl();
        posSlider->setGeometry(stackWidget->x()+185,stackWidget->y()+483,630,8);
        curTimeLabel->setGeometry(stackWidget->x()+227,stackWidget->y()+503,80,20);
        lengthLabel->setGeometry(stackWidget->x()+312,stackWidget->y()+503,100,20);
        btnPlay->setGeometry(stackWidget->x()+195,stackWidget->y()+503,18,20);
        btnVol->setGeometry(stackWidget->x()+727,stackWidget->y()+478,20,18);
        btnFull->setGeometry(stackWidget->x()+777,stackWidget->y()+503,22,16);
        volSlider->setGeometry(stackWidget->x()+752,stackWidget->y()+483,60,8);

        if (videoCtlTimer->isActive()) {
            videoCtlTimer->stop();
        }
    } else {
        emit fullScreen();
        isFullScreen = true;
        videoLabel->resize(rootWidget->width(),QApplication::desktop()->height());
        videoLabel->move(0,0);
        videoLabel->show();

        posSlider->setGeometry(10,QApplication::desktop()->height()-60,rootWidget->width()-20,8);
        curTimeLabel->setGeometry(60,QApplication::desktop()->height()-40,80,20);
        lengthLabel->setGeometry(145,QApplication::desktop()->height()-40,100,20);
        btnPlay->setGeometry(30,QApplication::desktop()->height()-40,18,20);
        btnVol->setGeometry(rootWidget->width()-120,QApplication::desktop()->height()-65,20,18);
        btnFull->setGeometry(rootWidget->width()-70,QApplication::desktop()->height()-40,22,16);
        volSlider->setGeometry(rootWidget->width()-90,QApplication::desktop()->height()-60,60,8);
        videoCtlRect->setGeometry(0,QApplication::desktop()->height()-65,rootWidget->width(),65);
        videoCtlRect->show();

        videoCtlRect->raise();
        posSlider->raise();
        curTimeLabel->raise();
        lengthLabel->raise();
        btnPlay->raise();
        btnVol->raise();
        btnFull->raise();
        volSlider->raise();

        if (videoCtlTimer->isActive()) {
            videoCtlTimer->stop();
        }
        videoCtlTimer->start(5000);
    }
}

void SfktVideoPage::onVideoClick()
{
    if (videoStop) {
        video->stop();
        video->play();

        setPauseIcon();
        videoStop = false;
        return;
    }
    if (video->getVideoState()) {
        setPlayIcon();
        video->pause();
    } else {
        setPauseIcon();
        video->play();
    }
}

void SfktVideoPage::changePosition(int pos)
{
    qDebug() << pos;
    float posf = (float)(pos)/100.0;
    video->setPosition(posf);
}

void SfktVideoPage::onPressed()
{
    int pos = posSlider->value();
    qDebug() << pos;
    float posf = (float)(pos)/100.0;
    video->setPosition(posf);
}

void SfktVideoPage::updateInfo()
{
    curTimeLabel->setText(video->getTime());
    lengthLabel->setText("/ " + video->getLength());

    if (video->getLengthInt() > 0 && (video->getLengthInt()-video->getTimeInt()) < 1000) {
        videoStop = true;
        video->pause();
        setPlayIcon();
    }

    if (video->getLengthInt() != 0) {
        int pos = video->getTimeInt()*100/video->getLengthInt();
        qDebug() << pos;
        posSlider->setValue(pos);
    }
}

bool SfktVideoPage::eventFilter(QObject *obj, QEvent *event)
{
    if(obj==posSlider)
    {
        if (event->type()==QEvent::MouseButtonPress)           //判断类型
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton)	//判断左键
            {
               int dur = posSlider->maximum() - posSlider->minimum();
               int pos = posSlider->minimum() + dur * ((double)mouseEvent->x() / posSlider->width());
               if(pos != posSlider->sliderPosition())
                {
                   changePosition(pos);
                }
            }

        }
    } else if (obj==volSlider) {
        if (event->type()==QEvent::MouseButtonPress)           //判断类型
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton)	//判断左键
            {
               int dur = volSlider->maximum() - volSlider->minimum();
               int pos = volSlider->minimum() + dur * ((double)mouseEvent->x() / volSlider->width());
               if(pos != volSlider->sliderPosition())
                {
                  volSlider->setValue(pos);
                  qDebug() << "vol value:" << pos;
                  video->setVolume(pos);
                  middle->setVolume(pos);
                }
            }

        }
    }
    return QObject::eventFilter(obj,event);
}

void SfktVideoPage::resizeVideo()
{
    if (!videoLabel->isHidden()) {
        videoLabel->setGeometry(stackWidget->x()+180,stackWidget->y()+115,640,360);
        posSlider->setGeometry(stackWidget->x()+185,stackWidget->y()+483,630,8);
        curTimeLabel->setGeometry(stackWidget->x()+227,stackWidget->y()+503,80,20);
        lengthLabel->setGeometry(stackWidget->x()+312,stackWidget->y()+503,100,20);
        btnPlay->setGeometry(stackWidget->x()+195,stackWidget->y()+503,18,20);
        btnVol->setGeometry(stackWidget->x()+727,stackWidget->y()+478,20,18);
        btnFull->setGeometry(stackWidget->x()+777,stackWidget->y()+503,22,16);
        volSlider->setGeometry(stackWidget->x()+752,stackWidget->y()+483,60,8);
    }
}

void SfktVideoPage::hideVideoCtl()
{
    posSlider->hide();
    curTimeLabel->hide();
    lengthLabel->hide();
    volSlider->hide();
    btnPlay->hide();
    btnVol->hide();
    btnFull->hide();
    videoBgLabel->hide();
    videoCtlRect->hide();
}

void SfktVideoPage::showVideoCtl()
{
    posSlider->show();
    curTimeLabel->show();
    lengthLabel->show();
//    volSlider->show();
    btnPlay->show();
//    btnVol->show();
    btnFull->show();
    videoBgLabel->show();
}

//void SfktVideoPage::updateVideoCtl()
//{
//    posSlider->hide();
//    curTimeLabel->hide();
//    lengthLabel->hide();
//}

void SfktVideoPage::onMouseMove()
{
    if (isFullScreen) {
        showVideoCtl();
        videoCtlRect->show();

        if (videoCtlTimer->isActive()) {
            videoCtlTimer->stop();
        }
        videoCtlTimer->start(5000);
    }
}

void SfktVideoPage::onVideoEndReached()
{
    video->stop();
    QPixmap playP1(":/player/play01.png");
    QPixmap playP2(":/player/play02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    btnPlay->repaint();
}

void SfktVideoPage::initVideoCtl()
{
    QFont font;
    font.setFamily("宋体");
    font.setPixelSize(20);

    QPalette pa;
    pa.setColor(QPalette::WindowText,QColor(211,211,211));

    // 设置控制栏背景
    //videoLabel->setGeometry(stackWidget->x()+174,stackWidget->y()+116,640,500);
    // videoLabel->setGeometry(stackWidget->x()+50,stackWidget->y()+86,640,470);
    // videoLabel->setGeometry(stackWidget->x()+149,stackWidget->y()+83,700,380);
    videoBgLabel->setGeometry(180,453,640,60);
    videoBgLabel->raise();
    videoBgLabel->hide();
    videoBgLabel->setStyleSheet("background-color: black");

    // 设置进度条
    posSlider->raise();
    posSlider->setGeometry(stackWidget->x()+185,stackWidget->y()+483,630,8);

    // 设置视频时间显示
    curTimeLabel->setGeometry(stackWidget->x()+227,stackWidget->y()+503,80,20);
    curTimeLabel->setFont(font);
    curTimeLabel->setPalette(pa);
    curTimeLabel->raise();
    curTimeLabel->setAlignment(Qt::AlignRight);
    lengthLabel->setGeometry(stackWidget->x()+312,stackWidget->y()+503,100,20);
    lengthLabel->setFont(font);
    lengthLabel->setPalette(pa);
    lengthLabel->raise();
    lengthLabel->setAlignment(Qt::AlignLeft);
    timer->start(1000);

    // 设置控制按钮图标
    QPixmap playP1(":/player/play01.png");
    QPixmap playP2(":/player/play02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    QPixmap volP1(":/player/vol01.png");
    QPixmap volP2(":/player/vol02.png");
    btnVol->setNormalImage(volP1);
    btnVol->setPressImage(volP2);
    QPixmap fullP1(":/player/full01.png");
    QPixmap fullP2(":/player/full02.png");
    btnFull->setNormalImage(fullP1);
    btnFull->setPressImage(fullP2);

    // 设置控制按钮坐标
    btnPlay->raise();
    btnPlay->setGeometry(stackWidget->x()+195,stackWidget->y()+503,18,20);
    btnVol->raise();
    btnVol->setGeometry(stackWidget->x()+727,stackWidget->y()+478,20,18);
    btnFull->raise();
    btnFull->setGeometry(stackWidget->x()+777,stackWidget->y()+503,22,16);

    // 设置声音控制
    volSlider->raise();
    volSlider->setGeometry(stackWidget->x()+752,stackWidget->y()+483,60,8);
}

void SfktVideoPage::setVideoCtl()
{
    // 设置播放按钮
    if (video->getVideoState()) {
        setPauseIcon();
    } else {
        setPlayIcon();
    }

    // 设置声音
    int vol = video->getVolume();
    qDebug() << "get volume:" << vol;
    volSlider->setValue(vol);
}

void SfktVideoPage::onVolClick()
{
    qDebug() << video->getMute();

    if (video->getMute()) { //当前状态为静音
        video->setMute(0);
        Common::muteStatus = false;

        QPixmap volP1(":/player/vol01.png");
        QPixmap volP2(":/player/vol02.png");
        btnVol->setNormalImage(volP1);
        btnVol->setPressImage(volP2);
        btnVol->repaint();
    } else {
        video->setMute(1);
        Common::muteStatus = true;

        QPixmap muteP1(":/player/mute01.png");
        QPixmap muteP2(":/player/mute02.png");
        btnVol->setNormalImage(muteP1);
        btnVol->setPressImage(muteP2);
        btnVol->repaint();
    }
}

void SfktVideoPage::initVolCtl()
{
    qDebug() << Common::muteStatus;

    if (Common::muteStatus) { //当前状态为静音
        video->setMute(1);

        QPixmap muteP1(":/player/mute01.png");
        QPixmap muteP2(":/player/mute02.png");
        btnVol->setNormalImage(muteP1);
        btnVol->setPressImage(muteP2);
        btnVol->repaint();
    } else {
        video->setMute(0);

        QPixmap volP1(":/player/vol01.png");
        QPixmap volP2(":/player/vol02.png");
        btnVol->setNormalImage(volP1);
        btnVol->setPressImage(volP2);
        btnVol->repaint();
    }
}

void SfktVideoPage::onVolSliderMoved(int pos)
{
    video->setVolume(pos);
    middle->setVolume(pos);
}

void SfktVideoPage::setViewList(int num, QString code)
{
    QList<QVariant> list;

    if (num != 2) {
        imageList->hide();
        return;
    }

    list = middle->getCalligraphyLessonktlxList(code);
    if (list.count() == 0)
        return;
    QString videoUrl = list[0].toMap()["videourl"].toString();
    int videoSize = list[0].toMap()["videosize"].toInt();

    videoLabel->show();
    qDebug() << "videoLabel" << videoUrl;
    video->stop();
    videoLabel->clear();
    if (videoUrl == "0") {
        videoLabel->setText("视频正在努力制作中，敬请期待！");
    }
//    video->loadVlc("c:\\work\\test2.wmv", (void*)videoLabel->winId());
    QString videoUrl_ = middle->getVideoPath("ggxz", videoUrl, videoSize);
    video->loadVlc(videoUrl_, (void*)videoLabel->winId());
    video->setVolume(middle->getVolume());
    initVolCtl();
    video->play();

    setVideoCtl();
    initVolCtl();

    setView(list);
    middle->download(videoUrl, videoSize);
}

void SfktVideoPage::setView(QList<QVariant> list)
{
    imageList->clear();
    imageList->setGeometry(180,523,640,100);
    imageList->show();
    imageList->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    imageList->setFlow(QListView::TopToBottom);
    //设置QListWidget的显示模式
    imageList->setViewMode(QListView::IconMode);
    //设置QListWidget中单元项的图片大小
    imageList->setIconSize(QSize(80,80));
    //设置QListWidget中单元项的间距
    imageList->setSpacing(12);
    //设置自动适应布局调整（Adjust适应，Fixed不适应），默认不适应
//    imageList->setResizeMode(QListWidget::Adjust);
    //设置不能移动
    imageList->setMovement(QListWidget::Static);
    //去掉边框
    imageList->setFrameStyle(0);
    //设置鼠标样式
    imageList->setCursor(QCursor(Qt::PointingHandCursor));
    // 滚动条样式
    imageList->horizontalScrollBar()->setStyleSheet("QScrollBar:horizontal{height:6px;background:transparent;background-color:rgb(248, 248, 248);margin:0px,0px,0px,0px;padding-left:10px;padding-right:10px;}"
                                                    "QScrollBar::handle:horizontal{height:6px;background:lightgray ;border-radius:5px;min-height:20px;}"
                                                    "QScrollBar::handle:horizontal:hover{height:6px;background:gray;border-radius:5px;min-height:20px;}"
                                                    "QScrollBar::add-line:horizontal{height:10px;width:10px;border-image:url(:/button/images/button/down.png);subcontrol-position:bottom;}"
                                                    "QScrollBar::sub-line:horizontal{height:10px;width:10px;border-image:url(:/button/images/button/up.png);subcontrol-position:top;}"
                                                    "QScrollBar::add-line:horizontal:hover{height:10px;width:10px;border-image:url(:/button/images/button/down_mouseDown.png);subcontrol-position:bottom;}"
                                                    "QScrollBar::sub-line:horizontal:hover{height:10px;width:10px;border-image:url(:/button/images/button/up_mouseDown.png);subcontrol-position:top;}"
                                                    "QScrollBar::add-page:horizontal,QScrollBar::sub-page:vertical{background:transparent;border-radius:5px;}");
    imageList->show();

    imageList->blockSignals(true);

    foreach (QVariant word, list) {
        if (stop)
            return;
        //定义QListWidgetItem对象
        QListWidgetItem *imageItem = new QListWidgetItem;
        //为单元项设置属性
        QPixmap image;
        //TODO
        // 添加BaseUrl
        bool ret = image.loadFromData(middle->getImageData(word.toMap()["imgurl"].toString()));
        if (!ret) {
            Sleep(1000);
            image.loadFromData(middle->getImageData(word.toMap()["imgurl"].toString()));
        }
//        image = image.scaled(QSize(80,80),Qt::IgnoreAspectRatio);
        imageItem->setIcon(QIcon(image));
        //重新设置单元项图片的宽度和高度
        imageItem->setSizeHint(QSize(80,80));
        //将单元项添加到QListWidget中
        imageList->addItem(imageItem);
        imageItem->setData(Qt::WhatsThisRole, word.toMap()["videourl"].toString() + "," + word.toMap()["videosize"].toString());
    }
    imageList->blockSignals(false);
}

void SfktVideoPage::itemClicked(QModelIndex index)
{
    QString data = index.data(Qt::WhatsThisRole).toString();
    QStringList list = data.split(",");

    QString videoUrl = list[0];
    int videoSize = list[1].toInt();

    videoLabel->show();
    qDebug() << "videoLabel" << videoUrl;
    video->stop();
    videoLabel->clear();
    if (videoUrl == "0") {
        videoLabel->setText("视频正在努力制作中，敬请期待！");
    }
//    video->loadVlc("c:\\work\\test2.wmv", (void*)videoLabel->winId());
    QString videoUrl_ = middle->getVideoPath("ggxz", videoUrl, videoSize);
    video->loadVlc(videoUrl_, (void*)videoLabel->winId());
    video->setVolume(middle->getVolume());
    initVolCtl();
    video->play();
    setPauseIcon();
    middle->download(videoUrl, videoSize);
}

void SfktVideoPage::setPlayIcon()
{
    QPixmap playP1(":/player/play01.png");
    QPixmap playP2(":/player/play02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    btnPlay->repaint();
}

void SfktVideoPage::setPauseIcon()
{
    QPixmap playP1(":/player/pause01.png");
    QPixmap playP2(":/player/pause02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    btnPlay->repaint();
}
