#include "sxzf2page.h"
#include <QDebug>
#include "qtooltipper.h"
#include <QToolTip>
#include <QAbstractItemModel>
#include <QScrollBar>

Sxzf2Page* Sxzf2Page::sxzf2Instance = NULL;

Sxzf2Page::Sxzf2Page(QObject *parent) : QObject(parent)
{
    middle = Middle::getMiddleInstance();
    sxzf3Page = Sxzf3Page::getSxzf3Instance();
}

Sxzf2Page* Sxzf2Page::getSxzf2Instance()
{
    if (sxzf2Instance == NULL)
        sxzf2Instance = new Sxzf2Page();

    return sxzf2Instance;
}

void Sxzf2Page::setWidget(QWidget *sxzf2, QStackedWidget *stack, QWidget *next)
{
    sxzf2Page = sxzf2;
    nextPage = next;
    stackWidget = stack;

    // 页面元素初始化
    topImageLabel = new QLabel(sxzf2Page);
    btnBack = new ImageButton(sxzf2Page);
    leftView = new QListView(sxzf2Page);
    leftView->hide();
    rightView = new QListView(sxzf2Page);
    rightView->hide();

    connect(leftView,SIGNAL(clicked(QModelIndex)),this,SLOT(leftItemClicked(QModelIndex)));
    connect(rightView,SIGNAL(clicked(QModelIndex)),this,SLOT(rightItemClicked(QModelIndex)));
    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));

}

void Sxzf2Page::setSxzf2Page(QWidget *parent, ImageButton *btnP, QString grade)
{
    parentPage = parent;
    btnParent = btnP;
    stop = false;

    setBackGround();
    setButtons();
    setList(grade);
}

void Sxzf2Page::setList(QString grade)
{
    QList<QVariant> articleList;
    QStringList leftList, rightList;
    int count;

    articleInfo.clear();
    articleList = middle->getClassicalList(grade);
    count = articleList.count();
    half = qCeil(double(count)/2);

    for (int i=0; i<half; i++) {
        if (stop)
            return;
        QVariant tmp = articleList[i];
        leftList.insert(i, tmp.toMap()["title"].toString());
        QList<QString> info;
        info.append(tmp.toMap()["img_url"].toString());
        info.append(tmp.toMap()["code"].toString());
        articleInfo.append(info);
    }
    qDebug() << __FUNCTION__  << leftList;

    for (int j=half; j<count; j++) {
        if (stop)
            return;
        QVariant tmp = articleList[j];
        rightList.insert(j, tmp.toMap()["title"].toString());
        QList<QString> info;
        info.append(tmp.toMap()["img_url"].toString());
        info.append(tmp.toMap()["code"].toString());
        articleInfo.append(info);
    }
    qDebug() << __FUNCTION__  << rightList;

    setListView(leftList, rightList);
}

void Sxzf2Page::slotEntered(const QModelIndex &index)
{
    qDebug() << __FUNCTION__  << "enter slotEntered";
    QToolTip::showText(QCursor::pos() + QPoint(10, 0), index.data().toString());
}

void Sxzf2Page::setListView(QStringList leftList, QStringList rightList)
{
    QFont font;

    font.setFamily("楷体");
    font.setPixelSize(35);
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);
//    font.setWeight(70);

    QStringListModel *leftListModel = new QStringListModel(leftList);
    QStringListModel *righeListModel = new QStringListModel(rightList);

    leftView->setModel(leftListModel);
    leftView->setGeometry(410,260,260,270);
    leftView->show();
    leftView->setSpacing(7);
    leftView->setCursor(QCursor(Qt::PointingHandCursor));
    leftView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    // 去掉滚动条
    leftView->verticalScrollBar()->setStyleSheet("QScrollBar:vertical{width:6px;background:transparent;background-color:rgb(248, 248, 248);margin:0px,0px,0px,0px;padding-top:10px;padding-bottom:10px;}"
                       "QScrollBar::handle:vertical{width:6px;background:lightgray ;border-radius:5px;min-height:20px;}"
                       "QScrollBar::handle:vertical:hover{width:6px;background:gray;border-radius:5px;min-height:20px;}"
                       "QScrollBar::add-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/down.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/up.png);subcontrol-position:top;}"
                       "QScrollBar::add-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/down_mouseDown.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/up_mouseDown.png);subcontrol-position:top;}"
                       "QScrollBar::add-page:vertical,QScrollBar::sub-page:vertical{background:transparent;border-radius:5px;}");
//    leftView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    leftView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    // 去边框
    leftView->setFrameStyle(0);
    leftView->setFont(font);
    // 设置样式
    leftView->setStyleSheet("QListView::item:selected:!active{background-color: transparent}"
                            "QListView::item:hover:active{background-color: rgb(251,151,5)}"
                            "QListView::item:hover{background-color: rgb(251,151,5)}"
                            "QListView{background-color:transparent}");

    rightView->setModel(righeListModel);
    rightView->setGeometry(675,260,260,270);
    rightView->show();
    rightView->setSpacing(7);
    rightView->setCursor(QCursor(Qt::PointingHandCursor));
    rightView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    // 去掉滚动条
    rightView->verticalScrollBar()->setStyleSheet("QScrollBar:vertical{width:6px;background:transparent;background-color:rgb(248, 248, 248);margin:0px,0px,0px,0px;padding-top:10px;padding-bottom:10px;}"
                       "QScrollBar::handle:vertical{width:6px;background:lightgray ;border-radius:5px;min-height:20px;}"
                       "QScrollBar::handle:vertical:hover{width:6px;background:gray;border-radius:5px;min-height:20px;}"
                       "QScrollBar::add-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/down.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical{height:10px;width:10px;border-image:url(:/button/images/button/up.png);subcontrol-position:top;}"
                       "QScrollBar::add-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/down_mouseDown.png);subcontrol-position:bottom;}"
                       "QScrollBar::sub-line:vertical:hover{height:10px;width:10px;border-image:url(:/button/images/button/up_mouseDown.png);subcontrol-position:top;}"
                       "QScrollBar::add-page:vertical,QScrollBar::sub-page:vertical{background:transparent;border-radius:5px;}");
//    rightView->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    rightView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    // 去边框
    rightView->setFrameStyle(0);
    rightView->setFont(font);
    // 设置样式
    rightView->setStyleSheet("QListView::item:selected:!active{background-color: transparent}"
                             "QListView::item:hover:active{background-color: rgb(251,151,5)}"
                             "QListView::item:hover{background-color: rgb(251,151,5)}"
                             "QListView{background-color:transparent}"); 
}

void Sxzf2Page::setBackGround()
{
//    sxzf2Page->setStyleSheet("image:url(:/sxzf/bj_13.jpg)");
    sxzf2Page->setStyleSheet("QWidget#sxzf2Page{image:url(:/sxzf/bj_13.jpg)}");
    topImageLabel->setGeometry(99,99,331,81);
    topImageLabel->setStyleSheet("image:url(:sxzf/sxzf_gssx.png)");
}

void Sxzf2Page::setButtons()
{
    // 返回
    btnBack->setGeometry(646,522,283,95);
    QPixmap backP1(":/sxzf/fh_09.png");
    QPixmap backP2(":/sxzf/fh_09.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void Sxzf2Page::leftItemClicked(QModelIndex index)
{
    int i = index.row();
    QString title = index.data().toString();
    qDebug() << __FUNCTION__  << i << title;
    qDebug() << __FUNCTION__  << articleInfo[i];
    leftView->blockSignals(true);

    sxzf3Page->setSxzf3Page(sxzf2Page, leftView, articleInfo[i]);
    stackWidget->setCurrentWidget(nextPage);
}

void Sxzf2Page::rightItemClicked(QModelIndex index)
{
    int i = index.row() + half;
    QString title = index.data().toString();
    qDebug() << __FUNCTION__  << i << title;
    qDebug() << __FUNCTION__  << articleInfo[i];
    rightView->blockSignals(true);

    sxzf3Page->setSxzf3Page(sxzf2Page, rightView, articleInfo[i]);
    stackWidget->setCurrentWidget(nextPage);
}

void Sxzf2Page::onBackClick()
{
    stop = true;
    btnParent->blockSignals(false);
    stackWidget->setCurrentWidget(parentPage);
    QStringListModel *listModel = new QStringListModel(QStringList{});
    leftView->setModel(listModel);
    rightView->setModel(listModel);
}
