#include "sxzfvideopage.h"
#include <QDebug>
#include <QApplication>
#include <QDesktopWidget>
#include <QFont>

SxzfVideoPage* SxzfVideoPage::sxzfVideoInstance = NULL;

SxzfVideoPage::SxzfVideoPage(QObject *parent) : QObject(parent)
{
    middle = Middle::getMiddleInstance();
    video = new Video();
//    connect(video, SIGNAL(endReached()), this, SLOT(onVideoEndReached()));
}

SxzfVideoPage* SxzfVideoPage::getSxzfVideoInstance()
{
    if (sxzfVideoInstance == NULL)
        sxzfVideoInstance = new SxzfVideoPage();

    return sxzfVideoInstance;
}

void SxzfVideoPage::setWidget(QWidget *sxzfVideo, QStackedWidget *stack, QWidget *root, QWidget *next,
                              MyLabel *video, QSlider *slider, QLabel *timeLabel, QLabel *lenLabel,
                              QSlider *_volSlider, ImageButton *play, ImageButton *vol, ImageButton *full, QLabel *ctlRect)
{
    sxzfVideoPage = sxzfVideo;
    rootWidget = root;
    nextPage = next;
    stackWidget = stack;
    videoLabel = video;

    bottomImageLabel = new QLabel(sxzfVideoPage);
    leftImageLabel = new QLabel(sxzfVideoPage);
    btnBack = new ImageButton(sxzfVideoPage);

    posSlider = slider;
    timer = new QTimer();
    videoCtlTimer = new QTimer();
    curTimeLabel = timeLabel;
    lengthLabel = lenLabel;

    posSlider->installEventFilter(this);

    videoBgLabel = new QLabel(sxzfVideoPage);
    volSlider = _volSlider;
    btnPlay = play;
    btnVol = vol;
    btnFull = full;
    videoCtlRect = ctlRect;
    volSlider->installEventFilter(this);

    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
    connect(videoLabel, SIGNAL(LabelDoubleClicked()), this, SLOT(onVideoDoubleClick()));
//    connect(videoLabel, SIGNAL(LabelClicked()), this, SLOT(onVideoClick()));
    connect(videoLabel, SIGNAL(LabelMouseMove()), this, SLOT(onMouseMove()));

    connect(posSlider, SIGNAL(sliderMoved(int)), this, SLOT(changePosition(int)));
//    connect(posSlider, SIGNAL(sliderPressed()), this, SLOT(onPressed()));
    connect(timer, SIGNAL(timeout()), this, SLOT(updateInfo()));
    connect(videoCtlTimer, SIGNAL(timeout()), this, SLOT(hideVideoCtl()));

    connect(btnPlay, SIGNAL(clicked()), this, SLOT(onVideoClick()));
    connect(btnFull, SIGNAL(clicked()), this, SLOT(onVideoDoubleClick()));
    connect(btnVol, SIGNAL(clicked()), this, SLOT(onVolClick()));
    connect(volSlider, SIGNAL(sliderMoved(int)), this, SLOT(onVolSliderMoved(int)));
}

void SxzfVideoPage::setSxzfVideoPage(QWidget *parent, ImageButton *btnP, QString videoUrl, int videoSize,
                                     QString leftImageUrl, QString bottomImageUrl, QRect videoRect)
{
    parentPage = parent;
    btnParent = btnP;

    setBackGround(leftImageUrl, bottomImageUrl);
    setVideo(videoUrl, videoSize, videoRect);
    setButtons();
}

void SxzfVideoPage::setBackGround(QString leftImageUrl, QString bottomImageUrl)
{
    sxzfVideoPage->setStyleSheet("QWidget#sxzfVideoPage{image:url(:/sxzf/bj_19.jpg)}");

    bottomImageLabel->setGeometry(125,435,750,124);
    QPixmap image;
    qDebug() << __FUNCTION__  << bottomImageUrl;
    image.loadFromData(middle->getImageData(bottomImageUrl));
    bottomImageLabel->setPixmap(image);

    leftImageLabel->setGeometry(137,138,54,239);
    leftImageLabel->setStyleSheet(leftImageUrl);
}

void SxzfVideoPage::setVideo(QString videoUrl, int videoSize, QRect rect)
{
    QFont font;
    QFont fontVideo;
    QPalette paVideo;

    font.setFamily("黑体");
    font.setPixelSize(15);

    QPalette pa;
    pa.setColor(QPalette::WindowText,QColor(211,211,211));

    fontVideo.setFamily("楷体");
    fontVideo.setPixelSize(30);
    fontVideo.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontVideo.setBold(true);
    paVideo.setColor(QPalette::WindowText,QColor(2,173,237));

    videoRect = rect;
    initVideoCtl();
    showVideoCtl();
    videoStop = false;

    videoLabel->show();
    videoLabel->setGeometry(stackWidget->x()+videoRect.x(), stackWidget->y()+videoRect.y(), videoRect.width(), videoRect.height()-35);
    videoLabel->setFont(fontVideo);
    videoLabel->setPalette(paVideo);
    videoLabel->setAlignment(Qt::AlignCenter);
//    video->loadVlc("c:\\chong2-encyp.mp4", (void*)videoLabel->winId());
    videoLabel->clear();
    if (videoUrl == "0") {
        videoLabel->setText("视频正在努力制作中，敬请期待！");
    }
    QString videoUrl_ = middle->getVideoPath("ggxz", videoUrl, videoSize);
    qDebug() << __FUNCTION__  << "videoUrl:" << videoUrl_;
    video->loadVlc(videoUrl_, (void*)videoLabel->winId());
    video->setVolume(middle->getVolume());
    video->play();

    setVideoCtl();
    initVolCtl();
    middle->download(videoUrl, videoSize);
}

void SxzfVideoPage::setButtons()
{
    // 返回
    btnBack->setGeometry(696,542,283,95);
    QPixmap backP1(":/sxzf/fh_09.png");
    QPixmap backP2(":/sxzf/fh_09.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void SxzfVideoPage::onBackClick()
{
    timer->stop();
    video->stop();
    videoLabel->hide();

    btnParent->blockSignals(false);
    stackWidget->setCurrentWidget(parentPage);
    hideVideoCtl();
    if (videoCtlTimer->isActive()) {
        videoCtlTimer->stop();
    }
    lengthLabel->clear();
    curTimeLabel->clear();
}

void SxzfVideoPage::onVideoDoubleClick()
{
    qDebug() << __FUNCTION__  << "video double click";
    if (isFullScreen) {
        isFullScreen = false;
        emit showNormal();

        videoLabel->setGeometry(stackWidget->x()+videoRect.x(), stackWidget->y()+videoRect.y(), videoRect.width(), videoRect.height()-35);
        videoLabel->show();

        showVideoCtl();
        posSlider->setGeometry(stackWidget->x()+videoRect.x()+5,stackWidget->y()+videoRect.y()+videoRect.height()-30,videoRect.width()-10,8);
        curTimeLabel->setGeometry(stackWidget->x()+videoRect.x()+47,stackWidget->y()+videoRect.y()+videoRect.height()-10,80,20);
        lengthLabel->setGeometry(stackWidget->x()+videoRect.x()+132,stackWidget->y()+videoRect.y()+videoRect.height()-10,100,20);
        btnPlay->setGeometry(stackWidget->x()+videoRect.x()+10,stackWidget->y()+videoRect.y()+videoRect.height()-10,18,20);
        btnVol->setGeometry(stackWidget->x()+videoRect.x()+videoRect.width()-93,stackWidget->y()+videoRect.y()+videoRect.height()-35,20,18);
        btnFull->setGeometry(stackWidget->x()+videoRect.x()+videoRect.width()-43,stackWidget->y()+videoRect.y()+videoRect.height()-10,22,16);
        volSlider->setGeometry(stackWidget->x()+videoRect.x()+videoRect.width()-68,stackWidget->y()+videoRect.y()+videoRect.height()-30,60,8);
        if (videoCtlTimer->isActive()) {
            videoCtlTimer->stop();
        }
    } else {
        emit fullScreen();
        isFullScreen = true;
        videoLabel->resize(rootWidget->width(),QApplication::desktop()->height());
        videoLabel->move(0,0);
        videoLabel->show();

        posSlider->setGeometry(10,QApplication::desktop()->height()-60,rootWidget->width()-20,8);
        curTimeLabel->setGeometry(60,QApplication::desktop()->height()-40,80,20);
        lengthLabel->setGeometry(145,QApplication::desktop()->height()-40,100,20);
        btnPlay->setGeometry(30,QApplication::desktop()->height()-40,18,20);
        btnVol->setGeometry(rootWidget->width()-120,QApplication::desktop()->height()-65,20,18);
        btnFull->setGeometry(rootWidget->width()-70,QApplication::desktop()->height()-40,22,16);
        volSlider->setGeometry(rootWidget->width()-90,QApplication::desktop()->height()-60,60,8);
        videoCtlRect->setGeometry(0,QApplication::desktop()->height()-65,rootWidget->width(),65);
        videoCtlRect->show();

        videoCtlRect->raise();
        posSlider->raise();
        curTimeLabel->raise();
        lengthLabel->raise();
        btnPlay->raise();
        btnVol->raise();
        btnFull->raise();
        volSlider->raise();

        if (videoCtlTimer->isActive()) {
            videoCtlTimer->stop();
        }
        videoCtlTimer->start(5000);
    }
}

void SxzfVideoPage::onVideoClick()
{
    if (videoStop) {
        video->stop();
        video->play();

        setPauseIcon();
        videoStop = false;
        return;
    }
    if (video->getVideoState()) {
        setPlayIcon();
        video->pause();
    } else {
        setPauseIcon();
        video->play();
    }
}

void SxzfVideoPage::changePosition(int pos)
{
    qDebug() << __FUNCTION__  << pos;
    float posf = (float)(pos)/100.0;
    video->setPosition(posf);
}

void SxzfVideoPage::onPressed()
{
    int pos = posSlider->value();
    qDebug() << __FUNCTION__  << pos;
    float posf = (float)(pos)/100.0;
    video->setPosition(posf);
}

void SxzfVideoPage::updateInfo()
{
    curTimeLabel->setText(video->getTime());
    lengthLabel->setText("/ " + video->getLength());

    if (video->getLengthInt() > 0 && (video->getLengthInt()-video->getTimeInt()) < 1100) {
        videoStop = true;
        video->pause();
        setPlayIcon();
    }

    if (video->getLengthInt() != 0) {
        int pos = video->getTimeInt()*100/video->getLengthInt();
        qDebug() << __FUNCTION__  << pos;
        posSlider->setValue(pos);
    }
}

bool SxzfVideoPage::eventFilter(QObject *obj, QEvent *event)
{
    if(obj==posSlider)
    {
        if (event->type()==QEvent::MouseButtonPress)           //判断类型
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton)	//判断左键
            {
               int dur = posSlider->maximum() - posSlider->minimum();
               int pos = posSlider->minimum() + dur * ((double)mouseEvent->x() / posSlider->width());
               if(pos != posSlider->sliderPosition())
                {
                   changePosition(pos);
                }
            }

        }
    } else if (obj==volSlider) {
        if (event->type()==QEvent::MouseButtonPress)           //判断类型
        {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton)	//判断左键
            {
               int dur = volSlider->maximum() - volSlider->minimum();
               int pos = volSlider->minimum() + dur * ((double)mouseEvent->x() / volSlider->width());
               if(pos != volSlider->sliderPosition())
                {
                  volSlider->setValue(pos);
                  qDebug() << __FUNCTION__  << "vol value:" << pos;
                  video->setVolume(pos);
                  middle->setVolume(pos);
                }
            }

        }
    }
    return QObject::eventFilter(obj,event);
}

void SxzfVideoPage::resizeVideo()
{
    if (!videoLabel->isHidden()) {
        videoLabel->setGeometry(stackWidget->x()+videoRect.x(), stackWidget->y()+videoRect.y(), videoRect.width(), videoRect.height()-35);
        posSlider->setGeometry(stackWidget->x()+videoRect.x()+5,stackWidget->y()+videoRect.y()+videoRect.height()-30,videoRect.width()-10,8);
        curTimeLabel->setGeometry(stackWidget->x()+videoRect.x()+47,stackWidget->y()+videoRect.y()+videoRect.height()-10,80,20);
        lengthLabel->setGeometry(stackWidget->x()+videoRect.x()+132,stackWidget->y()+videoRect.y()+videoRect.height()-10,100,20);
        btnPlay->setGeometry(stackWidget->x()+videoRect.x()+10,stackWidget->y()+videoRect.y()+videoRect.height()-10,18,20);
        btnVol->setGeometry(stackWidget->x()+videoRect.x()+videoRect.width()-93,stackWidget->y()+videoRect.y()+videoRect.height()-35,20,18);
        btnFull->setGeometry(stackWidget->x()+videoRect.x()+videoRect.width()-43,stackWidget->y()+videoRect.y()+videoRect.height()-10,22,16);
        volSlider->setGeometry(stackWidget->x()+videoRect.x()+videoRect.width()-68,stackWidget->y()+videoRect.y()+videoRect.height()-30,60,8);
    }
}

void SxzfVideoPage::hideVideoCtl()
{
    posSlider->hide();
    curTimeLabel->hide();
    lengthLabel->hide();
    volSlider->hide();
    btnPlay->hide();
    btnVol->hide();
    btnFull->hide();
    videoBgLabel->hide();
    videoCtlRect->hide();
}

void SxzfVideoPage::showVideoCtl()
{
    posSlider->show();
    curTimeLabel->show();
    lengthLabel->show();
//    volSlider->show();
    btnPlay->show();
//    btnVol->show();
    btnFull->show();
    videoBgLabel->show();
}

//void SxzfVideoPage::updateVideoCtl()
//{
//    posSlider->hide();
//    curTimeLabel->hide();
//    lengthLabel->hide();
//}

void SxzfVideoPage::onMouseMove()
{
    if (isFullScreen) {
        showVideoCtl();
        videoCtlRect->show();

        if (videoCtlTimer->isActive()) {
            videoCtlTimer->stop();
        }
        videoCtlTimer->start(5000);
    }
}

void SxzfVideoPage::onVideoEndReached()
{
    video->stop();
    setPlayIcon();
}

void SxzfVideoPage::initVideoCtl()
{
    QFont font;
    font.setFamily("宋体");
    font.setPixelSize(20);

    QPalette pa;
    pa.setColor(QPalette::WindowText,QColor(211,211,211));

    // 设置控制栏背景

    videoBgLabel->setGeometry(videoRect.x(),videoRect.y()+videoRect.height()-60,videoRect.width(),60);
    videoBgLabel->raise();
    videoBgLabel->hide();
//    videoBgLabel->show();
    videoBgLabel->setStyleSheet("background-color: black");


    // 设置进度条
    posSlider->raise();
    posSlider->setGeometry(stackWidget->x()+videoRect.x()+5,stackWidget->y()+videoRect.y()+videoRect.height()-30,videoRect.width()-10,8);

    // 设置视频时间显示
    curTimeLabel->setGeometry(stackWidget->x()+videoRect.x()+47,stackWidget->y()+videoRect.y()+videoRect.height()-10,80,20);
    curTimeLabel->setFont(font);
    curTimeLabel->setPalette(pa);
    curTimeLabel->raise();
    curTimeLabel->setAlignment(Qt::AlignRight);
    lengthLabel->setGeometry(stackWidget->x()+videoRect.x()+132,stackWidget->y()+videoRect.y()+videoRect.height()-10,100,20);
    lengthLabel->setFont(font);
    lengthLabel->setPalette(pa);
    lengthLabel->raise();
    lengthLabel->setAlignment(Qt::AlignLeft);
    timer->start(1000);

    // 设置控制按钮图标
    QPixmap playP1(":/player/play01.png");
    QPixmap playP2(":/player/play02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    QPixmap volP1(":/player/vol01.png");
    QPixmap volP2(":/player/vol02.png");
    btnVol->setNormalImage(volP1);
    btnVol->setPressImage(volP2);
    QPixmap fullP1(":/player/full01.png");
    QPixmap fullP2(":/player/full02.png");
    btnFull->setNormalImage(fullP1);
    btnFull->setPressImage(fullP2);

    // 设置控制按钮坐标
    btnPlay->raise();
    btnPlay->setGeometry(stackWidget->x()+videoRect.x()+10,stackWidget->y()+videoRect.y()+videoRect.height()-10,18,20);
    btnVol->raise();
    btnVol->setGeometry(stackWidget->x()+videoRect.x()+videoRect.width()-93,stackWidget->y()+videoRect.y()+videoRect.height()-35,20,18);
    btnFull->raise();
    btnFull->setGeometry(stackWidget->x()+videoRect.x()+videoRect.width()-43,stackWidget->y()+videoRect.y()+videoRect.height()-10,22,16);



    // 设置声音控制
    volSlider->raise();
    volSlider->setGeometry(stackWidget->x()+videoRect.x()+videoRect.width()-68,stackWidget->y()+videoRect.y()+videoRect.height()-30,60,8);
}

void SxzfVideoPage::setVideoCtl()
{
    // 设置播放按钮
    if (video->getVideoState()) {
        setPauseIcon();
    } else {
        setPlayIcon();
    }

    // 设置声音
    int vol = video->getVolume();
    qDebug() << __FUNCTION__  << "get volume:" << vol;
    volSlider->setValue(vol);
}

void SxzfVideoPage::onVolClick()
{
    qDebug() << __FUNCTION__  << video->getMute();

    if (video->getMute()) { //当前状态为静音
        video->setMute(0);
        Common::muteStatus = false;

        QPixmap volP1(":/player/vol01.png");
        QPixmap volP2(":/player/vol02.png");
        btnVol->setNormalImage(volP1);
        btnVol->setPressImage(volP2);
        btnVol->repaint();
    } else {
        video->setMute(1);
        Common::muteStatus = true;

        QPixmap muteP1(":/player/mute01.png");
        QPixmap muteP2(":/player/mute02.png");
        btnVol->setNormalImage(muteP1);
        btnVol->setPressImage(muteP2);
        btnVol->repaint();
    }
}

void SxzfVideoPage::initVolCtl()
{
    qDebug() << __FUNCTION__  << Common::muteStatus;

    if (Common::muteStatus) { //当前状态为静音
        video->setMute(1);

        QPixmap muteP1(":/player/mute01.png");
        QPixmap muteP2(":/player/mute02.png");
        btnVol->setNormalImage(muteP1);
        btnVol->setPressImage(muteP2);
        btnVol->repaint();
    } else {
        video->setMute(0);

        QPixmap volP1(":/player/vol01.png");
        QPixmap volP2(":/player/vol02.png");
        btnVol->setNormalImage(volP1);
        btnVol->setPressImage(volP2);
        btnVol->repaint();
    }
}

void SxzfVideoPage::onVolSliderMoved(int pos)
{
    video->setVolume(pos);
    middle->setVolume(pos);
}

void SxzfVideoPage::setPlayIcon()
{
    QPixmap playP1(":/player/play01.png");
    QPixmap playP2(":/player/play02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    btnPlay->repaint();
}

void SxzfVideoPage::setPauseIcon()
{
    QPixmap playP1(":/player/pause01.png");
    QPixmap playP2(":/player/pause02.png");
    btnPlay->setNormalImage(playP1);
    btnPlay->setPressImage(playP2);
    btnPlay->repaint();
}
