#include "sztxpage.h"
#include <QAbstractItemView>
#include <qmath.h>
#include <QDebug>

SztxPage* SztxPage::sztxInstance = NULL;

SztxPage::SztxPage(QWidget *parent) : QWidget(parent)
{
    middle = Middle::getMiddleInstance();
    sztxSubPage = SztxSubPage::getSztxSubInstance();
}

SztxPage* SztxPage::getSztxInstance()
{
    if (sztxInstance == NULL) {
        sztxInstance = new SztxPage();
    }

    return sztxInstance;
}

void SztxPage::setWidget(QWidget *sztx, QStackedWidget *stack, QWidget *next)
{
    sztxPage = sztx;
    nextPage = next;
    stackWidget = stack;

//    bgLabel = new QLabel(sztxPage);
    image = new QLabel(sztxPage);
    titleText = new QLabel(sztxPage);
//    containerList = new QListWidget(sztxPage);
//    containerList->hide();
    imageList = new QListWidget(sztxPage);
    imageList->hide();
//    wordItemList = new ItemWidget(sztxPage);
//    wordItemList->hide();
//    groupItemList = new ItemWidget(sztxPage);
//    groupItemList->hide();
//    chengYuItemList = new ItemWidget(sztxPage);
//    chengYuItemList->hide();
    btnOk = new ImageButton(sztxPage);
    btnCancle = new ImageButton(sztxPage);
    checkBox = new QCheckBox(sztxPage);

    connect(btnOk, SIGNAL(clicked()), this, SLOT(onOkClick()));
    connect(btnCancle, SIGNAL(clicked()), this, SLOT(onCancleClick()));
    connect(checkBox, SIGNAL(clicked(bool)), this, SLOT(onCheckBoxClick(bool)));
}

void SztxPage::setSztxPage(QWidget *parent, QList<QVariant> wordList, QList<QVariant> groupWordList,
                           QList<QVariant> chengYuList, QString title, QList<QString> videoInfo_,
                           ImageButton *btnP, QString grade, QString type, QWidget *flashP)
{
    parentPage = parent;
    btnParent = btnP;
    videoInfo = videoInfo_;
    title_ = title;
    flashPage = flashP;

    setBackGround(title);
    setButtons();
    setItemStyle();

    if (type == "zctx" && grade == "1") {
        chengYuList.clear();
    }
    if (type == "zctx" && grade != "1") {
        wordList.clear();
    }

    if (type == "whr") {
        groupWordList.clear();
        chengYuList.clear();
    }

    setWordList(wordList, groupWordList, chengYuList);
}

void SztxPage::setBackGround(QString title)
{
    QFont fontTitle, fontCheck;
    QPalette pa;

    pa.setColor(QPalette::WindowText,QColor(251,151,5));
    fontTitle.setFamily("黑体");
    fontTitle.setPixelSize(40);
    fontTitle.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontTitle.setBold(true);

    fontCheck.setFamily("楷体");
    fontCheck.setPixelSize(25);
    fontCheck.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontCheck.setBold(true);

    sztxPage->setStyleSheet("QWidget#sztxPage{image:url(:/nianji/yiji/bj_03.jpg)}");

    titleText->setText(title);
    titleText->setAlignment(Qt::AlignCenter);
    titleText->setFont(fontTitle);
    titleText->setPalette(pa);
    titleText->setGeometry(0,35,1000,50);

    image->setGeometry(50,130,907,535);
    image->setStyleSheet("image:url(:/nianji/tingxie/bj_021.png)");

    checkBox->setGeometry(797,168,100,50);
    checkBox->setText("全选");
    checkBox->setFont(fontCheck);
}

void SztxPage::setItemStyle()
{
    imageList->clear();
    imageList->setSelectionMode(QAbstractItemView::MultiSelection);
    imageList->setGeometry(140,211,720,390);
    imageList->show();
    imageList->setFocusPolicy(Qt::NoFocus);
    imageList->setMovement(QListWidget::Static);
    imageList->setFlow(QListView::TopToBottom);
    imageList->setSpacing(1);
    // 去掉滚动条
    imageList->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    imageList->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    //去掉选中的背景框
    imageList->setSelectionMode(QAbstractItemView::NoSelection);
    //去掉边框
    imageList->setFrameStyle(0);
    imageList->setStyleSheet("background-color:transparent");
    /*
    //设置QListWidget的显示模式
    imageList->setViewMode(QListView::IconMode);
    //设置QListWidget中单元项的图片大小
    imageList->setIconSize(QSize(320,80));
    //设置QListWidget中单元项的间距
    imageList->setSpacing(12);
    //设置自动适应布局调整（Adjust适应，Fixed不适应），默认不适应
    imageList->setResizeMode(QListWidget::Adjust);
    //设置不能移动
    imageList->setMovement(QListWidget::Static);
    //去掉边框
    imageList->setFrameStyle(0);
    //设置鼠标样式
    imageList->setCursor(QCursor(Qt::PointingHandCursor));
    // 去掉滚动条
    imageList->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    imageList->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    imageList->show();
    */
}

void SztxPage::setItems(ItemWidget **widget, int row_count, QList<QVariant> list, QString type)
{
    int height;
    int tmp = qCeil(double(list.count())/row_count);
    QListWidgetItem *listItem = new QListWidgetItem();
    *widget = new ItemWidget;

    if (tmp == 0)
        height = 0;
    else
        height = tmp*85;

    (*widget)->setSztxItemWidget(720, height, list, type);
    listItem->setSizeHint(QSize(720,height));

    imageList->addItem(listItem);
    imageList->setItemWidget(listItem, *widget);
}

void SztxPage::setWordList(QList<QVariant> wordList, QList<QVariant> groupWordList, QList<QVariant> chengYuList)
{
    setItems(&wordItemList, 6, wordList, "word");
    setItems(&groupItemList, 4, groupWordList, "groupword");
    setItems(&chengYuItemList, 3, chengYuList, "idiom");
}

void SztxPage::setButtons()
{
    // 确定
    btnOk->setGeometry(865,410,109,101);
    QPixmap okP1(":/nianji/tingxie/sztx_qd.png");
    QPixmap okP2(":/nianji/tingxie/sztx_qd.png");
    btnOk->setNormalImage(okP1);
    btnOk->setPressImage(okP2);
    btnOk->setCursor(QCursor(Qt::PointingHandCursor));

    // 取消
    btnCancle->setGeometry(855,518,109,101);
    QPixmap cancleP1(":/nianji/tingxie/sztx_qx.png");
    QPixmap cancleP2(":/nianji/tingxie/sztx_qx.png");
    btnCancle->setNormalImage(cancleP1);
    btnCancle->setPressImage(cancleP2);
    btnCancle->setCursor(QCursor(Qt::PointingHandCursor));
}

void SztxPage::onOkClick()
{
    QList<QString> listenUrl;
    QString url;

    btnParent->blockSignals(false);

    QList<QListWidgetItem*> word = wordItemList->getSelectedItems();
    foreach (QListWidgetItem *item, word) {
        url = item->data(Qt::WhatsThisRole).toString();
        if (url.length() > 0) {
            qDebug() << __FUNCTION__  << url;
            listenUrl.append(item->data(Qt::WhatsThisRole).toString());
        }
    }

    QList<QListWidgetItem*> groupWord = groupItemList->getSelectedItems();
    foreach (QListWidgetItem *item, groupWord) {
        url = item->data(Qt::WhatsThisRole).toString();
        if (url.length() > 0) {
            qDebug() << __FUNCTION__  << url;
            listenUrl.append(item->data(Qt::WhatsThisRole).toString());
        }
    }

    QList<QListWidgetItem*> chengYu = chengYuItemList->getSelectedItems();
    foreach (QListWidgetItem *item, chengYu) {
        url = item->data(Qt::WhatsThisRole).toString();
        if (url.length() > 0) {
            qDebug() << __FUNCTION__  << url;
            listenUrl.append(item->data(Qt::WhatsThisRole).toString());
        }
    }

    if (listenUrl.length() == 0) {
        middle->errorMBox("请至少选择一个字");
        return;
    }

    stackWidget->setCurrentWidget(nextPage);
    sztxSubPage->setSztxPage(sztxPage, title_, videoInfo, listenUrl, btnOk, flashPage);
}

void SztxPage::onCancleClick()
{
    checkBox->setChecked(false);
    btnParent->blockSignals(false);
    stackWidget->setCurrentWidget(parentPage);
    titleText->clear();
    imageList->clear();
}

void SztxPage::onCheckBoxClick(bool check)
{
    qDebug() << __FUNCTION__  << check;
    if (check) {
//        imageList->selectAll();
//        groupItemList->selectAll();
        wordItemList->selectAllItems();
        groupItemList->selectAllItems();
        chengYuItemList->selectAllItems();
    }
    else {
        wordItemList->clearAllItems();
        groupItemList->clearAllItems();
        chengYuItemList->clearAllItems();
    }
}
