#include "szzbsubpage.h"
#include <QDebug>

SzzbSubPage* SzzbSubPage::szzbSubInstance = NULL;

SzzbSubPage::SzzbSubPage(QObject *parent) : QObject(parent)
{
    middle = Middle::getMiddleInstance();
    wordPage = WordPage::getWordPageInstance();
//    szzbPage = SzzbPage::getSzzbInstance();
}

SzzbSubPage* SzzbSubPage::getSzzbSubInstance()
{
    if (szzbSubInstance == NULL)
        szzbSubInstance = new SzzbSubPage();

    return szzbSubInstance;
}

void SzzbSubPage::setWidget(QWidget *szzbSub, QStackedWidget *stack, QWidget *next)
{
    szzbSubPage = szzbSub;
    nextPage = next;
    stackWidget = stack;

//    btnPinyin = new ImageButton(szzbSubPage);
//    btnBushou = new ImageButton(szzbSubPage);
    btnBack = new ImageButton(szzbSubPage);
    indexWidget = new QTableWidget(szzbSubPage);
    indexWidget->hide();
    imageList = new QListWidget(szzbSubPage);
    imageList->hide();

    connect(indexWidget, SIGNAL(clicked(QModelIndex)), this, SLOT(onIndexItemClicked(QModelIndex)));
    connect(imageList,SIGNAL(clicked(QModelIndex)),this,SLOT(onItemClicked(QModelIndex)));
    QObject::connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClick()));
}

void SzzbSubPage::setSzzbSubPage(QWidget *parent, QListWidget *listP, QString index_, QString type_, QString grade_, QString level_)
{
    parentPage = parent;
    listParent = listP;
    type = type_;
    itemIndex = index_;
    grade = grade_;
    level = level_;
    stop = false;

    setBackGround();
    setButtons();
    setIndexList();
    setListView("");
}

void SzzbSubPage::setBackGround()
{
    szzbSubPage->setStyleSheet("background-color: rgb(73, 127, 191);");
}

void SzzbSubPage::setIndexList()
{
    if (type == "pinyin"){
        indexList = middle->getZiBihuaIndexList(itemIndex, grade, level);
    } else {
        indexList = middle->getWbBihuaIndexList(itemIndex, grade, level);
    }

    indexWidget->setGeometry(0,0,80,640);
    indexWidget->show();

    QHeaderView* headerViewV = indexWidget->verticalHeader();
    QHeaderView* headerViewH = indexWidget->horizontalHeader();
    headerViewV->setHidden(true);
    headerViewH->setHidden(true);

    indexWidget->setRowCount(indexList.count());
    indexWidget->setColumnCount(1);
    indexWidget->setColumnWidth(0,80);

    indexWidget->setFrameShape(QFrame::NoFrame); //设置边框
    indexWidget->setEditTriggers(QAbstractItemView::NoEditTriggers);
    indexWidget->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);//去掉水平滚动条
    indexWidget->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);  //去掉垂直滚动条
    indexWidget->setAlternatingRowColors(true);
    indexWidget->setFocusPolicy(Qt::NoFocus); //去除选中虚线框
    indexWidget->setCursor(QCursor(Qt::PointingHandCursor));

    indexWidget->setStyleSheet("QTableWidget{font-size: 20px; font-family:宋体}"
                               "QTableWidget::item:hover{background-color: rgb(251,151,5)}"
                               "QTableWidget::item:hover{color: white}"
                               "QTableWidget{background-color:white}");

    for (int i=0; i<indexList.count(); i++) {
        indexWidget->setRowHeight(i, 46);
        QVariant tmp = indexList[i];
        if (type == "pinyin") {
            indexWidget->setItem(i, 0, new QTableWidgetItem(tmp.toMap()["zi_bihua"].toString() + "画"));
        } else {
            indexWidget->setItem(i, 0, new QTableWidgetItem("剩" + tmp.toMap()["bw_bihua"].toString() + "画"));
        }
        indexWidget->item(i,0)->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
    }
}

void SzzbSubPage::setListView(QString bihua)
{
    QFont font;

    font.setFamily("HaiCi Dict");
    font.setPixelSize(20);
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);
//    font.setBold(true);

    if (bihua.length() > 0) {
        if (type == "pinyin") {
            itemList = middle->getWordByPinyinBihua(itemIndex, bihua, grade, level);
        } else {
            itemList = middle->getWordByBushouBihua(itemIndex, bihua, grade, level);
        }
    } else {
        if (type == "pinyin") {
            itemList = middle->getWordByPinyin(itemIndex, grade, level);
        } else {
            itemList = middle->getWordByBushou(itemIndex, grade, level);
        }
    }

    // 设置表格样式
    imageList->clear();
    imageList->setGeometry(80,35,855,560);
    imageList->show();
    //设置QListWidget的显示模式
    imageList->setViewMode(QListView::IconMode);
    //设置QListWidget中单元项的图片大小
    imageList->setIconSize(QSize(83,110));
    //设置QListWidget中单元项的间距
    imageList->setSpacing(12);
    //设置自动适应布局调整（Adjust适应，Fixed不适应），默认不适应
    imageList->setResizeMode(QListWidget::Adjust);
    //设置不能移动
    imageList->setMovement(QListWidget::Static);
    //去掉边框
    imageList->setFrameStyle(0);
    //设置鼠标样式
    imageList->setCursor(QCursor(Qt::PointingHandCursor));
    // 去掉滚动条
    imageList->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    imageList->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    //去掉选中的背景框
    imageList->setSelectionMode(QAbstractItemView::NoSelection);
    imageList->setStyleSheet("QListWidget{background-color:white}");

    //填充表格内容
    imageList->blockSignals(true);
    foreach (QVariant item, itemList) {
        if (stop)
            return;
        //定义QListWidgetItem对象
        QListWidgetItem *imageItem = new QListWidgetItem;
        //为单元项设置属性
        QPixmap image;
        QString imageFile = middle->getImageFile(item.toMap()["img_url"].toString(), "/zi/");
//        image.loadFromData(middle->getImageData(item.toMap()["img_url"].toString()));
        if (imageFile.length() > 0)
            image.load(imageFile);
        else
            image.loadFromData(middle->getImageData(item.toMap()["img_url"].toString()));
//        image = image.scaled(QSize(80,80),Qt::KeepAspectRatio);
        imageItem->setIcon(QIcon(image));
        imageItem->setText(item.toMap()["pinyin"].toString());
        imageItem->setData(Qt::WhatsThisRole, item.toMap()["zi"].toString());
        imageItem->setFont(font);
        //设置文字位置
        imageItem->setTextAlignment(Qt::AlignTop | Qt::AlignHCenter);
//        imageItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
        //重新设置单元项图片的宽度和高度
        imageItem->setSizeHint(QSize(83,110));
        //将单元项添加到QListWidget中
        imageList->addItem(imageItem);
    }
    imageList->blockSignals(false);
}

void SzzbSubPage::setButtons()
{
//    // 拼音查字
//    btnPinyin->setGeometry(248,648,164,46);
//    QPixmap pinyinP1(":szzb/szzb_pycz01.png");
//    QPixmap pinyinP2(":szzb/szzb_pycz02.png");
//    btnPinyin->setNormalImage(pinyinP1);
//    btnPinyin->setPressImage(pinyinP2);
//    btnPinyin->setCursor(QCursor(Qt::PointingHandCursor));
//    QObject::connect(btnPinyin, SIGNAL(clicked()), this, SLOT(onPinyinClick()));

//    // 部首查字
//    btnBushou->setGeometry(456,648,164,46);
//    QPixmap bushouP1(":szzb/pizb_bscz01.png");
//    QPixmap bushouP2(":szzb/pizb_bscz02.png");
//    btnBushou->setNormalImage(bushouP1);
//    btnBushou->setPressImage(bushouP2);
//    btnBushou->setCursor(QCursor(Qt::PointingHandCursor));
//    QObject::connect(btnBushou, SIGNAL(clicked()), this, SLOT(onBushouClick()));

    // 返回
    btnBack->setGeometry(935,470,35,103);
    QPixmap backP1(":szzb/fh.png");
    QPixmap backP2(":szzb/fh2.png");
    btnBack->setNormalImage(backP1);
    btnBack->setPressImage(backP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void SzzbSubPage::onIndexItemClicked(QModelIndex index)
{
    QString bihua;
    QString str = index.data().toString();
    qDebug() << __FUNCTION__  << "str" << str << str.length();
    int bhNum = str.length() - 1;

    if (bhNum < 1) {
        qDebug() << "笔画数异常！";
        return;
    }

    if (type == "pinyin")
        bihua = str.mid(0,bhNum);
    else
        bihua = str.mid(1,bhNum-1);

    qDebug() << bihua;
    setListView(bihua);
}

void SzzbSubPage::onItemClicked(QModelIndex index)
{
    QString word = index.data(Qt::WhatsThisRole).toString();

    qDebug() << __FUNCTION__  << "word" << word;
    imageList->blockSignals(true);
    wordPage->setWordPage(szzbSubPage, word, imageList, grade, level);
    stackWidget->setCurrentWidget(nextPage);
}

void SzzbSubPage::onPinyinClick()
{
//    szzbPage->onPinyinClick();
//    stackWidget->setCurrentWidget(parentPage);
}

void SzzbSubPage::onBushouClick()
{
//    szzbPage->onBushouClick();
//    stackWidget->setCurrentWidget(parentPage);
}

void SzzbSubPage::onBackClick()
{
    stop = true;
    if (listParent)
        listParent->blockSignals(false);
    stackWidget->setCurrentWidget(parentPage);
    imageList->clear();
}
