#ifndef VIDEOPAGE_H
#define VIDEOPAGE_H

#include <QWidget>
#include <QStackedWidget>
#include "video.h"
#include "imagebutton.h"
#include "middle.h"
#include "mylabel.h"
#include <QLabel>
#include <QSlider>
#include <QTimer>
#include <QMap>
#include "common.h"

class VideoPage : public QWidget
{
    Q_OBJECT
public:
    explicit VideoPage(QWidget *parent = 0);
    static VideoPage* getVideoPageInstance();

    void setWidget(QWidget *videoP, QStackedWidget *stack, QWidget *root, MyLabel *video,
                   QSlider *slider, QLabel *timeLabel, QLabel *lenLabel, QSlider *_volSlider,
                   ImageButton *play, ImageButton *vol, ImageButton *full, QLabel *ctlRect);
    void setVideoPage(QString bgImage, QString topImage, QString leftImage,
                      QString rightImage, QString videoUrl, int videoSize, ImageButton *btnP, QWidget *parent, QString type_);
    void setVideoPage(QString bgImage, QString topImage, QString leftImage,
                      QString rightImage, QMap<QString, int> videoMap, ImageButton *btnP, QWidget *parent, QString type_);
    void setVideoGeometry(int x, int y, int width, int height);
    void setBottonImage(QRect rect, QString imageUrl);
    bool eventFilter(QObject *obj, QEvent *event);
    void initVideoCtl();
    void showVideoCtl();
    void setVideoCtl();
    void initVolCtl();
    void setPlayIcon();
    void setPauseIcon();
    void playVideos();

signals:
    void fullScreen();
    void showNormal();

private:
    void setBackButton();
    static VideoPage* videoPageInstance;

    Middle *middle = NULL;
    QString type;

    QStackedWidget *stackWidget = NULL;
    QWidget *parentPage = NULL;
    QWidget *rootWidget = NULL;
    QWidget *videoPage = NULL;
    QMap<QString, int> videoMap_;

    bool videoStop;
    bool isFullScreen;
    QTimer *videoCtlTimer;
    QTimer *timer;
    QSlider *posSlider = NULL;
    QLabel *curTimeLabel = NULL;
    QLabel *lengthLabel = NULL;

    Video *video = NULL;
    MyLabel *videoLabel = NULL;
    QLabel *videoBgLabel = NULL;
    ImageButton *btnParent = NULL;
    QSlider *volSlider = NULL;
    ImageButton *btnPlay = NULL;
    ImageButton *btnVol = NULL;
    ImageButton *btnFull = NULL;
    QLabel *videoCtlRect = NULL;

    // 页面元素
    QLabel *BgLabel = NULL;
    QLabel *topLabel = NULL;
    QLabel *leftLabel = NULL;
    QLabel *rightLabel = NULL;
    ImageButton *btnBack = NULL;

public slots:
    void onBackClick();
    void onVideoDoubleClick();
    void onVideoClick();
    void changePosition(int pos);
    void updateInfo();
    void onPressed();
    void resizeVideo();
    void onMouseMove();
    void onVideoEndReached();
    void hideVideoCtl();
    void onVolClick();
    void onVolSliderMoved(int pos);
};

#endif // VIDEOPAGE_H
