#include "whrpage.h"
#include <QDebug>
#include <QFont>

WhrPage* WhrPage::whrInstance = NULL;

WhrPage::WhrPage(QWidget *parent) : QWidget(parent)
{
    middle = Middle::getMiddleInstance();
    wordPage = WordPage::getWordPageInstance();
}

WhrPage* WhrPage::getWhrInstance()
{
    if (whrInstance == NULL) {
        whrInstance = new WhrPage();
    }

    return whrInstance;
}

void WhrPage::setWidget(QWidget *whr, QStackedWidget *stack, QWidget *next)
{
    whrPage = whr;
    nextPage = next;
    stackWidget = stack;

    image = new QLabel(whrPage);
    image->hide();
    title = new QLabel(whrPage);
    title->hide();
    btnBack = new ImageButton(whrPage);
    btnBack->hide();
    imageList = new QListWidget(whrPage);
    imageList->show();

    connect(btnBack, SIGNAL(clicked()), this, SLOT(onBackClicked()));
    connect(imageList,SIGNAL(clicked(QModelIndex)),this,SLOT(itemClicked(QModelIndex)));
}

void WhrPage::setWhrPage(QWidget *parent, QList<QVariant> wordList, ImageButton *btnP,
                         QString grade_, QString level_, QString title_)
{
    parentPage = parent;
    btnParent = btnP;
    grade = grade_;
    level = level_;

    setBackGround(title_);
    setButtons();
    setWordList(wordList);
}

void WhrPage::setWordList(QList<QVariant> wordList)
{
    QFont font;

    font.setFamily("华文楷体");
    font.setPixelSize(65);
    font.setLetterSpacing(QFont::AbsoluteSpacing,1);
//    font.setBold(true);

    imageList->show();
    imageList->setFont(font);
    imageList->setSelectionMode(QAbstractItemView::MultiSelection);
    imageList->setGeometry(130,211,720,390);
    //设置QListWidget中单元项的间距
    imageList->setSpacing(15);
    //设置QListWidget的显示模式
    imageList->setViewMode(QListView::IconMode);
    //设置自动适应布局调整（Adjust适应，Fixed不适应），默认不适应
    imageList->setResizeMode(QListWidget::Adjust);
    //设置不能移动
    imageList->setMovement(QListWidget::Static);
    //去掉边框
    imageList->setFrameStyle(0);
    // 去掉滚动条
    imageList->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    imageList->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    imageList->setCursor(QCursor(Qt::PointingHandCursor));

    imageList->setStyleSheet("QListWidget::item:hover{background-color: rgb(251,151,5);border-color:rgb(251,151,5)}"
                               "QListWidget::item:selected{background:lightgray;}"
                               "QListWidget::item{border-width: 2px; border-style: solid; border-color: rgb(243,112,94);}"
                               "QListWidget{background-color:transparent}");

    foreach (QVariant word, wordList) {
        QListWidgetItem *item = new QListWidgetItem;
        item->setData(Qt::WhatsThisRole, word.toMap()["zi"].toString());
        item->setText(word.toMap()["zi"].toString());
        item->setSizeHint(QSize(80,80));
        imageList->addItem(item);
    }
}

void WhrPage::setBackGround(QString title_)
{
    QFont fontTitle;
    QPalette pa;

    pa.setColor(QPalette::WindowText,QColor(251,151,5));
    fontTitle.setFamily("黑体");
    fontTitle.setPixelSize(40);
    fontTitle.setLetterSpacing(QFont::AbsoluteSpacing,1);
    fontTitle.setBold(true);

    whrPage->setStyleSheet("QWidget#whrPage{image:url(:/nianji/yiji/bj_03.jpg)}");

    title->setText(title_);
    title->setAlignment(Qt::AlignCenter);
    title->setFont(fontTitle);
    title->setPalette(pa);
    title->setGeometry(0,35,1000,50);
    title->show();

    image->show();
    image->setGeometry(50,130,907,535);
    image->setStyleSheet("image:url(:/nianji/yiji/bg_whr.png)");
}

void WhrPage::setButtons()
{
    // 返回
    btnBack->show();
    btnBack->setGeometry(855,518,97,89);
    QPixmap btnbackP1(":/nianji/yiji/whr_fh.png");
    QPixmap btnbackP2(":/nianji/yiji/whr_fh.png");
    btnBack->setNormalImage(btnbackP1);
    btnBack->setPressImage(btnbackP2);
    btnBack->setCursor(QCursor(Qt::PointingHandCursor));
}

void WhrPage::onBackClicked()
{
    btnParent->blockSignals(false);
    stackWidget->setCurrentWidget(parentPage);
    imageList->clear();
}

void WhrPage::itemClicked(QModelIndex index)
{
    imageList->blockSignals(true);
    QString word = index.data(Qt::WhatsThisRole).toString();

    wordPage->setWordPage(whrPage, word, imageList, grade, level);
    stackWidget->setCurrentWidget(nextPage);
}
