#ifndef WORDPAGE_H
#define WORDPAGE_H

#include <QObject>
#include <QWidget>
#include <QAxWidget>
#include <QStackedWidget>
#include <QLabel>
#include <QPixmap>
#include <QListView>
#include <QListWidget>
#include <QItemDelegate>
#include <QScrollArea>
#include <QScrollBar>
#include <QVBoxLayout>
#include <QSlider>
#include <QTimer>
#include "video.h"
#include "middle.h"
#include "imagebutton.h"
#include "sfxspage.h"
#include "mylabel.h"
#include "swjzpage.h"
#include "common.h"
#include "flashpage.h"

class WordPage : public QObject
{
    Q_OBJECT
public:
    explicit WordPage(QObject *parent = 0);
    ~WordPage();

    static WordPage* getWordPageInstance();
    void setWidget(QWidget* word, QStackedWidget *stack, QWidget *next, QWidget *root,
                   MyLabel *video, QWidget *swjz, QSlider *Slider, QLabel *timeLabel,
                   QLabel *lenLabel, QSlider *_volSlider, ImageButton *play,
                   ImageButton *vol, ImageButton *full, QLabel *ctlRect, QWidget *flashP);
    void setWordPage(QWidget *parent, QString word_, QListWidget *listWidgetP, QString grade_, QString level_);
    void setButton();
    void setFlash(QString flashUrl, int flashSize);
    void setVideo();
    void setInfo(QString pinyi, QString bushou, QString zuci, QString zxjg, QString jiegou);
    bool eventFilter(QObject *obj, QEvent *event);
    void initVideoCtl();
    void showVideoCtl();
    void setVideoCtl();
    void initVolCtl();
    void setPlayIcon();
    void setPauseIcon();
    void onDoubleClick();

signals:
    void fullScreen();
    void showNormal();

public slots:
    void bssfClick();
    void gbsfClick();
    void mbsfClick();
    void sfxsClick();
    void onBackClick();
    void onVideoDoubleClick();
    void onVideoClick();
    void onMouseMove();
    void changePosition(int pos);
    void updateInfo();
    void onPressed();
    void resizeVideo();
    void hideVideoCtl();
    void onVideoEndReached();
    void onVolClick();
    void onVolSliderMoved(int pos);

private:
    static WordPage *wordPageInstance;
    Middle *middle = NULL;
    SfxsPage *sfxsPage = NULL;
    SwjzPage *swjz = NULL;
    bool videoStop;
    FlashPage *flashInstance = NULL;

//    QLabel *bgLabel = NULL;
    QLabel *topLabel = NULL;
    QLabel *bsLabel = NULL;
    QLabel *pinyinLabel = NULL;
    QLabel *bushouLabel = NULL;
    QLabel *jiegouLabel = NULL;
    QLabel *jiegouText = NULL;
    QLabel *jiegouRect = NULL;
    ImageButton *bssfButton = NULL;
    ImageButton *gbsfButton = NULL;
    ImageButton *mbsfButton = NULL;
    ImageButton *sfxsButton = NULL;
    ImageButton *btnBack = NULL;
//    QListView *zuciView = NULL;
    QListWidget *listWidgetParent = NULL;
    QWidget *swjzPage = NULL;
    QWidget *flashPage = NULL;

    QScrollArea *scrollArea = NULL;
//    QWidget *zuciWidget = NULL;

    Video *video = NULL;
    MyLabel *videoLabel = NULL;
    QLabel *videoBgLabel = NULL;
    QLabel *sjqpLabel = NULL;
    QAxWidget *flash = NULL;
    QString word;
    bool isFullScreen;
    bool flashIsFull;
    QString bsVideo;
    QString ybVideo;
    QString mbVideo;
    int bsVideoSize;
    int ybVideoSize;
    int mbVideoSize;
    QString grade, level;
    QString swjzFlashUrl;
    int swjzFlashSize;

    QTimer *timer;
    QTimer *videoCtlTimer;
    QSlider *posSlider = NULL;
    QLabel *curTimeLabel = NULL;
    QLabel *lengthLabel = NULL;
    QSlider *volSlider = NULL;
    ImageButton *btnPlay = NULL;
    ImageButton *btnVol = NULL;
    ImageButton *btnFull = NULL;
    QLabel *videoCtlRect = NULL;

    QWidget *wordPage = NULL;
    QWidget *parentPage = NULL;
    QWidget *nextPage = NULL;
    QStackedWidget *stackWidget = NULL;
    QWidget *rootWidget = NULL;
};

class ItemDelegate:public QItemDelegate
{
    Q_OBJECT

    public:ItemDelegate(){}

    void drawDisplay(QPainter*painter,const QStyleOptionViewItem&option,const QRect&rect,const QString&text)const
    {
        painter->drawText(rect.x(),rect.y(),rect.width(),rect.height(),Qt::AlignHCenter,text);
        QItemDelegate::drawDisplay(painter,option,rect,text);
    }
};

#endif // WORDPAGE_H
